//25 11 2003//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// File created with Carad, a CAD source code generator by Martin Waldegger
// Contact the author at martin.waldegger@chello.at or visit
// http://mcad.delphigl.com
//
// This code is distributed in the hope that it will be useful but
// WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
// DISCLAIMED. This includes but is not limited to warranties of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

{$INCLUDE includes.inc}
unit SETHImportForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons,
  OpenGL,
  bg,
  SETHImport;

type
  TFormSETHImport = class(TForm)
    Panel: TPanel;
    Button1: TButton;
    Button2: TButton;
    OpenDialog: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private-Deklarationen }
    LastTime: integer;
  public
    { Public-Deklarationen }
    procedure DrawSETHImport;
    procedure OnIdle(Sender: TObject; var Done: Boolean);
  end;

var
  FormSETHImport: TFormSETHImport;
  DisplayList: Cardinal;

implementation

{$R *.dfm}

var OldWndProc: integer;
function bgWndProc(wnd: HWND; msg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
begin
  case msg of     // check the message
    WM_PAINT    : begin FormSETHImport.DrawSETHImport; Result:=DefWindowProc(wnd,msg,wParam,lParam); end;
    WM_SIZE     : begin bgResizeViewport(LOWORD(lParam),HIWORD(lParam)); Result:=CallWindowProc(Pointer(OldWndProc),wnd,msg,wParam,lParam); end;
    else Result:=CallWindowProc(Pointer(OldWndProc),wnd,msg,wParam,lParam);
  end;
end;

procedure TFormSETHImport.DrawSETHImport;
var elapsed: integer;
begin
  elapsed := integer(GetTickCount) - LastTime;
  LastTime := LastTime + elapsed;
  bgClearScreen;
  glLoadIdentity;
  glTranslatef(0, 0, -150);
  glRotatef(LastTime / 10, 1, 0, 0);
  glRotatef(LastTime / 20, 0, 1, 0);
  glCallList(DisplayList);
  bgSwapBuffers;
end;

procedure TFormSETHImport.OnIdle(Sender: TObject; var Done: Boolean);
begin
  DrawSETHImport;
  Done := False;
end;

procedure TFormSETHImport.FormCreate(Sender: TObject);
begin
  if not InitOpenGL then begin
    ShowError('could not load OpenGL DLLs');
    Close;
  end;
  Panel.HandleNeeded;
  bgWnd := Panel.Handle;
  bgDC := GetDC(Panel.Handle);
  bgRC := CreateRenderingContext(bgDC, bgPixelFormat, bgPFD);
  if bgRC=0 then begin
    ShowError('could not create rendering context');
    Close;
  end;
  ActivateRenderingContext(bgDC, bgRC);

  glGetIntegerv(GL_MAX_TEXTURE_SIZE, @bgMaxTextureSize);
  {$IFDEF GL_13} if GL_13 then glGetIntegerv(GL_MAX_TEXTURE_UNITS, @bgMaxTextureUnits); {$ENDIF}
  glEnable(GL_TEXTURE_2D);

  glClearColor(0.0, 0.0, 1.0, 0.0);    // clear the background color to blue
  glClearDepth(1.0);                   // enables clearing of the depth buffer
  glDepthFunc(GL_LESS);                // the type of depth test to do
  glEnable(GL_DEPTH_TEST);             // enables depth testing
  glShadeModel(GL_SMOOTH);             // enables smooth color shading
  {$IFDEF GL_12} glLightModeli (GL_LIGHT_MODEL_COLOR_CONTROL,GL_SEPARATE_SPECULAR_COLOR); {$ENDIF}
  glEnable(GL_LIGHTING);               // enable lighting
  LightEnable(0);

  glMatrixMode(GL_MODELVIEW);          // set view matrices to identity
  glLoadIdentity;
  bgViewMatrix := MatrixIdentity;
  bgResizeViewport(Panel.Width, Panel.Height);
  bgRun := True;
  DisplayList := glGenLists(1);
  glNewList(DisplayList, GL_COMPILE);
  glBegin(GL_TRIANGLES);
  glVertex3f(0, -1, 0);
  glVertex3f(1, 0, 0);
  glVertex3f(-1, 0, 0);
  glEnd;
  glEndList;

  // initialize environment and objects
  InitSETHImport;

  // make sure that the panel works as wanted
  OldWndProc := SetWindowLong(bgWnd, GWL_WNDPROC, Integer(Addr(bgWndProc)));
  Application.OnIdle := OnIdle;
  LastTime := GetTickCount;
end;

procedure TFormSETHImport.Button2Click(Sender: TObject);
begin
  Halt(0);
end;

(******************************************************************************)

type
  TSETHVertex = packed record
    Position: TVec;
    Normal: TVec;
    TexCoord: TVec2f;
    sTangent: TVec;
    tTangent: TVec;
    TSpaceLight: TVec;
  end;
  PSETHVertex = ^TSETHVertex;
  TSETHVertices = array[0..9999] of TSETHVertex;
  PSETHVertices = ^TSETHVertices;

procedure TFormSETHImport.Button1Click(Sender: TObject);
var
  i, j: integer;
  Stream: TFileStream;
  vnum: integer;
  vert: PSETHVertices;
  inum: integer;
  ind: PIntegerTable;
  seth: PSETHVertex;
  Bodies: integer;
begin
  if OpenDialog.Execute then begin
    Stream := NIL;
    try
      Stream := TFileStream.Create(OpenDialog.FileName, fmOpenRead);
      Stream.Read(Bodies, 4);
      for i := 0 to Bodies-1 do begin
        Stream.Read(vnum, 4);
        GetMem(vert, vnum * sizeof(TSETHVertex));
        Stream.Read(vert^, vnum * sizeof(TSETHVertex));
        Stream.Read(inum, 4);
        GetMem(ind, inum * sizeof(Integer));
        Stream.Read(ind^, inum * sizeof(Integer));

        glNewList(DisplayList, GL_COMPILE);
        glBegin(GL_TRIANGLES);
        for j:=0 to inum-1 do begin
          seth := @vert[ind[j]];
          glNormal3fv(Addr(seth.Normal));
          glTexCoord2fv(Addr(seth.TexCoord));
          glVertex3fv(Addr(seth.Position));
        end;
        glEnd;
        glEndList;

        FreeMem(vert);
        FreeMem(ind);
        Stream.Free;
      end;

    except
      MessageBox(0, 'ERROR LOADING SETH FILE', 'ERROR', MB_OK + MB_ICONERROR);
      if Stream<>NIL then Stream.Free;
      Bodies := 0;
    end;
  end;
end;


end.
