//24 11 2003//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  BaseGraph support routines for Delphi
//  (c) Martin Waldegger
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  You may use this file to create and distribute binaries without paying
//  any fees, but the BaseGraph library itself is not Open Source and must
//  not be distributed outside of the entire Carad package.
//
//  Please read the copyright informations in LEGAL.TXT or visit
//  http://mcad.delphigl.com for further information, tutorials and
//  updates of the Carad software.
//
//  mailto:martin.waldegger@chello.at if you find any bugs, feel that
//  something should be changed or just to give feedback about Carad and the
//  BaseGraph library.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  This code is distributed in the hope that it will be useful but
//  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
//  DISCLAMED. This includes but is not limited to warranties of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

{$INCLUDE includes.inc}
unit bg;

interface

uses {$INCLUDE bglibs.inc};


type
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  simple types and structures
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  Float = GLfloat;

  TFloatTable = array[0..9999] of Float;
  PFloatTable = ^TFloatTable;

  PMatrix4f = ^TMatrix4f;
  TMatrices4f = array[0..9999] of TMatrix4f;
  PMatrices4f = ^TMatrices4f;

  TVec = record
    x,y,z : GLfloat;
  end;
  PVec = ^TVec;
  TVecs = array[0..9999] of TVec;
  PVecs = ^TVecs;
  TVecsTable = array[0..9999] of PVecs;
  PVecsTable = ^TVecsTable;

  TVec2 = record
    x,y : GLfloat;
  end;
  PVec2 = ^TVec2;
  TVecs2 = array [0..9999] of TVec2;
  pVecs2 = ^TVecs2;

  TNameString = String[63];
  PNameString = ^TNameString;

  TRGBA = packed record
            r,g,b,a : byte;    // align rgb for windows bitmaps
          end;
  PRGBA = ^TRGBA;
  TRGBAtable = array[0..4096*4096] of tRGBA; // reserve enough space
  PRGBAtable = ^TRGBAtable;

  TRGBALine = record
    length : integer;
    data : PRGBAtable;
  end;
  PRGBALine = ^TRGBALine;

  TRGBAImage = record
    width, height : integer;
    data : PRGBAtable;
  end;
  PRGBAImage = ^TRGBAImage;

  TRGBABox = record
    width, height, depth : integer;
    data : PRGBAtable;
  end;
  PRGBABox = ^TRGBABox;

  TRGBACubeMap = record
    side : integer;
    positive_x : PRGBATable;
    negative_x : PRGBATable;
    positive_y : PRGBATable;
    negative_y : PRGBATable;
    positive_z : PRGBATable;
    negative_z : PRGBATable;
  end;
  PRGBACubeMap = ^TRGBACubeMap;

  TGLuintTable = array[0..9999] of GLuint;
  PGLuintTable = ^TGLuintTable;

  TIntegerTable = Array[0..9999] of Integer;
  PIntegerTable = ^TIntegerTable;

  TBoolTable = Array[0..9999] of Boolean;
  PBoolTable = ^TBoolTable;

  TVec2f = Array[0..1] of GLfloat;
  PVec2f = ^TVec2f;
  TVecs2f = array [0..9999] of TVec2f;
  PVecs2f = ^TVecs2f;

  TVec3f = Array[0..2] of GLfloat;
  PVec3f = ^TVec2f;
  TVecs3f = array [0..9999] of TVec3f;
  PVecs3f = ^TVecs3f;

  TVec4f = Array[0..3] of GLfloat;
  PVec4f = ^TVec4f;
  TVecs4f = array [0..9999] of TVec4f;
  PVecs4f = ^TVecs4f;

  TQuaternion = record
    x, y, z, w : float;
  end;
  PQuaternion = ^TQuaternion;
  TQuaternions = array [0..9999] of TQuaternion;
  PQuaternions = ^TQuaternions;

  TTexCoord4f = record
    s, t, r, q : GLfloat;
  end;
  PTexCoord4f = ^TTexCoord4f;
  TTexCoords4f = array[0..9999] of TTexCoord4f;
  PTexCoords4f = ^TTexCoords4f;

  TColor4f = record
    r, g, b, a : GLfloat;
  end;
  PColor4f = ^TColor4f;
  TColors4f = array[0..9999] of TColor4f;
  PColors4f = ^TColors4f;

  TTri  = Array[0..2] of Integer; // vertex indices of triangle
  PTri  = ^TTri;
  TTris = Array[0..9999] of TTri;
  PTris = ^TTris;

  TTriShadowInfo = record
    PlaneEq : Array[0..3] of float;    // plane equation of triangle
    neigh   : Array[0..2] of integer;  // neighbours of triangle
    norm    : TVec;                    // normal vector of triangle
    visible : boolean;                 // visibility info
  end;
  PTriShadowInfo = ^TTriShadowInfo;
  TTriShadowInfos = Array[0..9999] of TTriShadowInfo;
  PTriShadowInfos = ^TTriShadowInfos;

  TVertShadowInfo = record
    v : TVec;
    computed : Boolean;
  end;
  PVertShadowInfo = ^TVertShadowInfo;
  TVertShadowInfos = Array[0..9999] of TVertShadowInfo;
  PVertShadowInfos = ^TVertShadowInfos;

  TPlane = record
    Poly    : Array[0..2] of TVec;
    Normal  : TVec;
    OrgDist : float;
  end;
  PPlane = ^TPlane;

  TTransparency = (Opaque, Transparent, Transparent2);

  TShadowStateState = (NoShadowing, FrustumShadowing, FastShadowing, Shadowing);
  TShadowState = record
    State: TShadowStateState;
    SelfShadowing: boolean;
  end;
  PShadowState = ^TShadowState;

  TTGAHeader = packed record
    idlength            : byte;
    colourmaptype       : byte;
    datatypecode        : byte;
    colourmaporigin     : smallint;
    colourmaplength     : smallint;
    colourmapdepth      : byte;
    x_origin            : smallint;
    y_origin            : smallint;
    width               : smallint;
    height              : smallint;
    bitsperpixel        : byte;
    imagedescriptor     : byte;
  end;

  // enumeration of frustum sides
  TFrustumSide = (fsRight, fsLeft, fsBottom, fsTop, fsBack, fsFront);
  // enumeration of normalvector (a,b,c) and distance(d) of the frustum plane
  TPlaneData = (pdA, pdB, pdC, pdD);
  // holds a whole frustum
  TFrustum = Array[TFrustumSide] of Array[TPlaneData] of GLfloat;
  // returns the orientation of a sphere to a plane
  TSphereClassification = (scBehind, scIntersects, scFront);


  TPolyStippleMask = Array[0..127] of GLubyte;

  TCapState = (csIgnore, csEnable, csUpdate, csDisable);

  TLight = record
    Position,
    Ambient,
    Diffuse,
    Specular,
    Shadow        : array[0..3] of GLfloat;
    SpotExponent,
    SpotCutoff    : GLfloat;
    SpotDirection : array[0..2] of GLfloat;
    ConstantAttenuation,
    LinearAttenuation,
    QuadraticAttenuation : GLfloat;
    Enabled,
    Shadowing     : boolean;
  end;
  PLight = ^TLight;

  // patch for for evaluating meshes
  TPatch2D = Array[0..2] of Array[0..2] of TVec;

  // window callback routine
  TWndProc = function (hWnd: HWND;
                       msg : UINT;
                       wParam: WPARAM;
                       lParam: LPARAM): LRESULT; stdcall;
  PWndProc = ^TWndProc;

  // binary file header
  TBFHeader = array[0..15] of integer;
  PBFHeader = ^TBFHeader;




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mesh
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  TMesh = class
  public
    meshVerts    : integer;   // number of verticies in mesh
    meshVert     : PVecs;     // vertex list

    meshTris     : integer;   // number of triangles in mesh;
    meshTri      : PTris;     // triangle list

    meshStaticVert,
    meshStaticTri   : boolean;// info if vertices or triangles should be freed

    meshTransformed : PVecs;  // transformed vertex list
    meshTriMin,
    meshTriMax      : PVecs;     // minimum and maximum triangle coordinates
    meshTSI         : PTriShadowInfos;  // shadow information for triangles
    meshSilhouette  : PIntegerTable;  // 2D-silhouette of vertices
    meshVSI         : PVertShadowInfos; // shadow information for vertices
    meshSilhouetteEdges : integer; // computed silhouette vertices (temporary)

    meshMatrixP  : PMatrix4f; // transformation matrix of mesh
    meshPosition : TVec;      // global position of mesh (temporary)
    meshDist     : float;     // mesh distance to light source (temporary)

    constructor Create(matrix : PMatrix4f);
    constructor CreateByData(matrix : PMatrix4f; vnum, tnum : integer; v,t : pointer);
    destructor Destroy; OVERRIDE;

    procedure Transform;
    procedure TriMinMax;
    function QuickIntersect(mesh : TMesh) : Boolean;
    function Intersect(mesh : TMesh; var intersection : TVec) : Boolean;
    function IntersectSphere(Center : TVec; radius : float) : Boolean;
    function IntersectLine(v1, v2 : TVec; Intersection : PVecs; max : Integer) : integer;
    procedure CreateShadowInfo;
    procedure ComputeShadowPlaneEq;
    procedure ComputeShadowPlaneNorm;
    procedure CastFastShadow(lp: TVec; SelfShadowing: boolean);
    procedure CastShadow(lp : TVec; SelfShadowing: boolean);

    function GetX(y,z : float; var x : float) : boolean;
    function GetY(x,z : float; var y : float) : boolean;
    function GetZ(x,y : float; var z : float) : boolean;

    procedure CreateVertices(verts : integer);
    procedure CreateTriangles(tris : integer);
    procedure SetVertex(index : integer; x, y, z : float);
    procedure SetVertexVec(index : integer; v : TVec);
    procedure SetTriangle(index : integer; i1, i2, i3 : integer);
    procedure SetTriangleTri(index : integer; tri : TTri);

    procedure ReadFromFile(var F : File);
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Obj
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  TObj = class;
  TObjTable = array[0..9999] of TObj;
  PObjTable = ^TObjTable;

  TObjCallback = procedure(Sender: TObj);
  TObjBeforeDrawObjCallback = procedure(Sender: TObj; var CanDisplay: boolean);
  TObjCycleCallback = procedure(Sender: TObj; AnimationIndex, CycleIndex: integer);

  TKeyframe = record
    Key   : integer;       // index of key frame
    Time  : integer;       // time when key frame is to be displayed in milliseconds
  end;
  PKeyframe = ^TKeyFrame;
  TKeyframes = array[0..9999] of TKeyFrame;
  PKeyframes = ^TKeyFrames;


  // skeleton bone joint
  TJoint = record
    Name                   : TNameString; // name of joint
    ParentJointIndex       : integer;  // index of parent joint
    Connection             : TObj;     // pointer to optional connected object (NIL if none)

    LocalX, LocalY, LocalZ,            // local origin of joint
    LocalXA, LocalYA, LocalZA : float; // local original rotation of joint as Euler angles

    AbsoluteTransformation,            // absolute transformation matrix of joint
    RelativeTransformation,            // transformation relative to joint origin
    FinalTransformation    : TMatrix4f;// final transformation matrix applied to vertices

    TranslationKeys,                   // number of translation keyframes
    RotationKeys           : integer;  // number of rotation keyframes
    TranslationKey         : PVecs;    // translation keyframes
    RotationKey            : PQuaternions; // rotation keyframes

    CurrentTranslationKey,             // currently displayed translation keyframe
    CurrentRotationKey     : integer;  // currently displayed rotation keyframe
    PrevTranslation        : TVec;     // previous translation
    PrevRotation           : TQuaternion;// previous rotation

    Tag                    : integer;  // user definable tag
    Data                   : pointer;  // user definable pointer
  end;
  PJoint = ^TJoint;
  TJoints = array[0..9999] of TJoint;
  PJoints = ^TJoints;

  // animation cycle for a joint
  TCycle = record
    CurrentTime,                       // current time of cycle in msecs
    TotalTime              : integer;  // total time of cycle in msecs
    JointIndex             : integer;  // index of animated joint
    TranslationFrames,                 // number of translation frames of joint
    RotationFrames         : integer;  // number of rotation frames of joint
    TranslationFrame,                  // translation frames of cycle
    RotationFrame          : PKeyFrames;// rotation frames of cycle
    Loop                   : boolean;  // true= cycle starts again if finished

    OnRunCycle             : TObjCycleCallback; // callback routine when cycle runs
    OnEndCycle             : TObjCycleCallback; // callback routine when cycle ends/loops
  end;
  PCycle = ^TCycle;
  TCycles = array[0..9999] of TCycle;
  PCycles = ^TCycles;

  // animation consisting of several cycles
  TAnimation = record
    Name                   : TNameString; // this stores the name of the animation (Jump, Pain, etc..)
    Cycles                 : integer;  // number of cycles in animation
    Cycle                  : PCycles;  // animation cycles
    Tag                    : integer;  // user definable integer value
    Data                   : pointer;  // pointer to user definable data
  end;
  PAnimation = ^TAnimation;
  TAnimations = Array[0..9999] of TAnimation;
  PAnimations = ^TAnimations;

  TObj = class
  public
    objHandle : GLuint;            // unique handle for every created object

    objX, objY, objZ : GLfloat;    // position of object in space
    objXS, objYS, objZS : GLfloat; // size of object

    objRMatrix : TMatrix4f;        // rotation matrix of object

    objMin, objMax, objCent : TVec;// minimum, maximum and center of object
    objBSRad : GLfloat;            // radius of bounding sphere

    objChildren : integer;         // number of child objects
    objChild : PObjTable;          // child objects
    objParent : TObj;              // parent object, NIL if undefined

    objTransparency : TTransparency; // transparency information
    objVisible : boolean;          // info if object shall be drawn
    objFrustumTest : boolean;      // false means object is sent to OpenGL even if outside frustum
    objShadowState : TShadowState; // info if object casts shadows

    objMatrix : TMatrix4f;         // transformation matrix of object
    objScaleFactor : GLfloat;      // single biggest scalefactor used on object

    objMesh : TMesh;               // object mesh (used for collision detection)

    objDistance : GLfloat;         // distance of object from viewer

    objOnBeforeDrawObj : TObjBeforeDrawObjCallback; // if set this is called before the obj is drawn
    objOnAfterDrawObj  : TObjCallback; // if set this is called after the obj is drawn

    // the following data is for animation
    objJoints : integer;            // number of joints of object
    objJoint : PJoints;             // object joints
    objAnimations : integer;        // number of animations of object
    objAnimation : PAnimations;     // object animations
    objStaticJoints : boolean;      // joint data is not to be freed
    objStaticAnimations : boolean;  // animation data is not to be freed

    Name : TNameString;             // name of object
    Tag : integer;                  // user definable tag
    Data : Pointer;                 // user definable data pointer

    constructor Create;
    destructor Destroy; override;

    procedure SetName(n : ShortString);

    procedure GetPos(var x,y,z : GLfloat);
    function GetPosVec : TVec;
    procedure SetPos(x,y,z : GLfloat);
    procedure SetPosVec(pos : TVec);
    procedure GetCoordAngle(var xa,ya,za : GLfloat);
    procedure SetCoordAngle(xa,ya,za : GLfloat);
    procedure GetScale(var xs,ys,zs : GLfloat);
    procedure SetScale(xs,ys,zs : GLfloat);

    procedure Move(x,y,z : GLfloat);
    procedure MoveVec(vec : TVec);
    procedure RotateCoord(xa,ya,za : GLfloat);
    procedure CoordAxisUnitLength;

    procedure SetVisible(Visible: boolean);
    function GetTransparency : TTransparency;
    procedure SetTransparency(t : TTransparency);
    function GetShadowState : TShadowState;
    procedure SetShadowState(State : TShadowStateState; SelfShadowing: boolean);

    procedure SetXAxis(ax,ay,az : GLfloat);
    procedure SetXAxisVec(vec : TVec);
    procedure SetYAxis(ax,ay,az : GLfloat);
    procedure SetYAxisVec(vec : TVec);
    procedure SetZAxis(ax,ay,az : GLfloat);
    procedure SetZAxisVec(vec : TVec);
    procedure GetXAxis(var ax,ay,az : GLfloat);
    function GetXAxisVec : TVec;
    procedure GetYAxis(var ax,ay,az : GLfloat);
    function GetYAxisVec : TVec;
    procedure GetZAxis(var ax,ay,az : GLfloat);
    function GetZAxisVec : TVec;
    procedure GetPosAngle(var px,py,pz,ax,ay,az : GLfloat);

    procedure PrepareModelMatrix;
    procedure SetOwnerMatrix;
    procedure SetMatrix(ParentMatrix: PMatrix4f);
    procedure MakeGlobal(var v : TVec);
    procedure GetGlobalRotationMatrix(var rmat : TMatrix4f);

    procedure Transform(var v : TVec);
    procedure LoadGLMatrix;
    procedure MultGLMatrix;
    function GetMatrix : TMatrix4f;
    function GetRMatrix : TMatrix4f;

    function GetParent : TObj;
    function GetChildren : Integer;
    procedure AddChild(chld : TObj);
    function GetChild(index : integer) : TObj;
    function GetChildByName(ChildName : String) : TObj;
    procedure SetChild(index : integer; chld : TObj);
    function IsChild(chld : TObj) : Boolean;
    procedure RemoveChild(chld : TObj);
    procedure RemoveChildIndex(index : integer);
    procedure RemoveChildrenTbl;

    procedure CreateJoints(number: integer);
    procedure CreateJointData(JointIndex, TranslationKeys, RotationKeys: integer;
                              Name: string; ParentJointIndex, Tag: integer);
    procedure SetJointOrigin(JointIndex: integer; x,y,z, xa,ya,za: float);
    procedure SetJointConnection(JointIndex: integer; Child: TObj);
    procedure SetJointTranslationKey(JointIndex, KeyIndex: integer; x,y,z: float);
    procedure SetJointRotationKey(JointIndex, KeyIndex: integer; xa,ya,za: float);
    procedure CreateAnimations(number: integer); VIRTUAL;
    procedure CreateAnimationData(AnimationIndex, Cycles: integer; Name: string; Tag: integer);
    procedure CreateCycleData(AnimationIndex, CycleIndex, JointIndex, TranslationFrames, RotationFrames: integer; Loop: boolean);
    procedure SetCycleTranslationFrame(AnimationIndex, CycleIndex, Index, Key, Time: integer);
    procedure SetCycleRotationFrame(AnimationIndex, CycleIndex, Index, Key, Time: integer);
    function GetJointIndex(Name: string): integer;
    function GetAnimationIndex(Name: string): integer;
    procedure SetRunCycleCallback(AnimationIndex, JointIndex: integer; CallBack: TObjCycleCallback);
    procedure SetEndCycleCallback(AnimationIndex, JointIndex: integer; CallBack: TObjCycleCallback);
    procedure SetupAnimations; VIRTUAL;
    procedure Animate(AnimationIndex, TimeDifference: integer); VIRTUAL;
    procedure ResetAnimation(AnimationIndex: integer); VIRTUAL;
    procedure ShowAnimation(AnimationIndex: integer); VIRTUAL;
    procedure HideAnimation(AnimationIndex: integer); VIRTUAL;
    procedure ReplaceAnimation(AnimationIndex, NewAnimationIndex, TimeGap: integer); VIRTUAL;
    procedure SetJoints(Joints: PJoints; number: integer);
    procedure SetAnimations(Animations: PAnimations; number: integer);
    procedure SetJointAnimByFile(FileName: String);

    procedure GetMin(var mx, my, mz : GLfloat);
    function GetMinVec : TVec;
    procedure GetMax(var mx, my, mz : GLfloat);
    function GetMaxVec : TVec;
    procedure GetCent(var cx, cy, cz : GLfloat);
    function GetCentVec : TVec;
    function GetBRad : GLfloat;
    procedure SetMin(mx, my, mz : GLfloat);
    procedure SetMinVec(min : TVec);
    procedure SetMax(mx, my, mz : GLfloat);
    procedure SetMaxVec(max : TVec);
    procedure SetCent(cx, cy, cz : GLfloat);
    procedure SetCentVec(cent : TVec);
    procedure SetBRad(br : GLfloat);

    function GetMesh : TMesh;
    procedure SetMeshByFile(FileName : String);

    function  SphereIntersect(obj : TObj) : boolean;
    function  QuickIntersect(obj : TObj) : boolean;
    function  Intersect(obj : TObj; var Intersection : TVec) : boolean;
    function  IntersectLine(v1, v2 : TVec; var Intersection : TVec) : boolean;

    procedure SetAsCamera;
    procedure DrawObj; VIRTUAL; ABSTRACT;
    procedure Display;
    procedure FillSelectionBuffer;
  end;

  PObjList = ^TObjList;
  TObjList = record
    me : TObj;
    next : PObjList;
  end;

  TObjAnimation = record
    me: TObj;
    AnimationIndex: integer;
  end;
  PObjAnimation = ^TObjAnimation;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Shader prototype
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  TShader = class
  protected
    sdIdentity : integer;             // shader identity

  public
    Name : TNameString;

    constructor Create;
    destructor Destroy; OVERRIDE;

    function GetIdentity: integer;    // return shader identity
    procedure SetName(aName: String);
    procedure Display; VIRTUAL;       // display shader
  end;

  TShaderTable = array[0..63] of TShader;
  PShaderTable = ^TShaderTable;



{$IFDEF GL_PROGRAM}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  vertex and fragment programs
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  TProgramParameter = record
    Param: TVec4f;
    Index: Integer;
  end;
  PProgramParameter = ^TProgramParameter;
  TProgramParameters = array[0..255] of TProgramParameter;
  PProgramParameters = ^TProgramParameters;

  TProgramMatrix = record
    Bound: TObj;
    Matrix: TMatrix4f;
    Index: integer;
  end;
  PProgramMatrix = ^TProgramMatrix;
  TProgramMatrices = array[0..31] of TProgramMatrix;
  PProgramMatrices = ^TProgramMatrices;

  TProgram = class(TShader)
  protected
    prTarget  : GLenum;              // program target
    prHandle  : GLuint;              // program handle
    prString  : AnsiString;          // program string, ready to be sent to OpenGL

  public
    prParam : TProgramParameters; // local parameters of program
    prParams : integer;              // number of local parameters of program

    constructor Create;
    destructor Destroy; OVERRIDE;

    procedure Compile; VIRTUAL;
    procedure Display; OVERRIDE;

    function GetProgramString: AnsiString;
    procedure SetProgramString(Prg: AnsiString);

    procedure UpdateParams;
    procedure SetParam(index: integer; a, b, c, d: float);
    procedure SetParamv(index: integer; p: TVec4f);
    function GetParam(index: integer): TVec4f;
    procedure RemoveParam(index: integer);
  end;

  TVertexProgram = class(TProgram)
  public
    constructor Create;
  end;


  TFragmentProgram = class(TProgram)
  public
    constructor Create;
  end;

  TVertexProgramTable = array[0..9999] of TVertexProgram;
  PVertexProgramTable = ^TVertexProgramTable;

  TFragmentProgramTable = array[0..9999] of TFragmentProgram;
  PFragmentProgramTable = ^TFragmentProgramTable;
{$ENDIF}


  // triangle list connected with shaders
  TTriangleList = record
    Shaders : integer;             // number of shaders in triangle list
    Shader : TShaderTable;         // shaders in triangle list
    First, Last : integer;         // index of first and last triangle in list
  end;
  PTriangleList = ^TTriangleList;
  TTriangleLists = array[0..9999] of TTriangleList;
  PTriangleLists = ^TTriangleLists;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Texture classes
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  TTexture = class(TShader)
  protected
    texComponents : GLint;     // texture components
    texTarget : GLenum;        // texture target
    texHandle : GLuint;        // texture handle

    texMinFilter : GLint;      // filter for pixels > texels
    texMagFilter : GLint;      // filter for pixels < texels

  public
    constructor Create;
    destructor  Destroy; OVERRIDE;

    function GetMinFilter: GLint;
    function GetMagFilter: GLint;
    function GetComponents: GLint;

    procedure SetMinFilter(filter: GLint);
    procedure SetMagFilter(filter: GLint);
    procedure SetComponents(components: GLint);

    procedure Display; OVERRIDE; // binds handle to target
    procedure Compile; VIRTUAL; ABSTRACT; // recreates texture object in handle
  end;

  PTextureTable = ^TTextureTable;
  TTextureTable = Array [0..9999] of TTexture;


  TTexture1D = class(TTexture)
  private
    t1dLine      : PRGBALine; // store 1D-texture data
    t1dWrapS     : GLint;     // wrap texture s parameter
    t1dBorderColor : TColor4f;   // border color of texture

  public
    constructor Create;
    destructor  Destroy; OVERRIDE;

    function GetBorderColor: TColor4f;
    procedure SetBorderColor(r,g,b,a : GLfloat);
    function GetLine: PRGBALine;
    procedure SetLine(Line: PRGBALine);
    function GetWrapS: GLint;
    procedure SetWrapS(value : GLint);
    procedure Rescale(NewLength : integer);
    procedure Compile; OVERRIDE;// recreates texture object in handle
  end;


  TTexture2D = class(TTexture)
  private
    t2dImage     : PRGBAImage;   // store 2D-texture data
    t2dWrapS     : GLint;        // wrap texture s parameter
    t2dWrapT     : GLint;        // wrap texture t parameter
    t2dBorderColor : TColor4f;   // border color of texture

  public
    constructor Create;
    destructor  Destroy; OVERRIDE;

    function GetImage: PRGBAImage;
    procedure SetImage(Image : PRGBAImage);

    function GetBorderColor : TColor4f;
    procedure SetBorderColor(r,g,b,a : GLfloat);
    function GetWrapS : GLint;
    procedure SetWrapS(value : GLint);
    function GetWrapT : GLint;
    procedure SetWrapT(value : GLint);
    procedure Rescale(NewWidth, NewHeight : integer);
    procedure Compile; OVERRIDE;// recreates texture object in handle
  end;


{$IFDEF GL_12}
  TTexture3D = class(TTexture)
  private
    t3dBox       : PRGBABox;     // store 3D-texture data GL 1.2
    t3dWrapS     : GLint;        // wrap texture s parameter
    t3dWrapT     : GLint;        // wrap texture t parameter
    t3dWrapR     : GLint;        // wrap texture r parameter
    t3dBorderColor : TColor4f;   // border color of texture

  public
    constructor Create;
    destructor  Destroy; OVERRIDE;

    function GetBox : PRGBABox;
    procedure SetBox(Box : PRGBABox);

    function GetBorderColor : TColor4f;
    procedure SetBorderColor(r,g,b,a : GLfloat);
    function GetWrapS : GLint;
    procedure SetWrapS(value : GLint);
    function GetWrapT : GLint;
    procedure SetWrapT(value : GLint);
    function GetWrapR : GLint;
    procedure SetWrapR(value : GLint);

    procedure Rescale(NewWidth, NewHeight, NewDepth : integer); VIRTUAL;
    procedure Compile; OVERRIDE;// recreates texture object in handle
  end;
{$ENDIF}


{$IFDEF GL_13}
  TTextureCubemap = class(TTexture)
  private
    tcmCubemap : PRGBACubemap;   // store cube map data GL 1.3
    tcmWrapS     : GLint;        // wrap texture s parameter
    tcmWrapT     : GLint;        // wrap texture t parameter
    tcmWrapR     : GLint;        // wrap texture r parameter
    tcmBorderColor : TColor4f;   // border color of texture

  public
    constructor Create;
    destructor  Destroy; OVERRIDE;

    function GetCubemap : PRGBACubemap;
    procedure SetCubemap(Cubemap : PRGBACubemap);

    function GetBorderColor : TColor4f;
    procedure SetBorderColor(r,g,b,a : GLfloat);
    function GetWrapS : GLint;
    procedure SetWrapS(value : GLint);
    function GetWrapT : GLint;
    procedure SetWrapT(value : GLint);
    function GetWrapR : GLint;
    procedure SetWrapR(value : GLint);

    procedure Rescale(NewSide : integer);
    procedure Compile; OVERRIDE;// recreates texture object in handle
    procedure DisplaySkyBox(side : float);   // displays sky-box around viewer
  end;
{$ENDIF}


  TDynamicTexture2D = class(TTexture)
  private
    dt2Width  : GLint;       // texture width
    dt2Height : GLint;       // texture height
    dt2WrapS  : GLint;       // wrap texture s parameter
    dt2WrapT  : GLint;       // wrap texture t parameter
    dt2BorderColor : TColor4f; // border color of texture
    dt2CreateNew : Boolean;  // if true the texture will be created instead of copied

    dt2Matrix : TMatrix4f;   // transformation matrix for the texture
    dt2FoV    : GLdouble;    // field of view
    dt2AspRat : GLdouble;    // apect / ratio
    dt2NearZ  : GLdouble;    // near z border
    dt2FarZ   : GLdouble;    // far z border
    dt2Bound  : TObj;        // bound object

  public
    constructor Create;
    function GetBorderColor : TColor4f;
    procedure SetBorderColor(r,g,b,a : GLfloat);
    function GetWrapS : GLint;
    procedure SetWrapS(value : GLint);
    function GetWrapT : GLint;
    procedure SetWrapT(value : GLint);
    procedure Rescale(NewWidth, NewHeight : integer);

    function GetWidth : GLint;
    function GetHeight : GLint;

    function GetMatrix : TMatrix4f;
    procedure SetMatrix(Matrix : TMatrix4f);
    procedure SetMatrixCoeff(c00,c01,c02,c03,c04, c10,c11,c12,c13, c20,c21,c22,c23, c30,c31,c32,c33 : float);
    function GetFoV : GLdouble;
    procedure SetFoV(FoV : GLdouble);
    function GetAspRat : GLdouble;
    procedure SetAspRat(AspRat : GLdouble);
    function GetNearZ : GLdouble;
    procedure SetNearZ(NearZ : GLdouble);
    function GetFarZ : GLdouble;
    procedure SetFarZ(FarZ : GLdouble);

    function GetBoundObj : TObj;
    procedure Bind(obj: TObj);
    procedure Compile; OVERRIDE;// recreates texture object in handle
  end;


{$IFDEF GL_13}
  TDynamicTextureCubemap = class(TTexture)
  private
    dcmSide   : GLint;       // length of cube map sides
    dcmWrapS     : GLint;        // wrap texture s parameter
    dcmWrapT     : GLint;        // wrap texture t parameter
    dcmWrapR     : GLint;        // wrap texture r parameter
    dcmBorderColor : TColor4f;   // border color of texture
    dcmCreateNew : Boolean;  // if true the texture will be created instead of copied

    dcmMatrix : TMatrix4f;   // transformation matrix for the texture
    dcmFoV    : GLdouble;    // field of view
    dcmAspRat : GLdouble;    // apect / ratio
    dcmNearZ  : GLdouble;    // near z border
    dcmFarZ   : GLdouble;    // far z border
    dcmBound  : TObj;        // bound object

  public
    constructor Create;
    procedure Rescale(NewSide : integer);
    procedure Bind(obj: TObj);
    function GetBoundObj : TObj;
    procedure Compile; OVERRIDE;// recreates texture object in handle
    procedure DisplaySkyBox(side : float);   // displays sky-box around viewer

    function GetSide : GLint;

    function GetBorderColor : TColor4f;
    procedure SetBorderColor(r,g,b,a : GLfloat);
    function GetWrapS : GLint;
    procedure SetWrapS(value : GLint);
    function GetWrapT : GLint;
    procedure SetWrapT(value : GLint);
    function GetWrapR : GLint;
    procedure SetWrapR(value : GLint);

    function GetMatrix : TMatrix4f;
    procedure SetMatrix(Matrix : TMatrix4f);
    procedure SetMatrixCoeff(c00,c01,c02,c03,c04, c10,c11,c12,c13, c20,c21,c22,c23, c30,c31,c32,c33 : float);
    function GetFoV : GLdouble;
    procedure SetFoV(FoV : GLdouble);
    function GetAspRat : GLdouble;
    procedure SetAspRat(AspRat : GLdouble);
    function GetNearZ : GLdouble;
    procedure SetNearZ(NearZ : GLdouble);
    function GetFarZ : GLdouble;
    procedure SetFarZ(FarZ : GLdouble);
  end;
{$ENDIF}


  TTextureDefault = class(TTexture)
    constructor Create;
    procedure Display; OVERRIDE;
    procedure Compile; OVERRIDE;
  end;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Material
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  TMatShininess = record
    change : boolean; // info if value should be updated
    value : GLfloat;  // update value
  end;

  TMatColor = record
    change : boolean;   // info if color should be updated
    color  : TColor4f;  // update color
  end;

  TMaterial = class(TShader)    // material definition
  private
    matFace : GLenum;           // info which faces are updated
    matList : GLuint;           // display list

    // material definition
    matShininess : TMatShininess;
    matAmbient   : TMatColor;
    matDiffuse   : TMatColor;
    matSpecular  : TMatColor;
    matEmission  : TMatColor;

  public
    constructor Create;
    destructor Destroy; OVERRIDE;

    function GetFace : GLenum;
    function GetShininess : TMatShininess;
    function GetAmbient : TMatColor;
    function GetDiffuse : TMatColor;
    function GetSpecular : TMatColor;
    function GetEmission : TMatColor;

    procedure SetFace(face : GLenum);
    procedure SetShininess(v : GLfloat; update : boolean);
    procedure SetAmbient(r,g,b,a : GLfloat; update : boolean);
    procedure SetDiffuse(r,g,b,a : GLfloat; update : boolean);
    procedure SetSpecular(r,g,b,a : GLfloat; update : boolean);
    procedure SetEmission(r,g,b,a : GLfloat; update : boolean);

    procedure SetShininessv(v : TMatShininess);
    procedure SetAmbientv(v : TMatColor);
    procedure SetDiffusev(v : TMatColor);
    procedure SetSpecularv(v : TMatColor);
    procedure SetEmissionv(v : TMatColor);

    procedure Compile;
    procedure Display; OVERRIDE;
    procedure InitMaterial;
  end;

  PMaterialTable = ^TMaterialTable;
  TMaterialTable = Array [0..9999] of TMaterial;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  RenderSettings
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  TRSAlphaTest = record
    State   : TCapState;  // alpha test
    Func    : GLenum;     // alpha test function
    Ref     : GLclampf;   // alpha test reference value
  end;

  TRSBlendFunc = record
    State   : TCapState;  // blending with background
    SFactor,              // source factor
    DFactor : GLenum;     // destination factor
  end;

  TRSColor = record
    Change  : boolean;    // change current color
    r,g,b,a : GLfloat;    // color values
  end;

  TRSColorMask = record
    Change  : boolean;    // change active color mask
    r,g,b,a : GLboolean;  // color mask values
  end;

  TRSColorMaterial = record
    State   : TCapState;  // material color tracking
    Face,                 // affected faces (front or back)
    Mode    : GLenum;     // tracked material parameter
  end;

  TRSCullFace = record
    State    : TCapState; // polygon culling
    Face     : GLenum;    // affected faces (front or back)
  end;

  TRSDepthTest = record
    State    : TCapState; // enable/disable depth testing
    Func     : GLenum;    // depth buffer function
  end;

  TRSFrontFace = record
    Change   : Boolean;   // polygon winding
    Winding  : GLenum;    // winding of front faced polygons
  end;

  TRSLineStipple = record
    State   : TCapState;  // line stippling
    Factor  : GLint;      // pattern multiplication factor
    Pattern : GLushort;   // stipple pattern
  end;

  TRSLineWidth = record
    Change  : boolean;    // line width
    Width   : GLfloat;
  end;

  TRSPointParam = record
    Change  : boolean;    // pointparameters for particle effects
    Sizemin : GLfloat;    // minimum size of attenuated points
    Sizemax : GLfloat;    // maximum size of attenuated points
    Fade    : GLfloat;    // fading threshold
    Attenuation : Array[0..2] of GLfloat; // parameters for attenuation equation
  end;

  TRSPointSize = record
    Change  : boolean;    // point size
    Size    : GLfloat;
  end;

  TRSPolyMode = record
    Change  : boolean;    // polygon mode
    Front,                // front faced polygons (line/point/filled)
    Back    : GLenum;     // back faced polygons (line/point/filled)
  end;

  TRSPolyStipple = record
    State   : TCapState;  // polygon stippling
    Mask    : TPolyStippleMask; // stipple mask
  end;

  TRSStencilTest = record
    State   : TCapState;  // stencil test
    Func    : GLenum;    // test function
    Ref     : GLint;     // reference value
    Mask    : GLuint;    // stencil mask
    OpFail  : GLenum;    // stencil fail operation
    OpZFail : GLenum;    // stencil pass z fail operation
    OpZPass : GLenum;    // stencil pass z pass operation
  end;


  TRenderSettings = class(TShader)
  private
    rsList : GLuint;            // display list

    rsAlphaTest : TRSAlphaTest;
    rsBlendFunc : TRSBlendFunc;
    rsColor : TRSColor;
    rsColorMask : TRSColorMask;
    rsColorMaterial : TRSColorMaterial;
    rsCullFace  : TRSCullFace;
    rsDepthTest : TRSDepthTest;
    rsDepthWrite : TCapState;   // depth buffer writing
    rsFog       : TCapState;    // fog is computed
    rsFragmentProgram : TCapState; // fragment programs are accepted
    rsFrontFace : TRSFrontFace;
    rsLighting  : TCapState;    // light is computed
    rsLinesmooth: TCapState;    // smoothed or aliased lines
    rsLinestipple : TRSLineStipple;
    rsLinewidth : TRSLineWidth;
    rsNormalize : TCapState;    // normal vectors are normalized
    rsPointParam : TRSPointParam;
    rsPointSize : TRSPointSize;
    rsPointSmooth : TCapState;  // smoothed or aliased points
    rsPolyMode : TRSPolyMode;
    rsPolySmooth : TCapState;   // smoothed or aliased polygons
    rsPolyStipple : TRSPolyStipple;
    rsRescaleNormal : TCapState; // rescale normals GL 1.2
    rsSeparateColor : TCapState;// separate or implicite specular color GL 1.2
    rsSmoothShading : TCapState;// smooth or flat shaded polygons
    rsStencilTest : TRSStencilTest;
    rsTexturing : TCapState;    // textures are accepted
    rsTwoSided  : TCapState;    // two sided lighting model
    rsVertexProgram : TCapState; // vertex programs are accepted

  public
    constructor Create;
    destructor Destroy; OVERRIDE;

    function GetAlphaTest : TRSAlphaTest;
    function GetBlendFunc : TRSBlendFunc;
    function GetColor : TRSColor;
    function GetColorMask : TRSColorMask;
    function GetColorMaterial : TRSColorMaterial;
    function GetCullFace : TRSCullFace;
    function GetDepthTest : TRSDepthTest;
    function GetDepthWrite : TCapState;
    function GetFog : TCapState;
    function GetFragmentProgram : TCapState;
    function GetFrontFace : TRSFrontFace;
    function GetLighting : TCapState;
    function GetLinesmooth : TCapState;
    function GetLineStipple : TRSLineStipple;
    function GetLineWidth : TRSLineWidth;
    function GetNormalize : TCapState;
    function GetPointParam : TRSPointParam;
    function GetPointSize : TRSPointSize;
    function GetPointSmooth : TCapState;
    function GetPolyMode : TRSPolyMode;
    function GetPolySmooth : TCapState;
    function GetPolyStipple : TRSPolyStipple;
    function GetRescaleNormal : TCapState;
    function GetSeparateColor : TCapState;
    function GetSmoothShading : TCapState;
    function GetStencilTest : TRSStencilTest;
    function GetTexturing : TCapState;
    function GetTwoSided : TCapState;
    function GetVertexProgram : TCapState;

    procedure SetAlphaTest(state : TCapState; func : GLenum; ref : GLclampf);
    procedure SetBlendFunc(state : TCapState; sfactor, dfactor : GLenum);
    procedure SetColor(change : boolean; r, g, b, a : GLfloat);
    procedure SetColorMask(change : boolean; r, g, b, a : GLboolean);
    procedure SetColorMaterial(state : TCapState; face, mode : GLenum);
    procedure SetCullFace(state : TCapState; face : GLenum);
    procedure SetDepthTest(state : TCapState; func : GLenum);
    procedure SetDepthWrite(state : TCapState);
    procedure SetFog(state : TCapState);
    procedure SetFragmentProgram(state : TCapState);
    procedure SetFrontFace(change : boolean; winding : GLenum);
    procedure SetLighting(state : TCapState);
    procedure SetLineSmooth(state : TCapState);
    procedure SetLineStipple(state : TCapState; factor : GLint; pattern : GLushort);
    procedure SetLineWidth(change : boolean; width : GLfloat);
    procedure SetNormalize(state : TCapState);
    procedure SetPointParam(change : boolean; min, max, fade, attconst, attlinear, attquadr : GLfloat);
    procedure SetPointSize(change : boolean; size : GLfloat);
    procedure SetPointSmooth(state : TCapState);
    procedure SetPolyMode(change : boolean; front, back : GLenum);
    procedure SetPolySmooth(state : TCapState);
    procedure SetPolyStipple(state : TCapState; mask : Array of Byte);
    procedure SetPolyStippleState(state : TCapState);
    procedure SetPolyStippleMask(m00,m01,m02,m03,m04,m05,m06,m07,m08,m09,m10,m11,m12,m13,m14,m15,
                                 m16,m17,m18,m19,m20,m21,m22,m23,m24,m25,m26,m27,m28,m29,m30,m31 : GLuint);
    procedure SetRescaleNormal(state : TCapState);
    procedure SetSeparateColor(state : TCapState);
    procedure SetSmoothShading(state : TCapState);
    procedure SetStencilTest(state : TCapState; func : GLenum; ref : GLint; mask : GLuint; opfail, opzfail, opzpass : GLenum);
    procedure SetTexturing(state : TCapState);
    procedure SetTwoSided(state : TCapState);
    procedure SetVertexProgram(state : TCapState);

    procedure SetAlphaTestv(v : TRSAlphaTest);
    procedure SetBlendFuncv(v : TRSBlendFunc);
    procedure SetColorv(v : TRSColor);
    procedure SetColorMaskv(v : TRSColorMask);
    procedure SetColorMaterialv(v : TRSColorMaterial);
    procedure SetCullFacev(v : TRSCullFace);
    procedure SetDepthTestv(v : TRSDepthTest);
    procedure SetFrontFacev(v : TRSFrontFace);
    procedure SetLineStipplev(v : TRSLineStipple);
    procedure SetLineWidthv(v : TRSLineWidth);
    procedure SetPointParamv(v : TRSPointParam);
    procedure SetPointSizev(v : TRSPointSize);
    procedure SetPolyModev(v : TRSPolyMode);
    procedure SetPolyStipplev(v : TRSPolyStipple);
    procedure SetStencilTestv(v : TRSStencilTest);

    procedure InitRenderSettings;
    procedure Compile;
    procedure Display; OVERRIDE;
  end;

  PRenderSettingsTable = ^TRenderSettingsTable;
  TRenderSettingsTable = Array [0..9999] of TRenderSettings;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  TextureSettings
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  TTSTexActive = record
    Change  : Boolean;    // texture unit change
    Value   : GLenum;     // set active texture unit
  end;

  TTSTexEnv = record
    Change   : boolean;   // texture environment
    Mode  : GLint;        // texture environment mode (replace, modulate, decal, blend, combine)
    Color : TColor4f;     // environment color of texture

    CombineRGB : GLenum;     // color combination mode
    CombineAlpha : GLenum;   // alpha combination mode
    Source0RGB : GLenum;     // combine rgb source 0
    Operand0RGB : GLenum;    // combine rgb operand source 0
    Source0Alpha : GLenum;   // combine alpha source 0
    Operand0Alpha : GLenum;  // combine alpha operand source 0
    Source1RGB : GLenum;     // combine rgb source 1
    Operand1RGB : GLenum;    // combine rgb operand source 1
    Source1Alpha : GLenum;   // combine alpha source 1
    Operand1Alpha : GLenum;  // combine alpha operand source 1
    Source2RGB : GLenum;     // combine rgb source 2
    Operand2RGB : GLenum;    // combine rgb operand source 2
    Source2Alpha : GLenum;   // combine alpha source 2
    Operand2Alpha : GLenum;  // combine alpha operand source 2
    RGBScale : GLint;        // rgb scale factor
    AlphaScale : GLint;      // alpha scale factor
  end;

  TTSTexGen = record
    State   : TCapState;  // texture s/t/r/q parameter generation
    Map     : GLint;      // eye, object, sphere map, normal map, reflection map
    Coeff   : Array[0..3] of GLfloat; // coefficients of equation
  end;

  TTSTexMatrix = record
    Change  : boolean;    // texture transformation matrix
    Matrix  : TMatrix4f;
  end;

  TTextureSettings = class(TShader)
  private
    tsList : GLuint;            // display list

    tsTexActive : TTSTexActive;
    tsTexture1D : TCapState;    // onedimensional textures
    tsTexture2D : TCapState;    // twodimensional textures
    tsTexture3D : TCapState;    // threedimensial textures GL 1.2
    tsTextureCubemap : TCapState; // cubemap textures GL 1.2
    tsTexEnv : TTSTexEnv;
    tsTexGenS : TTSTexGen;      // eye, object, sphere map, normal map, reflection map
    tsTexGenT : TTSTexGen;      // eye, object, sphere map, normal map, reflection map
    tsTexGenR : TTSTexGen;      // eye, object, normal map, reflection map
    tsTexGenQ : TTSTexGen;      // eye, object
    tsTexMatrix : TTSTexMatrix; // set texture transformation matrix
    tsTexReflect : TCapState;   // texture matrix reflects view matrix

  public
    constructor Create;
    destructor Destroy; OVERRIDE;

    function GetTexActive : TTSTexActive;
    function GetTexture1D : TCapState;
    function GetTexture2D : TCapState;
    function GetTexture3D : TCapState;
    function GetTextureCubemap : TCapState;
    function GetTexEnv : TTSTexEnv;
    function GetTexGenS : TTSTexGen;
    function GetTexGenT : TTSTexGen;
    function GetTexGenR : TTSTexGen;
    function GetTexGenQ : TTSTexGen;
    function GetTexMatrix : TTSTexMatrix;
    function GetTexReflect : TCapState;

    procedure SetTexActive(change : boolean; value : GLenum);
    procedure SetTexture1D(state : TCapState);
    procedure SetTexture2D(state : TCapState);
    procedure SetTexture3D(state : TCapState);
    procedure SetTextureCubemap(state : TCapState);
    procedure SetTexEnv(change : boolean; mode : GLint; r,g,b,a : GLfloat);
    procedure SetTexEnvCombineMode(CombineRGB, CombineAlpha : GLenum);
    procedure SetTexEnvCombine0(Source0RGB, Operand0RGB, Source0Alpha, Operand0Alpha : GLenum);
    procedure SetTexEnvCombine1(Source1RGB, Operand1RGB, Source1Alpha, Operand1Alpha : GLenum);
    procedure SetTexEnvCombine2(Source2RGB, Operand2RGB, Source2Alpha, Operand2Alpha : GLenum);
    procedure SetTexEnvCombineScale(RGBScale, AlphaScale : GLint);
    procedure SetTexGenS(state : TCapState; map : GLint; c1, c2, c3, c4 : GLfloat);
    procedure SetTexGenT(state : TCapState; map : GLint; c1, c2, c3, c4 : GLfloat);
    procedure SetTexGenR(state : TCapState; map : GLint; c1, c2, c3, c4 : GLfloat);
    procedure SetTexGenQ(state : TCapState; map : GLint; c1, c2, c3, c4 : GLfloat);
    procedure SetTexMatrix(change: boolean; c00,c01,c02,c03, c10,c11,c12,c13, c20,c21,c22,c23, c30,c31,c32,c33 : GLfloat);
    procedure SetTexReflect(state : TCapState);

    procedure SetTexActivev(v : TTSTexActive);
    procedure SetTexEnvv(v : TTSTexEnv);
    procedure SetTexGenSv(v : TTSTexGen);
    procedure SetTexGenTv(v : TTSTexGen);
    procedure SetTexGenRv(v : TTSTexGen);
    procedure SetTexGenQv(v : TTSTexGen);
    procedure SetTexMatrixv(v : TTSTexMatrix);

    procedure InitTextureSettings;
    procedure Compile;
    procedure Display; OVERRIDE;
  end;

  PTextureSettingsTable = ^TTextureSettingsTable;
  TTextureSettingsTable = Array [0..9999] of TTextureSettings;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  AniMesh
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  TAniMesh = class;

  TAniMeshAnimationCallback = procedure(Sender: TAniMesh; AnimationIndex: integer);

  TFrameAnimation = record
    CurrentTime,                  // current time of animation
    TotalTime      : integer;     // total time of animation
    Frames         : integer;     // number of frames in animation
    Frame          : PKeyFrames;  // frames of animation
    Loop           : boolean;     // true= animation starts again if finished

    OnRunAnimation : TAniMeshAnimationCallback; // callback routine when animation runs
    OnEndAnimation : TAniMeshAnimationCallback; // callback routine when animation ends/loops
  end;
  PFrameAnimation = ^TFrameAnimation;
  TFrameAnimations = array[0..9999] of TFrameAnimation;
  PFrameAnimations = ^TFrameAnimations;

  TAniMesh = class(TObj)
  public
    amFrames      : integer;      // number of frames in object
    amFrame       : PVecsTable;   // vertex positions for every frame in object
    amFrameNorm   : PVecsTable;   // vertex normals for every frame in object

    amFrameMin    : PVecs;        // minimum coordinate for every frame
    amFrameMax    : PVecs;        // maximum coordinate for every frame
    amFrameCent   : PVecs;        // center for every frame
    amFrameBSRad  : PFloatTable;  // bounding sphere radius for every frame

    amFrameAnimation: PFrameAnimations; // frame animations
    amCurrentFrame,               // currently displayed frame
    amPrevFrame   : integer;      // previously displayed frame

    amTexUnits    : integer;      // texture units in animator
    amTexUnit     : array[0..31] of GLenum; // filled texture units

    amNorm        : PVecs;        // place to precompute animated normals
    amTexCoord    : array[0..31] of PTexCoords4f; // texture positions of vertices
    amColor       : PColors4f;        // vertex colors of vertices

    amTriangleLists : integer;       // number of triangle lists to display
    amTriangleList: PTriangleLists; // pointer to triangle lists

    amAlphaChannel: boolean;      // if false no alphachannel is sent to OpenGL
    amTexComponents : integer;    // number of texture components sent to OpenGL (1 - 4)
    amStaticData : boolean;       // if true data is not to be freed

    constructor Create;
    constructor CreateByData(lists,frames,verts,tris: integer; normals: boolean);
    constructor CreateByFile(FileName : String);
    destructor Destroy; OVERRIDE;

    procedure SetFrameVerts(f: integer; pv: PVecs);
    procedure SetFrameNorms(f: integer; pn: PVecs);
    procedure SetTriangles(pt: PTris);
    procedure SetFrameAnimations(FrameAnimations: PFrameAnimations);

    procedure SetTexComponents(texcomponents: integer);
    procedure SetAlphaChannel(alphachannel: boolean);

    procedure SetTexCoord(texunit: integer; pvt: PTexCoords4f);
    procedure SetColor(pvc: PColors4f);
    procedure SetTexUnit(texunit, value: integer);

    procedure SetFrameMinCoords(pmn: PVecs);
    procedure SetFrameMaxCoords(pmx: PVecs);
    procedure SetBoundingSphereCenters(pct: PVecs);
    procedure SetBoundingSphereRadii(pbs: PFloatTable);

    function GetTriListNum : integer;
    procedure SetTriList(index: integer; first, last: integer);
    procedure AddShader(TriListIndex: integer; Shader: TShader);
    function TriList(TriListIndex: integer) : PTriangleList;

    procedure CreateAnimations(number: integer); OVERRIDE;
    procedure CreateFrameAnimationData(AnimationIndex, Frames: integer; Loop: boolean);
    procedure SetAnimationFrame(AnimationIndex, FrameIndex, Key, Time: integer);
    procedure SetupAnimations; OVERRIDE;
    procedure Animate(AnimationIndex, TimeDifference: integer); OVERRIDE;
    procedure ResetAnimation(AnimationIndex: integer); OVERRIDE;
    procedure ReplaceAnimation(AnimationIndex, NewAnimationIndex, TimeGap: integer); OVERRIDE;

    procedure ComputeMinMax;
    procedure DrawObj; OVERRIDE;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  TSkinMesh
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



  TSkinVert = record
    Pos: TVec;                          // position of vertex
    Norm: TVec;                         // normal vector of vertex
    JointIndex: integer;                // joint vertex is connected to
  end;
  PSkinVert = ^TSkinVert;
  TSkinVerts = array[0..9999] of TSkinVert;
  PSkinVerts = ^TSkinVerts;

  TSkinMesh = class(TObj)
    smVert        : PSkinVerts;         // skinned vertices (computation takes place in objMesh)
    smNorm        : PVecs;              // place to precompute normals
    smColor       : PColors4f;          // vertex colors
    smTexUnits    : integer;            // texture units in SkinMesh
    smTexUnit     : array[0..31] of GLenum; // filled texture units
    smTexCoord    : array[0..31] of PTexCoords4f; // texture positions of vertices

    smTriangleLists : integer;          // number of triangle lists to display
    smTriangleList : PTriangleLists;    // pointer to triangle lists

    smAlphaChannel : boolean;           // if false no alphachannel is sent to OpenGL
    smTexComponents : integer;          // number of texture components sent to OpenGL (1 - 4)
    smStaticData   : boolean;           // if true then data is not to be freed

    constructor Create;
    constructor CreateByData(lists,verts,tris: integer; normals: boolean);
    constructor CreateByFile(FileName : String);
    destructor Destroy; OVERRIDE;

    procedure SetVerts(SkinVerts: PSkinVerts);
    procedure SetTriangles(pt: PTris);

    procedure SetTexComponents(texcomponents: integer);
    procedure SetAlphaChannel(alphachannel: boolean);

    procedure SetTexCoord(texunit: integer; pvt: PTexCoords4f);
    procedure SetColor(pvc: PColors4f);
    procedure SetTexUnit(texunit, value: integer);

    function GetTriListNum : integer;
    procedure SetTriList(index, first, last: integer);
    procedure AddShader(TriListIndex: integer; Shader: TShader);
    function TriList(TriListIndex: integer) : PTriangleList;

    procedure SetupAnimations; OVERRIDE;
    procedure Animate(AnimationIndex, TimeDifference: integer); OVERRIDE;

    procedure DrawObj; OVERRIDE;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  1D-Line evaluator
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  TEvaluator1DSegment = record
    Start : float;                     // relative start of segment in evaluator
    Length : float;                    // length of segment relative to evaluator length
  end;
  PEvaluator1DSegment = ^TEvaluator1DSegment;
  TEvaluator1DSegments = array[0..9999] of TEvaluator1DSegment;
  PEvaluator1DSegments = ^TEvaluator1DSegments;

  TEvaluator1D = class(TObj)
  public
    e1dPoints : Integer;                // number of control points
    e1dPoint : PVecs;                   // control point table
    e1dLineWidth : float;               // line width if evaluator is drawn
    e1dLineColor : TColor4f;            // line color if evaluator is drawn

    e1dNWidth : integer;                // lines per segment
    e1dLength : float;                  // automatically computed
    e1dSegments : integer;              // number of spline segments
    e1dSegment : PEvaluator1DSegments;  // spline segments

    constructor Create;
    destructor Destroy; OVERRIDE;

    procedure AddPoint(x,y,z: float);
    procedure Close;
    procedure DrawObj; OVERRIDE;

    procedure RecomputeSegments;
    function Evaluate(position: float) : TVec;
    procedure EvaluateDirection(position: float; var v1, v2: TVec);
  end;

  TEv1DAnimator = class
  public
    e1aEv        : TEvaluator1D;  // Evaluator to animate position/angle
    e1aUp        : TVec;          // up vector
    e1aCycleTime : integer;       // time for full evaluator cycle
    e1aAdvance   : float;         // animation progress
    e1aLoop      : boolean;       // animation loops forever
    e1aFinished  : boolean;       // animation is done

    constructor Create(Ev: TEvaluator1D; CycleTime: integer);
    procedure Start(Advance: float);
    function Run(Time: integer) : TMatrix4f;
    procedure Stop;
    procedure Advance(Adv: float);
    procedure SetCycleTime(CycleTime: integer);
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  2D-Map evaluator
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


  TEvaluator2DPatch = record
    list : GLuint;                      // display list of the patch
    ControlPoint : Array[0..2] of Array[0..2] of PVec; // control points of the patch
    Center       : TVec;                // patch center
    BSRad        : float;               // bounding sphere radius
    Recompute,                          // true = display list has to be compiled
    Visible,                            // false = patch is not displayed
    Flat         : Boolean;             // true = patch is drawn as quad
    EdgeFlat     : Byte;                // bits: 0=top flat, 1=right flat, 2=bottom flat, 3=left flat

    Shaders      : integer;             // number of shaders
    Shader       : TShaderTable;        // shaders in patch

    BoundObjects : Integer;             // number of objects bound to patch
    BoundObject  : PObjTable;           // objects bound to patch
  end;
  PEvaluator2DPatch = ^TEvaluator2DPatch;
  TEvaluator2DPatchTable = Array[0..9999] of TEvaluator2DPatch;
  PEvaluator2DPatchTable = ^TEvaluator2DPatchTable;

  TEvaluator2D = class(TObj)
  public
    e2dLists       : GLuint;            // OpenGL display lists for all patches
    e2dAutoNormal  : Boolean;           // true = normals are generated
    e2dTexCoords   : Boolean;           // true = texture coordinates are generated
    e2dInvertOrientation : Boolean;     // true = backfaces are swapped
    e2dFillStyle   : GLenum;            // fillstyle of patch (GL_POINT, GL_LINE, GL_FILL)

    e2dWidthSegments,                   // number of width segments (u-patches)
    e2dHeightSegments : integer;        // number of height segments (v-patches)

    e2dLeft,                            // u-start of evaluator map
    e2dTop,                             // v-start of evaluator map
    e2dWidth,                           // u-size of evaluator map
    e2dHeight      : float;             // v-size of evaluator map
    e2dNWidth,                          // number of u-interpolations per patch
    e2dNHeight     : integer;           // number of v-interpolations per patch
    e2dPatchWidth,                      // u-size of patch (automatically computed)
    e2dPatchHeight : float;             // v-size of patch (automatically computed)

    e2dVert        : PVecs;             // vertices to build map patches
    e2dPatch       : PEvaluator2DPatchTable; // evaluator patches the map consists of
    e2dBinary      : Boolean;           // false = vertex data is linked statically

    e2dDefaultShaders : integer;        // number of default shaders;
    e2dDefaultShader : TShaderTable;    // default shaders
    e2dResetDefaultShaders : Boolean;   // true = settings are set back if a patch changes them

    e2dPatchChild  : PObjTable;         // table containing all patch children
    e2dPatchChildren : integer;         // number of all patch children
    e2dDispChild   : PObjTable;         // table containing all displayed children (automatically computed)
    e2dDispChildren : integer;          // number of children to displayed (automatically computed)

    constructor Create;
    constructor CreateByData(ControlPoints: PVecs; WidthSegments, HeightSegments: integer);
    constructor CreateByFile(FileName: String);
    destructor Destroy; OVERRIDE;

    procedure AddDefaultShader(aShader: TShader);
    function GetPatch(i, j: integer) : PEvaluator2DPatch;
    procedure AddPatchShader(i, j: integer; aShader: TShader);
    procedure SetDimensions(Left, Top, Width, Height: GLfloat; WidthSegments, HeightSegments: integer);
    procedure SetResolution(NWidth, NHeight: integer);
    procedure RecomputeAll;

    procedure DrawPatch(Patch : PEvaluator2DPatch);
    procedure DrawObj; OVERRIDE;
    function Evaluate(u, v: float) : TVec;
    function EvalNormal(u, v: float) : TVec;
    procedure SetPatchBounding(i, j: integer);
    procedure SetBoundings;

    procedure SetFlatPatchEdges(iStart, jStart, iEnd, jEnd: integer);
    procedure BindToPatches(obj: TObj);
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  BillBoard
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



  TBillBoard = class(TObj)
    bbShaders : integer;
    bbShader : TShaderTable;

    bbWidth : float;
    bbHeight : float;
    bbTexCoord : Array[0..3] of TTexCoord4f;
    bbDisableOtherTexUnits : boolean;

    constructor Create;
    procedure SetDimensions(Width, Height : float);
    procedure DrawObj; OVERRIDE;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  math routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

function ArcCos(const X: Extended): Extended;
function ArcSin(const X: Extended): Extended;
function ArcTan2(const Y, X: Extended): Extended;
function BinComb(n, k: integer) : integer;
function bgPower(x,y : float) : float;

function VectorLength(v: TVec): float;
function VectorAdd(v1, v2: TVec): TVec;
function VectorSub(v1, v2: TVec): TVec;
procedure VectorNeg(var v: TVec);
function VectorCombine(v1, v2: TVec; f1, f2: Float): TVec;
function VectorCross(v1, v2: TVec): TVec;
function VectorScale(v: TVec; f: Float): TVec;
function VectorDot(V1, V2: TVec): Float;
procedure VectorSet(var v: TVec; x, y, z: float);
function Vector(x, y, z: float): TVec;
function VectorStandard(var v: TVec): float;
function VectorStandardVec(v: TVec): TVec;
procedure VectorTransform(var v: TVec; var m: TMatrix4f);
function VectorTransformation(v: TVec; var m: TMatrix4f): TVec;
procedure VectorTransformTo(v: TVec; var t: TVec; var m: TMatrix4f);
procedure VectorTransformTo3(v: TVec; var t: TVec; var m: TMatrix4f);
procedure Vector4fTransform(var v: TVec4f; var m: TMatrix4f);
function VectorInverseTranslate(v: TVec; var m: TMatrix4f): TVec;
function VectorInverseRotate(v: TVec; var m: TMatrix4f): TVec;
function VectorDist(v1, v2: TVec): float;
function VectorAngle(v1, v2: TVec): float;

function MatrixCreate(c00,c01,c02,c03, c10,c11,c12,c13, c20,c21,c22,c23, c30,c31,c32,c33: float): TMatrix4f;
procedure MatrixAdjoint(var M: TMatrix4f);
function MatrixDet3x3(a1, a2, a3, b1, b2, b3, c1, c2, c3: Float): Float;
function MatrixDeterminant(M: TMatrix4f): float;
procedure MatrixExtractRotation(M: TMatrix4f; var xa, ya, za : float);
procedure MatrixExtractTranslation(M: TMatrix4f; var x, y, z : float);
function MatrixFromQuaternion(Q: TQuaternion) : TMatrix4f;
function MatrixIdentity : TMatrix4f;
procedure MatrixInvert(var M: TMatrix4f);
function MatrixIsIdentity(M: TMatrix4f) : boolean;
function MatrixMirrorPlane(p, v: TVec) : TMatrix4f;
function MatrixMultiply(M1, M2: TMatrix4f): TMatrix4f;
function MatrixPostMultiply(M1, M2: TMatrix4f): TMatrix4f;
function MatrixRotationX(AngleX: float) : TMatrix4f;
function MatrixRotationY(AngleY: float) : TMatrix4f;
function MatrixRotationZ(AngleZ: float) : TMatrix4f;
procedure MatrixScale(var M: TMatrix4f; ScaleX, ScaleY, ScaleZ: float);
procedure MatrixSetInverseTranslation(var M: TMatrix4f; x, y, z: float);
procedure MatrixSetInverseRotation(var M: TMatrix4f; xa, ya, za: float);
procedure MatrixSetRotation(var M: TMatrix4f; xa, ya, za: float);
procedure MatrixSetTranslation(var M: TMatrix4f; x, y, z: float);
procedure MatrixSetRotationQuaternion(var M: TMatrix4f; Q: TQuaternion);
function MatrixShadowPlane(plane, light: TVec4f) : TMatrix4f;
function MatrixTranslation(TranslateX, TranslateY, TranslateZ: float) : TMatrix4f;

function QuaternionAdd(q1, q2: TQuaternion) : TQuaternion;
procedure QuaternionConjugate(var Q: TQuaternion);
procedure QuaternionNormalize(var Q: TQuaternion);
function QuaternionFromMatrix(M: TMatrix4f) : TQuaternion;
function QuaternionMultiply(q1, q2: TQuaternion) : TQuaternion;
procedure QuaternionScale(var Q: TQuaternion; s: float);
function QuaternionSlerp(q1, q2: TQuaternion; t: float) : TQuaternion;

function AxisRotationToMatrix(Axis: TVec; Angle: float) : TMatrix4f;
function AxisRotation(ax,ay,az: float; var axis: TVec) : float;

procedure PlanePrepareTri(vecs : PVecs; tri : TTri);
procedure PlanePrepare(p1,p2,p3 : PVec);
function PlaneIntersectLine(v1, v2 : TVec; var Intersection : TVec) : boolean;
function PlaneX(y, z : Float; var x : Float) : boolean;
function PlaneY(x, z : Float; var y : Float) : boolean;
function PlaneZ(x, y : Float; var z : Float) : boolean;
function ClassifySphere(var Center, Normal, Point : TVec; radius : float) : TSphereClassification;
function PlaneIntersectSphere(Center : TVec; radius : float) : boolean;

// various help functions
function Color4f(r, g, b, a : GLfloat) : TColor4f;
function PointIsOccluded(p: TVec) : boolean;
function RectIsOccluded(Center: TVec; hw, hh: integer) : float;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  bg commands
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


function LoadTGAImage(FileName : ShortString) : PRGBAImage;

function NewRenderSettings : TRenderSettings;
function RemoveRenderSettings(rs: TRenderSettings) : boolean;
function GetRenderSettingsIndex(rs: TRenderSettings) : integer;

function NewTextureSettings : TTextureSettings;
function RemoveTextureSettings(ts: TTextureSettings) : boolean;
function GetTextureSettingsIndex(ts: TTextureSettings) : integer;

{$IFDEF GL_11} function NewTexture1D(FName: ShortString; components: GLint) : TTexture1D; {$ENDIF}
{$IFDEF GL_11} function NewTexture2D(FName: ShortString; components: GLint) : TTexture2D; {$ENDIF}
{$IFDEF GL_12} function NewTexture3D(FName: ShortString; components: GLint) : TTexture3D; {$ENDIF}
{$IFDEF GL_13} function NewTextureCubemap(FName: ShortString; components : GLint) : TTextureCubeMap; {$ENDIF}
{$IFDEF GL_13} function NewTextureCubemapFromTGAs(N1,N2,N3,N4,N5,N6: ShortString; components : GLint) : TTextureCubeMap; {$ENDIF}
{$IFDEF GL_11} function NewDynamicTexture2D(width, height: integer; components : GLint) : TDynamicTexture2D; {$ENDIF}
{$IFDEF GL_13} function NewDynamicTextureCubeMap(side: integer; components : GLint) : TDynamicTextureCubeMap; {$ENDIF}
{$IFDEF GL_11} function NewTextureDefault: TTextureDefault; {$ENDIF}
function RemoveTexture(t : TTexture) : Boolean;
function GetTextureIndex(t : TTexture) : integer;
procedure UpdateDynamicTextures;

function NewMaterial: TMaterial;
function RemoveMaterial(m: TMaterial): boolean;
function GetMaterialIndex(m: TMaterial): integer;

{$IFDEF GL_PROGRAM}
function NewVertexProgram: TVertexProgram;
function RemoveVertexProgram(vp: TVertexProgram): boolean;
function GetVertexProgramIndex(vp: TVertexProgram): integer;
procedure UpdateVPEnvParams;
procedure SetVPEnvParam(index: integer; a,b,c,d: float);
procedure SetVPEnvParamv(index: integer; p: TVec4f);
procedure SetProgramMatrix(index: integer; m: TMatrix4f);
procedure BindProgramMatrix(index: integer; obj: TObj);

function NewFragmentProgram: TFragmentProgram;
function RemoveFragmentProgram(fp: TFragmentProgram): boolean;
function GetFragmentProgramIndex(fp: TFragmentProgram): integer;
procedure UpdateFPEnvParams;
procedure SetFPEnvParam(index: integer; a,b,c,d: float);
procedure SetFPEnvParamv(index: integer; p: TVec4f);
{$ENDIF}

function GetObj(handle : GLuint) : TObj;
procedure ShowObj(b:TObj);
procedure HideObj(b:TObj);
procedure BindLight(index : integer; obj : TObj);
procedure BindSpotLight(index : integer; obj : TObj);
procedure BindView(obj : TObj);
function DisplayedList : PObjList;
procedure ReadPrimitive(var F : File);

procedure bgAnimate(TimeDifference: integer);
procedure bgSetBackColor(r,g,b,a : GLfloat);
procedure bgClearScreen;
procedure bgCloseWindow;
procedure bgDisplayWindow;
procedure bgDrawBackground;
procedure bgDrawObjects;
procedure bgDrawScene;
procedure bgFastHint;
function bgInitWindow(Left, Top, Width, Height : Integer;
                       Title : ShortString;
                       FullScreen : Boolean;
                       Parent : HWnd;
                       WndProc : PWndProc) : Boolean;
procedure bgNiceHint;
procedure bgResizeViewport(Width: GLsizei; Height: GLsizei);
procedure bgSetMatrices;
procedure bgShowScene;
procedure bgSwapBuffers;

procedure ViewComputeInvertedTextureMatrixList;
function ViewContainsAt(left,top,width,height: GLint; bt : PGLuintTable; max: integer) : integer;
procedure ViewDirectionX(length : float; var v : TVec);
procedure ViewDirectionY(length : float; var v : TVec);
procedure ViewDirectionZ(length : float; var v : TVec);
procedure ViewMove(x,y,z : float);
procedure ViewPerspective(FoV, Aspect, NearZ, FarZ : GLDouble);
procedure ViewRotate(xa,ya,za : float);
procedure ViewSet(x,y,z, xa,ya,za : Float);

procedure LightAmbient(index : integer; r, g, b, a : GLfloat);
procedure LightDiffuse(index : integer; r, g, b, a : GLfloat);
procedure LightSpecular(index : integer; r, g, b, a : GLfloat);
procedure LightShadow(index : integer; r, g, b, a : GLfloat);
procedure LightPosition(index : integer; x, y, z : GLfloat);
procedure LightDirection(index : integer; x, y, z : GLfloat);
procedure LightSpotExponent(index : integer; v :  GLfloat);
procedure LightSpotCutOff(index : integer; v : GLfloat);
procedure LightSpotDirection(index : integer; x, y, z : GLfloat);
procedure LightConstantAttenuation(index : integer; v : GLfloat);
procedure LightLinearAttenuation(index : integer; v : GLfloat);
procedure LightQuadraticAttenuation(index : integer; v : GLfloat);
procedure LightEnable(index : integer);
procedure LightDisable(index : integer);
function LightAddFlare(index : integer; Position, HSize, VSize, cr,cg,cb,ca: float; Tex: TTexture2D): integer;
procedure LightFlareOcclusion(index: integer; HOccSize, VOccSize: integer);
procedure LightRemoveFlare(index, FlareIndex : integer);
procedure LightRemoveFlares(index : integer);
procedure LightDrawFlares;

procedure FogColor(r, g, b, a : GLfloat);
procedure FogDensity(density : GLfloat);
procedure FogStart(start : GLfloat);
procedure FogEnd(fogend : GLfloat);
procedure FogLinear;
procedure FogExp;
procedure FogExp2;
procedure FogEnable;
procedure FogDisable;

procedure FrustumNormalizePlane(side : TFrustumSide);
procedure FrustumCalculate;
function PointInFrustum(x, y, z : float) : boolean;
function SphereInFrustum(x, y, z, radius : float) : boolean;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  BaseGraph constants
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


const

  // maximum number of shaders per primitive
  MAXSHADERS = 64;

  MAXLIGHTS = 8;
  MAXLIGHTFLARES = 16;
  MAXTRANSPARENTOBJECTS = 1024;
  MAXSHADOWEDOBJECTS = 1024;
  MAXDYNAMICTEXTURES = 128;
  MAXCONCURRENTANIMATIONS = 4096;

  // OpenGL file tokens
  TOKEN_BEGIN = 0;
  TOKEN_NONE = 1;
  TOKEN_V3F = 13;
  TOKEN_V4F = 14;
  TOKEN_N3F = 23;
  TOKEN_C3F = 43;
  TOKEN_C4F = 44;
  TOKEN_F1F = 51;
  TOKEN_A04F = 60;
  TOKEN_A314F = 91;
  // the values 60 to 91 are reserved for four component vertex parameters (indexed from 0 to 31)
  TOKEN_U0T1F = 100;
  TOKEN_U0T2F = 101;
  TOKEN_U0T3F = 102;
  TOKEN_U0T4F = 103;
  TOKEN_U31T4F = 227;
  // the values 105 to 227 are reserved for multi texture coordinates for up to 32 texture units
  TOKEN_END = 255;

  // shader identity
  SD_NONE = 0;              // no/undefined shader

  SD_TEX_NONE = 1;          // undefined texture shader
  SD_TEX_1D   = 2;          // 1D-texture shader
  SD_TEX_2D   = 3;          // 2D-texture shader
  SD_TEX_3D   = 4;          // 3D-texture shader
  SD_TEX_CUBE = 5;          // cubemap texture shader
  SD_TEX_DYN_2D = 6;        // dynamic 2D-texture shader
  SD_TEX_DYN_CUBE = 7;      // dynamic cubemap texture shader
  SD_TEX_DEFAULT = 8;       // default texture shader

  SD_MAT = 9;               // material shader
  SD_RS = 10;               // render settings shader
  SD_TS = 11;               // texture settings shader

  SD_VP = 12;               // vertex program shader
  SD_FP = 13;               // fragment program shader



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  BaseGraph variables
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


var
  NearZeroValue : float = 1/16384;
  bgPlane : TPlane;

  bgMaxTextureSize : integer = 256;
  bgMaxTextureUnits : integer = 1;

  bgObjAnimations : integer = 0;
  bgObjAnimation : array[0..MAXCONCURRENTANIMATIONS - 1] of TObjAnimation;

  bgTransparentList : PObjList;
  bgTransparentTable : Array[0..MAXTRANSPARENTOBJECTS - 1] of TObjList;
  bgTransparentObjects : Integer;

  bgShadowTable : Array[0..MAXSHADOWEDOBJECTS - 1] of TObj;
  bgShadowObjects : Integer;
  bgDrawShadows : Boolean = True;
  bgShadowLength : float = 50.0;
  bgShadowStencilStart : GLuint = 128;
  bgShadowViewDist : float = 100.0;

  bgViewMatrix : TMatrix4f;
  bgViewInvertedMatrix : TMatrix4f;
  bgViewTextureMatrix : TMatrix4f;
  bgViewInvertedTextureMatrixList : GLenum;
  bgFieldOfView : GLDouble = 45.0;
  bgAspectRatio : GLDouble = 1.0;
  bgZNear : GLDouble = 1.0;
  bgZFar  : GLDouble = 255.0;
  bgFrustum : TFrustum;

  bgObjList : PObjList = NIL;
  bgActive : Boolean = False;
  bgRun : Boolean = False;

  bgSafeCoordRotation : Boolean = FALSE; // enables extra code to make sure that coordinate
                                          // axii stay in range, when rotated
  bgUseMipMaps : Boolean = FALSE;        // enables mipmapped textures
  bgUseDlgFrame : Boolean = FALSE;

  bgBackColor : TColor4f; // Background Color
  bgClearBufferBits : GLenum = GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT + GL_STENCIL_BUFFER_BIT;

  bgBoundView : TObj = NIL;
  bgBoundLight : array[0..MAXLIGHTS-1] of TObj = (NIL, NIL, NIL, NIL, NIL, NIL, NIL, NIL);
  bgBoundSpotLight : array[0..MAXLIGHTS-1] of TObj  = (NIL, NIL, NIL, NIL, NIL, NIL, NIL, NIL);

  bgLight : array[0..MAXLIGHTS-1] of TLight = (
    (
      position  : (0.0, 0.0, 1.0, 0.0);  ambient   : (0.0, 0.0, 0.0, 1.0);  diffuse   : (1.0, 1.0, 1.0, 1.0);
      specular  : (1.0, 1.0, 1.0, 1.0);  shadow    : (0.0, 0.0, 0.0, 0.5);
      spotexponent : 0.0;                spotcutoff : 180.0;                spotdirection : (0.0, 0.0, -1.0);
      constantattenuation : 1.0;         linearattenuation : 0.0;           quadraticattenuation : 0.0;
      enabled   : true;                  shadowing : true;
    ),
    (
      position  : (0.0, 0.0, 1.0, 0.0);  ambient   : (0.0, 0.0, 0.0, 1.0);  diffuse   : (0.0, 0.0, 0.0, 0.0);
      specular  : (0.0, 0.0, 0.0, 1.0);  shadow    : (0.0, 0.0, 0.0, 0.5);
      spotexponent : 0.0;                spotcutoff : 180.0;                spotdirection : (0.0, 0.0, -1.0);
      constantattenuation : 1.0;         linearattenuation : 0.0;           quadraticattenuation : 0.0;
      enabled   : false;                 shadowing : false;
    ),
    (
      position  : (0.0, 0.0, 1.0, 0.0);  ambient   : (0.0, 0.0, 0.0, 1.0);  diffuse   : (0.0, 0.0, 0.0, 0.0);
      specular  : (0.0, 0.0, 0.0, 1.0);  shadow    : (0.0, 0.0, 0.0, 0.5);
      spotexponent : 0.0;                spotcutoff : 180.0;                spotdirection : (0.0, 0.0, -1.0);
      constantattenuation : 1.0;         linearattenuation : 0.0;           quadraticattenuation : 0.0;
      enabled   : false;                 shadowing : false;
    ),
    (
      position  : (0.0, 0.0, 1.0, 0.0);  ambient   : (0.0, 0.0, 0.0, 1.0);  diffuse   : (0.0, 0.0, 0.0, 0.0);
      specular  : (0.0, 0.0, 0.0, 1.0);  shadow    : (0.0, 0.0, 0.0, 0.5);
      spotexponent : 0.0;                spotcutoff : 180.0;                spotdirection : (0.0, 0.0, -1.0);
      constantattenuation : 1.0;         linearattenuation : 0.0;           quadraticattenuation : 0.0;
      enabled   : false;                 shadowing : false;
    ),
    (
      position  : (0.0, 0.0, 1.0, 0.0);  ambient   : (0.0, 0.0, 0.0, 1.0);  diffuse   : (0.0, 0.0, 0.0, 0.0);
      specular  : (0.0, 0.0, 0.0, 1.0);  shadow    : (0.0, 0.0, 0.0, 0.5);
      spotexponent : 0.0;                spotcutoff : 180.0;                spotdirection : (0.0, 0.0, -1.0);
      constantattenuation : 1.0;         linearattenuation : 0.0;           quadraticattenuation : 0.0;
      enabled   : false;                 shadowing : false;
    ),
    (
      position  : (0.0, 0.0, 1.0, 0.0);  ambient   : (0.0, 0.0, 0.0, 1.0);  diffuse   : (0.0, 0.0, 0.0, 0.0);
      specular  : (0.0, 0.0, 0.0, 1.0);  shadow    : (0.0, 0.0, 0.0, 0.5);
      spotexponent : 0.0;                spotcutoff : 180.0;                spotdirection : (0.0, 0.0, -1.0);
      constantattenuation : 1.0;         linearattenuation : 0.0;           quadraticattenuation : 0.0;
      enabled   : false;                 shadowing : false;
    ),
    (
      position  : (0.0, 0.0, 1.0, 0.0);  ambient   : (0.0, 0.0, 0.0, 1.0);  diffuse   : (0.0, 0.0, 0.0, 0.0);
      specular  : (0.0, 0.0, 0.0, 1.0);  shadow    : (0.0, 0.0, 0.0, 0.5);
      spotexponent : 0.0;                spotcutoff : 180.0;                spotdirection : (0.0, 0.0, -1.0);
      constantattenuation : 1.0;         linearattenuation : 0.0;           quadraticattenuation : 0.0;
      enabled   : false;                 shadowing : false;
    ),
    (
      position  : (0.0, 0.0, 1.0, 0.0);  ambient   : (0.0, 0.0, 0.0, 1.0);  diffuse   : (0.0, 0.0, 0.0, 0.0);
      specular  : (0.0, 0.0, 0.0, 1.0);  shadow    : (0.0, 0.0, 0.0, 0.5);
      spotexponent : 0.0;                spotcutoff : 180.0;                spotdirection : (0.0, 0.0, -1.0);
      constantattenuation : 1.0;         linearattenuation : 0.0;           quadraticattenuation : 0.0;
      enabled   : false;                 shadowing : false;
    )
  );

// light flare type information
type
  TLightFlare = record
    Position,             // position of flare relative to distance viewer/lightsource
    HSize,                // horizontal size
    VSize     : float;    // vertical size
    Color     : TColor4f; // color of light flare
    Tex       : TTexture2D;
  end;
  PLightFlare = ^TLightFlare;
  TLightFlares = record
    HOccSize  : integer;  // horizontal occlusion size
    VOccSize  : integer;  // vertical occlusion size
    Flares    : integer;
    Flare     : array[0..MAXLIGHTFLARES-1] of TLightFlare;
  end;


var
  bgLightFlare : array[0..MAXLIGHTS-1] of TLightFlares;
  bgLightFlares : boolean = True;

  bgRenderSet : PRenderSettingsTable = NIL;
  bgRenderSets : integer = 0;

  bgTextureSet : PTextureSettingsTable = NIL;
  bgTextureSets : integer = 0;

  bgObject : PObjTable = NIL;
  bgObjects : integer = 0;

  bgTexture : PTextureTable = NIL;
  bgTextures : integer = 0;
  bgDynamicTextures : Boolean = True;
  bgBoundSkyBox : TTexture = NIL;
  bgDynTexture : Array[0..MAXDYNAMICTEXTURES - 1] of TTexture;
  bgDynTextures : integer = 0;

  bgMaterial : PMaterialTable = NIL;
  bgMaterials : integer = 0;



{$IFDEF GL_PROGRAM}
  bgVertexProgram : PVertexProgramTable = NIL;
  bgVertexPrograms : integer = 0;

  bgFragmentProgram : PFragmentProgramTable = NIL;
  bgFragmentPrograms : integer = 0;

  // environment program parameters
  bgVPEnvParam: TProgramParameters;  // environment parameters of vertex programs
  bgVPEnvParams: integer = 0;  // number of environment paramters of vertex programs
  bgProgramMatrix: TProgramMatrices; // program matrices
  bgProgramMatrices: integer = 0; // number of program matrices

  bgFPEnvParam: TProgramParameters; // environment parameters of fragment programs
  bgFPEnvParams: integer = 0;  // number of environment parameters of fragment programs
{$ENDIF}

  bgAutoCompile : Boolean = True;

  bgWND: HWND;         // Permanent Window Handle
  bgRC : HGLRC;        // Permanent Rendering Context
  bgDC : HDC;          // Permanent GDI Device Context

  // pfd tells Windows how we want things to be');
  bgPFD: TPixelFormatDescriptor = (
    nSize:         SizeOf(TPixelFormatDescriptor); // size of pixel format descriptor
    nVersion:      1;                        // version number
    dwFlags:       PFD_DRAW_TO_WINDOW OR     // format must support window
                   PFD_SUPPORT_OPENGL OR     // format must support OpenGL
                   PFD_DOUBLEBUFFER;         // must support double buffering
    iPixelType:    PFD_TYPE_RGBA;            // request a RGBA format
    cColorBits:    24;                       // select color depth
    cRedBits:      0;                        // red bits ignored
    cRedShift:     0;                        // shift bits ignored
    cGreenBits:    0;                        // green bits ignored
    cGreenShift:   0;                        // shift bits ignored
    cBlueBits:     0;                        // blue bits ignored
    cBlueShift:    0;                        // color bits ignored
    cAlphaBits:    0;                        // alpha buffer bits
    cAlphaShift:   0;                        // shift bits ignored
    cAccumBits:    16;                       // accumulation buffer
    cAccumRedBits: 0;                        // accumulation red bits ignored
    cAccumGreenBits: 0;                      // accumulation green bits ignored
    cAccumBlueBits: 0;                       // accumulation blue bits ignored
    cAccumAlphaBits: 0;                      // accumulation alpha bits ignored
    cDepthBits:    32;                       // select z-buffer depth
    cStencilBits:  8;                        // stencil buffer bits
    cAuxBuffers:   0;                        // auxiliary buffers
    iLayerType:    PFD_MAIN_PLANE;           // main drawing layer
    bReserved:     0;                        // reserved
    dwLayerMask:   0;                        // layer masks ignored
    dwVisibleMask: 0;                        // visible masks ignored
    dwDamageMask:  0;                        // damage masks ignored
  );

  bgPixelFormat : integer = 0;              // currently used PixelFormat

  bgMouseCursor : PChar = IDC_ARROW;        // standard mouse cursor



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  start of the BaseGraph implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


implementation


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mesh implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



constructor TMesh.Create(matrix : PMatrix4f);

  begin
    meshMatrixP := matrix;
    meshVerts := 0;
    meshTris := 0;
    meshStaticVert := false;
    meshStaticTri := false;
    meshVert := NIL;
    meshTri := NIL;
    meshTransformed := NIL;
    meshTriMin := NIL;
    meshTriMax := NIL;
    meshTSI := NIL;
    meshSilhouette := NIL;
    meshVSI := NIL;
    meshSilhouetteEdges := 0;
  end;



constructor TMesh.CreateByData(matrix : PMatrix4f; vnum, tnum : integer; v, t : pointer);

  begin
    meshMatrixP := matrix;
    meshVerts := vnum;
    meshTris := tnum;
    meshStaticVert := true;
    meshStaticTri := true;
    meshVert := v;
    meshTri := t;
    GetMem(meshTransformed, meshVerts * SizeOf(TVec));
    GetMem(meshTriMin, meshTris * SizeOf(TVec));
    GetMem(meshTriMax, meshTris * SizeOf(TVec));
    meshTSI := NIL;
    meshSilhouette := NIL;
    meshVSI := NIL;
    meshSilhouetteEdges := 0;
  end;



destructor TMesh.Destroy;

  begin
    if not meshStaticVert then if meshVert<>NIL then FreeMem(meshVert);
    if not meshStaticTri then if meshTri<>NIL then FreeMem(meshTri);
    if meshTransformed<>NIL then FreeMem(meshTransformed);
    if meshTriMin<>NIL then FreeMem(meshTriMin);
    if meshTriMax<>NIL then FreeMem(meshTriMax);
    if meshTSI<>NIL then FreeMem(meshTSI);
    if meshSilhouette<>NIL then FreeMem(meshSilhouette);
    if meshVSI<>NIL then FreeMem(meshVSI);
    inherited Destroy;
  end;



procedure TMesh.Transform;

  var
    i : integer;

  begin
    for i := 0 to meshVerts-1 do
      VectorTransformTo(meshVert[i], meshTransformed[i], meshMatrixP^);
  end;



procedure TMesh.TriMinMax;

  var
    t : integer;
    pt : PTri;
    v1, v2, v3 : PVec;
    m : float;

  begin
    for t := 0 to meshTris-1 do begin
      pt := @meshTri[t];
      v1 := @meshTransformed[pt[0]];
      v2 := @meshTransformed[pt[1]];
      v3 := @meshTransformed[pt[2]];

      if v1.x<v2.x then m:=v1.x else m:=v2.x;
      if m<v3.x then meshTriMin[t].x:=m else meshTriMin[t].x:=v3.x;
      if v1.x>v2.x then m:=v1.x else m:=v2.x;
      if m>v3.x then meshTriMax[t].x:=m else meshTriMax[t].x:=v3.x;

      if v1.y<v2.y then m:=v1.y else m:=v2.y;
      if m<v3.y then meshTriMin[t].y:=m else meshTriMin[t].y:=v3.y;
      if v1.y>v2.y then m:=v1.y else m:=v2.y;
      if m>v3.y then meshTriMax[t].y:=m else meshTriMax[t].y:=v3.y;

      if v1.z<v2.z then m:=v1.z else m:=v2.z;
      if m<v3.z then meshTriMin[t].z:=m else meshTriMin[t].z:=v3.z;
      if v1.z>v2.z then m:=v1.z else m:=v2.z;
      if m>v3.z then meshTriMax[t].z:=m else meshTriMax[t].z:=v3.z;

    end;
  end;



function TMesh.QuickIntersect(mesh : TMesh) : Boolean;

  var
    i, j : integer;
    xmin,xmax,ymin,ymax,zmin,zmax : float;
    Tris : integer;
    TriMax, TriMin : PVecs;

  begin
    tris := mesh.meshTris;
    trimax := mesh.meshTriMax;
    trimin := mesh.meshTriMin;
    for j := 0 to meshTris-1 do begin
      xmin := meshTriMin[j].x;
      ymin := meshTriMin[j].y;
      zmin := meshTriMin[j].z;
      xmax := meshTriMax[j].x;
      ymax := meshTriMax[j].y;
      zmax := meshTriMax[j].z;
      for i := 0 to Tris-1 do begin
        if (xmin<=TriMax[i].x) and (xmax>=TriMin[i].x) and
           (ymin<=TriMax[i].y) and (ymax>=TriMin[i].y) and
           (zmin<=TriMax[i].z) and (zmax>=TriMin[i].z) then begin
          result := true;
          exit;
        end;
      end;
    end;
    result := false;
  end;



function TMesh.Intersect(mesh : TMesh; var intersection : TVec) : Boolean;

  var
    i, j : integer;
    xmin,xmax,ymin,ymax,zmin,zmax : float;
    pt : PTri;
    TriMax, TriMin : PVecs;
    Transformed : PVecs;
    Tri : PTris;
    TSI : PTriShadowInfos;
    ps : PTriShadowInfo;

  begin
    TriMax := mesh.meshTriMax;
    TriMin := mesh.meshTriMin;
    Transformed := mesh.meshTransformed;
    Tri := mesh.meshTri;
    TSI := mesh.meshTSI;

    if TSI = NIL then begin // mesh.meshTSI not defined
      for j := 0 to meshTris-1 do begin
        xmin := meshTriMin[j].x;
        ymin := meshTriMin[j].y;
        zmin := meshTriMin[j].z;
        xmax := meshTriMax[j].x;
        ymax := meshTriMax[j].y;
        zmax := meshTriMax[j].z;
        PlanePrepareTri(meshTransformed, meshTri[j]);
        for i := 0 to mesh.meshTris-1 do begin
          if (xmin<=TriMax[i].x) and (xmax>=TriMin[i].x) and
             (ymin<=TriMax[i].y) and (ymax>=TriMin[i].y) and
             (zmin<=TriMax[i].z) and (zmax>=TriMin[i].z) then begin
            pt := @Tri[i];
            if PlaneIntersectLine(Transformed[pt[0]], Transformed[pt[1]], intersection) then begin
              result := true;
              exit;
            end
            else if PlaneIntersectLine(Transformed[pt[1]], Transformed[pt[2]], intersection) then begin
              result := true;
              exit;
            end
            else if PlaneIntersectLine(mesh.meshTransformed[pt[2]], Transformed[pt[0]], intersection) then begin
              result := true;
              exit;
            end;
          end;
        end;
      end;
    end
    else begin // mesh.meshTSI defined
      for j := 0 to meshTris-1 do begin
        xmin := meshTriMin[j].x;
        ymin := meshTriMin[j].y;
        zmin := meshTriMin[j].z;
        xmax := meshTriMax[j].x;
        ymax := meshTriMax[j].y;
        zmax := meshTriMax[j].z;
        PlanePrepareTri(meshTransformed, meshTri[j]);
        for i := mesh.meshTris-1 downto 0 do begin
          if (xmin<=TriMax[i].x) and (xmax>=TriMin[i].x) and
             (ymin<=TriMax[i].y) and (ymax>=TriMin[i].y) and
             (zmin<=TriMax[i].z) and (zmax>=TriMin[i].z) then begin
            pt := @Tri[i];
            ps := @TSI[i];
            if (ps.neigh[0]<=i) then if PlaneIntersectLine(Transformed[pt[0]], Transformed[pt[1]], intersection) then begin
              result := true;
              exit;
            end;
            if (ps.neigh[1]<=i) then if PlaneIntersectLine(Transformed[pt[1]], Transformed[pt[2]], intersection) then begin
              result := true;
              exit;
            end;
            if (ps.neigh[2]<=i) then if PlaneIntersectLine(mesh.meshTransformed[pt[2]], Transformed[pt[0]], intersection) then begin
              result := true;
              exit;
            end;
          end;
        end;
      end;
    end;
    result := false;
  end;



function TMesh.IntersectSphere(Center : TVec; radius : float) : Boolean;

  var
    i : integer;
    xmin,xmax,ymin,ymax,zmin,zmax : float;

  begin
    xmin := Center.x - radius;
    ymin := Center.y - radius;
    zmin := Center.z - radius;
    xmax := Center.x + radius;
    ymax := Center.y + radius;
    zmax := Center.z + radius;
    for i := 0 to meshTris-1 do begin
      if (xmin<=meshTriMax[i].x) and (xmax>=meshTriMin[i].x) and
         (ymin<=meshTriMax[i].y) and (ymax>=meshTriMin[i].y) and
         (zmin<=meshTriMax[i].z) and (zmax>=meshTriMin[i].z) then begin
        PlanePrepareTri(meshTransformed, meshTri[i]);
        if PlaneIntersectSphere(Center, radius) then begin
          result := true;
          exit;
        end;
      end;
    end;
    result := false;
  end;



function TMesh.IntersectLine(v1, v2 : TVec; Intersection : PVecs; max : Integer) : integer;

  var
    i, n : integer;

  begin
    n := 0;
    for i := 0 to meshTris-1 do begin
      PlanePrepareTri(meshTransformed, meshTri[i]);
      if PlaneIntersectLine(v1, v2, Intersection[n]) then begin
        inc(n);
        if n>=max then break;
      end;
    end;
    IntersectLine := n;
  end;



procedure TMesh.CreateShadowInfo;

  var
    p1i, p2i, p1j, p2j : integer;
    i, j, ki, kj : integer;

  begin
    ReallocMem(meshTSI, meshTris * SizeOf(TTriShadowInfo));
    ReallocMem(meshSilhouette, 2 * meshVerts * SizeOf(Integer));
    ReallocMem(meshVSI, meshVerts * SizeOf(TVertShadowInfo));
    FillChar(meshTSI^, meshTris * SizeOf(TTriShadowInfo), 0);
    FillChar(meshSilhouette^, 2 * meshVerts * SizeOf(Integer), 0);
    FillChar(meshVSI^, meshVerts * SizeOf(TVertShadowInfo), 0);

    for i:=0 to meshTris-1 do
      for j:=i+1 to meshTris-1 do
        for ki:=0 to 2 do
          if meshTSI[i].neigh[ki]=0 then
            for kj:=0 to 2 do begin
              p1i := ki;
              p1j := kj;
              if ki=2 then p2i:=0 else p2i:=ki+1;
              if kj=2 then p2j:=0 else p2j:=kj+1;

              p1i := meshTri[i][p1i];
              p2i := meshTri[i][p2i];
              p1j := meshTri[j][p1j];
              p2j := meshTri[j][p2j];

              if ((p1i=p1j) and (p2i=p2j)) or ((p1i=p2j) and (p2i=p1j)) then begin
                meshTSI[i].neigh[ki] := j+1;
                meshTSI[j].neigh[kj] := i+1;
              end;
            end;
    ComputeShadowPlaneEq;
  end;



procedure TMesh.ComputeShadowPlaneEq;

  var
    i : integer;
    v : array[0..2] of TVec;

  begin
    if meshTSI<>NIL then for i:=0 to meshTris-1 do begin
      v[0] := meshVert[meshTri[i][0]];
      v[1] := meshVert[meshTri[i][1]];
      v[2] := meshVert[meshTri[i][2]];
      meshTSI[i].PlaneEq[0] := v[0].y*(v[1].z-v[2].z) + v[1].y*(v[2].z-v[0].z) + v[2].y*(v[0].z-v[1].z);
      meshTSI[i].PlaneEq[1] := v[0].z*(v[1].x-v[2].x) + v[1].z*(v[2].x-v[0].x) + v[2].z*(v[0].x-v[1].x);
      meshTSI[i].PlaneEq[2] := v[0].x*(v[1].y-v[2].y) + v[1].x*(v[2].y-v[0].y) + v[2].x*(v[0].y-v[1].y);
      meshTSI[i].PlaneEq[3] :=-( v[0].x*(v[1].y*v[2].z - v[2].y*v[1].z) +
                         v[1].x*(v[2].y*v[0].z - v[0].y*v[2].z) +
                         v[2].x*(v[0].y*v[1].z - v[1].y*v[0].z) );
    end;
  end;



procedure TMesh.ComputeShadowPlaneNorm;

  var
    i : integer;
    v : array[0..2] of TVec;
    l : float;

  begin
    if meshTSI<>NIL then for i:=0 to meshTris-1 do begin
      v[0] := meshVert[meshTri[i][0]];
      v[1] := meshVert[meshTri[i][1]];
      v[2] := meshVert[meshTri[i][2]];
      meshTSI[i].PlaneEq[0] := v[0].y*(v[1].z-v[2].z) + v[1].y*(v[2].z-v[0].z) + v[2].y*(v[0].z-v[1].z);
      meshTSI[i].PlaneEq[1] := v[0].z*(v[1].x-v[2].x) + v[1].z*(v[2].x-v[0].x) + v[2].z*(v[0].x-v[1].x);
      meshTSI[i].PlaneEq[2] := v[0].x*(v[1].y-v[2].y) + v[1].x*(v[2].y-v[0].y) + v[2].x*(v[0].y-v[1].y);
      meshTSI[i].PlaneEq[3] :=-( v[0].x*(v[1].y*v[2].z - v[2].y*v[1].z) +
                         v[1].x*(v[2].y*v[0].z - v[0].y*v[2].z) +
                         v[2].x*(v[0].y*v[1].z - v[1].y*v[0].z) );
      l := 1.0 / sqrt(meshTSI[i].PlaneEq[0]*meshTSI[i].PlaneEq[0] +
                      meshTSI[i].PlaneEq[1]*meshTSI[i].PlaneEq[1] +
                      meshTSI[i].PlaneEq[2]*meshTSI[i].PlaneEq[2]);
      meshTSI[i].norm.x := meshTSI[i].PlaneEq[0] * l;
      meshTSI[i].norm.y := meshTSI[i].PlaneEq[1] * l;
      meshTSI[i].norm.z := meshTSI[i].PlaneEq[2] * l;
    end;
  end;



const NextEdge : Array[0..2] of Integer = (1,2,0);
procedure TMesh.CastFastShadow(lp: TVec; SelfShadowing: boolean);

  var
    i, j, k, e : integer;
    v1 : PVec;
    sv : PVertShadowInfo;
    side, mult : float;
    minv : TMatrix4f;

  begin
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(minv));
    MatrixInvert(minv);
    VectorTransform(lp, minv);

    // clear vertex shadow information
    FillChar(meshVSI^, meshVerts * SizeOf(TVertShadowInfo), 0);

    //set visible parameter
    for i:=0 to meshTris-1 do begin
      // check if light is in front or behind the plane (face plane)
      side := meshTSI[i].PlaneEq[0]*lp.x + meshTSI[i].PlaneEq[1]*lp.y + meshTSI[i].PlaneEq[2]*lp.z + meshTSI[i].PlaneEq[3];
      meshTSI[i].visible := side > 0.0;
    end;

    // compute silhouette
    e := 0;
    for i:=0 to meshTris-1 do begin
      if meshTSI[i].visible then begin
        for j:=0 to 2 do begin
          k := meshTSI[i].neigh[j];
          if (k=0) or not meshTSI[k-1].visible then begin
            // here we have an edge, we must draw a polygon
            meshSilhouette[e] := meshTri[i][j];
            inc(e);
            meshSilhouette[e] := meshTri[i][NextEdge[j]];
            inc(e);
          end;
        end;
      end; // if
    end; // for
    meshSilhouetteEdges := e;

    // first pass, stencil operation increases stencil value
    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);
{$IFDEF GL_14}
    if GL_14 then
      glStencilOp(GL_KEEP, GL_KEEP, GL_INCR_WRAP)
    else
      glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
{$ELSE}
    glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
{$ENDIF}
    glBegin(GL_QUADS);
    i := 0;
    while i<e do begin
      v1 := @meshVert[meshSilhouette[i]];
      sv := @meshVSI[meshSilhouette[i]];
      if not sv.computed then begin
        sv.v.x:=v1.x-lp.x;  sv.v.y:=v1.y-lp.y;  sv.v.z:=v1.z-lp.z;
        mult:=bgShadowLength / sqrt(sv.v.x*sv.v.x + sv.v.y*sv.v.y + sv.v.z*sv.v.z);
        sv.v.x:=v1.x+mult*sv.v.x;  sv.v.y:=v1.y+mult*sv.v.y;  sv.v.z:=v1.z+mult*sv.v.z;
        sv.computed := true;
      end;
      glVertex3fv(PGLfloat(v1));
      glVertex3fv(Addr(sv.v));
      inc(i);

      v1 := @meshVert[meshSilhouette[i]];
      sv := @meshVSI[meshSilhouette[i]];
      if not sv.computed then begin
        sv.v.x:=v1.x-lp.x;  sv.v.y:=v1.y-lp.y;  sv.v.z:=v1.z-lp.z;
        mult:=bgShadowLength / sqrt(sv.v.x*sv.v.x + sv.v.y*sv.v.y + sv.v.z*sv.v.z);
        sv.v.x:=v1.x+mult*sv.v.x;  sv.v.y:=v1.y+mult*sv.v.y;  sv.v.z:=v1.z+mult*sv.v.z;
        sv.computed := true;
      end;
      glVertex3fv(Addr(sv.v));
      glVertex3fv(PGLfloat(v1));
      inc(i);
    end;
    glEnd;

    // second pass, stencil operation decreases stencil value
    glCullFace(GL_FRONT);
{$IFDEF GL_14}
    if GL_14 then
      glStencilOp(GL_KEEP, GL_KEEP, GL_DECR_WRAP)
    else
      glStencilOp(GL_KEEP, GL_KEEP, GL_DECR);
{$ELSE}
    glStencilOp(GL_KEEP, GL_KEEP, GL_DECR);
{$ENDIF}
    glBegin(GL_QUADS);
    i := 0;
    while i<e do begin
      glVertex3fv(Addr(meshVert[meshSilhouette[i]]));
      glVertex3fv(Addr(meshVSI[meshSilhouette[i]].v));
      inc(i);

      glVertex3fv(Addr(meshVSI[meshSilhouette[i]].v));
      glVertex3fv(Addr(meshVert[meshSilhouette[i]]));
      inc(i);
    end;
    glEnd;
    glCullFace(GL_BACK);

    if not SelfShadowing then begin
      glStencilOp(GL_ZERO, GL_ZERO, GL_ZERO);
      glEnableClientState(GL_VERTEX_ARRAY);
      glVertexPointer(3, GL_FLOAT, 0, meshVert);
      glDrawElements(GL_TRIANGLES, meshTris * 3, GL_UNSIGNED_INT, meshTri);
      glDisableClientState(GL_VERTEX_ARRAY);
    end;
end;



procedure TMesh.CastShadow(lp: TVec; SelfShadowing: boolean);

  var
    i, j, k, e : integer;
    v1 : PVec;
    sv : PVertShadowInfo;
    side, mult : float;
    minv : TMatrix4f;

  begin
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(minv));
    MatrixInvert(minv);
    VectorTransform(lp, minv);

    //set visible parameter
    for i:=0 to meshTris-1 do begin
      // check if light is in front or behind the plane (face plane)
      side := meshTSI[i].PlaneEq[0]*lp.x + meshTSI[i].PlaneEq[1]*lp.y + meshTSI[i].PlaneEq[2]*lp.z + meshTSI[i].PlaneEq[3];
      meshTSI[i].visible := side >= 0.0;
    end;

    // compute vertex shadow vectors
    for i:=0 to meshVerts-1 do begin
      v1 := @meshVert[i];
      sv := @meshVSI[i];
      sv.v.x:=v1.x-lp.x;  sv.v.y:=v1.y-lp.y;  sv.v.z:=v1.z-lp.z;
      mult:=bgShadowLength / sqrt(sv.v.x*sv.v.x + sv.v.y*sv.v.y + sv.v.z*sv.v.z);
      sv.v.x:=v1.x+mult*sv.v.x;  sv.v.y:=v1.y+mult*sv.v.y;  sv.v.z:=v1.z+mult*sv.v.z;
      sv.computed := true;
    end;

    // compute silhouette
    e := 0;
    for i:=0 to meshTris-1 do begin
      if meshTSI[i].visible then begin
        for j:=0 to 2 do begin
          k := meshTSI[i].neigh[j];
          if (k=0) or not meshTSI[k-1].visible then begin
            // here we have an edge, we must draw a polygon
            meshSilhouette[e] := meshTri[i][j];
            inc(e);
            meshSilhouette[e] := meshTri[i][NextEdge[j]];
            inc(e);
          end;
        end;
      end; // if
    end; // for
    meshSilhouetteEdges := e;

    glEnable(GL_CULL_FACE);
    // first pass, stencil operation increases stencil value
    glCullFace(GL_FRONT);
{$IFDEF GL_14}
    if GL_14 then
      glStencilOp(GL_KEEP, GL_INCR_WRAP, GL_KEEP)
    else
      glStencilOp(GL_KEEP, GL_INCR, GL_KEEP);
{$ELSE}
    glStencilOp(GL_KEEP, GL_INCR, GL_KEEP);
{$ENDIF}
    glBegin(GL_QUADS);
    i := 0;
    while i<e do begin
      glVertex3fv(Addr(meshVert[meshSilhouette[i]]));
      glVertex3fv(Addr(meshVSI[meshSilhouette[i]].v));
      inc(i);

      glVertex3fv(Addr(meshVSI[meshSilhouette[i]].v));
      glVertex3fv(Addr(meshVert[meshSilhouette[i]]));
      inc(i);
    end;
    glEnd;

    // draw caps
    glBegin(GL_TRIANGLES);
    if SelfShadowing then begin
      // self shadowing
      for i:=0 to meshTris-1 do begin
        if meshTSI[i].visible then begin
          // top triangle
          glVertex3fv(Addr(meshVert[meshTri[i][0]]));
          glVertex3fv(Addr(meshVert[meshTri[i][1]]));
          glVertex3fv(Addr(meshVert[meshTri[i][2]]));
        end
        else begin
          // bottom triangle
          glVertex3fv(Addr(meshVSI[meshTri[i][0]].v));
          glVertex3fv(Addr(meshVSI[meshTri[i][1]].v));
          glVertex3fv(Addr(meshVSI[meshTri[i][2]].v));
        end;
      end;
    end
    else begin
      // no self shadowing
      for i:=0 to meshTris-1 do begin
        if not meshTSI[i].visible then begin
          // bottom triangle
          glVertex3fv(Addr(meshVSI[meshTri[i][0]].v));
          glVertex3fv(Addr(meshVSI[meshTri[i][1]].v));
          glVertex3fv(Addr(meshVSI[meshTri[i][2]].v));
        end;
      end;
    end;
    glEnd;

    // second pass, stencil operation decreases stencil value
    glCullFace(GL_BACK);
{$IFDEF GL_14}
    if GL_14 then
      glStencilOp(GL_KEEP, GL_DECR_WRAP, GL_KEEP)
    else
      glStencilOp(GL_KEEP, GL_DECR, GL_KEEP);
{$ELSE}
    glStencilOp(GL_KEEP, GL_DECR, GL_KEEP);
{$ENDIF}
    glBegin(GL_QUADS);
    i := 0;
    while i<e do begin
      glVertex3fv(Addr(meshVert[meshSilhouette[i]]));
      glVertex3fv(Addr(meshVSI[meshSilhouette[i]].v));
      inc(i);

      glVertex3fv(Addr(meshVSI[meshSilhouette[i]].v));
      glVertex3fv(Addr(meshVert[meshSilhouette[i]]));
      inc(i);
    end;
    glEnd;

    // draw caps
    glBegin(GL_TRIANGLES);
    if SelfShadowing then begin
      // self shadowing
      for i:=0 to meshTris-1 do begin
        if meshTSI[i].visible then begin
          // top triangle
          glVertex3fv(Addr(meshVert[meshTri[i][0]]));
          glVertex3fv(Addr(meshVert[meshTri[i][1]]));
          glVertex3fv(Addr(meshVert[meshTri[i][2]]));
        end
        else begin
          // bottom triangle
          glVertex3fv(Addr(meshVSI[meshTri[i][0]].v));
          glVertex3fv(Addr(meshVSI[meshTri[i][1]].v));
          glVertex3fv(Addr(meshVSI[meshTri[i][2]].v));
        end;
      end;
    end
    else begin
      // no self shadowing
      for i:=0 to meshTris-1 do begin
        if not meshTSI[i].visible then begin
          // bottom triangle
          glVertex3fv(Addr(meshVSI[meshTri[i][0]].v));
          glVertex3fv(Addr(meshVSI[meshTri[i][1]].v));
          glVertex3fv(Addr(meshVSI[meshTri[i][2]].v));
        end;
      end;
    end;
    glEnd;

    if not SelfShadowing then begin
      glStencilOp(GL_ZERO, GL_ZERO, GL_ZERO);
      glEnableClientState(GL_VERTEX_ARRAY);
      glVertexPointer(3, GL_FLOAT, 0, meshVert);
      glDrawElements(GL_TRIANGLES, meshTris * 3, GL_UNSIGNED_INT, meshTri);
      glDisableClientState(GL_VERTEX_ARRAY);
    end;

  end;



function TMesh.GetX(y,z : float; var x : float) : boolean;

  var
    i  : integer;

  begin
    for i:=0 to meshTris-1 do begin
      if (y>=meshTriMin[i].y) and (y<=meshTriMax[i].y) and (z>=meshTriMin[i].z) and (z<=meshTriMax[i].z) then begin
        PlanePrepareTri(meshTransformed, meshTri[i]);
        if PlaneX(y, z, x) then begin
          GetX := True;
          exit;
        end;
      end;
    end;
    GetX := False;
  end;



function TMesh.GetY(x,z : float; var y : float) : boolean;

  var
    i  : integer;

  begin
    for i:=0 to meshTris-1 do begin
      if (x>=meshTriMin[i].x) and (x<=meshTriMax[i].x) and (z>=meshTriMin[i].z) and (z<=meshTriMax[i].z) then begin
        PlanePrepareTri(meshTransformed, meshTri[i]);
        if PlaneY(x, z, y) then begin
          GetY := True;
          exit;
        end;
      end;
    end;
    GetY := False;
  end;



function TMesh.GetZ(x,y : float; var z : float) : boolean;

  var
    i  : integer;

  begin
    for i:=0 to meshTris-1 do begin
      if (x>=meshTriMin[i].x) and (x<=meshTriMax[i].x) and (y>=meshTriMin[i].y) and (y<=meshTriMax[i].y) then begin
        PlanePrepareTri(meshTransformed, meshTri[i]);
        if PlaneZ(x, y, z) then begin
          GetZ := True;
          exit;
        end;
      end;
    end;
    GetZ := False;
  end;



procedure TMesh.CreateVertices(verts : integer);

  begin
    meshVerts := verts;
    ReallocMem(meshVert, meshVerts * SizeOf(TVec));
    ReallocMem(meshTransformed, meshVerts * SizeOf(TVec));
  end;



procedure TMesh.CreateTriangles(tris : integer);

  begin
    meshTris := tris;
    ReallocMem(meshTri, meshTris * SizeOf(TTri));
    ReallocMem(meshTriMin, meshTris * SizeOf(TVec));
    ReallocMem(meshTriMax, meshTris * SizeOf(TVec));
  end;



procedure TMesh.SetVertex(index : integer; x, y, z : float);

  begin
    meshVert[index].x := x;
    meshVert[index].y := y;
    meshVert[index].z := z;
  end;



procedure TMesh.SetVertexVec(index : integer; v : TVec);

  begin
    meshVert[index] := v;
  end;



procedure TMesh.SetTriangle(index : integer; i1, i2, i3 : integer);

  begin
    meshTri[index][0] := i1;
    meshTri[index][1] := i2;
    meshTri[index][2] := i3;
  end;



procedure TMesh.SetTriangleTri(index : integer; tri : TTri);

  begin
    meshTri[index] := tri;
  end;



procedure TMesh.ReadFromFile(var F : File);

  begin
    BlockRead(F, meshVerts, 4);
    ReallocMem(meshVert, meshVerts * 12);
    BlockRead(F, meshVert^, meshVerts * 12);
    BlockRead(F, meshTris, 4);
    ReallocMem(meshTri, meshTris * 12);
    BlockRead(F, meshTri^, meshTris * 12);
    ReallocMem(meshTransformed, meshVerts * SizeOf(TVec));
    ReallocMem(meshTriMin, meshTris * SizeOf(TVec));
    ReallocMem(meshTriMax, meshTris * SizeOf(TVec));
    TriMinMax;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Obj implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



constructor TObj.Create;

  var
    i : integer;

  begin
    SetPos(0,0,-4);
    SetCoordAngle(0,0,0);
    SetScale(1,1,1);
    objChildren := 0;
    objChild := NIL;
    objParent := NIL;
    Name := 'Obj';
    with objMin do begin x:=0; y:=0; z:=0; end;
    with objMax do begin x:=0; y:=0; z:=0; end;
    with objCent do begin x:=0; y:=0; z:=0; end;
    objBSRad := 0;
    objTransparency := Opaque;
    objVisible := true;
    objFrustumTest := true;
    objShadowState.State := NoShadowing;
    objShadowState.SelfShadowing := true;
    objMesh := NIL;

    Tag := 0;
    Data := NIL;
    objAnimations := 0;
    objAnimation := NIL;
    objJoints := 0;
    objJoint := NIL;
    objOnBeforeDrawObj := NIL;
    objOnAfterDrawObj := NIL;
    objStaticJoints := false;
    objStaticAnimations := false;

    for i := 0 to bgObjects-1 do
      if bgObject[i]=NIL then begin
        bgObject[i] := self;
        objHandle := i+1;
        exit;
      end;
    inc(bgObjects);
    ReallocMem(bgObject, bgObjects * SizeOf(TObj));
    bgObject[bgObjects-1] := self;
    objHandle := bgObjects;
  end;



destructor TObj.Destroy;

  var
    i, j: integer;

  begin
    HideObj(Self);
    if bgBoundView = Self then bgBoundView := NIL;
    for i := 0 to MAXLIGHTS - 1 do begin
      if bgBoundLight[i] = Self then bgBoundLight[i] := NIL;
      if bgBoundSpotLight[i] = Self then bgBoundSpotLight[i] := NIL;
    end;
    for i := 0 to objChildren-1 do if objChild[i]<>NIL then begin
      objChild[i].objParent := NIL;
      objChild[i].Free;
    end;
    FreeMem(objChild);
    if objParent<>NIL then objParent.RemoveChild(Self);
    if objMesh<>NIL then objMesh.Free;
    if not objStaticAnimations then begin
      if objAnimations>0 then begin
        for i:=0 to objAnimations-1 do begin
          for j:=0 to objAnimation[i].Cycles-1 do begin
            FreeMem(objAnimation[i].Cycle[j].TranslationFrame);
            FreeMem(objAnimation[i].Cycle[j].RotationFrame);
          end;
          FreeMem(objAnimation[i].Cycle);
        end;
        FreeMem(objAnimation);
      end;
    end;
    if not objStaticJoints then begin
      if objJoints>0 then begin
        for i := 0 to objJoints-1 do begin
          FreeMem(objJoint[i].TranslationKey);
          FreeMem(objJoint[i].RotationKey);
        end;
        FreeMem(objJoint);
      end;
    end;
    for i:=0 to bgObjects-1 do
      if bgObject[i]=self then bgObject[i] := NIL;
    inherited Destroy;
  end;



procedure TObj.SetName(n : ShortString);

  begin
    name := n;
  end;



procedure TObj.GetPos(var x, y, z: float);

  begin
    x := objX;
    y := objY;
    z := objZ;
  end;



function TObj.GetPosVec : TVec;

  begin
    result.x := objX;
    result.y := objY;
    result.z := objZ;
  end;


procedure TObj.SetPos(x,y,z : GLfloat);

  begin
    objX := x;
    objY := y;
    objZ := z;
  end;



procedure TObj.SetPosVec(pos : TVec);

  begin
    objX := pos.x;
    objY := pos.y;
    objZ := pos.z;
  end;



procedure TObj.GetCoordAngle(var xa,ya,za : GLfloat);

  begin
    MatrixExtractRotation(objRMatrix, xa, ya, za);
  end;



procedure TObj.SetCoordAngle(xa,ya,za : GLfloat);

  begin
    objRMatrix := MatrixIdentity;
    MatrixSetRotation(objRMatrix, xa, ya, za);
  end;



procedure TObj.GetScale(var xs,ys,zs : GLfloat);

  begin
    xs := objXS;
    ys := objYS;
    zs := objZS;
  end;



procedure TObj.SetScale(xs,ys,zs : GLfloat);

  begin
    objXS := xs;
    objYS := ys;
    objZS := zs;
  end;



procedure TObj.Move(x,y,z : GLfloat);

  begin
    objX := objX + x;
    objY := objY + y;
    objZ := objZ + z;
  end;



procedure TObj.MoveVec(vec : TVec);

  begin
    objX := objX + vec.x;
    objY := objY + vec.y;
    objZ := objZ + vec.z;
  end;



procedure TObj.RotateCoord(xa,ya,za : GLfloat);

  begin
    objRMatrix := MatrixMultiply(MatrixRotationZ(za), objRMatrix);
    objRMatrix := MatrixMultiply(MatrixRotationY(ya), objRMatrix);
    objRMatrix := MatrixMultiply(MatrixRotationX(xa), objRMatrix);
    if bgSafeCoordRotation then CoordAxisUnitlength;
  end;



// assure that the rotationmatrix stays in range since rotate could be
// called quite often, so rounding errors will be eliminated
// the main coordinate axis (z) is left unchanged, x is the normal vector of z
// and y, then y is the normal vector of z and x. Afterwards all normal vectors
// are unified
procedure TObj.CoordAxisUnitLength;

  var
    xa,
    ya  : TVec;

  begin
    xa := VectorCross(TVec(Addr(objRMatrix[2,0])^), TVec(Addr(objRMatrix[1,0])^));
    ya := VectorCross(TVec(Addr(objRMatrix[0,0])^), TVec(Addr(objRMatrix[2,0])^));
    if (xa.x*objRMatrix[0,0] + xa.y*objRMatrix[0,1] + xa.z*objRMatrix[0,2]) < 0.0 then begin
      xa.x := - xa.x;
      xa.y := - xa.y;
      xa.z := - xa.z;
    end;
    if (ya.x*objRMatrix[1,0] + ya.y*objRMatrix[1,1] + ya.z*objRMatrix[1,2]) < 0.0 then begin
      ya.x := - ya.x;
      ya.y := - ya.y;
      ya.z := - ya.z;
    end;
    objRMatrix[0,0] := xa.x;
    objRMatrix[0,1] := xa.y;
    objRMatrix[0,2] := xa.z;
    objRMatrix[1,0] := ya.x;
    objRMatrix[1,1] := ya.y;
    objRMatrix[1,2] := ya.z;
    VectorStandard(TVec(Addr(objRMatrix[0,0])^));
    VectorStandard(TVec(Addr(objRMatrix[1,0])^));
    VectorStandard(TVec(Addr(objRMatrix[2,0])^));
  end;



procedure TObj.SetVisible(Visible: boolean);

  begin
    objVisible := Visible;
  end;



function TObj.GetTransparency : TTransparency;

  begin
    result := objTransparency;
  end;



procedure TObj.SetTransparency(t : TTransparency);

  begin
    objTransparency := t;
  end;



function TObj.GetShadowState : TShadowState;

  begin
    result := objShadowState;
  end;



procedure TObj.SetShadowState(State: TShadowStateState; SelfShadowing: boolean);

  begin
    objShadowState.State := State;
    objShadowState.SelfShadowing := SelfShadowing;
  end;



procedure TObj.GetXAxis(var ax,ay,az : GLfloat);

  begin
    ax := objRMatrix[0,0];
    ay := objRMatrix[0,1];
    az := objRMatrix[0,2];
  end;



function TObj.GetXAxisVec : TVec;

  begin
    result.x := objRMatrix[0,0];
    result.y := objRMatrix[0,1];
    result.z := objRMatrix[0,2];
  end;



procedure TObj.GetYAxis(var ax,ay,az : GLfloat);

  begin
    ax := objRMatrix[1,0];
    ay := objRMatrix[1,1];
    az := objRMatrix[1,2];
  end;



function TObj.GetYAxisVec : TVec;

  begin
    result.x := objRMatrix[1,0];
    result.y := objRMatrix[1,1];
    result.z := objRMatrix[1,2];
  end;



procedure TObj.GetZAxis(var ax,ay,az : GLfloat);

  begin
    ax := objRMatrix[2,0];
    ay := objRMatrix[2,1];
    az := objRMatrix[2,2];
  end;



function TObj.GetZAxisVec : TVec;

  begin
    result.x := objRMatrix[2,0];
    result.y := objRMatrix[2,1];
    result.z := objRMatrix[2,2];
  end;



procedure TObj.SetXAxis(ax,ay,az : GLfloat);

  begin
    objRMatrix[0,0] := ax;
    objRMatrix[0,1] := ay;
    objRMatrix[0,2] := az;
  end;



procedure TObj.SetXAxisVec(vec : TVec);

  begin
    objRMatrix[0,0] := vec.x;
    objRMatrix[0,1] := vec.y;
    objRMatrix[0,2] := vec.z;
  end;



procedure TObj.SetYAxis(ax,ay,az : GLfloat);

  begin
    objRMatrix[1,0] := ax;
    objRMatrix[1,1] := ay;
    objRMatrix[1,2] := az;
  end;



procedure TObj.SetYAxisVec(vec : TVec);

  begin
    objRMatrix[1,0] := vec.x;
    objRMatrix[1,1] := vec.y;
    objRMatrix[1,2] := vec.z;
  end;



procedure TObj.SetZAxis(ax,ay,az : GLfloat);

  begin
    objRMatrix[2,0] := ax;
    objRMatrix[2,1] := ay;
    objRMatrix[2,2] := az;
  end;



procedure TObj.SetZAxisVec(vec : TVec);

  begin
    objRMatrix[2,0] := vec.x;
    objRMatrix[2,1] := vec.y;
    objRMatrix[2,2] := vec.z;
  end;



procedure TObj.GetPosAngle(var px,py,pz, ax,ay,az : GLfloat);

  var
    L : TObj;
    M : TMatrix4f;

  begin
    glLoadIdentity;
    SetOwnerMatrix;
    PrepareModelMatrix;
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(M));
    px := -M[3,0];
    py := -M[3,1];
    pz := -M[3,2];
    L := Self;
    glLoadIdentity;
    While L<>NIL do begin
      glMultMatrixf(Addr(L.objRMatrix));
      L := L.objParent;
    end;
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(M));
    MatrixExtractRotation(M, ax, ay, az);
    ax := -ax;
    ay := -ay;
    az := -az;
  end;



procedure TObj.PrepareModelMatrix;

  begin
    glTranslatef(objX, objY, objZ);
    glMultMatrixf(Addr(objRMatrix));
    glScalef(objXS, objYS, objZS);
  end;



procedure TObj.SetOwnerMatrix;

  var Obj : TObj;
      p : array[0..63] of TObj;
      n, i : integer;

  begin
    n := 0;
    Obj := self;
    while Obj.objParent <> NIL do begin
      p[n] := Obj.objParent;
      inc(n);
      Obj := Obj.objParent;
    end;
    for i := n-1 downto 0 do
      p[i].preparemodelmatrix;
  end;



procedure TObj.SetMatrix(ParentMatrix: PMatrix4f);

  var
    i : integer;

  begin
    if ParentMatrix<>NIL then
      objMatrix := ParentMatrix^
    else
      objMatrix := MatrixIdentity;
    objMatrix := MatrixMultiply(MatrixTranslation(objX, objY, objZ), objMatrix);
    objMatrix := MatrixMultiply(objRMatrix, objMatrix);
    MatrixScale(objMatrix, objXS, objYS, objZS);
    for i := 0 to objChildren-1 do
      if objChild[i]<>NIL then objChild[i].SetMatrix(@objMatrix);
    // set the maximum scalefactor
    objScaleFactor := objXS;
    if objYS>objScaleFactor then objScaleFactor := objYS;
    if objZS>objScaleFactor then objScaleFactor := objZS;
    if objParent<>NIL then objScaleFactor := objScaleFactor * objParent.objScaleFactor;
  end;



procedure TObj.MakeGlobal(var v : TVec);

  var
    Matrix : TMatrix4f;

  begin
    glLoadIdentity;
    SetOwnerMatrix;
    PrepareModelMatrix;
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(Matrix));
    VectorTransform(v, Matrix);
  end;



procedure TObj.GetGlobalRotationMatrix(var rmat : TMatrix4f);

  var
    Obj : TObj;
    p : array[0..63] of TObj;
    n, i : integer;

  begin
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity;
    n := 0;
    Obj := self;
    while Obj <> NIL do begin
      p[n] := Obj;
      inc(n);
      Obj := Obj.objParent;
    end;
    for i := n-1 downto 0 do
      glMultMatrixf(Addr(p[i].objRMatrix));
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(rmat));
  end;




procedure TObj.Transform(var v : TVec);

  begin
    VectorTransform(v, objMatrix);
  end;



procedure TObj.LoadGLMatrix;

  begin
    glLoadMatrixf(Addr(objMatrix));
  end;



procedure TObj.MultGLMatrix;

  begin
    glMultMatrixf(Addr(objMatrix));
  end;



function TObj.GetMatrix : TMatrix4f;

  begin
    result := objMatrix;
  end;



function TObj.GetRMatrix : TMatrix4f;

  begin
    result := objRMatrix;
  end;



function TObj.GetParent : TObj;

  begin
    result := objParent;
  end;



function TObj.GetChildren : Integer;

  begin
    result := objChildren;
  end;



function TObj.GetChild(index : integer) : TObj;

  begin
    if (index<0) or (index>=objChildren) then
      GetChild := NIL
    else
      GetChild := objChild[index];
  end;



function TObj.GetChildByName(ChildName : String) : TObj;

  var
    i: integer;

  begin
    for i:=0 to objChildren-1 do begin
      if (objChild[i]<>NIL) and (objChild[i].Name=ChildName) then begin
        result := objChild[i];
        exit;
      end;
    end;
    result := NIL;
  end;



procedure TObj.SetChild(index : integer; chld : TObj);

  begin
    if chld=self then exit;
    if chld<>NIL then
      if chld.objParent<>NIL then chld.objParent.RemoveChild(chld);
    if index>=objChildren then begin
      ReallocMem(objChild, (index + 1) * SizeOf(TObj));
      FillChar(objChild[objChildren], (index - objChildren) * SizeOf(TObj), 0);
      objChild[index] := chld;
      objChildren := index + 1;
    end
    else begin
      if objChild[index]<>NIL then objChild[index].objParent := NIL;
      objChild[index] := chld;
    end;
    if chld<>NIL then
      chld.objParent := Self;
  end;



function  TObj.IsChild(chld : TObj) : Boolean;

  var
    i : integer;

  begin
    if chld<>NIL then begin
      for i := 0 to pred(objChildren) do
        if objChild[i]=chld then Begin
          IsChild := True;
          Exit;
        end;
      for i := 0 to pred(objChildren) do
        if objChild[i]<>NIL then if objChild[i].IsChild(chld) then begin
          IsChild := True;
          Exit;
        End;
    end;
    IsChild := False;
  end;



procedure TObj.RemoveChild(chld : TObj);

  var
    i : integer;

  begin
    for i := 0 to pred(objChildren) do
      if objChild[i]=chld then begin
        if chld<>NIL then chld.objParent := NIL;
        if i<objJoints then
          objChild[i] := NIL
        else begin
          system.move(objChild[i+1], objChild[i], (objChildren-i-1) * SizeOf(TObject));
          dec(objChildren);
          ReallocMem(objChild, objChildren * SizeOf(TObject));
        end;
      end;
  end;



procedure TObj.RemoveChildIndex(index : integer);

  begin
    if (index < objChildren) then begin
      if objChild[index]<>NIL then objChild[index].objParent := NIL;
      if index < objJoints then
        objChild[index] := NIL
      else begin
        system.move(objChild[index+1], objChild[index], (objChildren-index-1) * SizeOf(TObject));
        dec(objChildren);
        ReallocMem(objChild, objChildren * SizeOf(TObject));
      end;
    end;
  end;



procedure TObj.RemoveChildrenTbl;

  begin
    FreeMem(objChild);
    objChild := NIL;
    objChildren := 0;
  end;



procedure TObj.CreateJoints(number: integer);

  begin
    objJoints := number;
    GetMem(objJoint, number * sizeof(TJoint));
    FillChar(objJoint^, number * sizeof(TJoint), 0);
  end;



procedure TObj.CreateJointData(JointIndex, TranslationKeys, RotationKeys: integer;
                               Name: string; ParentJointIndex, Tag: integer);

  var
    joint: PJoint;

  begin
    joint := @objJoint[JointIndex];
    joint.Name := Name;
    joint.ParentJointIndex := ParentJointIndex;
    joint.Connection := NIL;
    joint.TranslationKeys := TranslationKeys;
    GetMem(joint.TranslationKey, TranslationKeys * sizeof(TVec));
    FillChar(joint.TranslationKey^, TranslationKeys * sizeof(TVec), 0);
    joint.RotationKeys := RotationKeys;
    GetMem(joint.RotationKey, RotationKeys * sizeof(TQuaternion));
    FillChar(joint.RotationKey^, RotationKeys * sizeof(TQuaternion), 0);
    joint.Tag := Tag;
  end;



procedure TObj.SetJointOrigin(JointIndex: integer; x,y,z, xa,ya,za: float);

  var
    joint: PJoint;

  begin
    joint := @objJoint[JointIndex];
    joint.LocalX := x;
    joint.LocalY := y;
    joint.LocalZ := z;

    joint.LocalXA := xa;
    joint.LocalYA := ya;
    joint.LocalZA := za;
  end;



procedure TObj.SetJointConnection(JointIndex: integer; Child: TObj);

  begin
    objJoint[JointIndex].Connection := Child;
  end;




procedure TObj.SetJointTranslationKey(JointIndex, KeyIndex: integer; x,y,z: float);

  begin
    objJoint[JointIndex].TranslationKey[KeyIndex] := Vector(x,y,z);
  end;



procedure TObj.SetJointRotationKey(JointIndex, KeyIndex: integer; xa,ya,za: float);

  var
    m: TMatrix4f;

  begin
    m := MatrixIdentity;
    MatrixSetRotation(m, xa, ya, za);
    objJoint[JointIndex].RotationKey[KeyIndex] := QuaternionFromMatrix(m);
  end;



procedure TObj.CreateAnimations(number: integer);

  begin
    objAnimations := number;
    GetMem(objAnimation, number * sizeof(TAnimation));
    FillChar(objAnimation^, number * sizeof(TAnimation), 0);
  end;



procedure TObj.CreateAnimationData(AnimationIndex, Cycles: integer; Name: string; Tag: integer);

  var
    ani: PAnimation;

  begin
    ani := @objAnimation[AnimationIndex];
    ani.Name := Name;
    ani.Cycles := Cycles;
    GetMem(ani.Cycle, Cycles * sizeof(TCycle));
    FillChar(ani.Cycle^, Cycles * sizeof(TCycle), 0);
    ani.Tag := Tag;
  end;



procedure TObj.CreateCycleData(AnimationIndex, CycleIndex, JointIndex, TranslationFrames, RotationFrames: integer;  Loop: boolean);

  var
    cyc: PCycle;

  begin
    cyc := @objAnimation[AnimationIndex].Cycle[CycleIndex];
    cyc.JointIndex := JointIndex;
    cyc.TranslationFrames := TranslationFrames;
    GetMem(cyc.TranslationFrame, TranslationFrames * sizeof(TKeyFrame));
    FillChar(cyc.TranslationFrame^, TranslationFrames * sizeof(TKeyFrame), 0);
    cyc.RotationFrames := RotationFrames;
    GetMem(cyc.RotationFrame, RotationFrames * sizeof(TKeyFrame));
    FillChar(cyc.RotationFrame^, RotationFrames * sizeof(TKeyFrame), 0);
    cyc.Loop := Loop;
  end;



procedure TObj.SetCycleTranslationFrame(AnimationIndex, CycleIndex, Index, Key, Time: integer);

  var
    KeyFrame: PKeyFrame;

  begin
    KeyFrame := @objAnimation[AnimationIndex].Cycle[CycleIndex].TranslationFrame[Index];
    KeyFrame.Key := Key;
    KeyFrame.Time := Time;
  end;



procedure TObj.SetCycleRotationFrame(AnimationIndex, CycleIndex, Index, Key, Time: integer);

  var
    KeyFrame: PKeyFrame;

  begin
    KeyFrame := @objAnimation[AnimationIndex].Cycle[CycleIndex].RotationFrame[Index];
    KeyFrame.Key := Key;
    KeyFrame.Time := Time;
  end;



function TObj.GetJointIndex(Name: string): integer;

  var
    i: integer;

  begin
    for i:=0 to objJoints-1 do
      if objJoint[i].Name = Name then begin
        result := i;
        exit;
      end;
    result := -1;
  end;



function TObj.GetAnimationIndex(Name: string): integer;

  var
    i: integer;

  begin
    for i:=0 to objAnimations-1 do
      if objAnimation[i].Name = Name then begin
        result := i;
        exit;
      end;
    result := -1;
  end;



procedure TObj.SetRunCycleCallback(AnimationIndex, JointIndex: integer; CallBack: TObjCycleCallback);

  var
    i: integer;

  begin
    for i:=0 to objAnimation[AnimationIndex].Cycles-1 do
      if objAnimation[AnimationIndex].Cycle[i].JointIndex = JointIndex then begin
        objAnimation[AnimationIndex].Cycle[i].OnRunCycle := CallBack;
      end;
  end;



procedure TObj.SetEndCycleCallback(AnimationIndex, JointIndex: integer; CallBack: TObjCycleCallback);

  var
    i: integer;

  begin
    for i:=0 to objAnimation[AnimationIndex].Cycles-1 do
      if objAnimation[AnimationIndex].Cycle[i].JointIndex = JointIndex then begin
        objAnimation[AnimationIndex].Cycle[i].OnEndCycle := CallBack;
      end;
  end;



procedure TObj.SetupAnimations;

  var
    i,j,a,c,count: integer;
    joint: PJoint;
    ani: PAnimation;
    cyc: PCycle;
    sorted: boolean;
    temp: TJoint;

  begin
    // make sure that parent joints are before their children
    count := 0;
    repeat
      sorted := true;
      for i:=0 to objJoints-1 do begin
        joint := @objJoint[i];
        if joint.ParentJointIndex=i then joint.ParentJointIndex := -1 else
        if joint.ParentJointIndex>i then begin
          temp := objJoint[i];
          objJoint[i] := objJoint[temp.ParentJointIndex];
          objJoint[temp.ParentJointIndex] := temp;
          // swap cycle joint indices
          for a:=0 to objAnimations-1 do begin
            ani := @objAnimation[a];
            for c:=0 to ani.Cycles-1 do begin
              cyc := @ani.Cycle[c];
              if cyc.JointIndex=i then cyc.JointIndex := temp.ParentJointIndex else
              if cyc.JointIndex=temp.ParentJointIndex then cyc.JointIndex := i;
            end;
          end;
          // swap parent joint indices
          for j:=0 to objJoints-1 do begin
            if objJoint[j].ParentJointIndex=i then objJoint[j].ParentJointIndex := temp.ParentJointIndex else
            if objJoint[j].ParentJointIndex=temp.ParentJointIndex then objJoint[j].ParentJointIndex := i;
          end;
          sorted := false;
        end;
      end;
      if count>=objJoints then begin
        for i:=0 to objJoints-1 do
          if objJoint[i].ParentJointIndex>=i then objJoint[i].ParentJointIndex := -1;
      end;
      inc(count);
    until sorted;

    // compute joint transformations
    for i:=0 to objJoints-1 do begin
      joint := @objJoint[i];
      joint.RelativeTransformation := MatrixIdentity;
      MatrixSetRotation(joint.RelativeTransformation, joint.LocalXA, joint.LocalYA, joint.LocalZA);
      MatrixSetTranslation(joint.RelativeTransformation, joint.LocalX, joint.LocalY, joint.LocalZ);
      if joint.ParentJointIndex<>-1 then begin
        joint.AbsoluteTransformation := objJoint[joint.ParentJointIndex].AbsoluteTransformation;
        joint.AbsoluteTransformation := MatrixPostMultiply(joint.AbsoluteTransformation, joint.RelativeTransformation);
      end
      else
        joint.AbsoluteTransformation := joint.RelativeTransformation;
      joint.FinalTransformation := joint.AbsoluteTransformation;
    end;

    // compute cycle times
    for a:=0 to objAnimations-1 do begin
      ani := @objAnimation[a];
      for c:=0 to ani.Cycles-1 do begin
        cyc := @ani.Cycle[c];
        if cyc.TranslationFrames>0 then
          i := cyc.TranslationFrame[cyc.TranslationFrames-1].Time
        else
          i := 0;
        if cyc.RotationFrames>0 then
          cyc.TotalTime := cyc.RotationFrame[cyc.RotationFrames-1].Time
        else
          cyc.TotalTime := 0;
        if i>cyc.TotalTime then cyc.TotalTime:=i;
      end;
    end;
  end;



procedure TObj.Animate(AnimationIndex, TimeDifference: integer);

  var
    c: integer;
    ani: PAnimation;
    cyc: PCycle;
    joint: PJoint;
    PrevTranslationKey, TranslationKey: PVec;
    PrevRotationKey, RotationKey: PQuaternion;
    frame: integer;
    Keyframe,PrevKeyframe: PKeyFrame;
    Translation: TVec;
    Rotation: TQuaternion;
    Transformation, RelativeFinal: TMatrix4f;
    interp: float;

  begin
    if AnimationIndex >= objAnimations then exit;
    ani := @objAnimation[AnimationIndex];
    for c:=0 to ani.Cycles-1 do begin
      cyc := @ani.Cycle[c];
      cyc.CurrentTime := cyc.CurrentTime + TimeDifference;
      joint := @objJoint[cyc.JointIndex];
      if cyc.CurrentTime > cyc.TotalTime then begin
        if addr(cyc.OnEndCycle)<>NIL then
          cyc.OnEndCycle(self, AnimationIndex, c);
        if cyc.Loop then begin
          while cyc.CurrentTime > cyc.TotalTime do
            dec(cyc.CurrentTime, cyc.TotalTime);
          if Cyc.TranslationFrames>0 then joint.CurrentTranslationKey := cyc.TranslationFrame[0].Key;
          if Cyc.RotationFrames>0 then joint.CurrentRotationKey := cyc.RotationFrame[0].Key;
        end
        else begin
          cyc.CurrentTime := cyc.TotalTime;
          if Cyc.TranslationFrames>0 then joint.CurrentTranslationKey := cyc.TranslationFrame[cyc.TranslationFrames-1].Key;
          if Cyc.RotationFrames>0 then joint.CurrentRotationKey := cyc.RotationFrame[cyc.RotationFrames-1].Key;
        end;
      end // CurrentTime > TotalTime
      else begin
        if addr(cyc.OnRunCycle)<>NIL then
          cyc.OnRunCycle(self, AnimationIndex, c);
      end;

      // find translation frame
      if cyc.TranslationFrames>0 then begin
        frame := 0;
        while (frame<cyc.TranslationFrames-1) and (cyc.TranslationFrame[frame].Time<cyc.CurrentTime) do
          inc(frame);
        Keyframe := @cyc.TranslationFrame[frame];
        joint.CurrentTranslationKey := Keyframe.Key;
        TranslationKey := @joint.TranslationKey[Keyframe.Key];

        if (frame=0) then begin
          if cyc.CurrentTime < 0 then begin
            // animation comes from other animation cycle
            interp := - TimeDifference / (cyc.CurrentTime - TimeDifference);
            Translation.x := joint.PrevTranslation.x + interp * (TranslationKey.x - joint.PrevTranslation.x);
            Translation.y := joint.PrevTranslation.y + interp * (TranslationKey.y - joint.PrevTranslation.y);
            Translation.z := joint.PrevTranslation.z + interp * (TranslationKey.z - joint.PrevTranslation.z);
          end
          else begin
            Translation := joint.TranslationKey[Keyframe.Key];
          end;
        end else begin
          PrevKeyframe := @cyc.TranslationFrame[frame-1];
          PrevTranslationKey := @joint.TranslationKey[PrevKeyframe.Key];
          interp := (cyc.CurrentTime - PrevKeyframe.Time) / (Keyframe.Time - PrevKeyframe.Time);
          Translation.x := PrevTranslationKey.x + interp * (TranslationKey.x - PrevTranslationKey.x);
          Translation.y := PrevTranslationKey.y + interp * (TranslationKey.y - PrevTranslationKey.y);
          Translation.z := PrevTranslationKey.z + interp * (TranslationKey.z - PrevTranslationKey.z);
        end;
      end;

      // find rotation frame
      if cyc.RotationFrames>0 then begin
        frame := 0;
        while (frame<cyc.RotationFrames-1) and (cyc.RotationFrame[frame].Time<cyc.CurrentTime) do
          inc(frame);
        Keyframe := @cyc.RotationFrame[frame];
        joint.CurrentRotationKey := Keyframe.Key;
        RotationKey := @joint.RotationKey[Keyframe.Key];

        if (frame=0) then begin
          if cyc.CurrentTime < 0 then begin
            // animation comes from other animation cycle
            interp := - TimeDifference / (cyc.CurrentTime - TimeDifference);
            Rotation := QuaternionSLerp(joint.PrevRotation, RotationKey^, interp);
          end
          else begin
            Rotation := joint.RotationKey[Keyframe.Key];
          end;
        end else begin
          PrevKeyframe := @cyc.RotationFrame[frame-1];
          PrevRotationKey := @joint.RotationKey[PrevKeyframe.Key];
          interp := (cyc.CurrentTime - PrevKeyframe.Time) / (Keyframe.Time - PrevKeyframe.Time);
          Rotation := QuaternionSLerp(PrevRotationKey^, RotationKey^, interp);
        end;
      end;

      // apply transformation
      Transformation := MatrixIdentity;
      if cyc.RotationFrames>0 then MatrixSetRotationQuaternion(Transformation, Rotation);
      if cyc.TranslationFrames>0 then MatrixSetTranslation(Transformation, Translation.x, Translation.y, Translation.z);

      RelativeFinal := MatrixPostMultiply(joint.RelativeTransformation, Transformation);
      if joint.ParentJointIndex < 0 then begin
        joint.FinalTransformation := RelativeFinal;
      end
      else begin
        joint.FinalTransformation := MatrixPostMultiply(objJoint[joint.ParentJointIndex].FinalTransformation, RelativeFinal);
      end;

      if joint.Connection<>NIL then begin
        Transformation := joint.FinalTransformation;
        if cyc.TranslationFrames>0 then MatrixExtractTranslation(Transformation, joint.Connection.objX, joint.Connection.objY, joint.Connection.objZ);
        if cyc.RotationFrames>0 then begin
          joint.Connection.objRMatrix := Transformation;
          MatrixSetTranslation(joint.Connection.objRMatrix, 0.0, 0.0, 0.0);
        end;
      end;

    end;
  end;



procedure TObj.ResetAnimation(AnimationIndex: integer);

  var
    i: integer;

  begin
    for i:=0 to objAnimation[AnimationIndex].Cycles-1 do begin
      objAnimation[AnimationIndex].Cycle[i].CurrentTime := 0;
    end;
  end;



procedure TObj.ShowAnimation(AnimationIndex: integer);

  var
    i: integer;

  begin
    for i:=0 to bgObjAnimations-1 do
      if (bgObjAnimation[i].me=self) and (bgObjAnimation[i].AnimationIndex=AnimationIndex) then
        exit;
    bgObjAnimation[bgObjAnimations].me := self;
    bgObjAnimation[bgObjAnimations].AnimationIndex := AnimationIndex;
    inc(bgObjAnimations);
  end;



procedure TObj.HideAnimation(AnimationIndex: integer);

  var
    i: integer;

  begin
    for i:=0 to bgObjAnimations-1 do
      if (bgObjAnimation[i].me=self) and (bgObjAnimation[i].AnimationIndex=AnimationIndex) then begin
        System.Move(bgObjAnimation[i+1], bgObjAnimation[i], (bgObjAnimations - i - 1) * sizeof(TObjAnimation));
        dec(bgObjAnimations);
        exit;
      end;
  end;



procedure TObj.ReplaceAnimation(AnimationIndex, NewAnimationIndex, TimeGap: integer);

  var
    i, j, k, l: integer;
    x, y, z: float;

  begin
    for i:=0 to bgObjAnimations-1 do begin
      if (bgObjAnimation[i].me=self) and (bgObjAnimation[i].AnimationIndex=AnimationIndex) then begin
        bgObjAnimation[i].AnimationIndex := NewAnimationIndex;
        for j:=0 to objAnimation[NewAnimationIndex].Cycles-1 do begin
          objAnimation[NewAnimationIndex].Cycle[j].CurrentTime := -TimeGap;
          l := objAnimation[NewAnimationIndex].Cycle[j].JointIndex;
          for k:=0 to objAnimation[AnimationIndex].Cycles-1 do begin
            if l=objAnimation[AnimationIndex].Cycle[k].JointIndex then begin
              MatrixExtractTranslation(objJoint[l].RelativeTransformation, x, y, z);
              objJoint[l].PrevTranslation := Vector(x, y, z);
              objJoint[l].PrevRotation := QuaternionFromMatrix(objJoint[l].RelativeTransformation);
              break;
            end;
          end; // for k
        end; // for j
      end; // if (bgObjAnimation[i].me=...
    end; // for i
  end;



procedure TObj.SetJoints(Joints: PJoints; number: integer);

  begin
    objJoints := number;
    objJoint := Joints;
    objStaticJoints := true;
  end;



procedure TObj.SetAnimations(Animations: PAnimations; number: integer);

  begin
    objAnimations := number;
    objAnimation := Animations;
    objStaticAnimations := true;
  end;



procedure TObj.SetJointAnimByFile(FileName: String);

  var
    Hdr: TBFHeader;
    F: File;
    j, a, c: integer;
    joint: PJoint;
    ani: PAnimation;
    cyc: PCycle;

  begin
    Assign(F, FileName);
    Reset(F, 1);
    if IOResult<>0 then exit;
    BlockRead(F, Hdr, 64);
    Seek(F, Hdr[1]);
    BlockRead(F, objJoints, 4);
    GetMem(objJoint, objJoints * sizeof(TJoint));
    FillChar(objJoint^, objJoints * sizeof(TJoint), 0);
    for j:=0 to objJoints-1 do begin
      joint := @objJoint[j];
      BlockRead(F, joint.Name, sizeof(joint.Name));
      BlockRead(F, joint.ParentJointIndex, 4);
      BlockRead(F, joint.LocalX, 4);
      BlockRead(F, joint.LocalY, 4);
      BlockRead(F, joint.LocalZ, 4);
      BlockRead(F, joint.LocalXA, 4);
      BlockRead(F, joint.LocalYA, 4);
      BlockRead(F, joint.LocalZA, 4);
      BlockRead(F, joint.TranslationKeys, 4);
      BlockRead(F, joint.RotationKeys, 4);
      GetMem(joint.TranslationKey, joint.TranslationKeys * sizeof(TVec));
      BlockRead(F, joint.TranslationKey^, joint.TranslationKeys * 12);
      GetMem(joint.Rotationkey, joint.RotationKeys * sizeof(TQuaternion));
      BlockRead(F, joint.RotationKey^, joint.RotationKeys * 16);
      BlockRead(F, joint.Tag, 4);
    end;

    BlockRead(F, objAnimations, 4);
    GetMem(objAnimation, objAnimations * sizeof(TAnimation));
    FillChar(objAnimation^, objAnimations * sizeof(TAnimation), 0);
    for a:=0 to objAnimations-1 do begin
      ani := @objAnimation[a];
      BlockRead(F, ani.Name, sizeof(ani.Name));
      BlockRead(F, ani.Cycles, 4);
      GetMem(ani.Cycle, ani.Cycles * sizeof(TCycle));
      FillChar(ani.Cycle^, ani.Cycles * sizeof(TCycle), 0);
      for c:=0 to ani.Cycles-1 do begin
        cyc := @ani.Cycle[c];
        BlockRead(F, cyc.JointIndex, 4);
        BlockRead(F, cyc.TranslationFrames, 4);
        BlockRead(F, cyc.RotationFrames, 4);
        GetMem(cyc.TranslationFrame, cyc.TranslationFrames * sizeof(TKeyFrame));
        BlockRead(F, cyc.TranslationFrame^, cyc.TranslationFrames * sizeof(TKeyFrame));
        GetMem(cyc.RotationFrame, cyc.RotationFrames * sizeof(TKeyFrame));
        BlockRead(F, cyc.RotationFrame^, cyc.RotationFrames * sizeof(TKeyFrame));
        BlockRead(F, cyc.Loop, 1);
      end;
    end;

    Close(F);
  end;


procedure TObj.GetMin(var mx, my, mz : GLfloat);

  begin
    mx := objMin.x;
    my := objMin.y;
    mz := objMin.z;
  end;



function TObj.GetMinVec : TVec;

  begin
    result := objMin;
  end;



procedure TObj.GetMax(var mx, my, mz : GLfloat);

  begin
    mx := objMax.x;
    my := objMax.y;
    mz := objMax.z;
  end;



function TObj.GetMaxVec : TVec;

  begin
    result := objMax;
  end;



procedure TObj.GetCent(var cx, cy, cz : GLfloat);

  begin
    cx := objCent.x;
    cy := objCent.y;
    cz := objCent.z;
  end;



function TObj.GetCentVec : TVec;

  begin
    result := objCent;
  end;



function TObj.GetBRad : GLfloat;

  begin
    result := objBSRad;
  end;



procedure TObj.SetMin(mx, my, mz : GLfloat);

  begin
    objMin.x := mx;
    objMin.y := my;
    objMin.z := mz;
  end;



procedure TObj.SetMinVec(min : TVec);

  begin
    objMin := min;
  end;



procedure TObj.SetMax(mx, my, mz : GLfloat);

  begin
    objMax.x := mx;
    objMax.y := my;
    objMax.z := mz;
  end;



procedure TObj.SetMaxVec(max : TVec);

  begin
    objMax := max;
  end;



procedure TObj.SetCent(cx, cy, cz : GLfloat);

  begin
    objCent.x := cx;
    objCent.y := cy;
    objCent.z := cz;
  end;



procedure TObj.SetCentVec(cent : TVec);

  begin
    objCent := cent;
  end;



procedure TObj.SetBRad(br : GLfloat);

  begin
    objBSRad := br;
  end;



function TObj.GetMesh : TMesh;

  begin
    result := objMesh;
  end;



procedure TObj.SetMeshByFile(FileName : String);

  var
    F : File;
    Hdr: TBFHeader;

  begin
    IOResult; // clear IOResult
    Assign(F, FileName);
    Reset(F, 1);
    if IOResult<>0 then exit;
    if objMesh<>NIL then objMesh.Free;
    BlockRead(F, Hdr, 64);
    Seek(F, Hdr[0]);
    BlockRead(F, objMesh, 4);
    if objMesh<>NIL then begin
      objMesh := TMesh.Create(@objMatrix);
      objMesh.ReadFromFile(F);
    end;
    Close(F);
  end;



function TObj.SphereIntersect(obj : TObj) : boolean;

  var
    c1, c2 : TVec;
    f,g : float;

  begin
    VectorTransformTo(objCent, c1, objMatrix);
    VectorTransformTo(obj.objCent, c2, obj.objMatrix);
    c1.x := c1.x - c2.x;
    c1.y := c1.y - c2.y;
    c1.z := c1.z - c2.z;
    f := c1.x*c1.x + c1.y*c1.y + c1.z*c1.z;
    g := abs(objBSRad*objScaleFactor) + abs(obj.objBSRad*obj.objScaleFactor);
    result := (f < g*g);
  end;



function TObj.QuickIntersect(obj : TObj) : boolean;

  begin
    if SphereIntersect(obj) then begin
      if (objMesh<>NIL) and (obj.objMesh<>NIL) then
        result := objMesh.QuickIntersect(obj.objMesh)
      else
        result := true;
      exit;
    end;
    result := false;
  end;



function TObj.Intersect(obj : TObj; var Intersection : TVec) : boolean;

  begin
    if SphereIntersect(obj) then begin
      if (objMesh<>NIL) and (obj.objMesh<>NIL) then
        result := objMesh.Intersect(obj.objMesh, Intersection)
      else begin
        Intersection := objMesh.meshVert[0];
        result := true;
      end;
      exit;
    end;
    result := false;
  end;



function TObj.IntersectLine(v1, v2 : TVec; var Intersection : TVec) : boolean;

  var
    i : integer;

  begin
    if objMesh<>NIL then begin
      for i := 0 to objMesh.meshTris-1 do begin
        PlanePrepareTri(objMesh.meshTransformed, objMesh.meshTri[i]);
        if PlaneIntersectLine(v1, v2, Intersection) then begin
          result := true;
          exit;
        end;
      end;
    end;
    result := false;
  end;



procedure TObj.SetAsCamera;

  begin
    glLoadIdentity;
    SetOwnerMatrix;
    PrepareModelMatrix;
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(bgViewMatrix));
    MatrixInvert(bgViewMatrix);
    glLoadMatrixf(Addr(bgViewMatrix));
    FrustumCalculate;
    ViewComputeInvertedTextureMatrixList;
  end;



procedure TObj.AddChild(chld : TObj);

  begin
    if (chld=self) or IsChild(chld) then exit;
    HideObj(chld);
    ReallocMem(objChild, Succ(objChildren)*SizeOf(TObject));
    objChild[objChildren] := chld;
    if chld<>NIL then begin
      if chld.objParent<>NIL then chld.objParent.RemoveChild(chld);
      chld.objParent := Self;
    end;
    inc(objChildren);
  end;



procedure TObj.Display;

  var
    i : integer;
    l,h : PObjList;
    v   : TVec;
    infrustum : Boolean;

  begin
    if objVisible then begin
      VectorTransformTo(objCent, v, objMatrix);
      if objFrustumTest then
        infrustum := SphereInFrustum(v.x, v.y, v.z, objBSRad*objScaleFactor)
      else
        infrustum := true;
      if infrustum then begin
        if objShadowState.State<>NoShadowing then begin
          bgShadowTable[bgShadowObjects] := Self;
          inc(bgShadowObjects);
        end;
        if objTransparency <> Opaque then begin
          VectorTransform(v, bgViewMatrix);
          objDistance := v.z;
          bgTransparentTable[bgTransparentObjects].me := Self;
          if bgTransparentObjects > 0 then begin
            if objDistance < bgTransparentList.me.objDistance then begin
              // new first element
              bgTransparentTable[bgTransparentObjects].next := bgTransparentList;
              bgTransparentList := @bgTransparentTable[bgTransparentObjects];
            end
            else begin
              // find element to insert after
              h := NIL;
              l := bgTransparentList;
              while (objDistance >= l.me.objDistance) and (l.next<>NIL) do begin
                h := l;
                l := l.next;
              end;
              if objDistance >= l.me.objDistance then begin
                bgTransparentTable[bgTransparentObjects].next := NIL;
                l.next := @bgTransparentTable[bgTransparentObjects];
              end
              else begin
                bgTransparentTable[bgTransparentObjects].next := l;
                if h<>NIL then
                  h.next := @bgTransparentTable[bgTransparentObjects]
                else
                  bgTransparentList := @bgTransparentTable[bgTransparentObjects];
              end;
            end;
          end
          else begin
            // start list
            bgTransparentTable[0].next := NIL;
            bgTransparentList := @bgTransparentTable[0];
          end;
          inc(bgTransparentObjects);
        end // transparent object
        else begin
          glLoadMatrixf(Addr(bgViewMatrix));
          glMultMatrixf(Addr(objMatrix));
          if Assigned(objOnBeforeDrawObj) then objOnBeforeDrawObj(Self, infrustum);
          if infrustum then begin
            DrawObj;
            if Assigned(objOnAfterDrawObj) then objOnAfterDrawObj(Self);
          end;
        end;
      end // object in frustum
      else
      if objShadowState.State>FrustumShadowing then begin
        bgShadowTable[bgShadowObjects] := Self;
        inc(bgShadowObjects);
      end;
    end; // visible
    for i := 0 to objChildren-1 do
      if objChild[i]<>NIL then objChild[i].display;
  end;



procedure TObj.FillSelectionBuffer;

  var
    i : integer;
    v : TVec;

  begin
    v := objCent;
    VectorTransform(v, objMatrix);
    if SphereInFrustum(v.x, v.y, v.z, objBSRad*objScaleFactor) then begin
      glLoadMatrixf(Addr(bgViewMatrix));
      glMultMatrixf(Addr(objMatrix));
      glLoadName(objHandle);
      DrawObj;
    end;
    for i := 0 to objChildren-1 do
      if objChild[i]<>NIL then objChild[i].FillSelectionBuffer;
  end;






//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  generic shader implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



constructor TShader.Create;

  begin
    Name := 'SD_NONE';
    sdIdentity := SD_NONE;
  end;



destructor TShader.Destroy;

  begin
    inherited Destroy;
  end;



function TShader.GetIdentity: integer;

  begin
    GetIdentity := sdIdentity;
  end;



procedure TShader.SetName(aName: String);

  begin
    Name := aName;
  end;



procedure TShader.Display;

  begin
  end;



{$IFDEF GL_PROGRAM}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  vertex and fragment programs
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



constructor TProgram.Create;

  var
    i: integer;

  begin
    inherited Create;
    prTarget := GL_VERTEX_PROGRAM_ARB;
    if GL_PROGRAM then
      glGenProgramsARB(1, Addr(prHandle));
    prParams := 0;
    prString := '';
    for i:=0 to 255 do begin
      prParam[i].Index := i;
      prParam[i].Param[0] := 0.0;
      prParam[i].Param[1] := 0.0;
      prParam[i].Param[2] := 0.0;
      prParam[i].Param[3] := 0.0;
    end;
  end;



destructor TProgram.Destroy;

  begin
    if GL_PROGRAM then
      glDeleteProgramsARB(1, Addr(prHandle));
    inherited Destroy;
  end;



procedure TProgram.Compile;

  begin
    if GL_PROGRAM then begin
      glBindProgramARB(prTarget, prHandle);
      glProgramStringARB(prTarget, GL_PROGRAM_FORMAT_ASCII_ARB, length(prString), Addr(prString[1]));
      UpdateParams;
    end;
  end;



procedure TProgram.Display;

  begin
    if GL_PROGRAM then
      glBindProgramARB(prTarget, prHandle);
  end;



function TProgram.GetProgramString: AnsiString;

  begin
    result := prString;
  end;



procedure TProgram.SetProgramString(Prg: AnsiString);

  begin
    prString := Prg;
    if bgAutoCompile then Compile;
  end;



procedure TProgram.UpdateParams;

  var
    i: integer;

  begin
    if GL_PROGRAM then
      for i:=0 to prParams-1 do
        if prParam[i].Index>=0 then
          glProgramLocalParameter4fvARB(prTarget, prParam[i].Index, addr(prParam[i].Param));
  end;



procedure TProgram.SetParam(index: integer; a, b, c, d: float);

  var
    i: integer;

  begin
    if GL_PROGRAM then begin
      for i:=0 to prParams-1 do begin
        if prParam[i].Index = index then with prParam[i] do begin
          Param[0] := a;
          Param[1] := b;
          Param[2] := c;
          Param[3] := d;
          glProgramLocalParameter4fvARB(prTarget, Index, addr(Param));
          exit;
        end;
      end;
      with prParam[prParams] do begin
        Param[0] := a;
        Param[1] := b;
        Param[2] := c;
        Param[3] := d;
        glProgramLocalParameter4fvARB(prTarget, Index, addr(Param));
      end;
      inc(prParams);
    end;
  end;



procedure TProgram.SetParamv(index: integer; p: TVec4f);

  var
    i: integer;

  begin
    if GL_PROGRAM then begin
      for i:=0 to prParams-1 do begin
        if prParam[i].Index = index then with prParam[i] do begin
          Param := p;
          glProgramLocalParameter4fvARB(prTarget, index, addr(Param));
          exit;
        end;
      end;
      with prParam[prParams] do begin
        Param := p;
        glProgramLocalParameter4fvARB(prTarget, index, addr(Param));
      end;
      inc(prParams);
    end;
  end;



function TProgram.GetParam(index: integer): TVec4f;

  var
    i: integer;

  begin
    for i:=0 to prParams-1 do begin
      if prParam[i].Index = index then with prParam[i] do begin
        result := Param;
        exit;
      end;
    end;
    for i:=0 to 3 do
      result[i] := 0.0;
  end;



procedure TProgram.RemoveParam(index: integer);

  var
    i: integer;

  begin
    for i:=0 to prParams-1 do begin
      if prParam[i].Index = index then begin
        System.Move(prParam[i+1], prParam[i], (prParams - i - 1) * sizeof(TProgramParameter));
        dec(prParams);
        if bgAutoCompile then UpdateParams;
        exit;
      end;
    end;
  end;




constructor TVertexProgram.Create;

  begin
    inherited Create;
    Name := 'VertexProgram';
    prTarget := GL_VERTEX_PROGRAM_ARB;
    sdIdentity := SD_VP;
  end;



constructor TFragmentProgram.Create;

  begin
    inherited Create;
    Name := 'FragmentProgram';
    prTarget := GL_FRAGMENT_PROGRAM_ARB;
    sdIdentity := SD_FP;
  end;
{$ENDIF}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  SkyBox routine for Cubemaps
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



{$IFDEF GL_13}
procedure DrawSkyBox(side: float; handle: GLenum);

  var
    s : float;

  begin
    s := side / 2;
    glPushAttrib(GL_ENABLE_BIT + GL_DEPTH_BUFFER_BIT + GL_TEXTURE_BIT);
    glActiveTexture(GL_TEXTURE0);
    glEnable(GL_TEXTURE_CUBE_MAP);
    glDisable(GL_TEXTURE_GEN_S);
    glDisable(GL_TEXTURE_GEN_T);
    glDisable(GL_TEXTURE_GEN_R);
    glBindTexture(GL_TEXTURE_CUBE_MAP, handle);
    glDisable(GL_LIGHTING);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_CULL_FACE);
    glDepthMask(False);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
    glMatrixMode(GL_TEXTURE);
    glLoadMatrixf(Addr(bgViewTextureMatrix));
    glBegin(GL_QUADS);
      glTexCoord3f(-1,-1,-1); glVertex3f(-s, -s , -s);
      glTexCoord3f( 1,-1,-1); glVertex3f( s, -s , -s);
      glTexCoord3f( 1, 1,-1); glVertex3f( s,  s , -s);
      glTexCoord3f(-1, 1,-1); glVertex3f(-s,  s , -s);

      glTexCoord3f(-1,-1, 1); glVertex3f(-s, -s ,  s);
      glTexCoord3f( 1,-1, 1); glVertex3f( s, -s ,  s);
      glTexCoord3f( 1, 1, 1); glVertex3f( s,  s ,  s);
      glTexCoord3f(-1, 1, 1); glVertex3f(-s,  s ,  s);

      glTexCoord3f(-1,-1,-1); glVertex3f(-s, -s , -s);
      glTexCoord3f( 1,-1,-1); glVertex3f( s, -s , -s);
      glTexCoord3f( 1,-1, 1); glVertex3f( s, -s ,  s);
      glTexCoord3f(-1,-1, 1); glVertex3f(-s, -s ,  s);

      glTexCoord3f(-1, 1,-1); glVertex3f(-s,  s , -s);
      glTexCoord3f( 1, 1,-1); glVertex3f( s,  s , -s);
      glTexCoord3f( 1, 1, 1); glVertex3f( s,  s ,  s);
      glTexCoord3f(-1, 1, 1); glVertex3f(-s,  s ,  s);

      glTexCoord3f(-1,-1,-1); glVertex3f(-s, -s , -s);
      glTexCoord3f(-1, 1,-1); glVertex3f(-s,  s , -s);
      glTexCoord3f(-1, 1, 1); glVertex3f(-s,  s ,  s);
      glTexCoord3f(-1,-1, 1); glVertex3f(-s, -s ,  s);

      glTexCoord3f( 1,-1,-1); glVertex3f( s, -s , -s);
      glTexCoord3f( 1, 1,-1); glVertex3f( s,  s , -s);
      glTexCoord3f( 1, 1, 1); glVertex3f( s,  s ,  s);
      glTexCoord3f( 1,-1, 1); glVertex3f( s, -s ,  s);
    glEnd;
    glLoadIdentity;
    glMatrixMode(GL_MODELVIEW);
    glPopAttrib;
  end;
{$ENDIF}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Texture implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



constructor TTexture.Create;

  begin
    sdIdentity := SD_TEX_NONE;
    glGenTextures(1, Addr(texHandle));
    texMinFilter := GL_LINEAR;   // filter for pixels > texels
    texMagFilter := GL_LINEAR;   // filter for pixels < texels
    texTarget := GL_TEXTURE_2D;
    texComponents := GL_RGB;
  end;



destructor TTexture.Destroy;

  begin
    glDeleteTextures(1, Addr(texHandle));
  end;



function TTexture.GetMinFilter: GLint;

  begin
    result := texMinFilter;
  end;



function TTexture.GetMagFilter: GLint;

  begin
    result := texMagFilter;
  end;



function TTexture.GetComponents: GLint;

  begin
    result := texComponents;
  end;



procedure TTexture.SetMinFilter(filter : GLint);

   begin
     texMinFilter := filter;
     if bgAutoCompile then begin
       glBindTexture(texTarget, texHandle); // texture binding info
       glTexParameteri(texTarget, GL_TEXTURE_MIN_FILTER, texMinFilter); // filter for pixels > texels
     end;
   end;



procedure TTexture.SetMagFilter(filter : GLint);

   begin
     texMagFilter := filter;
     if bgAutoCompile then begin
       glBindTexture(texTarget, texHandle); // texture binding info
       glTexParameteri(texTarget, GL_TEXTURE_MAG_FILTER, texMagFilter); // filter for pixels < texels
     end;
   end;



procedure TTexture.SetComponents(components : GLint);

   begin
     texComponents := components;
     if bgAutoCompile then Compile;  // must compile texture if components are changed
   end;



procedure TTexture.Display;

  begin
    glBindTexture(texTarget, texHandle) // texture binding info
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Texture1D implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



constructor TTexture1d.Create;

  begin
    inherited Create;
    sdIdentity := SD_TEX_1D;
    texTarget := GL_TEXTURE_1D;
    t1dWraps := GL_REPEAT;       // wrap texture s parameter
    t1dBorderColor.r := 0;      // texture border color
    t1dBorderColor.g := 0;
    t1dBorderColor.b := 0;
    t1dBorderColor.a := 1;
    t1dLine := NIL;
  end;



destructor TTexture1d.Destroy;

  begin
    if t1dLine<>NIL then begin
      FreeMem(t1dLine.data);
      FreeMem(t1dLine);
    end;
    inherited Destroy;
  end;



function TTexture1d.GetBorderColor: TColor4f;

  begin
    result := t1dBorderColor;
  end;



function TTexture1d.GetWrapS: GLint;

  begin
    result := t1dWraps;
  end;



function TTexture1d.GetLine: PRGBALine;

  begin
    result := t1dLine;
  end;



procedure TTexture1d.SetBorderColor(r,g,b,a : GLfloat);

  begin
    t1dBorderColor.r := r;
    t1dBorderColor.g := g;
    t1dBorderColor.b := b;
    t1dBorderColor.a := a;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameterfv(texTarget, GL_TEXTURE_BORDER_COLOR, Addr(t1dBorderColor)); // texture border color
    end;
  end;



procedure TTexture1d.SetWrapS(value : GLint);

  begin
    t1dWraps := value;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameteri(texTarget, GL_TEXTURE_WRAP_S, t1dWraps); // wrap texture s parameter
    end;
  end;



procedure TTexture1D.SetLine(Line: PRGBALine);

   begin
     t1dLine := Line;
     if bgAutoCompile then Compile;
   end;



procedure TTexture1d.Rescale(NewLength : integer);

  begin
    if t1dLine=NIL then
      New(t1dLine)
    else
      FreeMem(t1dLine.data);
    t1dLine.length := NewLength;
    GetMem(t1dLine.data, NewLength * 4);
  end;



procedure TTexture1d.Compile;

  begin
    glBindTexture(texTarget, texHandle); // texture binding info
    // either use mipmaps or set texture image directly
    if t1dLine<>NIL then with t1dLine^ do begin
      {$IFNDEF NO_GLU}
      if bgUseMipMaps and (Addr(gluBuild1DMipMaps)<>NIL) then
      gluBuild1dMipMaps(texTarget,             // target
                        texComponents,         // components
                        length,
                        GL_RGBA, GL_UNSIGNED_BYTE, data)
      else
     {$ENDIF}
      glTexImage1D(texTarget,             // target
                   0,                  // mipmap level
                   texComponents,         // components
                   length,
                   0,                  // border
                   GL_RGBA, GL_UNSIGNED_BYTE, data);
    end; // t1dLine defined
    glTexParameteri(texTarget, GL_TEXTURE_MIN_FILTER, texMinFilter); // filter for pixels > texels
    glTexParameteri(texTarget, GL_TEXTURE_MAG_FILTER, texMagFilter); // filter for pixels < texels
    glTexParameteri(texTarget, GL_TEXTURE_WRAP_S, t1dWraps); // wrap texture s parameter
    glTexParameterfv(texTarget, GL_TEXTURE_BORDER_COLOR, Addr(t1dBorderColor)); // texture border color
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Texture2D implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



constructor TTexture2d.Create;

  begin
    inherited Create;
    sdIdentity := SD_TEX_2D;
    texTarget := GL_TEXTURE_2D;
    t2dImage := NIL;
    t2dWrapS := GL_REPEAT;       // wrap texture s parameter
    t2dWrapT := GL_REPEAT;       // wrap texture t parameter
    t2dBorderColor.r := 0;      // texture border color
    t2dBorderColor.g := 0;
    t2dBorderColor.b := 0;
    t2dBorderColor.a := 1;
  end;



destructor TTexture2d.Destroy;

  begin
    if t2dImage<>NIL then begin
      FreeMem(t2dImage.data);
      FreeMem(t2dImage);
    end;
    inherited Destroy;
  end;



function TTexture2d.GetImage: PRGBAImage;

  begin
    result := t2dImage;
  end;



procedure TTexture2d.SetImage(Image : PRGBAImage);

  begin
    t2dImage := Image;
    if bgAutoCompile then Compile;
  end;



function TTexture2d.GetBorderColor : TColor4f;

  begin
    result := t2dBorderColor;
  end;



procedure TTexture2d.SetBorderColor(r,g,b,a : GLfloat);

  begin
    t2dBorderColor.r := r;
    t2dBorderColor.g := g;
    t2dBorderColor.b := b;
    t2dBorderColor.a := a;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameterfv(texTarget, GL_TEXTURE_BORDER_COLOR, Addr(t2dBorderColor)); // texture border color
    end;
  end;



function TTexture2D.GetWrapS : GLint;

  begin
    result := t2dWrapS;
  end;



procedure TTexture2d.SetWrapS(value : GLint);

  begin
    t2dWrapS := value;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameteri(texTarget, GL_TEXTURE_WRAP_S, t2dWrapS); // wrap texture s parameter
    end;
  end;



function TTexture2D.GetWrapT : GLint;

  begin
    result := t2dWrapT;
  end;



procedure TTexture2d.SetWrapT(value : GLint);

  begin
    t2dWrapT := value;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameteri(texTarget, GL_TEXTURE_WRAP_T, t2dWrapT); // wrap texture t parameter
    end;
  end;



procedure TTexture2d.Rescale(NewWidth, NewHeight : integer);

  begin
    if t2dImage=NIL then
      New(t2dImage)
    else
      FreeMem(t2dImage.data);
    t2dImage.width := NewWidth;
    t2dImage.height := NewHeight;
    GetMem(t2dImage.data, NewWidth * NewHeight * 4);
  end;



procedure TTexture2d.Compile;

  begin
    glBindTexture(texTarget, texHandle); // texture binding info
    // either use mipmaps or set texture image directly
    if t2dImage<>NIL then with t2dImage^ do begin
      {$IFNDEF NO_GLU}
      if bgUseMipMaps and (Addr(gluBuild2DMipMaps)<>NIL) then
      gluBuild2dMipMaps(texTarget,             // target
                        texComponents,         // components
                        width, height,
                        GL_RGBA, GL_UNSIGNED_BYTE, data)
      else
     {$ENDIF}
      glTexImage2D(texTarget,             // target
                   0,                  // mipmap level
                   texComponents,         // components
                   width, height,
                   0,                  // border
                   GL_RGBA, GL_UNSIGNED_BYTE, data);
    end;
    glTexParameteri(texTarget, GL_TEXTURE_MIN_FILTER, texMinFilter); // filter for pixels > texels
    glTexParameteri(texTarget, GL_TEXTURE_MAG_FILTER, texMagFilter); // filter for pixels < texels
    glTexParameteri(texTarget, GL_TEXTURE_WRAP_S, t2dWrapS); // wrap texture s parameter
    glTexParameteri(texTarget, GL_TEXTURE_WRAP_T, t2dWrapT); // wrap texture t parameter
    glTexParameterfv(texTarget, GL_TEXTURE_BORDER_COLOR, Addr(t2dBorderColor)); // texture border color
 end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Texture3D implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



{$IFDEF GL_12}
constructor TTexture3d.Create;

  begin
    inherited Create;
    sdIdentity := SD_TEX_3D;
    texTarget := GL_TEXTURE_3D;
    t3dBox := NIL;
    t3dWrapS := GL_REPEAT;       // wrap texture s parameter
    t3dWrapT := GL_REPEAT;       // wrap texture t parameter
    t3dWrapR := GL_REPEAT;       // wrap texture r parameter
    t3dBorderColor.r := 0;      // texture border color
    t3dBorderColor.g := 0;
    t3dBorderColor.b := 0;
    t3dBorderColor.a := 1;
    texComponents := GL_RGBA;
  end;



destructor TTexture3d.Destroy;

  begin
    if t3dBox<>NIL then begin
      FreeMem(t3dBox.data);
      FreeMem(t3dBox);
    end;
    inherited Destroy;
  end;



function TTexture3d.GetBox : PRGBABox;

  begin
    result := t3dBox;
  end;



procedure TTexture3d.SetBox(Box : PRGBABox);

  begin
    t3dBox := Box;
    if bgAutoCompile then Compile;
  end;



function TTexture3D.GetBorderColor : TColor4f;

  begin
    result := t3dBorderColor;
  end;



procedure TTexture3d.SetBorderColor(r,g,b,a : GLfloat);

  begin
    t3dBorderColor.r := r;
    t3dBorderColor.g := g;
    t3dBorderColor.b := b;
    t3dBorderColor.a := a;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameterfv(texTarget, GL_TEXTURE_BORDER_COLOR, Addr(t3dBorderColor)); // texture border color
    end;
  end;



function TTexture3d.GetWrapS : GLint;

  begin
    result := t3dWrapS;
  end;



procedure TTexture3d.SetWrapS(value : GLint);

  begin
    t3dWrapS := value;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameteri(texTarget, GL_TEXTURE_WRAP_S, t3dWrapS); // wrap texture s parameter
    end;
  end;



function TTexture3d.GetWrapT : GLint;

  begin
    result := t3dWrapT;
  end;



procedure TTexture3d.SetWrapT(value : GLint);

  begin
    t3dWrapT := value;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameteri(texTarget, GL_TEXTURE_WRAP_T, t3dWrapT); // wrap texture t parameter
    end;
  end;



function TTexture3d.GetWrapR : GLint;

  begin
    result := t3dWrapR;
  end;



procedure TTexture3d.SetWrapR(value : GLint);

  begin
    t3dWrapR := value;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameteri(texTarget, GL_TEXTURE_WRAP_R, t3dWrapR); // wrap texture s parameter
    end;
  end;



procedure TTexture3d.Rescale(NewWidth, NewHeight, NewDepth : integer);

  begin
    if t3dBox=NIL then
      New(t3dBox)
    else
      FreeMem(t3dBox.data);
    t3dBox.width := NewWidth;
    t3dBox.height := NewHeight;
    t3dBox.depth := NewDepth;
    GetMem(t3dBox.data, NewWidth * NewHeight * NewDepth * 4);
  end;



procedure TTexture3d.Compile;

  begin
    glBindTexture(texTarget, texHandle); // texture binding info
    // either use mipmaps or set texture image directly
    if t3dBox<>NIL then begin
      {$IFNDEF NO_GLU}
      if bgUseMipMaps and (Addr(gluBuild3dMipMaps)<>NIL) then
      gluBuild3dMipMaps(texTarget,             // target
                        texComponents,         // components
                        t3dBox.width, t3dBox.height, t3dBox.depth,
                        GL_RGBA, GL_UNSIGNED_BYTE, t3dBox.data)
      else
     {$ENDIF}
      glTexImage3D(texTarget,             // target
                   0,                  // mipmap level
                   texComponents,         // components
                   t3dBox.width, t3dBox.height, t3dBox.depth,
                   0,                  // border
                   GL_RGBA, GL_UNSIGNED_BYTE, t3dBox.data);
    end;
    glTexParameteri(texTarget, GL_TEXTURE_MIN_FILTER, texMinFilter); // filter for pixels > texels
    glTexParameteri(texTarget, GL_TEXTURE_MAG_FILTER, texMagFilter); // filter for pixels < texels
    glTexParameteri(texTarget, GL_TEXTURE_WRAP_S, t3dWrapS); // wrap texture s parameter
    glTexParameteri(texTarget, GL_TEXTURE_WRAP_T, t3dWrapT); // wrap texture t parameter
    glTexParameteri(texTarget, GL_TEXTURE_WRAP_R, t3dWrapR); // wrap texture r parameter
    glTexParameterfv(texTarget, GL_TEXTURE_BORDER_COLOR, Addr(t3dBorderColor)); // texture border color
  end;
{$ENDIF}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  TextureCubemap implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



{$IFDEF GL_13}
constructor TTextureCubemap.Create;

  begin
    inherited Create;
    sdIdentity := SD_TEX_CUBE;
    texTarget := GL_TEXTURE_CUBE_MAP;
    texComponents := GL_RGBA;
    texMinFilter := GL_LINEAR;   // filter for pixels > texels
    tcmCubemap := NIL;
    tcmWrapS := GL_CLAMP_TO_EDGE;  // wrap texture s parameter
    tcmWrapT := GL_CLAMP_TO_EDGE;  // wrap texture t parameter
    tcmWrapR := GL_CLAMP_TO_EDGE;  // wrap texture r parameter
    tcmBorderColor.r := 0;      // texture border color
    tcmBorderColor.g := 0;
    tcmBorderColor.b := 0;
    tcmBorderColor.a := 1;
  end;



destructor TTextureCubemap.Destroy;

  begin
    if tcmCubemap<>NIL then begin
      FreeMem(tcmCubemap.positive_x);
      FreeMem(tcmCubemap.negative_x);
      FreeMem(tcmCubemap.positive_y);
      FreeMem(tcmCubemap.negative_y);
      FreeMem(tcmCubemap.positive_z);
      FreeMem(tcmCubemap.negative_z);
      FreeMem(tcmCubemap);
      if bgBoundSkyBox = Self then bgBoundSkyBox := NIL;
    end;
    inherited Destroy;
  end;



function TTextureCubemap.GetCubemap : PRGBACubemap;

  begin
    result := tcmCubemap;
  end;



procedure TTextureCubemap.SetCubemap(Cubemap : PRGBACubemap);

  begin
    tcmCubemap := Cubemap;
    if bgAutoCompile then Compile;
  end;



function TTextureCubemap.GetBorderColor : TColor4f;

  begin
    result := tcmBorderColor;
  end;



procedure TTextureCubemap.SetBorderColor(r,g,b,a : GLfloat);

  begin
    tcmBorderColor.r := r;
    tcmBorderColor.g := g;
    tcmBorderColor.b := b;
    tcmBorderColor.a := a;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameterfv(texTarget, GL_TEXTURE_BORDER_COLOR, Addr(tcmBorderColor)); // texture border color
    end;
  end;



function TTextureCubemap.GetWrapS : GLint;

  begin
    result := tcmWrapS;
  end;



procedure TTextureCubemap.SetWrapS(value : GLint);

  begin
    tcmWrapS := value;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameteri(texTarget, GL_TEXTURE_WRAP_S, tcmWrapS); // wrap texture s parameter
    end;
  end;



function TTextureCubemap.GetWrapT : GLint;

  begin
    result := tcmWrapT;
  end;



procedure TTextureCubemap.SetWrapT(value : GLint);

  begin
    tcmWrapT := value;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameteri(texTarget, GL_TEXTURE_WRAP_T, tcmWrapT); // wrap texture t parameter
    end;
  end;



function TTextureCubemap.GetWrapR : GLint;

  begin
    result := tcmWrapR;
  end;



procedure TTextureCubemap.SetWrapR(value : GLint);

  begin
    tcmWrapR := value;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameteri(texTarget, GL_TEXTURE_WRAP_R, tcmWrapR); // wrap texture s parameter
    end;
  end;



procedure TTextureCubemap.Rescale(NewSide : integer);

  var
    newsize : integer;

  begin
    newsize := NewSide * NewSide * 4;
    if tcmCubemap=NIL then begin
      New(tcmCubemap);
    end
    else begin
      FreeMem(tcmCubemap.positive_x);
      FreeMem(tcmCubemap.negative_x);
      FreeMem(tcmCubemap.positive_y);
      FreeMem(tcmCubemap.negative_y);
      FreeMem(tcmCubemap.positive_z);
      FreeMem(tcmCubemap.negative_z);
    end;
    tcmCubemap.side := NewSide;
    GetMem(tcmCubemap.positive_x, newsize);
    GetMem(tcmCubemap.negative_x, newsize);
    GetMem(tcmCubemap.positive_y, newsize);
    GetMem(tcmCubemap.negative_y, newsize);
    GetMem(tcmCubemap.positive_z, newsize);
    GetMem(tcmCubemap.negative_z, newsize);
  end;



procedure TTextureCubemap.Compile;

  begin
    glBindTexture(texTarget, texHandle); // texture binding info
    if tcmCubemap<>NIL then begin
      glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X, 0, texComponents, tcmCubemap.side, tcmCubemap.side, 0, GL_RGBA, GL_UNSIGNED_BYTE, tcmCubemap.positive_x);
      glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_X, 0, texComponents, tcmCubemap.side, tcmCubemap.side, 0, GL_RGBA, GL_UNSIGNED_BYTE, tcmCubemap.negative_x);
      glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Y, 0, texComponents, tcmCubemap.side, tcmCubemap.side, 0, GL_RGBA, GL_UNSIGNED_BYTE, tcmCubemap.positive_y);
      glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y, 0, texComponents, tcmCubemap.side, tcmCubemap.side, 0, GL_RGBA, GL_UNSIGNED_BYTE, tcmCubemap.negative_y);
      glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Z, 0, texComponents, tcmCubemap.side, tcmCubemap.side, 0, GL_RGBA, GL_UNSIGNED_BYTE, tcmCubemap.positive_z);
      glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z, 0, texComponents, tcmCubemap.side, tcmCubemap.side, 0, GL_RGBA, GL_UNSIGNED_BYTE, tcmCubemap.negative_z);
    end;
    glTexParameteri(texTarget, GL_TEXTURE_MIN_FILTER, texMinFilter); // filter for pixels > texels
    glTexParameteri(texTarget, GL_TEXTURE_MAG_FILTER, texMagFilter); // filter for pixels < texels
    glTexParameteri(texTarget, GL_TEXTURE_WRAP_S, tcmWrapS); // wrap texture s parameter
    glTexParameteri(texTarget, GL_TEXTURE_WRAP_T, tcmWrapT); // wrap texture t parameter
    glTexParameteri(texTarget, GL_TEXTURE_WRAP_R, tcmWrapR); // wrap texture r parameter
    glTexParameterfv(texTarget, GL_TEXTURE_BORDER_COLOR, Addr(tcmBorderColor)); // texture border color
  end;



procedure TTextureCubemap.DisplaySkyBox(side : float);

  begin
    DrawSkyBox(side, texHandle);
  end;
{$ENDIF}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  DynamicTexture2D implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



constructor TDynamicTexture2d.Create;

  begin
    inherited Create;
    texTarget := GL_TEXTURE_2D;
    texMinFilter := GL_LINEAR;   // no mipmaps for dynamic textures
    dt2WrapS := GL_REPEAT;       // wrap texture s parameter
    dt2WrapT := GL_REPEAT;       // wrap texture t parameter
    dt2CreateNew := true;        // texture has to be newly created
    dt2BorderColor.r := 0;       // texture border color
    dt2BorderColor.g := 0;
    dt2BorderColor.b := 0;
    dt2BorderColor.a := 1;
    dt2Width := 64;
    dt2Height := 64;
    dt2FoV := 45.0;
    dt2AspRat := 1.0;
    dt2NearZ := bgZNear;
    dt2FarZ := bgZFar;
    sdIdentity := SD_TEX_DYN_2D;
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(dt2Matrix));
    dt2Bound := NIL;
  end;



function TDynamicTexture2d.GetBorderColor : TColor4f;

  begin
    result := dt2BorderColor;
  end;



procedure TDynamicTexture2d.SetBorderColor(r,g,b,a : GLfloat);

  begin
    dt2BorderColor.r := r;
    dt2BorderColor.g := g;
    dt2BorderColor.b := b;
    dt2BorderColor.a := a;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameterfv(texTarget, GL_TEXTURE_BORDER_COLOR, Addr(dt2BorderColor)); // texture border color
    end;
  end;



function TDynamicTexture2d.GetWrapS : GLint;

  begin
    result := dt2WrapS;
  end;



procedure TDynamicTexture2d.SetWrapS(value : GLint);

  begin
    dt2WrapS := value;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameteri(texTarget, GL_TEXTURE_WRAP_S, dt2WrapS); // wrap texture s parameter
    end;
  end;



function TDynamicTexture2d.GetWrapT : GLint;

  begin
    result := dt2WrapT;
  end;



procedure TDynamicTexture2d.SetWrapT(value : GLint);

  begin
    dt2WrapT := value;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameteri(texTarget, GL_TEXTURE_WRAP_T, dt2WrapT); // wrap texture t parameter
    end;
  end;



function TDynamicTexture2d.GetMatrix : TMatrix4f;

  begin
    result := dt2Matrix;
  end;



procedure TDynamicTexture2d.SetMatrix(Matrix : TMatrix4f);

  begin
    dt2Matrix := Matrix;
  end;



procedure TDynamicTexture2D.SetMatrixCoeff(c00,c01,c02,c03,c04, c10,c11,c12,c13, c20,c21,c22,c23, c30,c31,c32,c33 : float);

  begin
    dt2Matrix[0,0]:=c00;  dt2Matrix[0,1]:=c01;  dt2Matrix[0,2]:=c02;  dt2Matrix[0,3]:=c03;
    dt2Matrix[1,0]:=c10;  dt2Matrix[1,1]:=c01;  dt2Matrix[1,2]:=c02;  dt2Matrix[1,3]:=c03;
    dt2Matrix[2,0]:=c20;  dt2Matrix[2,1]:=c01;  dt2Matrix[2,2]:=c02;  dt2Matrix[2,3]:=c03;
    dt2Matrix[3,0]:=c30;  dt2Matrix[3,1]:=c01;  dt2Matrix[3,2]:=c02;  dt2Matrix[3,3]:=c03;
  end;



function TDynamicTexture2d.GetFoV : GLdouble;

  begin
    result := dt2FoV;
  end;



procedure TDynamicTexture2d.SetFoV(FoV : GLdouble);

  begin
    dt2FoV := FoV;
  end;



function TDynamicTexture2d.GetAspRat : GLdouble;

  begin
    result := dt2AspRat;
  end;



procedure TDynamicTexture2d.SetAspRat(AspRat : GLdouble);

  begin
    dt2AspRat := AspRat;
  end;



function TDynamicTexture2d.GetNearZ : GLdouble;

  begin
    result := dt2NearZ;
  end;



procedure TDynamicTexture2d.SetNearZ(NearZ : GLdouble);

  begin
    dt2NearZ := NearZ;
  end;



function TDynamicTexture2d.GetFarZ : GLdouble;

  begin
    result := dt2FarZ;
  end;



procedure TDynamicTexture2d.SetFarZ(FarZ : GLdouble);

  begin
    dt2FarZ := FarZ;
  end;



procedure TDynamicTexture2d.Rescale(NewWidth, NewHeight : integer);

  begin
    dt2Width := NewWidth;
    dt2Height := NewHeight;
    dt2CreateNew := true;
  end;



function TDynamicTexture2D.GetWidth : GLint;

  begin
    result := dt2Width;
  end;



function TDynamicTexture2D.GetHeight : GLint;

  begin
    result := dt2Height;
  end;



procedure TDynamicTexture2d.Bind(obj: TObj);

  begin
    dt2Bound := obj;
  end;



function TDynamicTexture2d.GetBoundObj : TObj;

  begin
    result := dt2Bound;
  end;



procedure TDynamicTexture2d.Compile;

  var
    v : TVector4i;
    temp : TMatrix4f;

  begin
    glGetIntegerv(GL_VIEWPORT, Addr(v));
    glViewport(0, 0, dt2Width, dt2Height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity;
    gluPerspective(dt2FoV, dt2AspRat, dt2NearZ, dt2FarZ);
    glMatrixMode(GL_MODELVIEW);
    temp := bgViewMatrix;
    if dt2Bound<>NIL then begin
      dt2Matrix := dt2Bound.objMatrix;
      MatrixInvert(dt2Matrix);
    end;
    bgViewMatrix := dt2Matrix;
    glLoadMatrixf(Addr(bgViewMatrix));
    FrustumCalculate;
    ViewComputeInvertedTextureMatrixList;

    bgDrawBackground;
{$IFDEF GL_PROGRAM}
    if GL_PROGRAM then begin
      UpdateVPEnvParams;
      UpdateFPEnvParams;
    end;
{$ENDIF}
    bgDrawObjects;
    glBindTexture(texTarget, texHandle);
    if dt2CreateNew then begin
      glCopyTexImage2d(texTarget, 0, texComponents, 0, 0, dt2Width, dt2Height, 0);
      dt2CreateNew := false;
    end
    else
      glCopyTexSubImage2d(texTarget, 0, 0, 0, 0, 0, dt2Width, dt2Height);
    glTexParameteri(texTarget, GL_TEXTURE_MIN_FILTER, texMinFilter); // filter for pixels > texels
    glTexParameteri(texTarget, GL_TEXTURE_MAG_FILTER, texMagFilter); // filter for pixels < texels
    glTexParameteri(texTarget, GL_TEXTURE_WRAP_S, dt2WrapS); // wrap texture s parameter
    glTexParameteri(texTarget, GL_TEXTURE_WRAP_T, dt2WrapT); // wrap texture t parameter
    glTexParameterfv(texTarget, GL_TEXTURE_BORDER_COLOR, Addr(dt2BorderColor)); // texture border color

    glViewPort(v[0], v[1], v[2], v[3]);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity;
    gluPerspective(bgFieldOfView, (bgAspectRatio*v[2])/v[3], bgZNear, bgZFar);
    glMatrixMode(GL_MODELVIEW);
    bgViewMatrix := temp;
    glLoadMatrixf(Addr(bgViewMatrix));
    FrustumCalculate;
    ViewComputeInvertedTextureMatrixList;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  DynamicTextureCubemap implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



{$IFDEF GL_13}
constructor TDynamicTextureCubemap.Create;

  begin
    inherited Create;
    texTarget := GL_TEXTURE_CUBE_MAP;
    texMinFilter := GL_LINEAR;   // no mipmaps for dynamic textures
    dcmCreateNew := true;        // texture has to be newly created
    dcmWrapS := GL_CLAMP_TO_EDGE;  // wrap texture s parameter
    dcmWrapT := GL_CLAMP_TO_EDGE;  // wrap texture t parameter
    dcmWrapR := GL_CLAMP_TO_EDGE;  // wrap texture r parameter
    dcmBorderColor.r := 0;      // texture border color
    dcmBorderColor.g := 0;
    dcmBorderColor.b := 0;
    dcmBorderColor.a := 1;
    dcmSide := 64;
    dcmFoV := 90.0;
    dcmAspRat := 1.0;
    dcmNearZ := bgZNear;
    dcmFarZ := bgZFar;
    sdIdentity := SD_TEX_DYN_CUBE;
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(dcmMatrix));
    dcmBound := NIL;
  end;



procedure TDynamicTextureCubemap.Rescale(NewSide : integer);

  begin
    dcmSide := NewSide;
    dcmCreateNew := true;
  end;



procedure TDynamicTextureCubemap.Bind(obj: TObj);

  begin
    dcmBound := obj;
  end;



function TDynamicTextureCubemap.GetBoundObj : TObj;

  begin
    result := dcmBound;
  end;



procedure TDynamicTextureCubemap.Compile;

  procedure ReadTexture(Target: GLenum);
  begin
    FrustumCalculate;
    ViewComputeInvertedTextureMatrixList;
    if bgBoundSkyBox<>self then
      bgDrawBackground
    else
      glClear(bgClearBufferBits);
{$IFDEF GL_PROGRAM}
    if GL_PROGRAM then begin
      UpdateVPEnvParams;
      UpdateFPEnvParams;
    end;
{$ENDIF}
    bgDrawObjects;
    glEnable(GL_TEXTURE_CUBE_MAP);
    glBindTexture(GL_TEXTURE_CUBE_MAP, texHandle);
    if dcmCreateNew then
      glCopyTexImage2d(Target, 0, texComponents, 0, 0, dcmSide, dcmSide, 0)
    else
      glCopyTexSubImage2d(Target, 0, 0, 0, 0, 0, dcmSide, dcmSide);
  end;

  var
    v : TVector4i;
    temp : TMatrix4f;

  begin
    glGetIntegerv(GL_VIEWPORT, Addr(v));
    glViewport(0, 0, dcmSide, dcmSide);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity;
    gluPerspective(dcmFoV, dcmAspRat, dcmNearZ, dcmFarZ);
    glMatrixMode(GL_MODELVIEW);
    temp := bgViewMatrix;
    if dcmBound<>NIL then begin
      dcmMatrix := dcmBound.objMatrix;
      MatrixInvert(dcmMatrix);
    end;

    // positive x
    glLoadIdentity;
    glRotatef(-90, 0, 1, 0);
    glRotatef(180, 0, 0, 1);
    glMultMatrixf(Addr(dcmMatrix));
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(bgViewMatrix));
    ReadTexture(GL_TEXTURE_CUBE_MAP_POSITIVE_X);

    // negative x
    glLoadIdentity;
    glRotatef( 90, 0, 1, 0);
    glRotatef(180, 0, 0, 1);
    glMultMatrixf(Addr(dcmMatrix));
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(bgViewMatrix));
    ReadTexture(GL_TEXTURE_CUBE_MAP_NEGATIVE_X);

    // positive y
    glLoadIdentity;
    glRotatef(-90, 1, 0, 0);
    glMultMatrixf(Addr(dcmMatrix));
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(bgViewMatrix));
    ReadTexture(GL_TEXTURE_CUBE_MAP_POSITIVE_Y);

    // negative y
    glLoadIdentity;
    glRotatef( 90, 1, 0, 0);
    glMultMatrixf(Addr(dcmMatrix));
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(bgViewMatrix));
    ReadTexture(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y);

    // positive z
    glLoadIdentity;
    glRotatef(180, 1, 0, 0);
    glMultMatrixf(Addr(dcmMatrix));
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(bgViewMatrix));
    ReadTexture(GL_TEXTURE_CUBE_MAP_POSITIVE_Z);

    // negative z
    glLoadIdentity;
    glRotatef(180, 0, 0, 1);
    glMultMatrixf(Addr(dcmMatrix));
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(bgViewMatrix));
    ReadTexture(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z);

    dcmCreateNew := false;
    glTexParameteri(texTarget, GL_TEXTURE_MIN_FILTER, texMinFilter); // filter for pixels > texels
    glTexParameteri(texTarget, GL_TEXTURE_MAG_FILTER, texMagFilter); // filter for pixels < texels
    glTexParameteri(texTarget, GL_TEXTURE_WRAP_S, dcmWrapS); // wrap texture s parameter
    glTexParameteri(texTarget, GL_TEXTURE_WRAP_T, dcmWrapT); // wrap texture t parameter
    glTexParameteri(texTarget, GL_TEXTURE_WRAP_R, dcmWrapR); // wrap texture r parameter
    glTexParameterfv(texTarget, GL_TEXTURE_BORDER_COLOR, Addr(dcmBorderColor)); // texture border color

    glViewPort(v[0], v[1], v[2], v[3]);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity;
    gluPerspective(bgFieldOfView, (bgAspectRatio*v[2])/v[3], bgZNear, bgZFar);
    glMatrixMode(GL_MODELVIEW);
    bgViewMatrix := temp;
    glLoadMatrixf(Addr(bgViewMatrix));
    FrustumCalculate;
    ViewComputeInvertedTextureMatrixList;
  end;



procedure TDynamicTextureCubemap.DisplaySkyBox(side : float);

  begin
    DrawSkyBox(side, texHandle);
  end;



function TDynamicTextureCubemap.GetSide : GLint;

  begin
    result := dcmSide;
  end;



function TDynamicTextureCubemap.GetBorderColor : TColor4f;

  begin
    result := dcmBorderColor;
  end;



procedure TDynamicTextureCubemap.SetBorderColor(r,g,b,a : GLfloat);

  begin
    dcmBorderColor.r := r;
    dcmBorderColor.g := g;
    dcmBorderColor.b := b;
    dcmBorderColor.a := a;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameterfv(texTarget, GL_TEXTURE_BORDER_COLOR, Addr(dcmBorderColor)); // texture border color
    end;
  end;



function TDynamicTextureCubemap.GetWrapS : GLint;

  begin
    result := dcmWrapS;
  end;



procedure TDynamicTextureCubemap.SetWrapS(value : GLint);

  begin
    dcmWrapS := value;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameteri(texTarget, GL_TEXTURE_WRAP_S, dcmWrapS); // wrap texture s parameter
    end;
  end;



function TDynamicTextureCubemap.GetWrapT : GLint;

  begin
    result := dcmWrapT;
  end;



procedure TDynamicTextureCubemap.SetWrapT(value : GLint);

  begin
    dcmWrapT := value;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameteri(texTarget, GL_TEXTURE_WRAP_T, dcmWrapT); // wrap texture t parameter
    end;
  end;



function TDynamicTextureCubemap.GetWrapR : GLint;

  begin
    result := dcmWrapR;
  end;



procedure TDynamicTextureCubemap.SetWrapR(value : GLint);

  begin
    dcmWrapR := value;
    if bgAutoCompile then begin
      glBindTexture(texTarget, texHandle); // texture binding info
      glTexParameteri(texTarget, GL_TEXTURE_WRAP_R, dcmWrapR); // wrap texture s parameter
    end;
  end;



function TDynamicTextureCubemap.GetMatrix : TMatrix4f;

  begin
    result := dcmMatrix;
  end;



procedure TDynamicTextureCubemap.SetMatrix(Matrix : TMatrix4f);

  begin
    dcmMatrix := Matrix;
  end;



procedure TDynamicTextureCubemap.SetMatrixCoeff(c00,c01,c02,c03,c04, c10,c11,c12,c13, c20,c21,c22,c23, c30,c31,c32,c33 : float);

  begin
    dcmMatrix[0,0]:=c00;  dcmMatrix[0,1]:=c01;  dcmMatrix[0,2]:=c02;  dcmMatrix[0,3]:=c03;
    dcmMatrix[1,0]:=c10;  dcmMatrix[1,1]:=c01;  dcmMatrix[1,2]:=c02;  dcmMatrix[1,3]:=c03;
    dcmMatrix[2,0]:=c20;  dcmMatrix[2,1]:=c01;  dcmMatrix[2,2]:=c02;  dcmMatrix[2,3]:=c03;
    dcmMatrix[3,0]:=c30;  dcmMatrix[3,1]:=c01;  dcmMatrix[3,2]:=c02;  dcmMatrix[3,3]:=c03;
  end;



function TDynamicTextureCubemap.GetFoV : GLdouble;

  begin
    result := dcmFoV;
  end;



procedure TDynamicTextureCubemap.SetFoV(FoV : GLdouble);

  begin
    dcmFoV := FoV;
  end;



function TDynamicTextureCubemap.GetAspRat : GLdouble;

  begin
    result := dcmAspRat;
  end;



procedure TDynamicTextureCubemap.SetAspRat(AspRat : GLdouble);

  begin
    dcmAspRat := AspRat;
  end;



function TDynamicTextureCubemap.GetNearZ : GLdouble;

  begin
    result := dcmNearZ;
  end;



procedure TDynamicTextureCubemap.SetNearZ(NearZ : GLdouble);

  begin
    dcmNearZ := NearZ;
  end;



function TDynamicTextureCubemap.GetFarZ : GLdouble;

  begin
    result := dcmFarZ;
  end;



procedure TDynamicTextureCubemap.SetFarZ(FarZ : GLdouble);

  begin
    dcmFarZ := FarZ;
  end;
{$ENDIF}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  TextureDefault implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



constructor TTextureDefault.Create;

  begin
    inherited Create;
    sdIdentity := SD_TEX_DEFAULT;
    name := 'bgDefaultTexture';
  end;



procedure TTextureDefault.Display;

  begin
    glBindTexture(GL_TEXTURE_1D, 0);
    glBindTexture(GL_TEXTURE_2D, 0);
    {$IFDEF GL_12} if GL_12 then glBindTexture(GL_TEXTURE_3D, 0); {$ENDIF}
    {$IFDEF GL_13} if GL_13 then glBindTexture(GL_TEXTURE_CUBE_MAP, 0); {$ENDIF}
  end;



procedure TTextureDefault.Compile;

  begin
    Display;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Material implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



constructor TMaterial.Create;

  begin
    sdIdentity := SD_MAT;
    name := 'Material';
    matList := glGenLists(1);
    InitMaterial;
    matShininess.change := true;
    matAmbient.change := true;
    matDiffuse.change := true;
    matSpecular.change := true;
    matEmission.change := true;
    if bgAutoCompile then Compile;
  end;



destructor TMaterial.Destroy;

  begin
    glDeleteLists(matList, 1);
    inherited Destroy;
  end;



function TMaterial.GetFace : GLenum;
  begin
    result := matFace;
  end;

function TMaterial.GetShininess : TMatShininess;
  begin
    result := matShininess;
  end;

function TMaterial.GetAmbient : TMatColor;
  begin
    result := matAmbient;
  end;

function TMaterial.GetDiffuse : TMatColor;
  begin
    result := matDiffuse;
  end;

function TMaterial.GetSpecular : TMatColor;
  begin
    result := matSpecular;
  end;

function TMaterial.GetEmission : TMatColor;
  begin
    result := matEmission;
  end;



procedure TMaterial.SetFace(face : GLenum);
  begin
    matFace := face;
    if bgAutoCompile then Compile;
  end;

procedure TMaterial.SetShininess(v : GLfloat; update : boolean);
  begin
    matShininess.value := v;
    matShininess.change := update;
    if bgAutoCompile then Compile;
  end;

procedure TMaterial.SetAmbient(r,g,b,a : GLfloat; update : boolean);
  begin
    matAmbient.color.r := r;
    matAmbient.color.g := g;
    matAmbient.color.b := b;
    matAmbient.color.a := a;
    matAmbient.change := update;
    if bgAutoCompile then Compile;
  end;

procedure TMaterial.SetDiffuse(r,g,b,a : GLfloat; update : boolean);
  begin
    matDiffuse.color.r := r;
    matDiffuse.color.g := g;
    matDiffuse.color.b := b;
    matDiffuse.color.a := a;
    matDiffuse.change := update;
    if bgAutoCompile then Compile;
  end;

procedure TMaterial.SetSpecular(r,g,b,a : GLfloat; update : boolean);
  begin
    matSpecular.color.r := r;
    matSpecular.color.g := g;
    matSpecular.color.b := b;
    matSpecular.color.a := a;
    matSpecular.change := update;
    if bgAutoCompile then Compile;
  end;

procedure TMaterial.SetEmission(r,g,b,a : GLfloat; update : boolean);
  begin
    matEmission.color.r := r;
    matEmission.color.g := g;
    matEmission.color.b := b;
    matEmission.color.a := a;
    matEmission.change := update;
    if bgAutoCompile then Compile;
  end;

procedure TMaterial.SetShininessv(v : TMatShininess);
  begin
    matShininess := v;
    if bgAutoCompile then Compile;
  end;

procedure TMaterial.SetAmbientv(v : TMatColor);
  begin
    matAmbient := v;
    if bgAutoCompile then Compile;
  end;

procedure TMaterial.SetDiffusev(v : TMatColor);
  begin
    matDiffuse := v;
    if bgAutoCompile then Compile;
  end;

procedure TMaterial.SetSpecularv(v : TMatColor);
  begin
    matSpecular := v;
    if bgAutoCompile then Compile;
  end;

procedure TMaterial.SetEmissionv(v : TMatColor);
  begin
    matEmission := v;
    if bgAutoCompile then Compile;
  end;



procedure TMaterial.Compile;

  begin
    glNewList(matList, GL_COMPILE);
      if matShininess.change then glMaterialf(matFace, GL_SHININESS, matShininess.value);
      if matAmbient.change then glMaterialfv(matFace, GL_AMBIENT, Addr(matAmbient.color));
      if matDiffuse.change then glMaterialfv(matFace, GL_DIFFUSE, Addr(matDiffuse.color));
      if matSpecular.change then glMaterialfv(matFace, GL_SPECULAR, Addr(matSpecular.color));
      if matEmission.change then glMaterialfv(matFace, GL_EMISSION, Addr(matEmission.color));
    glEndList;
  end;



procedure TMaterial.Display;

  begin
    glCallList(matList);
  end;



procedure TMaterial.InitMaterial;

  begin
    with matShininess do begin
      value := 0.0;
      change := false;
    end;
    with matAmbient do begin
      color.r := 0.2;
      color.g := 0.2;
      color.b := 0.2;
      color.a := 1.0;
      change := false;
    end;
    with matDiffuse do begin
      color.r := 0.8;
      color.g := 0.8;
      color.b := 0.8;
      color.a := 1.0;
      change := false;
    end;
    with matSpecular do begin
      color.r := 0.0;
      color.g := 0.0;
      color.b := 0.0;
      color.a := 1.0;
      change := false;
    end;
    with matEmission do begin
      color.r := 0.0;
      color.g := 0.0;
      color.b := 0.0;
      color.a := 1.0;
      change := false;
    end;
    matFace := GL_FRONT;
    if bgAutoCompile then Compile;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  RenderSettings implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



constructor TRenderSettings.Create;

  begin
    Name := 'RenderSettings';
    sdIdentity := SD_RS;
    rsList := glGenLists(1);
    InitRenderSettings;
  end;



destructor TRenderSettings.Destroy;

  begin
    glDeleteLists(rsList, 1);
    inherited Destroy;
  end;



function TRenderSettings.GetAlphaTest : TRSAlphaTest;
  begin
    result := rsAlphaTest;
  end;

function TRenderSettings.GetBlendFunc : TRSBlendFunc;
  begin
    result := rsBlendFunc;
  end;

function TRenderSettings.GetColor : TRSColor;
  begin
    result := rsColor;
  end;

function TRenderSettings.GetColorMask : TRSColorMask;
  begin
    result := rsColorMask;
  end;

function TRenderSettings.GetColorMaterial : TRSColorMaterial;
  begin
    result := rsColorMaterial;
  end;

function TRenderSettings.GetCullFace : TRSCullFace;
  begin
    result := rsCullFace;
  end;

function TRenderSettings.GetDepthTest : TRSDepthTest;
  begin
    result := rsDepthTest;
  end;

function TRenderSettings.GetDepthWrite : TCapState;
  begin
    result := rsDepthWrite;
  end;

function TRenderSettings.GetFog : TCapState;
  begin
    result := rsFog;
  end;

function TRenderSettings.GetFragmentProgram : TCapState;
  begin
    result := rsFragmentProgram;
  end;

function TRenderSettings.GetFrontFace : TRSFrontFace;
  begin
    result := rsFrontFace;
  end;

function TRenderSettings.GetLighting : TCapState;
  begin
    result := rsLighting;
  end;

function TRenderSettings.GetLinesmooth : TCapState;
  begin
    result := rsLineSmooth;
  end;

function TRenderSettings.GetLineStipple : TRSLineStipple;
  begin
    result := rsLineStipple;
  end;

function TRenderSettings.GetLineWidth : TRSLineWidth;
  begin
    result := rsLineWidth;
  end;

function TRenderSettings.GetNormalize : TCapState;
  begin
    result := rsNormalize;
  end;

function TRenderSettings.GetPointParam : TRSPointParam;
  begin
    result := rsPointParam;
  end;

function TRenderSettings.GetPointSize : TRSPointSize;
  begin
    result := rsPointSize;
  end;

function TRenderSettings.GetPointSmooth : TCapState;
  begin
    result := rsPointSmooth;
  end;

function TRenderSettings.GetPolyMode : TRSPolyMode;
  begin
    result := rsPolyMode;
  end;

function TRenderSettings.GetPolySmooth : TCapState;
  begin
    result := rsPolySmooth;
  end;

function TRenderSettings.GetPolyStipple : TRSPolyStipple;
  begin
    result := rsPolyStipple;
  end;

function TRenderSettings.GetRescaleNormal : TCapState;
  begin
    result := rsRescaleNormal;
  end;

function TRenderSettings.GetSeparateColor : TCapState;
  begin
    result := rsSeparateColor;
  end;

function TRenderSettings.GetSmoothShading : TCapState;
  begin
    result := rsSmoothShading;
  end;

function TRenderSettings.GetStencilTest : TRSStencilTest;
  begin
    result := rsStencilTest;
  end;

function TRenderSettings.GetTexturing : TCapState;
  begin
    result := rsTexturing;
  end;

function TRenderSettings.GetTwoSided : TCapState;
  begin
    result := rsTwoSided;
  end;

function TRenderSettings.GetVertexProgram : TCapState;
  begin
    result := rsVertexProgram;
  end;



procedure TRenderSettings.SetAlphaTest(state : TCapState; func : GLenum; ref : GLclampf);
  begin
    rsAlphaTest.state := state;
    rsAlphaTest.func := func;
    rsAlphaTest.ref := ref;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetBlendFunc(state : TCapState; sfactor, dfactor : GLenum);
  begin
    rsBlendFunc.state := state;
    rsBlendFunc.sfactor := sfactor;
    rsBlendFunc.dfactor := dfactor;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetColor(change : boolean; r, g, b, a : GLfloat);
  begin
    rsColor.change := change;
    rsColor.r := r;
    rsColor.g := g;
    rsColor.b := b;
    rsColor.a := a;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetColorMask(change : boolean; r, g, b, a : GLboolean);
  begin
    rsColorMask.change := change;
    rsColorMask.r := r;
    rsColorMask.g := g;
    rsColorMask.b := b;
    rsColorMask.a := a;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetColorMaterial(state : TCapState; face, mode : GLenum);
  begin
    rsColorMaterial.state := state;
    rsColorMaterial.face := face;
    rsColorMaterial.mode := mode;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetCullFace(state : TCapState; face : GLenum);
  begin
    rsCullFace.state := state;
    rsCullFace.face := face;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetDepthTest(state : TCapState; func : GLenum);
  begin
    rsDepthTest.state := state;
    rsDepthTest.func := func;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetDepthWrite(state : TCapState);
  begin
    rsDepthWrite := state;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetFog(state : TCapState);
  begin
    rsFog := state;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetFragmentProgram(state : TCapState);
  begin
    rsFragmentProgram := state;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetFrontFace(change : Boolean; winding : GLenum);
  begin
    rsFrontFace.change := change;
    rsFrontFace.winding := winding;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetLineStipple(state : TCapState; factor : GLint; pattern : GLushort);
  begin
    rsLineStipple.state := state;
    rsLineStipple.factor := factor;
    rsLineStipple.pattern := pattern;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetLighting(state : TCapState);
  begin
    rsLighting := state;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetLineSmooth(state : TCapState);
  begin
    rsLineSmooth := state;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetLineWidth(change : boolean; width : GLfloat);
  begin
    rsLineWidth.change := change;
    rsLineWidth.width := width;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetNormalize(state : TCapState);
  begin
    rsNormalize := state;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetPointParam(change : boolean; min, max, fade, attconst, attlinear, attquadr : GLfloat);
  begin
    rsPointParam.change := change;
    rsPointParam.sizemin := min;
    rsPointParam.sizemax := max;
    rsPointParam.fade := fade;
    rsPointParam.attenuation[0] := attconst;
    rsPointParam.attenuation[1] := attlinear;
    rsPointParam.attenuation[2] := attquadr;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetPointSize(change : boolean; size : GLfloat);
  begin
    rsPointSize.change := change;
    rsPointSize.size := size;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetPointSmooth(state : TCapState);
  begin
    rsPointSmooth := state;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetPolyMode(change : boolean; front, back : GLenum);
  begin
    rsPolyMode.change := change;
    rsPolyMode.front := front;
    rsPolyMode.back := back;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetPolySmooth(state : TCapState);
  begin
    rsPolySmooth := state;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetPolyStipple(state : TCapState; mask : Array of Byte);
  begin
    rsPolyStipple.state := state;
    Move(mask, rsPolyStipple.mask, SizeOf(rsPolyStipple.mask));
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetPolyStippleState(state : TCapState);
  begin
    rsPolyStipple.state := state;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetPolyStippleMask(
  m00,m01,m02,m03,m04,m05,m06,m07,m08,m09,m10,m11,m12,m13,m14,m15,
  m16,m17,m18,m19,m20,m21,m22,m23,m24,m25,m26,m27,m28,m29,m30,m31 : GLuint);
  begin
    with rsPolyStipple do begin
      Move(m00, mask[0], 4);    Move(m01, mask[4], 4);    Move(m02, mask[8], 4);    Move(m03, mask[12], 4);
      Move(m04, mask[16], 4);   Move(m05, mask[20], 4);   Move(m06, mask[24], 4);   Move(m07, mask[28], 4);
      Move(m08, mask[32], 4);   Move(m09, mask[36], 4);   Move(m10, mask[40], 4);   Move(m11, mask[44], 4);
      Move(m12, mask[48], 4);   Move(m13, mask[52], 4);   Move(m14, mask[56], 4);   Move(m15, mask[60], 4);
      Move(m16, mask[64], 4);   Move(m17, mask[68], 4);   Move(m18, mask[72], 4);   Move(m19, mask[76], 4);
      Move(m20, mask[80], 4);   Move(m21, mask[84], 4);   Move(m22, mask[88], 4);   Move(m23, mask[92], 4);
      Move(m24, mask[96], 4);   Move(m25, mask[100], 4);  Move(m26, mask[104], 4);  Move(m27, mask[108], 4);
      Move(m28, mask[112], 4);  Move(m29, mask[116], 4);  Move(m30, mask[120], 4);  Move(m31, mask[125], 4);
    end;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetRescaleNormal(state : TCapState);
  begin
    rsRescaleNormal := state;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetSeparateColor(state : TCapState);
  begin
    rsSeparateColor := state;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetSmoothShading(state : TCapState);
  begin
    rsSmoothShading := state;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetStencilTest(state : TCapState; func : GLenum; ref : GLint; mask : GLuint; opfail, opzfail, opzpass : GLenum);
  begin
    rsStencilTest.state := state;
    rsStencilTest.func := func;
    rsStencilTest.ref := ref;
    rsStencilTest.mask := mask;
    rsStencilTest.opfail := opfail;
    rsStencilTest.opzfail := opzfail;
    rsStencilTest.opzpass := opzpass;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetTexturing(state : TCapState);
  begin
    rsTexturing := state;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetTwoSided(state : TCapState);
  begin
    rsTwoSided := state;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetVertexProgram(state : TCapState);
  begin
    rsVertexProgram := state;
    if bgAutoCompile then Compile;
  end;



procedure TRenderSettings.SetAlphaTestv(v : TRSAlphaTest);
  begin
    rsAlphaTest := v;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetBlendFuncv(v : TRSBlendFunc);
  begin
    rsBlendFunc := v;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetColorv(v : TRSColor);
  begin
    rsColor := v;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetColorMaskv(v : TRSColorMask);
  begin
    rsColorMask := v;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetColorMaterialv(v : TRSColorMaterial);
  begin
    rsColorMaterial := v;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetCullFacev(v : TRSCullFace);
  begin
    rsCullFace := v;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetDepthTestv(v : TRSDepthTest);
  begin
    rsDepthTest := v;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetFrontFacev(v : TRSFrontFace);
  begin
    rsFrontFace := v;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetLineStipplev(v : TRSLineStipple);
  begin
    rsLineStipple := v;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetLineWidthv(v : TRSLineWidth);
  begin
    rsLineWidth := v;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetPointParamv(v : TRSPointParam);
  begin
    rsPointParam := v;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetPointSizev(v : TRSPointSize);
  begin
    rsPointSize := v;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetPolyModev(v : TRSPolyMode);
  begin
    rsPolyMode := v;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetPolyStipplev(v : TRSPolyStipple);
  begin
    rsPolyStipple := v;
    if bgAutoCompile then Compile;
  end;

procedure TRenderSettings.SetStencilTestv(v : TRSStencilTest);
  begin
    rsStencilTest := v;
    if bgAutoCompile then Compile;
  end;



procedure TRenderSettings.InitRenderSettings;

  begin

    with rsAlphaTest do begin
      state := csIgnore;
      func := GL_ALWAYS;
      ref := 0.0;
    end;

    with rsBlendFunc do begin
      state := csIgnore;
      sfactor := GL_SRC_ALPHA;
      dfactor := GL_ONE_MINUS_SRC_ALPHA;
    end;

    with rsColor do begin
      change := false;
      r := 1;
      g := 1;
      b := 1;
      a := 1;
    end;

    with rsColorMask do begin
      change := false;
      r := true;
      g := true;
      b := true;
      a := true;
    end;

    with rsColorMaterial do begin
      state := csIgnore;           // material color tracking
      face := GL_FRONT_AND_BACK; // affected faces (front or back)
      mode := GL_AMBIENT_AND_DIFFUSE; // tracked material parameter
    end;

    with rsCullFace do begin
      state := csIgnore;
      face := GL_BACK;
    end;

    with rsDepthTest do begin
      state := csIgnore;
      func := GL_LEQUAL;
    end;

    rsDepthWrite := csIgnore;
    rsFog := csIgnore;
    rsFragmentProgram := csIgnore;

    with rsFrontFace do begin
      change := false;
      winding := GL_CCW;
    end;

    rsLighting := csIgnore;
    rsLineSmooth := csIgnore;

    with rsLineStipple do begin
      state := csIgnore;
      factor := 1;
      pattern := $ffff;
    end;

    with rsLineWidth do begin
      change := false;
      width := 1.0;
    end;

    rsNormalize := csIgnore;

    with rsPointParam do begin
      change := false;
      sizemin := 0.0;
      sizemax := 1.0;
      fade := 1.0;
      attenuation[0] := 1.0;
      attenuation[1] := 0.0;
      attenuation[2] := 0.0;
    end;

    with rsPointSize do begin
      change := false;
      size := 1.0;
    end;

    rsPointSmooth := csIgnore;

    with rsPolyMode do begin
      change := false;
      front := GL_FILL;
      back := GL_FILL;
    end;

    rsPolySmooth := csIgnore;

    with rsPolyStipple do begin
       state := csIgnore;
       FillChar(mask, SizeOf(mask), $ff);
    end;

    rsRescaleNormal := csIgnore;
    rsSeparateColor := csIgnore;
    rsSmoothShading := csIgnore;

    with rsStencilTest do begin
      state := csIgnore;
      func := GL_ALWAYS;
      ref := 255;
      mask := Cardinal($7fffffff);
      opfail := GL_KEEP;
      opzfail := GL_KEEP;
      opzpass := GL_KEEP;
    end;

    rsTexturing := csIgnore;
    rsTwoSided := csIgnore;
    rsVertexProgram := csIgnore;

    if bgAutoCompile then Compile;
  end;



procedure TRenderSettings.Compile;

  var
    i: integer;

  begin
    glNewList(rsList, GL_COMPILE);

    with rsAlphaTest do begin
      if (state = csEnable) or (state = csUpdate) then glEnable(GL_ALPHA_TEST);
      if (state = csUpdate) then glAlphaFunc(func, ref);
      if (state = csDisable) then glDisable(GL_ALPHA_TEST);
    end;

    with rsBlendFunc do begin
      if (state = csEnable) or (state = csUpdate) then glEnable(GL_BLEND);
      if (state = csUpdate) then glBlendFunc(sfactor, dfactor);
      if (state = csDisable) then glDisable(GL_BLEND);
    end;

    if rsColor.change then
      glColor4f(rsColor.r, rsColor.g, rsColor.b, rsColor.a);

    if rsColorMask.change then
      glColorMask(rsColorMask.r, rsColorMask.g, rsColorMask.b, rsColorMask.a);

    with rsColorMaterial do begin
      if (state = csEnable) or (state = csUpdate) then glEnable(GL_COLOR_MATERIAL);
      if (state = csUpdate) then glColorMaterial(face, mode);
      if (state = csDisable) then glDisable(GL_COLOR_MATERIAL);
    end;

    with rsCullFace do begin
      if (state = csEnable) or (state = csUpdate) then glEnable(GL_CULL_FACE);
      if (state = csUpdate) then glCullFace(face);
      if (state = csDisable) then glDisable(GL_CULL_FACE);
    end;

    with rsDepthTest do begin
      if (state = csEnable) or (state = csUpdate) then glEnable(GL_DEPTH_TEST);
      if (state = csUpdate) then glDepthFunc(func);
      if (state = csDisable) then glDisable(GL_DEPTH_TEST);
    end;

    case rsDepthWrite of
      csEnable, csUpdate : glDepthMask(TRUE);
      csDisable        : glDepthMask(FALSE);
    end;

    case rsFog of
      csEnable, csUpdate : glEnable(GL_FOG);
      csDisable        : glDisable(GL_FOG);
    end;

{$IFDEF GL_PROGRAM}
    if GL_PROGRAM then
    case rsFragmentProgram of
      csEnable, csUpdate : glEnable(GL_FRAGMENT_PROGRAM_ARB);
      csDisable        : glDisable(GL_FRAGMENT_PROGRAM_ARB);
    end;
{$ENDIF}

    with rsFrontFace do begin
      if change then
        glFrontFace(winding);
    end;

    case rsLighting of
      csEnable, csUpdate : glEnable(GL_LIGHTING);
      csDisable        : glDisable(GL_LIGHTING);
    end;

    case rsLineSmooth of
      csEnable, csUpdate : glEnable(GL_LINE_SMOOTH);
      csDisable        : glDisable(GL_LINE_SMOOTH);
    end;

    with rsLineStipple do begin
      if (state = csEnable) or (state = csUpdate) then glEnable(GL_LINE_STIPPLE);
      if (state = csUpdate) then
        glLineStipple(factor, pattern);
      if (state = csDisable) then glDisable(GL_LINE_STIPPLE);
    end;

    with rsLineWidth do begin
      if change then glLineWidth(width);
    end;

    case rsNormalize of
      csEnable, csUpdate : glEnable(GL_NORMALIZE);
      csDisable        : glDisable(GL_NORMALIZE);
    end;

{$IFDEF GL_14}
    if GL_14 then
    with rsPointParam do begin
      if change then begin
        glPointParameterf(GL_POINT_SIZE_MIN, sizemin);
        glPointParameterf(GL_POINT_SIZE_MAX, sizemax);
        glPointParameterf(GL_POINT_FADE_THRESHOLD_SIZE, fade);
        glPointParameterfv(GL_POINT_DISTANCE_ATTENUATION, Addr(attenuation));
      end;
    end;
{$ENDIF}

    with rsPointSize do begin
      if change then glPointSize(size);
    end;

    case rsPointSmooth of
      csEnable, csUpdate : glEnable(GL_POINT_SMOOTH);
      csDisable        : glDisable(GL_POINT_SMOOTH);
    end;

    with rsPolyMode do begin
      if change then
        if front = back then
          glPolygonMode(GL_FRONT_AND_BACK, front)
        else begin
          glPolygonMode(GL_FRONT, front);
          glPolygonMode(GL_BACK, back);
        end;
    end;

    case rsPolySmooth of
      csEnable, csUpdate : glEnable(GL_POLYGON_SMOOTH);
      csDisable        : glDisable(GL_POLYGON_SMOOTH);
    end;

    with rsPolyStipple do begin
      if (state = csEnable) or (state = csUpdate) then glEnable(GL_POLYGON_STIPPLE);
      if (state = csUpdate) then glPolygonStipple(Addr(mask));
      if (state = csDisable) then glDisable(GL_POLYGON_STIPPLE);
    end;

{$IFDEF GL_12}
    if GL_12 then
    case rsRescaleNormal of
      csEnable, csUpdate : glEnable(GL_RESCALE_NORMAL);
      csDisable        : glDisable(GL_RESCALE_NORMAL);
    end;
{$ENDIF}

{$IFDEF GL_12}
    if GL_12 then
    case rsSeparateColor of
      csEnable, csUpdate : glLightModeli (GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR);
      csDisable        : glLightModeli (GL_LIGHT_MODEL_COLOR_CONTROL, GL_SINGLE_COLOR);
    end;
{$ENDIF}

    case rsSmoothShading of
      csEnable, csUpdate : glShadeModel(GL_SMOOTH);
      csDisable        : glShadeModel(GL_FLAT);
    end;

    with rsStencilTest do begin
      if (state = csEnable) or (state = csUpdate) then glEnable(GL_STENCIL_TEST);
      if (state = csUpdate) then begin
        glStencilFunc(func, ref, mask);
        glStencilOp(opfail, opzfail, opzpass);
      end;
      if (state = csDisable) then glDisable(GL_STENCIL_TEST);
    end;

{$IFDEF GL_13}
    if GL_13 then
    case rsTexturing of
      csEnable, csUpdate: ;
      csDisable: begin
          for i := GL_TEXTURE0 + bgMaxTextureUnits - 1 downto GL_TEXTURE0 do begin
            glActiveTexture(i);
            glDisable(GL_TEXTURE_1D);
            glDisable(GL_TEXTURE_2D);
            glDisable(GL_TEXTURE_3D);
            glDisable(GL_TEXTURE_CUBE_MAP);
          end;
        end;
    end;
{$ENDIF}

    case rsTwoSided of
      csEnable, csUpdate : glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, 1);
      csDisable        : glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, 0);
    end;

{$IFDEF GL_PROGRAM}
    if GL_PROGRAM then
    case rsVertexProgram of
      csEnable, csUpdate : glEnable(GL_VERTEX_PROGRAM_ARB);
      csDisable        : glDisable(GL_VERTEX_PROGRAM_ARB);
    end;
{$ENDIF}

    glEndList;
  end;



procedure TRenderSettings.Display;

  begin
    glCallList(rsList);
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  TextureSettings implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



constructor TTextureSettings.Create;

  begin
    Name := 'TextureSettings';
    sdIdentity := SD_TS;
    tsList := glGenLists(1);
    InitTextureSettings;
  end;



destructor TTextureSettings.Destroy;

  begin
    glDeleteLists(tsList, 1);
    inherited Destroy;
  end;



function TTextureSettings.GetTexActive : TTSTexActive;
  begin
    result := tsTexActive;
  end;

function TTextureSettings.GetTexture1D : TCapState;
  begin
    result := tsTexture1D;
  end;

function TTextureSettings.GetTexture2D : TCapState;
  begin
    result := tsTexture2D;
  end;

function TTextureSettings.GetTexture3D : TCapState;
  begin
    result := tsTexture3D;
  end;

function TTextureSettings.GetTextureCubeMap : TCapState;
  begin
    result := tsTextureCubeMap;
  end;

function TTextureSettings.GetTexEnv : TTSTexEnv;
  begin
    result := tsTexEnv;
  end;

function TTextureSettings.GetTexGenS : TTSTexGen;
  begin
    result := tsTexGenS;
  end;

function TTextureSettings.GetTexGenT : TTSTexGen;
  begin
    result := tsTexGenT;
  end;

function TTextureSettings.GetTexGenR : TTSTexGen;
  begin
    result := tsTexGenR;
  end;

function TTextureSettings.GetTexGenQ : TTSTexGen;
  begin
    result := tsTexGenQ;
  end;

function TTextureSettings.GetTexMatrix : TTSTexMatrix;
  begin
    result := tsTexMatrix;
  end;

function TTextureSettings.GetTexReflect : TCapState;
  begin
    result := tsTexReflect;
  end;



procedure TTextureSettings.SetTexActive(change : boolean; value : GLenum);
  begin
    tsTexActive.change := change;
    tsTexActive.value := value;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexEnv(change : boolean; mode : GLint; r,g,b,a : GLfloat);
  begin
    tsTexEnv.change := change;
    tsTexEnv.mode := mode;
    tsTexEnv.color.r := r;
    tsTexEnv.color.g := g;
    tsTexEnv.color.b := b;
    tsTexEnv.color.a := a;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexEnvCombineMode(CombineRGB, CombineAlpha : GLenum);
  begin
    tsTexEnv.CombineRGB := CombineRGB;
    tsTexEnv.CombineAlpha := CombineAlpha;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexEnvCombine0(Source0RGB, Operand0RGB, Source0Alpha, Operand0Alpha : GLenum);
  begin
    tsTexEnv.Source0RGB := Source0RGB;
    tsTexEnv.Operand0RGB := Operand0RGB;
    tsTexEnv.Source0Alpha := Source0Alpha;
    tsTexEnv.Operand0Alpha := Operand0Alpha;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexEnvCombine1(Source1RGB, Operand1RGB, Source1Alpha, Operand1Alpha : GLenum);
  begin
    tsTexEnv.Source1RGB := Source1RGB;
    tsTexEnv.Operand1RGB := Operand1RGB;
    tsTexEnv.Source1Alpha := Source1Alpha;
    tsTexEnv.Operand1Alpha := Operand1Alpha;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexEnvCombine2(Source2RGB, Operand2RGB, Source2Alpha, Operand2Alpha : GLenum);
  begin
    tsTexEnv.Source2RGB := Source2RGB;
    tsTexEnv.Operand2RGB := Operand2RGB;
    tsTexEnv.Source2Alpha := Source2Alpha;
    tsTexEnv.Operand2Alpha := Operand2Alpha;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexEnvCombineScale(RGBScale, AlphaScale : GLint);
  begin
    tsTexEnv.RGBScale := RGBScale;
    tsTexEnv.AlphaScale := AlphaScale;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexture1D(state : TCapState);
  begin
    tsTexture1D := state;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexture2D(state : TCapState);
  begin
    tsTexture2D := state;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexture3D(state : TCapState);
  begin
    tsTexture3D := state;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTextureCubeMap(state : TCapState);
  begin
    tsTextureCubeMap := state;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexGenS(state : TCapState; map : GLint; c1, c2, c3, c4 : GLfloat);
  begin
    tsTexGenS.state := state;
    tsTexGenS.map := map;
    tsTexGenS.coeff[0] := c1;
    tsTexGenS.coeff[1] := c2;
    tsTexGenS.coeff[2] := c3;
    tsTexGenS.coeff[3] := c4;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexGenT(state : TCapState; map : GLint; c1, c2, c3, c4 : GLfloat);
  begin
    tsTexGenT.state := state;
    tsTexGenT.map := map;
    tsTexGenT.coeff[0] := c1;
    tsTexGenT.coeff[1] := c2;
    tsTexGenT.coeff[2] := c3;
    tsTexGenT.coeff[3] := c4;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexGenR(state : TCapState; map : GLint; c1, c2, c3, c4 : GLfloat);
  begin
    tsTexGenR.state := state;
    tsTexGenR.map := map;
    tsTexGenR.coeff[0] := c1;
    tsTexGenR.coeff[1] := c2;
    tsTexGenR.coeff[2] := c3;
    tsTexGenR.coeff[3] := c4;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexGenQ(state : TCapState; map : GLint; c1, c2, c3, c4 : GLfloat);
  begin
    tsTexGenQ.state := state;
    tsTexGenQ.map := map;
    tsTexGenQ.coeff[0] := c1;
    tsTexGenQ.coeff[1] := c2;
    tsTexGenQ.coeff[2] := c3;
    tsTexGenQ.coeff[3] := c4;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexMatrix(change : boolean; c00,c01,c02,c03, c10,c11,c12,c13, c20,c21,c22,c23, c30,c31,c32,c33 : GLfloat);
  begin
    tsTexMatrix.Change := change;
    tsTexMatrix.Matrix[0,0]:=c00; tsTexMatrix.Matrix[0,1]:=c01; tsTexMatrix.Matrix[0,2]:=c02; tsTexMatrix.Matrix[0,3]:=c03;
    tsTexMatrix.Matrix[1,0]:=c10; tsTexMatrix.Matrix[1,1]:=c11; tsTexMatrix.Matrix[1,2]:=c12; tsTexMatrix.Matrix[1,3]:=c13;
    tsTexMatrix.Matrix[2,0]:=c20; tsTexMatrix.Matrix[2,1]:=c21; tsTexMatrix.Matrix[2,2]:=c22; tsTexMatrix.Matrix[2,3]:=c23;
    tsTexMatrix.Matrix[3,0]:=c30; tsTexMatrix.Matrix[3,1]:=c31; tsTexMatrix.Matrix[3,2]:=c32; tsTexMatrix.Matrix[3,3]:=c33;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexReflect(state : TCapState);
  begin
    tsTexReflect := state;
    if bgAutoCompile then Compile;
  end;



procedure TTextureSettings.SetTexActivev(v : TTSTexActive);
  begin
    tsTexActive := v;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexEnvv(v : TTSTexEnv);
  begin
    tsTexEnv := v;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexGenSv(v : TTSTexGen);
  begin
    tsTexGenS := v;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexGenTv(v : TTSTexGen);
  begin
    tsTexGenT := v;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexGenRv(v : TTSTexGen);
  begin
    tsTexGenR := v;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexGenQv(v : TTSTexGen);
  begin
    tsTexGenQ := v;
    if bgAutoCompile then Compile;
  end;

procedure TTextureSettings.SetTexMatrixv(v : TTSTexMatrix);
  begin
    tsTexMatrix := v;
    if bgAutoCompile then Compile;
  end;



procedure TTextureSettings.InitTextureSettings;

  begin

    with tsTexActive do begin
      change := FALSE;
      {$IFDEF GL_13} value := GL_TEXTURE0; {$ENDIF}
    end;

    tsTexture1D := csIgnore;
    tsTexture2D := csIgnore;
    tsTexture3D := csIgnore;
    tsTextureCubeMap := csIgnore;

    with tsTexEnv do begin
      change := false;
      mode := GL_MODULATE;
      color.r:=0; color.g:=0; color.b:=0; color.a:=0;
{$IFDEF GL_13}
      combinergb := GL_MODULATE;
      combinealpha := GL_MODULATE;
      source0rgb := GL_TEXTURE;
      operand0rgb := GL_SRC_COLOR;
      source0alpha := GL_TEXTURE;
      operand0alpha := GL_SRC_ALPHA;
      source1rgb := GL_PREVIOUS;
      operand1rgb := GL_SRC_COLOR;
      source1alpha := GL_PREVIOUS;
      operand1alpha := GL_SRC_ALPHA;
      source2rgb := GL_CONSTANT;
      operand2rgb := GL_SRC_ALPHA;
      source2alpha := GL_CONSTANT;
      operand2alpha := GL_SRC_ALPHA;
      rgbscale := 1;
      alphascale := 1;
{$ENDIF}
    end;

    with tsTexGenS do begin
      state := csIgnore;
      map := GL_EYE_LINEAR;
      coeff[0]:=1.0; coeff[1]:=0.0; coeff[2]:=0.0; coeff[3]:=0.0;
    end;

    with tsTexGenT do begin
      state := csIgnore;
      map := GL_EYE_LINEAR;
      coeff[0]:=0.0; coeff[1]:=1.0; coeff[2]:=0.0; coeff[3]:=0.0;
    end;

    with tsTexGenR do begin
      state := csIgnore;
      map := GL_EYE_LINEAR;
      coeff[0]:=0.0; coeff[1]:=0.0; coeff[2]:=0.0; coeff[3]:=0.0;
    end;

    with tsTexGenQ do begin
      state := csIgnore;
      map := GL_EYE_LINEAR;
      coeff[0]:=0.0; coeff[1]:=0.0; coeff[2]:=0.0; coeff[3]:=0.0;
    end;

    tsTexReflect := csIgnore;

    with tsTexMatrix do begin
      Change := false;
      Matrix := MatrixIdentity;
    end;

    if bgAutoCompile then Compile;
  end;



procedure TTextureSettings.Compile;

  begin
    glNewList(tsList, GL_COMPILE);

{$IFDEF GL_13}
    if GL_13 then
    with tsTexActive do begin
      if change then
        glActiveTexture(value);
    end;
{$ENDIF}

    case tsTexture1D of
      csEnable, csUpdate : glEnable(GL_TEXTURE_1D);
      csDisable        : glDisable(GL_TEXTURE_1D);
    end;

    case tsTexture2D of
      csEnable, csUpdate : glEnable(GL_TEXTURE_2D);
      csDisable        : glDisable(GL_TEXTURE_2D);
    end;

{$IFDEF GL_12}
    if GL_12 then
    case tsTexture3D of
      csEnable, csUpdate : glEnable(GL_TEXTURE_3D);
      csDisable        : glDisable(GL_TEXTURE_3D);
    end;
{$ENDIF}

{$IFDEF GL_13}
    if GL_13 then
    case tsTextureCubeMap of
      csEnable, csUpdate : glEnable(GL_TEXTURE_CUBE_MAP);
      csDisable        : glDisable(GL_TEXTURE_CUBE_MAP);
    end;
{$ENDIF}

    with tsTexEnv do begin
      if change then begin
        glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, mode); // texture environment mode (replace, modulate, decal, blend, combine)
        glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, Addr(color)); // environment color of texture
{$IFDEF GL_13}
        if mode=GL_COMBINE then begin
          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, combinergb);
          glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA, combinealpha);
          glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, source0rgb);
          glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB, operand0rgb);
          glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA, source0alpha);
          glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA, operand0alpha);
          glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB, source1rgb);
          glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_RGB, operand1rgb);
          glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_ALPHA, source1alpha);
          glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_ALPHA, operand1alpha);
          if combinergb=GL_INTERPOLATE then begin
            glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE2_RGB, source2rgb);
            glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND2_RGB, operand2rgb);
          end;
          if combinealpha=GL_INTERPOLATE then begin
            glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE2_ALPHA, source2alpha);
            glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND2_ALPHA, operand2alpha);
          end;
          glTexEnvi(GL_TEXTURE_ENV, GL_RGB_SCALE, rgbscale);
          glTexEnvi(GL_TEXTURE_ENV, GL_ALPHA_SCALE, alphascale);
        end;
{$ENDIF}
      end;
    end;

    with tsTexGenS do begin
      if (state = csEnable) or (state = csUpdate) then glEnable(GL_TEXTURE_GEN_S);
      if (state = csUpdate) then begin
        glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, map);
        case map of
          GL_OBJECT_LINEAR : glTexGenfv(GL_S, GL_OBJECT_PLANE, Addr(coeff));
          GL_EYE_LINEAR    : glTexGenfv(GL_S, GL_EYE_PLANE, Addr(coeff));
        end;
      end;
      if (state = csDisable) then glDisable(GL_TEXTURE_GEN_S);
    end;

    with tsTexGenT do begin
      if (state = csEnable) or (state = csUpdate) then glEnable(GL_TEXTURE_GEN_T);
      if (state = csUpdate) then begin
        glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, map);
        case map of
          GL_OBJECT_LINEAR : glTexGenfv(GL_T, GL_OBJECT_PLANE, Addr(coeff));
          GL_EYE_LINEAR    : glTexGenfv(GL_T, GL_EYE_PLANE, Addr(coeff));
        end;
      end;
      if (state = csDisable) then glDisable(GL_TEXTURE_GEN_T);
    end;

    with tsTexGenR do begin
      if (state = csEnable) or (state = csUpdate) then glEnable(GL_TEXTURE_GEN_R);
      if (state = csUpdate) then begin
        glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, map);
        case map of
          GL_OBJECT_LINEAR : glTexGenfv(GL_R, GL_OBJECT_PLANE, Addr(coeff));
          GL_EYE_LINEAR    : glTexGenfv(GL_R, GL_EYE_PLANE, Addr(coeff));
        end;
      end;
      if (state = csDisable) then glDisable(GL_TEXTURE_GEN_R);
    end;

    with tsTexGenQ do begin
      if (state = csEnable) or (state = csUpdate) then glEnable(GL_TEXTURE_GEN_Q);
      if (state = csUpdate) then begin
        glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, map);
        case map of
          GL_OBJECT_LINEAR : glTexGenfv(GL_Q, GL_OBJECT_PLANE, Addr(coeff));
          GL_EYE_LINEAR    : glTexGenfv(GL_Q, GL_EYE_PLANE, Addr(coeff));
        end;
      end;
      if (state = csDisable) then glDisable(GL_TEXTURE_GEN_Q);
    end;

    case tsTexReflect of
      csEnable, csUpdate : glCallList(bgViewInvertedTextureMatrixList);
      csDisable          : begin
                             glMatrixMode(GL_TEXTURE);
                             glLoadIdentity;
                             glMatrixMode(GL_MODELVIEW);
                           end;
    end;

    with tsTexMatrix do begin
      if Change then begin
        glMatrixMode(GL_TEXTURE);
        if tsTexReflect = csIgnore then
          glLoadMatrixf(Addr(Matrix))
        else
          glMultMatrixf(Addr(Matrix));
        glMatrixMode(GL_MODELVIEW);
      end;
    end;

    glEndList;
  end;



procedure TTextureSettings.Display;

  begin
    glCallList(tsList);
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  AniMesh implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



// internal description :
//    frames, frame hold the frame vertices
//    framenorm holds frame normals, if it is zero, norm must be zero too
//    the currently displayed vertices are in objMesh.meshVert
//    the triangles are in objMesh.meshTri
//    texture positions are in TexCoord, if there are none texunits must be zero
//    minimum, maximum, bounding sphere center and radius for every frame are stored in
//       framemin, framemax, framecent, framebsrad
constructor TAniMesh.Create;

  var
    i : integer;

  begin
    inherited Create;
    objMesh := TMesh.Create(@objMatrix);
    amFrames := 0;
    amFrame := NIL;
    amFrameNorm := NIL;
    amFrameMin := NIL;
    amFrameMax := NIL;
    amFrameCent := NIL;
    amFrameBSRad := NIL;
    amFrameAnimation := NIL;
    amNorm := NIL;
    amTexUnits := 0;
    for i := 0 to 31 do
      amTexUnit[i] := $84C0 + i;  // GL_TEXTURE0
    FillChar(amTexCoord, sizeof(amTexCoord), 0);
    amColor := NIL;
    amAlphaChannel := false;
    amTexComponents := 2;
    amStaticData := false;
    amTriangleLists := 0;
    amTriangleList := NIL;
  end;



// lists ...... number of triangle lists
// frames ..... number of frames
// verts ...... number of vertices
// tris ....... number of triangles
// normals .... there are frame normals
constructor TAniMesh.CreateByData(lists,frames,verts,tris: integer; normals: boolean);

  var
    i : integer;

  begin
    inherited Create;
    objMesh := TMesh.Create(@objMatrix);
    objMesh.CreateVertices(verts);
    amFrames := frames;
    GetMem(amFrame, amFrames * SizeOf(PVecs));
    FillChar(amFrame^, amFrames * SizeOf(PVecs), 0);
    objMesh.meshTris := tris;
    objMesh.meshTri := NIL;
    objMesh.meshStaticTri := true;
    GetMem(objMesh.meshTriMin, tris * SizeOf(TVec));
    GetMem(objMesh.meshTriMax, tris * SizeOf(TVec));
    if normals then begin
      GetMem(amNorm, verts * SizeOf(TVec));
      GetMem(amFrameNorm, amFrames * SizeOf(PVecs));
      FillChar(amFrameNorm^, amFrames * SizeOf(PVecs), 0);
    end
    else begin
      amNorm := NIL;
      amFrameNorm := NIL;
    end;

    for i := 0 to 31 do
      amTexUnit[i] := $84C0 + i;  // GL_TEXTURE0
    FillChar(amTexCoord, sizeof(amTexCoord), 0);
    amColor := NIL;

    amTriangleLists := lists;
    GetMem(amTriangleList, lists * SizeOf(TTriangleList));
    FillChar(amTriangleList^, lists * SizeOf(TTriangleList), 0);
    amStaticData := true;

    amFrameMin := NIL;
    amFrameMax := NIL;
    amFrameCent := NIL;
    amFrameBSRad := NIL;

    amTexUnits := 0;
    amAlphaChannel := false;
    amTexComponents := 2;
  end;



constructor TAniMesh.CreateByFile(FileName : String);

  var
    F: File;
    Hdr: TBFHeader;
    i: integer;
    a: integer;
    fa: PFrameAnimation;
    tl: PTriangleList;

  begin
    inherited Create;
    SetJointAnimByFile(FileName);
    Assign(F, FileName);
    Reset(F, 1);
    if IOResult<>0 then exit;
    BlockRead(F, Hdr, 64);

    objMesh := TMesh.Create(@objMatrix);
    BlockRead(F, objMesh.meshVerts, 4);
    GetMem(objMesh.meshVert, objMesh.meshVerts * sizeof(TVec));
    BlockRead(F, objMesh.meshTris, 4);
    GetMem(objMesh.meshTri, objMesh.meshTris * sizeof(TTri));
    BlockRead(F, objMesh.meshTri^, objMesh.meshTris * 12);
    GetMem(objMesh.meshTransformed, objMesh.meshVerts * SizeOf(TVec));
    GetMem(objMesh.meshTriMin, objMesh.meshTris * SizeOf(TVec));
    GetMem(objMesh.meshTriMax, objMesh.meshTris * SizeOf(TVec));

    amTriangleList := NIL;
    BlockRead(F, amTriangleLists, 4);
    GetMem(amTriangleList, amTriangleLists * sizeof(TTriangleList));
    FillChar(amTriangleList^, amTriangleLists * sizeof(TTriangleList), 0);
    for i:=0 to amTriangleLists-1 do begin
      tl := @amTriangleList[i];
      BlockRead(F, tl.First, 4);
      BlockRead(F, tl.Last, 4);
    end;

    BlockRead(F, amFrames, 4);
    GetMem(amFrame, amFrames * sizeof(PVecs));
    amFrameNorm := NIL;
    amNorm := NIL;
    BlockRead(F, i, 4);
    if i<>0 then begin
      GetMem(amFrameNorm, amFrames * sizeof(PVecs));
      GetMem(amNorm, objMesh.meshVerts * 12);
    end;
    for i:=0 to amFrames-1 do begin
      GetMem(amFrame[i], objMesh.meshVerts * 12);
      BlockRead(F, amFrame[i]^, objMesh.meshVerts * 12);
    end;
    if amFrameNorm<>NIL then begin
      for i:=0 to amFrames-1 do begin
        GetMem(amFrameNorm[i], objMesh.meshVerts * 12);
        BlockRead(F, amFrameNorm[i]^, objMesh.meshVerts * 12);
      end;
    end;

    GetMem(amFrameMin, amFrames * 12);
    BlockRead(F, amFrameMin^, amFrames * 12);
    GetMem(amFrameMax, amFrames * 12);
    BlockRead(F, amFrameMax^, amFrames * 12);
    GetMem(amFrameCent, amFrames * 12);
    BlockRead(F, amFrameCent^, amFrames * 12);
    GetMem(amFrameBSRad, amFrames * 4);
    BlockRead(F, amFrameBSRad^, amFrames * 4);

    GetMem(amFrameAnimation, objAnimations * sizeof(TFrameAnimation));
    FillChar(amFrameAnimation^, objAnimations * sizeof(TFrameAnimation), 0);
    for a:=0 to objAnimations-1 do begin
      fa := @amFrameAnimation[a];
      BlockRead(F, fa.Frames, 4);
      GetMem(fa.Frame, fa.Frames * sizeof(TKeyFrame));
      BlockRead(F, fa.Frame^, fa.Frames * 8);
      BlockRead(F, fa.Loop, 1);
    end;

    BlockRead(F, amTexUnits, 4);
    BlockRead(F, amTexUnit, amTexUnits * 4);
    FillChar(amTexCoord, sizeof(amTexCoord), 0);
    for i:=0 to amTexUnits-1 do begin
      GetMem(amTexCoord[i], objMesh.meshVerts * 16);
      BlockRead(F, amTexCoord[i]^, objMesh.meshVerts * 16);
    end;

    amColor := NIL;
    BlockRead(F, i, 4);
    if i<>0 then begin
      GetMem(amColor, objMesh.meshVerts * 16);
      BlockRead(F, amColor^, objMesh.meshVerts * 16);
    end;

    amTriangleLists := 0;
    amTriangleList := NIL;
    amAlphaChannel := false;
    BlockRead(F, amAlphaChannel, 1);
    BlockRead(F, amTexComponents, 4);
    Close(F);

    SetupAnimations;
    amStaticData := false;
  end;



// pv ... pointer to vertex table
procedure TAniMesh.SetFrameVerts(f: integer; pv: PVecs);

  begin
    amFrame[f] := pv;
  end;



// pn ... pointer to normal table
procedure TAniMesh.SetFrameNorms(f: integer; pn: PVecs);

  begin
    amFrameNorm[f] := pn;
  end;



// pt ... pointer to triangle table
procedure TAniMesh.SetTriangles(pt: PTris);

  begin
    objMesh.meshTri := pt;
  end;



// set frame animations
procedure TAniMesh.SetFrameAnimations(FrameAnimations: PFrameAnimations);

  begin
    amFrameAnimation := FrameAnimations;
  end;


// set number of texture components to read
procedure TAniMesh.SetTexComponents(texcomponents: integer);

  begin
    amTexComponents := texcomponents;
  end;



// use alpha channel when displaying vertex colors
procedure TAniMesh.SetAlphaChannel(alphachannel: boolean);

  begin
    amAlphaChannel := alphachannel;
  end;



// texunit ... texture unit to set, pvt ... pointer to vertex texture position table
procedure TAniMesh.SetTexCoord(texunit: integer; pvt: PTexCoords4f);

  begin
    amTexCoord[texunit] := pvt;
  end;



// pvc ... pointer to vertex color table
procedure TAniMesh.SetColor(pvc: PColors4f);

  begin
    amColor := pvc;
  end;



// texunit: index of texture coordinate set, value: texture unit value
procedure TAniMesh.SetTexUnit(texunit, value: integer);

  begin
    amTexUnit[texunit] := value;
    if texunit>=amTexUnits-1 then amTexUnits := texunit + 1;
  end;



// pmn .. pointer to minimum coordinates table (one coordinate for every frame)
procedure TAniMesh.SetFrameMinCoords(pmn: PVecs);

  begin
    amFrameMin := pmn;
  end;



// pmx .. pointer to maximum coordinates table (one coordinate for every frame)
procedure TAniMesh.SetFrameMaxCoords(pmx: PVecs);

  begin
    amFrameMax := pmx;
  end;



// pct .. pointer to bounding sphere center table (one bounding sphere for every frame)
procedure TAniMesh.SetBoundingSphereCenters(pct: PVecs);

  begin
    amFrameCent := pct;
  end;



// pbs .. pointer to bounding sphere radius table (one bounding sphere for every frame)
procedure TAniMesh.SetBoundingSphereRadii(pbs: PFloatTable);

  begin
    amFrameBSRad := pbs;
  end;



destructor TAniMesh.Destroy;

  var
    i : integer;

  begin
    if not amStaticData then begin
      for i := 0 to amFrames-1 do begin
        FreeMem(amFrame[i]);
        if amFrameNorm<>NIL then FreeMem(amFrameNorm[i]);
      end;
      if amColor<>NIL then FreeMem(amColor);
      for i := 0 to amTexUnits-1 do begin
        if amTexCoord[i]<>NIL then FreeMem(amTexCoord[i]);
      end;
      FreeMem(amFrameMin);
      FreeMem(amFrameMax);
      FreeMem(amFrameCent);
      FreeMem(amFrameBSRad);
      for i:=0 to objAnimations-1 do
        if amFrameAnimation[i].Frame<>NIL then FreeMem(amFrameAnimation[i].Frame);
      if amFrameAnimation<>NIL then FreeMem(amFrameAnimation);
    end;
    if amTriangleLists>0 then FreeMem(amTriangleList);
    FreeMem(amFrame);
    if amFrameNorm<>nil then FreeMem(amFrameNorm);
    if amNorm<>nil then FreeMem(amNorm);
    inherited Destroy;
  end;



procedure TAniMesh.CreateAnimations(number: integer);

  begin
    inherited CreateAnimations(number);
    GetMem(amFrameAnimation, number * sizeof(TFrameAnimation));
    FillChar(amFrameAnimation^, number * sizeof(TFrameAnimation), 0);
  end;



procedure TAniMesh.CreateFrameAnimationData(AnimationIndex, Frames: integer; Loop: boolean);

  var
    ani: PFrameAnimation;

  begin
    ani := @amFrameAnimation[AnimationIndex];
    ani.Frames := Frames;
    GetMem(ani.Frame, Frames * sizeof(TKeyFrame));
    FillChar(ani.Frame^, Frames * sizeof(TKeyFrame), 0);
    ani.Loop := Loop;
  end;



procedure TAniMesh.SetAnimationFrame(AnimationIndex, FrameIndex, Key, Time: integer);

  var
    KeyFrame: PKeyFrame;

  begin
    KeyFrame := @amFrameAnimation[AnimationIndex].Frame[FrameIndex];
    KeyFrame.Key := Key;
    KeyFrame.Time := Time;
  end;



procedure TAniMesh.SetupAnimations;

  var
    a: integer;
    ani: PFrameAnimation;

  begin
    inherited SetupAnimations;
    for a:=0 to objAnimations-1 do begin
      ani := @amFrameAnimation[a];
      ani.TotalTime := ani.Frame[ani.Frames-1].Time;
    end;
  end;



procedure TAniMesh.Animate(AnimationIndex, TimeDifference: integer);

  var
    frame: integer;
    f, n, fn, nn : pVecs;
    j : integer;
    v, v1, v2, n1, n2 : PVec;
    interp: float;
    ani: PFrameAnimation;

  begin
    if AnimationIndex >= objAnimations then begin
      if amFrames>0 then begin
        System.Move(amFrame[0]^, objMesh.meshVert^, objMesh.meshVerts * sizeof(TVec));
        if amFrameNorm<>NIL then
          System.Move(amFrameNorm[0]^, amNorm^, objMesh.meshVerts * sizeof(TVec));
      end;
      exit;
    end;
    inherited Animate(AnimationIndex, TimeDifference);

    ani := @amFrameAnimation[AnimationIndex];
    inc(ani.CurrentTime, TimeDifference);
    if ani.CurrentTime > ani.TotalTime then begin
      if addr(ani.OnEndAnimation)<>NIL then
        ani.OnEndAnimation(self, AnimationIndex);
      if ani.Loop then begin
        while ani.CurrentTime > ani.TotalTime do
          dec(ani.CurrentTime, ani.TotalTime);
        amCurrentFrame := ani.Frame[0].Key;
      end
      else begin
        ani.CurrentTime := ani.TotalTime;
        amCurrentFrame := ani.Frame[ani.Frames-1].Key;
      end;
    end // CurrentTime > TotalTime
    else begin
      if addr(ani.OnRunAnimation)<>NIL then
        ani.OnRunAnimation(self, AnimationIndex);
    end;

    // find animation frame
    frame := 0;
    while (frame<ani.Frames-1) and (ani.Frame[frame].Time<ani.CurrentTime) do
      inc(frame);
    amCurrentFrame := ani.Frame[frame].Key;

    if (frame=0) then begin
      if ani.CurrentTime < 0 then begin
        // animation comes from other animation cycle
        interp := - TimeDifference / (ani.CurrentTime - TimeDifference);
      end
      else begin
        amPrevFrame := amCurrentFrame;
        interp := 0.0;
      end;
    end
    else begin
      amPrevFrame := ani.Frame[frame-1].Key;
      interp := (ani.CurrentTime - ani.Frame[frame-1].Time) / (ani.Frame[frame].Time - ani.Frame[frame-1].Time);
    end;

    if (amPrevFrame=amCurrentFrame) or (interp=0.0) then begin
      // copy current frame into objMesh, since current and next frame are the same
      f := amFrame[amCurrentFrame];
      if amFrameNorm=NIL then
        System.Move(f^, objMesh.meshVert^, objMesh.meshVerts * SizeOf(TVec))
      else begin
        fn := amFrameNorm[amCurrentFrame];
        System.Move(f^, objMesh.meshVert^, objMesh.meshVerts * SizeOf(TVec));
        System.Move(fn^, amNorm^, objMesh.meshVerts * SizeOf(TVec));
      end;
    end
    else begin
      // interpolate between current and next frame
      f := amFrame[amPrevFrame];
      n := amFrame[amCurrentFrame];
      if amFrameNorm=NIL then
        for j := 0 to objMesh.meshVerts-1 do begin
          v1 := @f[j];
          v2 := @n[j];
          v := @objMesh.meshVert[j];
          v.x := v1.x+interp*(v2.x-v1.x);
          v.y := v1.y+interp*(v2.y-v1.y);
          v.z := v1.z+interp*(v2.z-v1.z);
        end
      else begin
        fn := amFrameNorm[amPrevFrame];
        nn := amFrameNorm[amCurrentFrame];
        for j := 0 to objMesh.meshVerts-1 do begin
          v1 := @f[j];
          v2 := @n[j];
          n1 := @fn[j];
          n2 := @nn[j];
          v := @objMesh.meshVert[j];
          v.x := v1.x+interp*(v2.x-v1.x);
          v.y := v1.y+interp*(v2.y-v1.y);
          v.z := v1.z+interp*(v2.z-v1.z);
          v := @amNorm[j];
          v.x := n1.x+interp*(n2.x-n1.x);
          v.y := n1.y+interp*(n2.y-n1.y);
          v.z := n1.z+interp*(n2.z-n1.z);
        end;
      end;
    end;

    ComputeMinMax;
    if objShadowState.State<>NoShadowing then objMesh.ComputeShadowPlaneEq;
  end;



procedure TAniMesh.ResetAnimation(AnimationIndex: integer);

  begin
    inherited ResetAnimation(AnimationIndex);
    amFrameAnimation[AnimationIndex].CurrentTime := 0;
  end;



procedure TAniMesh.ReplaceAnimation(AnimationIndex, NewAnimationIndex, TimeGap: integer);

  begin
    inherited ReplaceAnimation(AnimationIndex, NewAnimationIndex, TimeGap);
    amFrameAnimation[NewAnimationIndex].CurrentTime := -TimeGap;
  end;



function TAniMesh.GetTriListNum : integer;

  begin
    result := amTriangleLists;
  end;



// add next triangle list to triangle list table
procedure TAniMesh.SetTriList(index: integer; first, last: integer);

  begin
    amTriangleList[index].Shaders := 0;
    FillChar(amTriangleList[index].Shader, SizeOf(TShaderTable), 0);
    amTriangleList[index].first := first;
    amTriangleList[index].last := last;
  end;



// adds a shader to the last added triangle list
procedure TAniMesh.AddShader(TriListIndex: integer; Shader: TShader);

  begin
    amTriangleList[TriListIndex].Shader[amTriangleList[TriListIndex].Shaders] := Shader;
    inc(amTriangleList[TriListIndex].Shaders);
  end;



// returns triangle list by index
function TAniMesh.TriList(TriListIndex: integer) : PTriangleList;

  begin
    if TriListIndex<amTriangleLists then
      TriList := @amTriangleList[TriListIndex]
    else
      TriList := NIL;
  end;



procedure TAniMesh.ComputeMinMax;

  var
    v1, v2 : TVec;

  begin
    v1 := amFrameMin[amPrevFrame];
    v2 := amFrameMin[amCurrentFrame];
    if v1.x<v2.x then objMin.x:=v1.x else objMin.x:=v2.x;
    if v1.y<v2.y then objMin.y:=v1.y else objMin.y:=v2.y;
    if v1.z<v2.z then objMin.z:=v1.z else objMin.z:=v2.z;

    v1 := amFrameMax[amPrevFrame];
    v2 := amFrameMax[amCurrentFrame];
    if v1.x>v2.x then objMax.x:=v1.x else objMax.x:=v2.x;
    if v1.y>v2.y then objMax.y:=v1.y else objMax.y:=v2.y;
    if v1.z>v2.z then objMax.z:=v1.z else objMax.z:=v2.z;

    v1 := amFrameCent[amPrevFrame];
    v2 := amFrameCent[amCurrentFrame];
    objCent.x := (v1.x + v2.x) / 2;
    objCent.y := (v1.y + v2.y) / 2;
    objCent.z := (v1.z + v2.z) / 2;

    objBSRad := amFrameBSRad[amPrevFrame];
    if amFrameBSRad[amCurrentFrame]>objBSRad then objBSRad := amFrameBSRad[amCurrentFrame];
    objBSRad := objBSRad + sqrt(sqr(v1.x-objCent.x) + sqr(v1.y-objCent.y) + sqr(v1.z-objCent.z));
  end;



procedure TAniMesh.DrawObj;

  var
    i,j : integer;
    TriList : PTriangleList;
    f,l : integer;

  begin
    if objMesh.meshVert<>NIL then begin
      glEnableClientState(GL_VERTEX_ARRAY);
      glVertexPointer(3, GL_FLOAT, 0, objMesh.meshVert);
    end;

    if amNorm<>NIL then begin
      glEnableClientState(GL_NORMAL_ARRAY);
      glNormalPointer(GL_FLOAT, 0, amNorm);
    end;

    if amColor<>NIL then begin
      glEnableClientState(GL_COLOR_ARRAY);
      if amAlphaChannel then
        glColorPointer(4, GL_FLOAT, 0, amColor)
      else
        glColorPointer(3, GL_FLOAT, SizeOf(TColor4f), amColor);
    end;

    if amTexCoord[0]<>NIL then begin
      for i := 0 to amTexUnits-1 do begin
{$IFDEF GL_13}
        if GL_13 then glClientActiveTexture(amTexUnit[i]);
{$ENDIF}
        if amTexCoord[i]<>NIL then begin
          glEnableClientState(GL_TEXTURE_COORD_ARRAY);
          if amTexComponents=4 then
            glTexCoordPointer(4, GL_FLOAT, 0, amTexCoord[i])
          else
            glTexCoordPointer(amTexComponents, GL_FLOAT, SizeOf(TTexCoord4f), amTexCoord[i])
        end
        else
          glDisableClientState(GL_TEXTURE_COORD_ARRAY);
      end;
{$IFDEF GL_13}
      glClientActiveTexture(GL_TEXTURE0);
{$ENDIF}
    end;

    for i := 0 to amTriangleLists-1 do begin
      TriList := @amTriangleList[i];
      for j := 0 to TriList.Shaders-1 do
        TriList.Shader[j].Display;
      f := TriList.first;
      l := TriList.last;
      if not(l-f<0) then
        glDrawElements(GL_TRIANGLES, (l-f+1)*3, GL_UNSIGNED_INT, @objMesh.meshTri[f]);
    end;

    if objMesh.meshVert<>NIL then
      glDisableClientState(GL_VERTEX_ARRAY);
    if amNorm<>NIL then
      glDisableClientState(GL_NORMAL_ARRAY);
    if amColor<>NIL then
      glDisableClientState(GL_COLOR_ARRAY);
    if amTexUnits>0 then begin
      for i := 0 to amTexUnits-1 do begin
{$IFDEF GL_13}
        if GL_13 then glClientActiveTexture(amTexUnit[i]);
{$ENDIF}
        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
      end;
{$IFDEF GL_13}
      if GL_13 and (amTexUnit[amTexUnits-1]<>GL_TEXTURE0) then glClientActiveTexture(GL_TEXTURE0);
{$ENDIF}
    end;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  SkinMesh implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



procedure TSkinMesh.Animate(AnimationIndex, TimeDifference: integer);

  var
    i: integer;
    v: TVec;
    sv: PSkinVert;
    mv: PVec;

  begin
    inherited Animate(AnimationIndex, TimeDifference);
    if smNorm=NIL then begin
      // no normal vectors have to be computed
      if objMesh.meshVerts>0 then begin
        sv := @smVert[0];
        mv := @objMesh.meshVert[0];
        if sv.JointIndex>=0 then
          VectorTransformTo(sv.Pos, mv^, objJoint[sv.JointIndex].FinalTransformation)
        else
          mv^ := sv.Pos;
        objMin := mv^;
        objMax := mv^;
      end;
      for i:=1 to objMesh.meshVerts-1 do begin
        sv := @smVert[i];
        mv := @objMesh.meshVert[i];
        if smVert[i].JointIndex>=0 then
          VectorTransformTo(sv.Pos, mv^, objJoint[sv.JointIndex].FinalTransformation)
        else
          mv^ := sv.Pos;
        if mv.x<objMin.x then objMin.x:=mv.x else
        if mv.x>objMax.x then objMax.x:=mv.x;
        if mv.y<objMin.y then objMin.y:=mv.y else
        if mv.y>objMax.y then objMax.y:=mv.y;
        if mv.z<objMin.z then objMin.z:=mv.z else
        if mv.z>objMax.z then objMax.z:=mv.z;
      end;
    end
    else begin
      // compute with normal vectors
      if objMesh.meshVerts>0 then begin
        sv := @smVert[0];
        mv := @objMesh.meshVert[0];
        if sv.JointIndex>=0 then begin
          VectorTransformTo(sv.Pos, mv^, objJoint[sv.JointIndex].FinalTransformation);
          VectorTransformTo3(sv.Norm, smNorm[0], objJoint[sv.JointIndex].FinalTransformation);
        end
        else begin
          mv^ := sv.Pos;
          smNorm[0] := sv.Norm;
        end;
        objMin := mv^;
        objMax := mv^;
      end;
      for i:=1 to objMesh.meshVerts-1 do begin
        sv := @smVert[i];
        mv := @objMesh.meshVert[i];
        if smVert[i].JointIndex>=0 then begin
          VectorTransformTo(sv.Pos, mv^, objJoint[sv.JointIndex].FinalTransformation);
          VectorTransformTo3(sv.Norm, smNorm[i], objJoint[sv.JointIndex].FinalTransformation);
        end
        else
          mv^ := sv.Pos;
        if mv.x<objMin.x then objMin.x:=mv.x else
        if mv.x>objMax.x then objMax.x:=mv.x;
        if mv.y<objMin.y then objMin.y:=mv.y else
        if mv.y>objMax.y then objMax.y:=mv.y;
        if mv.z<objMin.z then objMin.z:=mv.z else
        if mv.z>objMax.z then objMax.z:=mv.z;
      end;
    end;
    objCent.x := (objMin.x + objMax.x) * 0.5;
    objCent.y := (objMin.y + objMax.y) * 0.5;
    objCent.z := (objMin.z + objMax.z) * 0.5;
    v.x := objCent.x - objMin.x;
    v.y := objCent.y - objMin.y;
    v.z := objCent.z - objMin.z;
    objBSRad := sqrt(v.x*v.x + v.y*v.y + v.z*v.z);
    if (objShadowState.State<>NoShadowing) then objMesh.ComputeShadowPlaneEq;
  end;



constructor TSkinMesh.Create;

  var
    i: integer;

  begin
    inherited Create;
    smVert := NIL;
    smNorm := NIL;
    smColor := NIL;
    smTexUnits := 0;
    for i := 0 to 31 do
      smTexUnit[i] := $84C0 + i;  // GL_TEXTURE0
    FillChar(smTexCoord, sizeof(smTexCoord), 0);
    smTriangleLists := 0;
    smTriangleList := NIL;
    smAlphaChannel := false;
    smTexComponents := 2;
    smStaticData := false;
  end;



constructor TSkinMesh.CreateByData(lists, verts, tris: integer; normals: boolean);

  var
    i: integer;

  begin
    inherited Create;
    objMesh := TMesh.Create(@objMatrix);
    objMesh.CreateVertices(verts);
    objMesh.meshTris := tris;
    objMesh.meshTri := NIL;
    objMesh.meshStaticTri := true;
    GetMem(smVert, verts * sizeof(TSkinVert));
    FillChar(smVert^, verts * sizeof(TSkinVert), 0);
    for i := 0 to verts-1 do begin
      smVert[i].JointIndex := -1;
    end;

    if normals then
      GetMem(smNorm, verts * sizeof(TVec))
    else
      smNorm := NIL;

    for i := 0 to 31 do
      smTexUnit[i] := $84C0 + i;  // GL_TEXTURE0
    FillChar(smTexCoord, sizeof(smTexCoord), 0);
    smColor := NIL;

    smTriangleLists := lists;
    GetMem(smTriangleList, lists * SizeOf(TTriangleList));
    FillChar(smTriangleList^, lists * SizeOf(TTriangleList), 0);
    smStaticData := true;

    smTexUnits := 0;
    smAlphaChannel := false;
    smTexComponents := 2;
  end;



constructor TSkinMesh.CreateByFile(FileName: String);

  var
    F: File;
    Hdr: TBFHeader;
    i: integer;
    tl: PTriangleList;

  begin
    inherited Create;
    SetJointAnimByFile(FileName);
    Assign(F, FileName);
    Reset(F, 1);
    if IOResult<>0 then exit;
    BlockRead(F, Hdr, 64);

    objMesh := TMesh.Create(@objMatrix);
    BlockRead(F, objMesh.meshVerts, 4);
    GetMem(objMesh.meshVert, objMesh.meshVerts * sizeof(TVec));
    BlockRead(F, objMesh.meshTris, 4);
    GetMem(objMesh.meshTri, objMesh.meshTris * sizeof(TTri));
    BlockRead(F, objMesh.meshTri^, objMesh.meshTris * 12);
    GetMem(objMesh.meshTransformed, objMesh.meshVerts * SizeOf(TVec));
    GetMem(objMesh.meshTriMin, objMesh.meshTris * SizeOf(TVec));
    GetMem(objMesh.meshTriMax, objMesh.meshTris * SizeOf(TVec));

    smTriangleList := NIL;
    BlockRead(F, smTriangleLists, 4);
    GetMem(smTriangleList, smTriangleLists * sizeof(TTriangleList));
    FillChar(smTriangleList^, smTriangleLists * sizeof(TTriangleList), 0);
    for i:=0 to smTriangleLists-1 do begin
      tl := @smTriangleList[i];
      BlockRead(F, tl.First, 4);
      BlockRead(F, tl.Last, 4);
    end;

    GetMem(smVert, objMesh.meshVerts * sizeof(TSkinVert));
    BlockRead(F, smVert^, objMesh.meshVerts * 28);
    smNorm := NIL;
    BlockRead(F, i, 4);
    if i<>0 then
      GetMem(smNorm, objMesh.meshVerts * sizeof(TVec));

    BlockRead(F, smTexUnits, 4);
    BlockRead(F, smTexUnit, smTexUnits * 4);
    FillChar(smTexCoord, sizeof(smTexCoord), 0);
    for i:=0 to smTexUnits-1 do begin
      GetMem(smTexCoord[i], objMesh.meshVerts * 16);
      BlockRead(F, smTexCoord[i]^, objMesh.meshVerts * 16);
    end;

    smColor := NIL;
    BlockRead(F, i, 4);
    if i<>0 then begin
      GetMem(smColor, objMesh.meshVerts * 16);
      BlockRead(F, smColor^, objMesh.meshVerts * 16);
    end;

    smAlphaChannel := false;
    BlockRead(F, smAlphaChannel, 1);
    BlockRead(F, smTexComponents, 4);
    Close(F);

    smStaticData := true; // do not let SetupAnimations transform vertex data
    SetupAnimations;
    smStaticData := false;
  end;


destructor TSkinMesh.Destroy;

  var
    i: integer;

  begin
    if not smStaticData then begin
      if smVert<>NIL then FreeMem(smVert);
      if smColor<>NIL then FreeMem(smColor);
      for i:=0 to smTexUnits-1 do
        if smTexCoord[i]<>NIL then FreeMem(smTexCoord[i]);
    end;
    if smNorm<>NIL then FreeMem(smNorm);
    if smTriangleLists>0 then FreeMem(smTriangleList);
    inherited Destroy;
  end;



procedure TSkinMesh.DrawObj;

  var
    i,j : integer;
    TriList : PTriangleList;
    f,l : integer;

  begin
    if objMesh.meshVert<>NIL then begin
      glEnableClientState(GL_VERTEX_ARRAY);
      glVertexPointer(3, GL_FLOAT, 0, objMesh.meshVert);
    end;

    if smNorm<>NIL then begin
      glEnableClientState(GL_NORMAL_ARRAY);
      glNormalPointer(GL_FLOAT, 0, smNorm);
    end;

    if smColor<>NIL then begin
      glEnableClientState(GL_COLOR_ARRAY);
      if smAlphaChannel then
        glColorPointer(4, GL_FLOAT, 0, smColor)
      else
        glColorPointer(3, GL_FLOAT, SizeOf(TColor4f), smColor);
    end;

    if smTexCoord[0]<>NIL then begin
      for i := 0 to smTexUnits-1 do begin
{$IFDEF GL_13}
        if GL_13 then glClientActiveTexture(smTexUnit[i]);
{$ENDIF}
        if smTexCoord[i]<>NIL then begin
          glEnableClientState(GL_TEXTURE_COORD_ARRAY);
          if smTexComponents=4 then
            glTexCoordPointer(4, GL_FLOAT, 0, smTexCoord[i])
          else
            glTexCoordPointer(smTexComponents, GL_FLOAT, SizeOf(TTexCoord4f), smTexCoord[i])
        end
        else
          glDisableClientState(GL_TEXTURE_COORD_ARRAY);
      end;
{$IFDEF GL_13}
      glClientActiveTexture(GL_TEXTURE0);
{$ENDIF}
    end;

    for i := 0 to smTriangleLists-1 do begin
      TriList := @smTriangleList[i];
      for j := 0 to TriList.Shaders-1 do
        TriList.Shader[j].Display;
      f := TriList.first;
      l := TriList.last;
      if not(l-f<0) then
        glDrawElements(GL_TRIANGLES, (l-f+1)*3, GL_UNSIGNED_INT, @objMesh.meshTri[f]);
    end;

    if objMesh.meshVert<>NIL then
      glDisableClientState(GL_VERTEX_ARRAY);
    if smNorm<>NIL then
      glDisableClientState(GL_NORMAL_ARRAY);
    if smColor<>NIL then
      glDisableClientState(GL_COLOR_ARRAY);
    if smTexUnits>0 then begin
      for i := 0 to smTexUnits-1 do begin
{$IFDEF GL_13}
        if GL_13 then glClientActiveTexture(smTexUnit[i]);
{$ENDIF}
        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
      end;
{$IFDEF GL_13}
      if GL_13 and (smTexUnit[smTexUnits-1]<>GL_TEXTURE0) then glClientActiveTexture(GL_TEXTURE0);
{$ENDIF}
    end;
  end;



function TSkinMesh.GetTriListNum: integer;

  begin
    result := smTriangleLists;
  end;



// adds a shader to the last added triangle list
procedure TSkinMesh.AddShader(TriListIndex: integer; Shader: TShader);

  begin
    smTriangleList[TriListIndex].Shader[smTriangleList[TriListIndex].Shaders] := Shader;
    inc(smTriangleList[TriListIndex].Shaders);
  end;



// add next triangle list to triangle list table
procedure TSkinMesh.SetTriList(index, first, last: integer);

  begin
    smTriangleList[index].Shaders := 0;
    FillChar(smTriangleList[index].Shader, SizeOf(TShaderTable), 0);
    smTriangleList[index].first := first;
    smTriangleList[index].last := last;
  end;



// use alpha channel when displaying vertex colors
procedure TSkinMesh.SetAlphaChannel(alphachannel: boolean);

  begin
    smAlphaChannel := alphachannel;
  end;



// pvc ... pointer to vertex color table
procedure TSkinMesh.SetColor(pvc: PColors4f);

  begin
    smColor := pvc;
  end;



// set number of texture components to be sent to OpenGL
procedure TSkinMesh.SetTexComponents(texcomponents: integer);

  begin
    smTexComponents := texcomponents;
  end;



// texunit ... texture unit to set, pvt ... pointer to vertex texture position table
procedure TSkinMesh.SetTexCoord(texunit: integer; pvt: PTexCoords4f);

  begin
    smTexCoord[texunit] := pvt;
  end;



// texunit: index of texture coordinate set, value: texture unit value
procedure TSkinMesh.SetTexUnit(texunit, value: integer);

  begin
    smTexUnit[texunit] := value;
    if texunit>=smTexUnits-1 then smTexUnits := texunit + 1;
  end;



// pt ... pointer to triangle table
procedure TSkinMesh.SetTriangles(pt: PTris);

  begin
    objMesh.meshTri := pt;
  end;



procedure TSkinMesh.SetupAnimations;

  var
    i: integer;
    sv: PSkinVert;
    m: PMatrix4f;

  begin
    inherited SetupAnimations;
    if not smStaticData then begin
      for i:=0 to objMesh.meshVerts-1 do begin
        sv := @smVert[i];
        if sv.JointIndex>=0 then begin
          m := @objJoint[sv.JointIndex].AbsoluteTransformation;
          sv.Pos := VectorInverseTranslate(sv.Pos, m^);
          sv.Pos := VectorInverseRotate(sv.Pos, m^);
          sv.Norm := VectorInverseRotate(sv.Norm, m^);
        end;
      end;
    end;
  end;



// pv ... pointer to skinned vertex table
procedure TSkinMesh.SetVerts(SkinVerts: PSkinVerts);

  var
    i: integer;

  begin
    smVert := SkinVerts;
    for i:=0 to objMesh.meshVerts-1 do begin
      objMesh.meshVert[i] := SkinVerts[i].Pos;
    end;
  end;



// returns triangle list by index
function TSkinMesh.TriList(TriListIndex: integer): PTriangleList;

  begin
    if TriListIndex<smTriangleLists then
      TriList := @smTriangleList[TriListIndex]
    else
      TriList := NIL;
  end;






//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Evaluator1D implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



constructor TEvaluator1D.Create;

  begin
    inherited Create;
    e1dPoints := 0;
    e1dPoint := NIL;
    e1dLineWidth := 1.0;
    e1dLineColor.r := 1.0;  e1dLineColor.g := 1.0;  e1dLineColor.b := 1.0;  e1dLineColor.a := 0.0;

    e1dNWidth := 10;
    e1dLength := 0;
    e1dSegments := 0;
    e1dSegment := NIL;
  end;



destructor TEvaluator1D.Destroy;

  begin
    if e1dPoints>0 then FreeMem(e1dPoint);
    if e1dSegments>0 then FreeMem(e1dSegment);
    inherited Destroy;
  end;



procedure TEvaluator1D.AddPoint(x,y,z: float);

  var
    a, b : integer;

  begin
    if e1dPoints = 0 then
      e1dPoints := 1
    else
      inc(e1dPoints, 2);
    ReallocMem(e1dPoint, e1dPoints * SizeOf(TVec));
    if e1dPoints = 1 then begin
      e1dPoint[0] := Vector(x,y,z);
    end
    else if e1dPoints = 3 then begin
      a := e1dPoints-1;
      b := e1dPoints-3;
      e1dPoint[a] := Vector(x,y,z);
      e1dPoint[e1dPoints-2] := Vector( (e1dPoint[a].x + e1dPoint[b].x) * 0.5,
                                       (e1dPoint[a].y + e1dPoint[b].y) * 0.5,
                                       (e1dPoint[a].z + e1dPoint[b].z) * 0.5);
    end
    else begin
      a := e1dPoints-1;
      b := e1dPoints-3;
      e1dPoint[a] := Vector(x,y,z);
      e1dPoint[e1dPoints-2] := Vector( (e1dPoint[a].x + e1dPoint[b].x) * 0.5,
                                       (e1dPoint[a].y + e1dPoint[b].y) * 0.5,
                                       (e1dPoint[a].z + e1dPoint[b].z) * 0.5);
      a := e1dPoints - 2;
      b := e1dPoints - 4;
      e1dPoint[e1dPoints-3] := Vector( (e1dPoint[a].x + e1dPoint[b].x) * 0.5,
                                       (e1dPoint[a].y + e1dPoint[b].y) * 0.5,
                                       (e1dPoint[a].z + e1dPoint[b].z) * 0.5);
    end;
  end;



procedure TEvaluator1D.Close;

  var
    a, b: integer;

  begin
    if e1dPoints<2 then exit;
    AddPoint(e1dPoint[0].x, e1dPoint[0].y, e1dPoint[0].z);
    a := e1dPoints - 2;
    b := 1;
    e1dPoint[0] := Vector( (e1dPoint[a].x + e1dPoint[b].x) * 0.5,
                           (e1dPoint[a].y + e1dPoint[b].y) * 0.5,
                           (e1dPoint[a].z + e1dPoint[b].z) * 0.5);
    e1dPoint[e1dPoints-1] := e1dPoint[0];
  end;



procedure TEvaluator1D.DrawObj;

  var
    i, n: integer;

  begin
    glDisable(GL_COLOR_MATERIAL);
    glDisable(GL_LIGHTING);
{$IFDEF GL_13}
    for i:=GL_TEXTURE0 + bgMaxTextureUnits - 1 downto GL_TEXTURE0 do begin
      if GL_13 then glActiveTexture(i);
      glDisable(GL_TEXTURE_1D);
      glDisable(GL_TEXTURE_2D);
      if GL_12 then glDisable(GL_TEXTURE_3D);
      if GL_13 then glDisable(GL_TEXTURE_CUBE_MAP);
    end;
{$ELSE}
    glDisable(GL_TEXTURE_1D);
    glDisable(GL_TEXTURE_2D);
{$IFDEF GL_12}
    if GL_12 then glDisable(GL_TEXTURE_3D);
{$ENDIF}
{$ENDIF}

    glColor4fv(Addr(e1dLineColor));
    glLineWidth(e1dLineWidth);

    glEnable(GL_MAP1_VERTEX_3);
    glDisable(GL_AUTO_NORMAL);
    glDisable(GL_MAP1_TEXTURE_COORD_2);
    glMapGrid1f(e1dNWidth, 0.0, 1.0);

    n := (e1dPoints div 2) - 1;
    for i := 0 to n do begin
      glMap1f(GL_MAP1_VERTEX_3, 0.0,1.0,3,3, Addr(e1dPoint[i*2]));
      glEvalMesh1(GL_LINE, 0, e1dNWidth);
    end;
  end;



procedure TEvaluator1D.RecomputeSegments;

  var
    i, j, k: integer;
    f, t : float;
    v: array[0..2] of PVec;
    u, B: float;
    Bu: array[0..2] of float;
    vec, last: TVec;

  begin
    if e1dPoints < 3 then exit;
    e1dSegments := (e1dPoints+1) div 2;
    ReallocMem(e1dSegment, e1dSegments * SizeOf(TEvaluator1DSegment));
    f := 0;
    last := e1dPoint[0];
    for i := 0 to e1dSegments-1 do begin
      e1dSegment[i].Start := f;
      e1dSegment[i].Length := 0;
      if i<>e1dSegments-1 then begin
        v[0] := @e1dPoint[i*2];
        v[1] := @e1dPoint[i*2+1];
        v[2] := @e1dPoint[i*2+2];
        for j := 1 to e1dNWidth do begin
          u := j / e1dNWidth;
          for k := 0 to 2 do
            Bu[k] := BinComb(2,k)*bgPower(u,k) * bgPower((1.0-u),2-k);

          vec := Vector(0,0,0);
          for k := 0 to 2 do begin
            B := Bu[k];
            vec.x := vec.x + B*v[k].x;
            vec.y := vec.y + B*v[k].y;
            vec.z := vec.z + B*v[k].z;
          end;

          t := VectorDist(last, vec);
          e1dSegment[i].Length := e1dSegment[i].Length + t;
          last := vec;
        end;
        f := f + e1dSegment[i].Length;
      end;
    end;

    if f<0.00001 then f:=1.0;
    e1dLength := f;
    for i := 0 to e1dSegments-1 do begin
      e1dSegment[i].Start := e1dSegment[i].Start / e1dLength;
      e1dSegment[i].Length := e1dSegment[i].Length / e1dLength;
    end;
  end;



function TEvaluator1D.Evaluate(position: float) : TVec;

  var
    i    : integer;
    v    : array[0..2] of PVec;
    u, B : float;
    Bu   : array[0..2] of float;
    vec  : TVec;

  begin
    if (position < 0.0) or (position > 1.0) or (e1dPoints<3) then begin
      result := Vector(0,0,0);
      exit;
    end;
    i := 0;
    while (i+2<e1dSegments) and (e1dSegment[i+1].Start<position) do
      inc(i);

    v[0] := @e1dPoint[i*2];
    v[1] := @e1dPoint[i*2+1];
    v[2] := @e1dPoint[i*2+2];

    u := (position - e1dSegment[i].Start) / e1dSegment[i].Length;
    if u<0.0 then u:=0.0;
    if u>1.0 then u:=1.0;

    for i := 0 to 2 do
      Bu[i] := BinComb(2,i)*bgPower(u,i) * bgPower((1.0-u),2-i);

    vec := Vector(0,0,0);
    for i := 0 to 2 do begin
      B := Bu[i];
      vec.x := vec.x + B*v[i].x;
      vec.y := vec.y + B*v[i].y;
      vec.z := vec.z + B*v[i].z;
    end;

    result := vec;
  end;



procedure TEvaluator1D.EvaluateDirection(position: float; var v1, v2: TVec);

  var
    f: float;

  begin
    f := 1.0 / (e1dSegments * e1dNWidth);
    if position < 0.0 then position := 0.0;
    if position > 1.0 then position := 1.0;
    if position + f < 1.0 then begin
      v1 := Evaluate(position);
      v2 := Evaluate(position + f);
    end
    else begin
      if VectorDist(e1dPoint[0], e1dPoint[e1dPoints-1]) < 0.0001 then begin
        v1 := Evaluate(position);
        v2 := Evaluate(position + f - 1.0);
      end
      else begin
        v1 := Evaluate(1.0 - f);
        v2 := Evaluate(1.0);
      end;
    end;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



constructor TEv1DAnimator.Create(Ev: TEvaluator1D; CycleTime: integer);

  begin
    e1aEv := Ev;
    e1aCycleTime := CycleTime;
    e1aAdvance := 0.0;
    e1aLoop := true;
    e1aFinished := false;
    e1aUp := Vector(0.0,1.0,0.0);
  end;



procedure TEv1DAnimator.Start(Advance: float);

  begin
    e1aAdvance := Advance;
    e1aFinished := false;
  end;



function TEv1DAnimator.Run(Time: integer) : TMatrix4f;

  var
    advance : float;
    v1, v2  : TVec;

  begin
    if e1aFinished then
      advance := e1aAdvance
    else
      advance := e1aAdvance + Time / e1aCycleTime;

    if advance > 1.0 then begin
      if e1aLoop then
        advance := Frac(advance)
      else
        advance := 1.0;
    end;
    e1aAdvance := advance;

    e1aEv.EvaluateDirection(e1aAdvance, v1, v2);
    glLoadIdentity;
    gluLookAt(v1.x,v1.y,v1.z, v2.x,v2.y,v2.z, e1aUp.x,e1aUp.y,e1aUp.z);
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(result));
    MatrixInvert(result);
    v1 := e1aEv.Evaluate(e1aAdvance);

    result[3,0] := v1.x;
    result[3,1] := v1.y;
    result[3,2] := v1.z;
  end;



procedure TEv1DAnimator.Stop;

  begin
    e1aFinished := true;
  end;



procedure TEv1DAnimator.Advance(Adv: float);

  begin
    e1aAdvance := Adv;
  end;



procedure TEv1DAnimator.SetCycleTime(CycleTime: integer);

  begin
    e1aCycleTime := abs(CycleTime);
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Evaluator2D implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



constructor TEvaluator2D.Create;

  begin
    inherited Create;
    Name := 'Evaluator2D';
    e2dAutoNormal := True;
    e2dTexCoords := True;
    e2dInvertOrientation := True;
    e2dFillStyle := GL_LINE;

    e2dWidthSegments := 0;
    e2dHeightSegments := 0;

    e2dLeft := 0.0;
    e2dTop := 0.0;
    e2dWidth := 0.0;
    e2dHeight := 0.0;
    e2dNWidth := 10;
    e2dNHeight := 10;
    e2dPatchWidth := 0.0;
    e2dPatchHeight := 0.0;

    e2dVert := NIL;
    e2dPatch := NIL;
    e2dBinary := True;

    e2dDefaultShaders := 0;
    FillChar(e2dDefaultShader, SizeOf(e2dDefaultShader), 0);
    e2dResetDefaultShaders := true;

    e2dPatchChild := NIL;
    e2dPatchChildren := 0;
    e2dDispChild := NIL;
    e2dDispChildren := 0;
  end;


constructor TEvaluator2D.CreateByData(ControlPoints: PVecs; WidthSegments, HeightSegments: integer);

  var
    LineVecs : integer;
    i, j : integer;
    Patch : PEvaluator2DPatch;

  begin
    inherited Create;
    Name := 'Evaluator2D';
    e2dAutoNormal := True;
    e2dTexCoords := True;
    e2dInvertOrientation := True;
    e2dFillStyle := GL_LINE;

    e2dVert := ControlPoints;
    e2dBinary := False;

    e2dWidthSegments := WidthSegments;
    e2dHeightSegments := HeightSegments;
    e2dLists := glGenLists(e2dHeightSegments * e2dWidthSegments);
    GetMem(e2dPatch, HeightSegments * WidthSegments * SizeOf(TEvaluator2DPatch));
    FillChar(e2dPatch^, HeightSegments * WidthSegments * SizeOf(TEvaluator2DPatch), 0);
    LineVecs := WidthSegments * 2 + 1;

    e2dLeft := ControlPoints[0].x;
    e2dTop := ControlPoints[0].z;
    i := (HeightSegments * 2 + 1) * LineVecs - 1;
    e2dWidth := ControlPoints[i].x - e2dLeft;
    e2dHeight := ControlPoints[i].z - e2dTop;

    e2dNWidth := 10;
    e2dNHeight := 10;
    e2dPatchWidth := e2dWidth / e2dWidthSegments;
    e2dPatchHeight := e2dHeight / e2dHeightSegments;

    for j := 0 to e2dHeightSegments-1 do
      for i := 0 to e2dWidthSegments-1 do begin
        Patch := @e2dPatch[j*e2dWidthSegments + i];

        Patch.ControlPoint[0][0] := @e2dVert[j*LineVecs*2 + i*2];
        Patch.ControlPoint[0][1] := @PVecs(Patch.ControlPoint[0][0])[1];
        Patch.ControlPoint[0][2] := @PVecs(Patch.ControlPoint[0][0])[2];

        Patch.ControlPoint[1][0] := @PVecs(Patch.ControlPoint[0][0])[LineVecs];
        Patch.ControlPoint[1][1] := @PVecs(Patch.ControlPoint[1][0])[1];
        Patch.ControlPoint[1][2] := @PVecs(Patch.ControlPoint[1][0])[2];

        Patch.ControlPoint[2][0] := @PVecs(Patch.ControlPoint[1][0])[LineVecs];
        Patch.ControlPoint[2][1] := @PVecs(Patch.ControlPoint[2][0])[1];
        Patch.ControlPoint[2][2] := @PVecs(Patch.ControlPoint[2][0])[2];

        Patch.list := e2dLists + GLuint(j*WidthSegments + i);
        Patch.Recompute := true;
        Patch.Visible := true;
        Patch.Flat := false;
        Patch.EdgeFlat := 0;
      end;

    for j := 0 to e2dHeightSegments-1 do
      for i := 0 to e2dWidthSegments-1 do
        SetPatchBounding(i, j);

    e2dDefaultShaders := 0;
    FillChar(e2dDefaultShader, SizeOf(e2dDefaultShader), 0);
    e2dResetDefaultShaders := true;

    e2dPatchChild := NIL;
    e2dPatchChildren := 0;
    e2dDispChild := NIL;
    e2dDispChildren := 0;

    objCent.x := e2dLeft + e2dWidth*0.5;
    objCent.y := 0.0;
    objCent.z := e2dTop + e2dHeight*0.5;
    objBSRad := sqrt(sqr(e2dWidth) + sqr(e2dHeight)) / 2.0;
  end;


constructor TEvaluator2D.CreateByFile(FileName: String);

  var
    F : File;
    LineVecs : integer;
    i, j : integer;
    Patch : PEvaluator2DPatch;

  begin
    inherited Create;
    Name := 'Evaluator2D';
    IOResult; // clear IOResult
    Assign(F, FileName);
    Reset(F, 1);
    if IOResult=0 then begin
      BlockRead(F, e2dWidthSegments, SizeOf(e2dWidthSegments));
      BlockRead(F, e2dHeightSegments, SizeOf(e2dHeightSegments));
      LineVecs := e2dWidthSegments * 2 + 1;
      GetMem(e2dVert, (e2dHeightSegments*2+1) * LineVecs * SizeOf(TVec));
      BlockRead(F, e2dVert^, (e2dHeightSegments*2+1) * LineVecs * SizeOf(TVec));

      GetMem(e2dPatch, e2dHeightSegments * e2dWidthSegments * SizeOf(TEvaluator2DPatch));
      FillChar(e2dPatch^, e2dHeightSegments * e2dWidthSegments * SizeOf(TEvaluator2DPatch), 0);
      e2dLists := glGenLists(e2dHeightSegments * e2dWidthSegments);

      e2dLeft := e2dVert[0].x;
      e2dTop := e2dVert[0].z;
      i := (e2dHeightSegments * 2 + 1) * LineVecs - 1;
      e2dWidth := e2dVert[i].x - e2dLeft;
      e2dHeight := e2dVert[i].z - e2dTop;

      e2dNWidth := 10;
      e2dNHeight := 10;
      e2dPatchWidth := e2dWidth / e2dWidthSegments;
      e2dPatchHeight := e2dHeight / e2dHeightSegments;

      for j := 0 to e2dHeightSegments-1 do
        for i := 0 to e2dWidthSegments-1 do begin
          Patch := @e2dPatch[j*e2dWidthSegments + i];

          Patch.ControlPoint[0][0] := @e2dVert[j*LineVecs*2 + i*2];
          Patch.ControlPoint[0][1] := @PVecs(Patch.ControlPoint[0][0])[1];
          Patch.ControlPoint[0][2] := @PVecs(Patch.ControlPoint[0][0])[2];

          Patch.ControlPoint[1][0] := @PVecs(Patch.ControlPoint[0][0])[LineVecs];
          Patch.ControlPoint[1][1] := @PVecs(Patch.ControlPoint[1][0])[1];
          Patch.ControlPoint[1][2] := @PVecs(Patch.ControlPoint[1][0])[2];

          Patch.ControlPoint[2][0] := @PVecs(Patch.ControlPoint[1][0])[LineVecs];
          Patch.ControlPoint[2][1] := @PVecs(Patch.ControlPoint[2][0])[1];
          Patch.ControlPoint[2][2] := @PVecs(Patch.ControlPoint[2][0])[2];

          Patch.list := e2dLists + GLuint(j*e2dWidthSegments + i);
          Patch.Recompute := true;
          Patch.Visible := true;
          Patch.Flat := false;
          Patch.EdgeFlat := 0;
        end;

      for j := 0 to e2dHeightSegments-1 do
        for i := 0 to e2dWidthSegments-1 do
          SetPatchBounding(i, j);

      e2dDefaultShaders := 0;
      FillChar(e2dDefaultShader, SizeOf(e2dDefaultShader), 0);
      e2dResetDefaultShaders := true;

      e2dPatchChild := NIL;
      e2dPatchChildren := 0;
      e2dDispChild := NIL;
      e2dDispChildren := 0;

      objCent.x := e2dLeft + e2dWidth*0.5;
      objCent.y := 0.0;
      objCent.z := e2dTop + e2dHeight*0.5;
      objBSRad := sqrt(sqr(e2dWidth) + sqr(e2dHeight)) / 2.0;
      Close(F);
    end;
  end;



destructor TEvaluator2D.Destroy;

  var
    i : integer;

  begin
    if e2dBinary then
      if e2dVert<>NIL then FreeMem(e2dVert);
    if e2dPatchChildren>0 then begin
      FreeMem(e2dPatchChild);
      FreeMem(e2dDispChild);
    end;
    for i := 0 to e2dHeightSegments * e2dWidthSegments - 1 do
      if e2dPatch[i].BoundObjects>0 then FreeMem(e2dPatch[i].BoundObject);
    if e2dPatch<>NIL then FreeMem(e2dPatch);
    if (e2dHeightSegments or e2dWidthSegments) <> 0 then
      glDeleteLists(e2dLists, e2dHeightSegments * e2dWidthSegments);
    inherited Destroy;
  end;



procedure TEvaluator2D.AddDefaultShader(aShader: TShader);

  begin
    e2dDefaultShader[e2dDefaultShaders] := aShader;
    inc(e2dDefaultShaders);
  end;



function TEvaluator2D.GetPatch(i, j: integer) : PEvaluator2DPatch;

  begin
    result := @e2dPatch[j * e2dWidthSegments + i];
  end;



procedure TEvaluator2D.AddPatchShader(i, j: integer; aShader: TShader);

  var
    Patch: PEvaluator2DPatch;

  begin
    Patch := @e2dPatch[j * e2dWidthSegments + i];
    Patch.Shader[Patch.Shaders] := aShader;
    inc(Patch.Shaders);
  end;



procedure TEvaluator2D.SetDimensions(Left, Top, Width, Height: GLfloat; WidthSegments, HeightSegments: integer);

  var
    i, j  : integer;
    Patch : PEvaluator2DPatch;
    bsrad : float;
    LineVecs : integer;

  begin
    e2dLeft := Left;
    e2dTop := Top;
    e2dWidth := Width;
    e2dHeight := Height;

    if (e2dHeightSegments or e2dWidthSegments) <> 0 then
      glDeleteLists(e2dLists, e2dHeightSegments * e2dWidthSegments);

    if e2dBinary then begin
      e2dWidthSegments := WidthSegments;
      e2dHeightSegments := HeightSegments;
      ReallocMem(e2dVert, (HeightSegments*2+1) * (WidthSegments*2+1) * SizeOf(TVec));
    end
    else begin
      WidthSegments := e2dWidthSegments;
      HeightSegments := e2dHeightSegments;
    end;
    ReallocMem(e2dPatch, HeightSegments * WidthSegments * SizeOf(TEvaluator2DPatch));
    FillChar(e2dPatch^, HeightSegments * WidthSegments * SizeOf(TEvaluator2DPatch), 0);

    e2dLists := glGenLists(e2dHeightSegments * e2dWidthSegments);

    e2dPatchWidth := Width / WidthSegments;
    e2dPatchHeight := Height / HeightSegments;

    bsrad := sqrt(sqr(e2dPatchWidth) + sqr(e2dPatchHeight)) / 2.0;
    LineVecs := WidthSegments*2 + 1;
    for j := 0 to HeightSegments-1 do
      for i := 0 to WidthSegments-1 do begin
        Patch := @e2dPatch[j*WidthSegments + i];

        Patch.ControlPoint[0][0] := @e2dVert[j*LineVecs*2 + i*2];
        Patch.ControlPoint[0][1] := @PVecs(Patch.ControlPoint[0][0])[1];
        Patch.ControlPoint[0][2] := @PVecs(Patch.ControlPoint[0][0])[2];

        Patch.ControlPoint[1][0] := @PVecs(Patch.ControlPoint[0][0])[LineVecs];
        Patch.ControlPoint[1][1] := @PVecs(Patch.ControlPoint[1][0])[1];
        Patch.ControlPoint[1][2] := @PVecs(Patch.ControlPoint[1][0])[2];

        Patch.ControlPoint[2][0] := @PVecs(Patch.ControlPoint[1][0])[LineVecs];
        Patch.ControlPoint[2][1] := @PVecs(Patch.ControlPoint[2][0])[1];
        Patch.ControlPoint[2][2] := @PVecs(Patch.ControlPoint[2][0])[2];

        Patch.ControlPoint[0][0]^ := Vector(Left + (i      *Width)/WidthSegments, 0.0, Top + (j      *Height)/HeightSegments);
        Patch.ControlPoint[0][1]^ := Vector(Left + ((i+0.5)*Width)/WidthSegments, 0.0, Top + (j      *Height)/HeightSegments);
        Patch.ControlPoint[0][2]^ := Vector(Left + ((i+1.0)*Width)/WidthSegments, 0.0, Top + (j      *Height)/HeightSegments);

        Patch.ControlPoint[1][0]^ := Vector(Left + (i      *Width)/WidthSegments, 0.0, Top + ((j+0.5)*Height)/HeightSegments);
        Patch.ControlPoint[1][1]^ := Vector(Left + ((i+0.5)*Width)/WidthSegments, 0.0, Top + ((j+0.5)*Height)/HeightSegments);
        Patch.ControlPoint[1][2]^ := Vector(Left + ((i+1.0)*Width)/WidthSegments, 0.0, Top + ((j+0.5)*Height)/HeightSegments);

        Patch.ControlPoint[2][0]^ := Vector(Left + (i      *Width)/WidthSegments, 0.0, Top + ((j+1.0)*Height)/HeightSegments);
        Patch.ControlPoint[2][1]^ := Vector(Left + ((i+0.5)*Width)/WidthSegments, 0.0, Top + ((j+1.0)*Height)/HeightSegments);
        Patch.ControlPoint[2][2]^ := Vector(Left + ((i+1.0)*Width)/WidthSegments, 0.0, Top + ((j+1.0)*Height)/HeightSegments);

        Patch.list := e2dLists + GLuint(j*WidthSegments + i);
        Patch.Center := Patch.ControlPoint[1][1]^;
        Patch.BSRad := bsrad;
        Patch.Recompute := true;
        Patch.Visible := true;
        Patch.Flat := false;
        Patch.EdgeFlat := 0;
      end;

    objCent.x := Left + Width*0.5;
    objCent.y := 0.0;
    objCent.z := Top + Height*0.5;
    objBSRad := sqrt(sqr(Width) + sqr(Height)) / 2.0;
  end;



procedure TEvaluator2D.SetResolution(NWidth, NHeight: integer);

  begin
    e2dNWidth := NWidth;
    e2dNHeight := NHeight;
    RecomputeAll;
  end;



procedure TEvaluator2D.RecomputeAll;

  var
    i, j  : integer;

  begin
    for j := 0 to e2dHeightSegments-1 do
      for i := 0 to e2dWidthSegments-1 do
        e2dPatch[j*e2dHeightSegments + i].Recompute := true;
  end;



procedure TEvaluator2D.DrawPatch(Patch : PEvaluator2DPatch);

  var
    PatchVecs: TPatch2D;
    i : integer;
    uRS, uTS, uTex, uMat, uVP, uFP : Boolean;

  begin
    if Patch.Recompute then begin
      if e2dInvertOrientation then begin
        PatchVecs[2][0] := Patch.ControlPoint[0][0]^;
        PatchVecs[2][1] := Patch.ControlPoint[0][1]^;
        PatchVecs[2][2] := Patch.ControlPoint[0][2]^;
        PatchVecs[1][0] := Patch.ControlPoint[1][0]^;
        PatchVecs[1][1] := Patch.ControlPoint[1][1]^;
        PatchVecs[1][2] := Patch.ControlPoint[1][2]^;
        PatchVecs[0][0] := Patch.ControlPoint[2][0]^;
        PatchVecs[0][1] := Patch.ControlPoint[2][1]^;
        PatchVecs[0][2] := Patch.ControlPoint[2][2]^;
      end
      else begin
        PatchVecs[2][2] := Patch.ControlPoint[0][0]^;
        PatchVecs[2][1] := Patch.ControlPoint[0][1]^;
        PatchVecs[2][0] := Patch.ControlPoint[0][2]^;
        PatchVecs[1][2] := Patch.ControlPoint[1][0]^;
        PatchVecs[1][1] := Patch.ControlPoint[1][1]^;
        PatchVecs[1][0] := Patch.ControlPoint[1][2]^;
        PatchVecs[0][2] := Patch.ControlPoint[2][0]^;
        PatchVecs[0][1] := Patch.ControlPoint[2][1]^;
        PatchVecs[0][0] := Patch.ControlPoint[2][2]^;
      end;

      glNewList(Patch.list, GL_COMPILE);
        for i := 0 to Patch.Shaders-1 do
          Patch.Shader[i].Display;
        glMap2f(GL_MAP2_VERTEX_3, 0.0,1.0,3,3, 1.0,0.0,9,3, Addr(PatchVecs));
        if Patch.Flat then begin
          case e2dFillStyle of
            GL_POINT: begin
                glBegin(GL_POINTS);
                  glEvalCoord2f(0, 0);
                  glEvalCoord2f(0, 1);
                  glEvalCoord2f(1, 1);
                  glEvalCoord2f(1, 0);
                glEnd;
              end;
            GL_LINE: begin
                glBegin(GL_LINE_STRIP);
                  glEvalCoord2f(0, 0);
                  glEvalCoord2f(0, 1);
                  glEvalCoord2f(1, 1);
                  glEvalCoord2f(1, 0);
                  glEvalCoord2f(0, 0);
                glEnd;
              end;
            GL_FILL: begin
                if Patch.EdgeFlat = $0F then begin
                  glBegin(GL_QUADS);
                    glEvalCoord2f(0, 0);
                    glEvalCoord2f(0, 1);
                    glEvalCoord2f(1, 1);
                    glEvalCoord2f(1, 0);
                  glEnd;
                end
                else begin
                  glBegin(GL_TRIANGLE_FAN);
                    glEvalCoord2f(0.5, 0.5);
                    // upper edge
                    if (Patch.EdgeFlat and $01)<>0 then begin
                      glEvalPoint2(e2dNWidth, 0);
                      glEvalPoint2(0, 0);
                    end
                    else
                    for i := e2dNWidth downto 0 do
                      glEvalPoint2(i, 0);
                    // left edge
                    if (Patch.EdgeFlat and $08)<>0 then
                      glEvalPoint2(0, e2dNHeight)
                    else
                    for i := 1 to e2dNHeight do
                      glEvalPoint2(0, i);
                    // lower edge
                    if (Patch.EdgeFlat and $04)<>0 then
                      glEvalPoint2(e2dNWidth, e2dNHeight)
                    else
                    for i := 1 to e2dNWidth do
                      glEvalPoint2(i, e2dNHeight);
                    // right edge
                    if (Patch.EdgeFlat and $02)<>0 then
                      glEvalPoint2(e2dNWidth, 0)
                    else
                    for i := e2dNHeight-1 downto 0 do
                      glEvalPoint2(e2dNWidth, i);
                  glEnd;
                end;
              end;
          end;
        end
        else
          glEvalMesh2(e2dFillStyle, 0, e2dNWidth, 0, e2dNHeight);
        if e2dResetDefaultShaders then begin
          uRS := False;
          uTS := False;
          uTex := False;
          uMat := False;
          uVP := False;
          uFP := False;
          for i := 0 to Patch.Shaders - 1 do
          case Patch.Shader[i].GetIdentity of
              SD_TEX_NONE,
              SD_TEX_1D,
              SD_TEX_2D,
              SD_TEX_3D,
              SD_TEX_CUBE,
              SD_TEX_DYN_2D,
              SD_TEX_DYN_CUBE,
              SD_TEX_DEFAULT    : uTex := true;
              SD_MAT            : uMat := true;
              SD_RS             : uRS := true;
              SD_TS             : uTS := true;
              SD_VP             : uVP := true;
              SD_FP             : uFP := true;
          end;
          for i := 0 to e2dDefaultShaders-1 do
          case e2dDefaultShader[i].GetIdentity of
              SD_TEX_NONE,
              SD_TEX_1D,
              SD_TEX_2D,
              SD_TEX_3D,
              SD_TEX_CUBE,
              SD_TEX_DYN_2D,
              SD_TEX_DYN_CUBE,
              SD_TEX_DEFAULT    : if uTex then e2dDefaultShader[i].Display;
              SD_MAT            : if uMat then e2dDefaultShader[i].Display;
              SD_RS             : if uRS then e2dDefaultShader[i].Display;
              SD_TS             : if uTS then e2dDefaultShader[i].Display;
              SD_VP             : if uVP then e2dDefaultShader[i].Display;
              SD_FP             : if uFP then e2dDefaultShader[i].Display;
          end;
        end;
      glEndList;
      Patch.Recompute := false;
    end;
    glCallList(Patch.list);
  end;



var e2dStdTexCoord : Array[0..1] of Array[0..1] of TVec2f =
  ( ((0,0), (0,1)), ((1,0), (1,1)) );

procedure TEvaluator2D.DrawObj;

  var
    Patch: PEvaluator2DPatch;
    Center : TVec;
    i, j, t : integer;
    Viewer : TVec;
    istart, iend, jstart, jend : integer;
    f : float;
    obj : TObj;

  begin
    for i := 0 to e2dDefaultShaders-1 do
      e2dDefaultShader[i].Display;

    glEnable(GL_MAP2_VERTEX_3);
    if e2dAutoNormal then
      glEnable(GL_AUTO_NORMAL)
    else
      glDisable(GL_AUTO_NORMAL);
    glMapGrid2f(e2dNWidth, 0.0, 1.0, e2dNHeight, 0.0, 1.0);
    if e2dTexCoords then begin
      glEnable(GL_MAP2_TEXTURE_COORD_2);
      glMap2f(GL_MAP2_TEXTURE_COORD_2, 0.0,1.0,2,2, 0.0,1.0,4,2, Addr(e2dStdTexCoord));
    end
    else
      glDisable(GL_MAP2_TEXTURE_COORD_2);

    e2dDispChildren := 0;
    for t := 0 to e2dPatchChildren-1 do
      e2dPatchChild[t].objScaleFactor := 0.0;

    if not MatrixIsIdentity(objMatrix) then begin
      j := e2dHeightSegments * e2dWidthSegments - 1;
      for i := 0 to j do begin
        Patch := @e2dPatch[i];
        if Patch.Visible then begin
          VectorTransformTo(Patch.Center, Center, objMatrix);
          if SphereInFrustum(Center.x, Center.y, Center.z, Patch.BSRad*objScaleFactor) then
            DrawPatch(Patch);
        end;
        for t := 0 to Patch.BoundObjects-1 do begin
          obj := Patch.BoundObject[t];
          if obj.objScaleFactor=0.0 then begin
            e2dDispChild[e2dDispChildren] := obj;
            inc(e2dDispChildren);
            obj.objScaleFactor := 1.0;
          end;
        end;
      end;
    end  // MatrixIsIdentity
    else begin
      Viewer.x := bgViewInvertedMatrix[3,0];
      Viewer.y := bgViewInvertedMatrix[3,1];
      Viewer.z := bgViewInvertedMatrix[3,2];
      i := Round((Viewer.x - e2dLeft) / e2dPatchWidth - 0.5);
      j := Round((Viewer.z - e2dTop) / e2dPatchHeight - 0.5);

      f := e2dPatchWidth;
      if e2dPatchHeight > f then f := e2dPatchHeight;
      t := Round((bgZFar*1.4142136) / f + 0.5);

      istart := i - t;
      if istart < 0 then istart := 0;
      iend := i + t;
      if iend>=e2dWidthSegments then iend := e2dWidthSegments-1;

      jstart := j - t;
      if jstart < 0 then jstart := 0;
      jend := j + t;
      if jend >= e2dHeightSegments then jend := e2dHeightSegments-1;

      for j := jstart to jend do
        for i := istart to iend do begin
          Patch := @e2dPatch[j*e2dWidthSegments + i];
          if Patch.Visible then begin
            if SphereInFrustum(Patch.Center.x, Patch.Center.y, Patch.Center.z, Patch.BSRad) then
              DrawPatch(Patch);
          end;
          for t := 0 to Patch.BoundObjects-1 do begin
            obj := Patch.BoundObject[t];
            if obj.objScaleFactor=0.0 then begin
              e2dDispChild[e2dDispChildren] := obj;
              inc(e2dDispChildren);
              obj.objScaleFactor := 1.0;
            end;
          end;
        end;
    end;

    for t := 0 to e2dDispChildren-1 do
      e2dDispChild[t].SetMatrix(@objMatrix);
    for t := 0 to e2dDispChildren-1 do
      e2dDispChild[t].Display;
  end;



function TEvaluator2D.Evaluate(u, v: float) : TVec;

  var
    i, j : integer;
    patch : PEvaluator2DPatch;
    vec : TVec;
    Bu, Bv : Array[0..2] of float;
    B : float;
    du, dv : float;
    PatchVecs : TPatch2D;

  begin
    vec.x:=0.0;  vec.y:=0.0;  vec.z:=0.0;
    du := u - e2dLeft;
    dv := v - e2dTop;
    if (du<0) or (dv<0) or (du>e2dWidth) or (dv>e2dHeight) then begin
      vec.x := u;
      vec.z := v;
      result := vec;
      exit;
    end;
    i := Round(du / e2dPatchWidth - 0.5);
    j := Round(dv / e2dPatchHeight - 0.5);
    if i<0 then i:=0;
    if j<0 then j:=0;
    if i>=e2dWidthSegments then i:=e2dWidthSegments-1;
    if j>=e2dHeightSegments then j:=e2dHeightSegments-1;
    Patch := @e2dPatch[j*e2dWidthSegments + i];

    if e2dInvertOrientation then begin
      PatchVecs[2][0] := Patch.ControlPoint[0][0]^;
      PatchVecs[2][1] := Patch.ControlPoint[0][1]^;
      PatchVecs[2][2] := Patch.ControlPoint[0][2]^;
      PatchVecs[1][0] := Patch.ControlPoint[1][0]^;
      PatchVecs[1][1] := Patch.ControlPoint[1][1]^;
      PatchVecs[1][2] := Patch.ControlPoint[1][2]^;
      PatchVecs[0][0] := Patch.ControlPoint[2][0]^;
      PatchVecs[0][1] := Patch.ControlPoint[2][1]^;
      PatchVecs[0][2] := Patch.ControlPoint[2][2]^;
    end
    else begin
      PatchVecs[2][2] := Patch.ControlPoint[0][0]^;
      PatchVecs[2][1] := Patch.ControlPoint[0][1]^;
      PatchVecs[2][0] := Patch.ControlPoint[0][2]^;
      PatchVecs[1][2] := Patch.ControlPoint[1][0]^;
      PatchVecs[1][1] := Patch.ControlPoint[1][1]^;
      PatchVecs[1][0] := Patch.ControlPoint[1][2]^;
      PatchVecs[0][2] := Patch.ControlPoint[2][0]^;
      PatchVecs[0][1] := Patch.ControlPoint[2][1]^;
      PatchVecs[0][0] := Patch.ControlPoint[2][2]^;
    end;

    u := (u - PatchVecs[0][0].x) / (PatchVecs[0][2].x - PatchVecs[0][0].x);
    v := (v - PatchVecs[0][0].z) / (PatchVecs[2][0].z - PatchVecs[0][0].z);
    if u<0.0 then u := 0.0;
    if u>1.0 then u := 1.0;
    if v<0.0 then v := 0.0;
    if v>1.0 then v := 1.0;

    for i := 0 to 2 do begin
      Bu[i] := BinComb(2,i)*bgPower(u,i) * bgPower((1.0-u),2-i);
      Bv[i] := BinComb(2,i)*bgPower(v,i) * bgPower((1.0-v),2-i);
    end;

    for j := 0 to 2 do
      for i := 0 to 2 do begin
        B := Bu[i]*Bv[j];
        vec.x := vec.x + B*PatchVecs[j][i].x;
        vec.y := vec.y + B*PatchVecs[j][i].y;
        vec.z := vec.z + B*PatchVecs[j][i].z;
      end;
    result := vec;
  end;



function TEvaluator2D.EvalNormal(u, v: float) : TVec;

  var
    v1, v2, v3 : TVec;
    f1, f2 : float;
    Normal : TVec;
  begin
    f1 := e2dPatchWidth / 100.0;
    f2 := e2dPatchHeight / 100.0;
    v1 := Evaluate(u, v-f2);
    v2 := Evaluate(u-f1, v+f2);
    v3 := Evaluate(u+f1, v+f2);
    Normal.x := v1.y*(v2.z-v3.z) + v2.y*(v3.z-v1.z) + v3.y*(v1.z-v2.z);
    Normal.y := v1.z*(v2.x-v3.x) + v2.z*(v3.x-v1.x) + v3.z*(v1.x-v2.x);
    Normal.z := v1.x*(v2.y-v3.y) + v2.x*(v3.y-v1.y) + v3.x*(v1.y-v2.y);
    VectorStandard(Normal);
    result := Normal;
  end;



procedure TEvaluator2D.SetPatchBounding(i, j : integer);

  var
    Patch : PEvaluator2DPatch;
    PatchVert : Array[0..2] of Array[0..2] of TVec;
    f : float;

  begin
    if i<0 then exit;
    if j<0 then exit;
    if i>= e2dWidthSegments then exit;
    if j>= e2dHeightSegments then exit;
    Patch := @e2dPatch[j*e2dHeightSegments + i];
    Patch.Center := Vector(0,0,0);

    PatchVert[0][0] := Patch.ControlPoint[0][0]^;
    PatchVert[0][1] := Evaluate(Patch.ControlPoint[0][1].x, Patch.ControlPoint[0][1].z);
    PatchVert[0][2] := Patch.ControlPoint[0][2]^;

    PatchVert[1][0] := Evaluate(Patch.ControlPoint[1][0].x, Patch.ControlPoint[1][0].z);
    PatchVert[1][1] := Evaluate(Patch.ControlPoint[1][1].x, Patch.ControlPoint[1][1].z);
    PatchVert[1][2] := Evaluate(Patch.ControlPoint[1][2].x, Patch.ControlPoint[1][2].z);

    PatchVert[2][0] := Patch.ControlPoint[2][0]^;
    PatchVert[2][1] := Evaluate(Patch.ControlPoint[2][1].x, Patch.ControlPoint[2][1].z);
    PatchVert[2][2] := Patch.ControlPoint[2][2]^;

    for j := 0 to 2 do
      for i := 0 to 2 do
        Patch.Center := VectorAdd(Patch.Center, PatchVert[j][i]);
    Patch.Center.x := Patch.Center.x / 9;
    Patch.Center.y := Patch.Center.y / 9;
    Patch.Center.z := Patch.Center.z / 9;

    Patch.BSRad := 0;
    for j := 0 to 2 do
      for i := 0 to 2 do begin
        f := VectorDist(Patch.Center, Patch.ControlPoint[j][i]^);
        if f>Patch.BSRad then Patch.BSRad := f;
      end;
  end;



procedure TEvaluator2D.SetBoundings;

  var
    i, j : integer;

  begin
    for j:=0 to e2dHeightSegments-1 do
      for i:=0 to e2dWidthSegments-1 do
        SetPatchBounding(i,j);
  end;



procedure TEvaluator2D.SetFlatPatchEdges(iStart, jStart, iEnd, jEnd: integer);

  var
    i, j : integer;
    Patch : PEvaluator2DPatch;

  begin
    for j:=jStart to jEnd do
      for i:=iStart to iEnd do begin
        if (i>=0) and (i<e2dWidthSegments) and (j>=0) and (j<e2dHeightSegments) then begin
          Patch := GetPatch(i, j);
          Patch.EdgeFlat := 0;
          if j>0 then
            Patch.EdgeFlat := Patch.EdgeFlat or (Ord(GetPatch(i, j-1).Flat) * $01)
          else
            Patch.EdgeFlat := Patch.EdgeFlat or $01;
          if i<e2dWidthSegments-1 then
            Patch.EdgeFlat := Patch.EdgeFlat or (Ord(GetPatch(i+1, j).Flat) * $02)
          else
            Patch.EdgeFlat := Patch.EdgeFlat or $02;
          if j<e2dHeightSegments-1 then
            Patch.EdgeFlat := Patch.EdgeFlat or (Ord(GetPatch(i, j+1).Flat) * $04)
          else
            Patch.EdgeFlat := Patch.EdgeFlat or $04;
          if i>0 then
            Patch.EdgeFlat := Patch.EdgeFlat or (Ord(GetPatch(i-1, j).Flat) * $08)
          else
            Patch.EdgeFlat := Patch.EdgeFlat or $08;
          Patch.Recompute := True;
        end;
      end;
  end;



procedure TEvaluator2D.BindToPatches(obj: TObj);

  var
    i, j, k, l : integer;
    Patch : PEvaluator2DPatch;
    BindObj : Boolean;
    rad : float;
    umin, umax,
    vmin, vmax : integer;

  begin
    // ensure that obj is removed from patches
    BindObj := False;
    for i := 0 to e2dPatchChildren-1 do begin
      if e2dPatchChild[i]=obj then begin
        k := e2dWidthSegments * e2dHeightSegments;
        for j := 0 to k-1 do begin
          Patch := @e2dPatch[j];
          for l := 0 to Patch.BoundObjects - 1 do begin
            if Patch.BoundObject[l] = obj then begin
              System.Move(Patch.BoundObject[l+1], Patch.BoundObject[l], (Patch.BoundObjects-l-1) * SizeOf(TObj));
              Dec(Patch.BoundObjects);
              break;
            end;
          end;
        end;
        BindObj := True;
        break;
      end;
    end;
    // assign obj to patch children if needed
    if BindObj then begin
      ReallocMem(e2dPatchChild, (e2dPatchChildren + 1) * SizeOf(TObj));
      ReallocMem(e2dDispChild, (e2dPatchChildren + 1) * SizeOf(TObj));
      e2dPatchChild[e2dPatchChildren] := obj;
      inc(e2dPatchChildren);
    end;
    // bind obj to correct patches
    rad := obj.objXS;
    if obj.objYS>rad then rad := obj.objYS;
    if obj.objZS>rad then rad := obj.objZS;
    rad := rad * obj.objBSRad;
    if obj.objParent<>NIL then
      obj.objParent.RemoveChild(obj);
    umin := Round( (obj.objX - rad) / e2dPatchWidth - 0.5);
    umax := Round( (obj.objX + rad) / e2dPatchWidth - 0.5);
    vmin := Round( (obj.objZ - rad) / e2dPatchHeight - 0.5);
    vmax := Round( (obj.objZ + rad) / e2dPatchHeight - 0.5);
    for j := vmin to vmax do
      for i := umin to umax do begin
        Patch := GetPatch(i, j);
        ReallocMem(Patch.BoundObject, (Patch.BoundObjects + 1) * SizeOf(TObj));
        Patch.BoundObject[Patch.BoundObjects] := obj;
        inc(Patch.BoundObjects);
      end;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  BillBoard implementation
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



constructor TBillBoard.Create;

  begin
    inherited Create;
    bbShaders := 0;
    FillChar(bbShader, SizeOf(bbShader), 0);
    bbDisableOtherTexUnits := true;

    SetDimensions(1.0, 1.0);
    FillChar(bbTexCoord, SizeOf(bbTexCoord), 0);
    bbTexCoord[0].s := 0.0;  bbTexCoord[0].t := 0.0;
    bbTexCoord[1].s := 1.0;  bbTexCoord[1].t := 0.0;
    bbTexCoord[2].s := 1.0;  bbTexCoord[2].t := 1.0;
    bbTexCoord[3].s := 0.0;  bbTexCoord[3].t := 1.0;

    bbDisableOtherTexUnits := true;
  end;



procedure TBillBoard.SetDimensions(Width, Height : float);

  begin
    bbWidth := Width;
    bbHeight := Height;
    objCent := Vector(0, 0, 0);
    objBSRad := sqrt(bbWidth*bbWidth + bbHeight*bbHeight) / 2;
    objMin := Vector(-objBSRad, -objBSRad, -objBSRad);
    objMax := Vector( objBSRad,  objBSRad,  objBSRad);
  end;



procedure TBillBoard.DrawObj;

  var
    i : integer;
    XV, YV, ZV : TVec;
    matrix : TMatrix4f;

  begin
{$IFDEF GL_13}
    if bbDisableOtherTexUnits and GL_13 then begin
      for i := GL_TEXTURE0 + bgMaxTextureUnits - 1 downto GL_TEXTURE0 do begin
        glActiveTexture(i);
        glDisable(GL_TEXTURE_1D);
        glDisable(GL_TEXTURE_2D);
        glDisable(GL_TEXTURE_3D);
        glDisable(GL_TEXTURE_CUBE_MAP);
      end;
    end;
{$ENDIF}
    for i := 0 to bbShaders - 1 do
      bbShader[i].Display;

    Matrix := MatrixMultiply(objMatrix, bgViewMatrix);
    XV := Vector(Matrix[0][0], Matrix[1][0], Matrix[2][0]);
    YV := Vector(Matrix[0][1], Matrix[1][1], Matrix[2][1]);
    ZV := Vector(Matrix[0][2], Matrix[1][2], Matrix[2][2]);
    XV := VectorScale(XV, bbWidth * 0.5);
    YV := VectorScale(YV, bbHeight * 0.5);

    glBegin(GL_QUADS);
      glNormal3fv(Addr(ZV));
      glTexCoord4fv(Addr(bbTexCoord[0]));
      glVertex3f(-XV.x - YV.x, -XV.y - YV.y, -XV.z - YV.z);
      glTexCoord4fv(Addr(bbTexCoord[1]));
      glVertex3f( XV.x - YV.x,  XV.y - YV.y,  XV.z - YV.z);
      glTexCoord4fv(Addr(bbTexCoord[2]));
      glVertex3f( XV.x + YV.x,  XV.y + YV.y,  XV.z + YV.z);
      glTexCoord4fv(Addr(bbTexCoord[3]));
      glVertex3f(-XV.x + YV.x, -XV.y + YV.y, -XV.z + YV.z);
    glEnd;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  math help routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



function ArcCos(const X: Extended): Extended;
begin
  Result := ArcTan2(Sqrt(1 - X * X), X);
end;



function ArcSin(const X: Extended): Extended;
begin
  Result := ArcTan2(X, Sqrt(1 - X * X))
end;



function ArcTan2(const Y, X: Extended): Extended;
asm
        FLD     Y
        FLD     X
        FPATAN
        FWAIT
end;



var Faculty : Array[0..6] of integer = (1,1,2,6,24,120,720);
function BinComb(n, k: integer) : integer;

  begin
    result := Faculty[n] div (Faculty[k] * Faculty[n-k]);
  end;



function bgPower(x,y : float) : float;

  begin
    if y=0.0 then
      result := 1.0
    else
      if x=0.0 then
        result := 0.0
      else
        result := exp(y*ln(x));
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  vector routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



// sets a vector
procedure VectorSet(var v : TVec; x, y, z : float);

  begin
    v.x := x;
    v.y := y;
    v.z := z;
  end;


function Vector(x, y, z : float) : TVec;

  begin
    result.x := x;
    result.y := y;
    result.z := z;
  end;


// amount of a vector
function VectorLength(v : TVec) : float;

  begin
    result := sqrt(v.x*v.x + v.y*v.y + v.z*v.z);
  end;


// makes unit length vector
function VectorStandard(var v : TVec) : float;

  var
    a : float;

  begin
    a := VectorLength(v);
    if a>0 then begin
      v.x := v.x / a;
      v.y := v.y / a;
      v.z := v.z / a;
    end
    else
      VectorSet(v, 0, 0, 1);
    result := a;
  end;


function VectorStandardVec(v : TVec) : TVec;

  var
    a : float;

  begin
    a := VectorLength(v);
    if a>0 then begin
      result.x := v.x / a;
      result.y := v.y / a;
      result.z := v.z / a;
    end
    else
      VectorSet(result, 0, 0, 1);
  end;



// transforms vector by 4x4 matrix
procedure VectorTransform(var v : TVec; var m : TMatrix4f);

  var
    x, y, z : float;

  begin
    x := v.x*m[0,0] + v.y*m[1,0] + v.z*m[2,0] + m[3,0];
    y := v.x*m[0,1] + v.y*m[1,1] + v.z*m[2,1] + m[3,1];
    z := v.x*m[0,2] + v.y*m[1,2] + v.z*m[2,2] + m[3,2];
    v.x := x;
    v.y := y;
    v.z := z;
  end;



function VectorTransformation(v : TVec; var m : TMatrix4f) : TVec;

  begin
    result.x := v.x*m[0,0] + v.y*m[1,0] + v.z*m[2,0] + m[3,0];
    result.y := v.x*m[0,1] + v.y*m[1,1] + v.z*m[2,1] + m[3,1];
    result.z := v.x*m[0,2] + v.y*m[1,2] + v.z*m[2,2] + m[3,2];
  end;



procedure VectorTransformTo(v : TVec; var t : TVec; var m : TMatrix4f);

  begin
    t.x := v.x*m[0,0] + v.y*m[1,0] + v.z*m[2,0] + m[3,0];
    t.y := v.x*m[0,1] + v.y*m[1,1] + v.z*m[2,1] + m[3,1];
    t.z := v.x*m[0,2] + v.y*m[1,2] + v.z*m[2,2] + m[3,2];
  end;



// transforms 3d Vector by first 3x3 components of 4x4 matrix
procedure VectorTransformTo3(v : TVec; var t : TVec; var m : TMatrix4f);

  begin
      t.x := v.x*m[0,0] + v.y*m[1,0] + v.z*m[2,0];
      t.y := v.x*m[0,1] + v.y*m[1,1] + v.z*m[2,1];
      t.z := v.x*m[0,2] + v.y*m[1,2] + v.z*m[2,2];
  end;


// transforms 4d vector by 4x4 matrix
procedure Vector4fTransform(var v : TVec4f; var m : TMatrix4f);

  var
    x, y, z, w : float;

  begin
    x := v[0]*m[0,0] + v[1]*m[1,0] + v[2]*m[2,0] + v[3]*m[3,0];
    y := v[0]*m[0,1] + v[1]*m[1,1] + v[2]*m[2,1] + v[3]*m[3,1];
    z := v[0]*m[0,2] + v[1]*m[1,2] + v[2]*m[2,2] + v[3]*m[3,2];
    w := v[0]*m[0,3] + v[1]*m[1,3] + v[2]*m[2,3] + v[3]*m[3,3];
    v[0] := x;
    v[1] := y;
    v[2] := z;
    v[3] := w;
  end;


// translates vector by inverse translation part of a matrix
function VectorInverseTranslate(v: TVec; var m: TMatrix4f): TVec;

  begin
      result.x := v.x - m[3,0];
      result.y := v.y - m[3,1];
      result.z := v.z - m[3,2];
  end;


// rotates vector by inverse rotation part of a matrix
function VectorInverseRotate(v: TVec; var m: TMatrix4f): TVec;

  begin
    result.x := v.x*m[0,0] + v.y*m[0,1] + v.z*m[0,2];
    result.y := v.x*m[1,0] + v.y*m[1,1] + v.z*m[1,2];
    result.z := v.x*m[2,0] + v.y*m[2,1] + v.z*m[2,2];
  end;


// computes distance between points
function VectorDist(v1, v2 : TVec) : float;

  var v : TVec;

  begin
    v.x := v1.x-v2.x;
    v.y := v1.y-v2.y;
    v.z := v1.z-v2.z;
    result := VectorLength(v);
  end;


// calculates the angle between Vector1 and Vector2
function VectorAngle(v1, v2 : TVec) : float;

  var
    v, w : float;

  begin
    v := v1.x*v2.x + v1.y*v2.y + v1.z*v2.z;
    w := (sqr(v1.x)+sqr(v1.y)+sqr(v1.z)) * (sqr(v2.x)+sqr(v2.y)+sqr(v2.z));
    result := ArcCos(v / sqrt(w));
  end;


// adds v2 to v1
function VectorAdd(v1, v2 : TVec) : TVec;

  var
    v : TVec;

  begin
    v.x := v1.x + v2.x;
    v.y := v1.y + v2.y;
    v.z := v1.z + v2.z;
    result := v;
  end;


// subtracts v2 from v1
function VectorSub(v1, v2 : TVec) : TVec;

  var
    v : TVec;

  begin
    v.x := v1.x - v2.x;
    v.y := v1.y - v2.y;
    v.z := v1.z - v2.z;
    result := v;
  end;


// negates v
procedure VectorNeg(var v : TVec);

  begin
    v.x := -v.x;
    v.y := -v.y;
    v.z := -v.z;
  end;


// makes a linear combination of two vectors and return the result
function VectorCombine(V1, V2: TVec; F1, F2: Float): TVec;

  begin
    Result.X := F1*V1.X + F2*V2.X;
    Result.Y := F1*V1.Y + F2*V2.Y;
    Result.Z := F1*V1.Z + F2*V2.Z;
  end;


// calculates the cross product between vector 1 and 2
function VectorCross(v1, v2 : TVec): TVec;

  var
    Temp : TVec;

  begin
    Temp.x := V1.y*V2.z - V1.z*V2.y;
    Temp.y := V1.z*V2.x - V1.x*V2.z;
    Temp.z := V1.x*V2.y - V1.y*V2.x;
    Result := Temp;
  end;



function VectorScale(v : TVec; f : Float) : TVec;

  var
    r : TVec;

  begin
    r.x := v.x * f;
    r.y := v.y * f;
    r.z := v.z * f;
    result := r;
  end;


// calculates the dot product of vector 1 and 2
function VectorDot(V1, V2: TVec): Float; register;

  begin
    Result := V1.x*V2.x + V1.y*V2.y + V1.z*V2.z;
  end;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  matrix routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



// converts 16 coefficients to a 4x4 Matrix
function MatrixCreate(c00,c01,c02,c03, c10,c11,c12,c13, c20,c21,c22,c23, c30,c31,c32,c33: float): TMatrix4f;

  begin
    result[0,0] := c00;
    result[0,1] := c01;
    result[0,2] := c02;
    result[0,3] := c03;

    result[1,0] := c10;
    result[1,1] := c11;
    result[1,2] := c12;
    result[1,3] := c13;

    result[2,0] := c20;
    result[2,1] := c21;
    result[2,2] := c22;
    result[2,3] := c23;

    result[3,0] := c30;
    result[3,1] := c31;
    result[3,2] := c32;
    result[3,3] := c33;
  end;



// Determinant of a 3x3 matrix
function MatrixDet3x3(a1, a2, a3, b1, b2, b3, c1, c2, c3: Float): Float;

  begin
    Result := a1 * (b2 * c3 - b3 * c2) -
              b1 * (a2 * c3 - a3 * c2) +
              c1 * (a2 * b3 - a3 * b2);
  end;



// Determinant of a 4x4 matrix
function MatrixDeterminant(M: TMatrix4f): Float;

  var
    a1, a2, a3, a4,
    b1, b2, b3, b4,
    c1, c2, c3, c4,
    d1, d2, d3, d4  : Single;

  begin
    a1 := M[0,0];  b1 := M[0,1];  c1 := M[0,2];  d1 := M[0,3];
    a2 := M[1,0];  b2 := M[1,1];  c2 := M[1,2];  d2 := M[1,3];
    a3 := M[2,0];  b3 := M[2,1];  c3 := M[2,2];  d3 := M[2,3];
    a4 := M[3,0];  b4 := M[3,1];  c4 := M[3,2];  d4 := M[3,3];

    Result := a1 * MatrixDet3x3(b2, b3, b4, c2, c3, c4, d2, d3, d4) -
              b1 * MatrixDet3x3(a2, a3, a4, c2, c3, c4, d2, d3, d4) +
              c1 * MatrixDet3x3(a2, a3, a4, b2, b3, b4, d2, d3, d4) -
              d1 * MatrixDet3x3(a2, a3, a4, b2, b3, b4, c2, c3, c4);
  end;



// Adjoint of a 4x4 matrix - used in the computation of the inverse of a 4x4 matrix
procedure MatrixAdjoint(var M: TMatrix4f);

  var
    a1, a2, a3, a4,
    b1, b2, b3, b4,
    c1, c2, c3, c4,
    d1, d2, d3, d4: Float;

  begin
      a1 :=  M[0,0]; b1 :=  M[0,1];
      c1 :=  M[0,2]; d1 :=  M[0,3];
      a2 :=  M[1,0]; b2 :=  M[1,1];
      c2 :=  M[1,2]; d2 :=  M[1,3];
      a3 :=  M[2,0]; b3 :=  M[2,1];
      c3 :=  M[2,2]; d3 :=  M[2,3];
      a4 :=  M[3,0]; b4 :=  M[3,1];
      c4 :=  M[3,2]; d4 :=  M[3,3];

      // row column labeling reversed since we transpose rows & columns
      M[0,0] :=  MatrixDet3x3(b2, b3, b4, c2, c3, c4, d2, d3, d4);
      M[1,0] := -MatrixDet3x3(a2, a3, a4, c2, c3, c4, d2, d3, d4);
      M[2,0] :=  MatrixDet3x3(a2, a3, a4, b2, b3, b4, d2, d3, d4);
      M[3,0] := -MatrixDet3x3(a2, a3, a4, b2, b3, b4, c2, c3, c4);

      M[0,1] := -MatrixDet3x3(b1, b3, b4, c1, c3, c4, d1, d3, d4);
      M[1,1] :=  MatrixDet3x3(a1, a3, a4, c1, c3, c4, d1, d3, d4);
      M[2,1] := -MatrixDet3x3(a1, a3, a4, b1, b3, b4, d1, d3, d4);
      M[3,1] :=  MatrixDet3x3(a1, a3, a4, b1, b3, b4, c1, c3, c4);

      M[0,2] :=  MatrixDet3x3(b1, b2, b4, c1, c2, c4, d1, d2, d4);
      M[1,2] := -MatrixDet3x3(a1, a2, a4, c1, c2, c4, d1, d2, d4);
      M[2,2] :=  MatrixDet3x3(a1, a2, a4, b1, b2, b4, d1, d2, d4);
      M[3,2] := -MatrixDet3x3(a1, a2, a4, b1, b2, b4, c1, c2, c4);

      M[0,3] := -MatrixDet3x3(b1, b2, b3, c1, c2, c3, d1, d2, d3);
      M[1,3] :=  MatrixDet3x3(a1, a2, a3, c1, c2, c3, d1, d2, d3);
      M[2,3] := -MatrixDet3x3(a1, a2, a3, b1, b2, b3, d1, d2, d3);
      M[3,3] :=  MatrixDet3x3(a1, a2, a3, b1, b2, b3, c1, c2, c3);
  end;



// identity matrix
function MatrixIdentity : TMatrix4f;

  begin
    result[0,0]:=1.0;  result[0,1]:=0.0;  result[0,2]:=0.0;  result[0,3]:=0.0;
    result[1,0]:=0.0;  result[1,1]:=1.0;  result[1,2]:=0.0;  result[1,3]:=0.0;
    result[2,0]:=0.0;  result[2,1]:=0.0;  result[2,2]:=1.0;  result[2,3]:=0.0;
    result[3,0]:=0.0;  result[3,1]:=0.0;  result[3,2]:=0.0;  result[3,3]:=1.0;
  end;



// returns if m is identity matrix
function MatrixIsIdentity(M: TMatrix4f) : boolean;

  begin
    if M[0,0]<>1.0 then begin result:=false; exit; end;
    if M[0,1]<>0.0 then begin result:=false; exit; end;
    if M[0,2]<>0.0 then begin result:=false; exit; end;
    if M[0,3]<>0.0 then begin result:=false; exit; end;

    if M[1,0]<>0.0 then begin result:=false; exit; end;
    if M[1,1]<>1.0 then begin result:=false; exit; end;
    if M[1,2]<>0.0 then begin result:=false; exit; end;
    if M[1,3]<>0.0 then begin result:=false; exit; end;

    if M[2,0]<>0.0 then begin result:=false; exit; end;
    if M[2,1]<>0.0 then begin result:=false; exit; end;
    if M[2,2]<>1.0 then begin result:=false; exit; end;
    if M[2,3]<>0.0 then begin result:=false; exit; end;

    if M[3,0]<>0.0 then begin result:=false; exit; end;
    if M[3,1]<>0.0 then begin result:=false; exit; end;
    if M[3,2]<>0.0 then begin result:=false; exit; end;
    if M[3,3]<>1.0 then begin result:=false; exit; end;

    result := true;
  end;



// finds the inverse of a 4x4 matrix
procedure MatrixInvert(var M: TMatrix4f);

  var
    Det : Float;

  begin
    Det := MatrixDeterminant(M);
    if Abs(Det) < nearzerovalue then
      M := MatrixIdentity
    else
      begin
        Det := 1/Det;
        MatrixAdjoint(M);
        M[0,0]:=M[0,0]*Det;  M[0,1]:=M[0,1]*Det;  M[0,2]:=M[0,2]*Det;  M[0,3]:=M[0,3]*Det;
        M[1,0]:=M[1,0]*Det;  M[1,1]:=M[1,1]*Det;  M[1,2]:=M[1,2]*Det;  M[1,3]:=M[1,3]*Det;
        M[2,0]:=M[2,0]*Det;  M[2,1]:=M[2,1]*Det;  M[2,2]:=M[2,2]*Det;  M[2,3]:=M[2,3]*Det;
        M[3,0]:=M[3,0]*Det;  M[3,1]:=M[3,1]*Det;  M[3,2]:=M[3,2]*Det;  M[3,3]:=M[3,3]*Det;
      end;
  end;



// computes a mirror matrix to a given plane (p=point, v=normal) to multiply with the current matrix
function MatrixMirrorPlane(p, v: TVec) : TMatrix4f;

  var
    dot: float;

  begin
    dot := p.x*v.x + p.y*v.y + p.z*v.z;

    result[0,0] := 1.0 - 2.0*v.x*v.x;
    result[1,0] :=     - 2.0*v.x*v.y;
    result[2,0] :=     - 2.0*v.x*v.z;
    result[3,0] := 2.0*dot*v.x;

    result[0,1] := 1.0 - 2.0*v.y*v.x;
    result[1,1] :=     - 2.0*v.y*v.y;
    result[2,1] :=     - 2.0*v.y*v.z;
    result[3,1] := 2.0*dot*v.y;

    result[0,2] := 1.0 - 2.0*v.z*v.x;
    result[1,2] :=     - 2.0*v.z*v.y;
    result[2,2] :=     - 2.0*v.z*v.z;
    result[3,2] := 2.0*dot*v.z;

    result[0,3] := 0.0;
    result[1,3] := 0.0;
    result[2,3] := 0.0;
    result[3,3] := 1.0;
  end;



// computes the shadow matrix for a given plane and light source
function MatrixShadowPlane(plane, light: TVec4f) : TMatrix4f;

  var
    dot: float;

  begin
    dot := plane[0]*light[0] + plane[1]*light[1] + plane[2]*light[2] + plane[3]*light[3];

    result[0,0] := dot - light[0]*plane[0];
    result[1,0] :=     - light[0]*plane[1];
    result[2,0] :=     - light[0]*plane[2];
    result[3,0] :=     - light[0]*plane[3];

    result[0,1] :=     - light[1]*plane[0];
    result[1,1] := dot - light[1]*plane[1];
    result[2,1] :=     - light[1]*plane[2];
    result[3,1] :=     - light[1]*plane[3];

    result[0,2] :=     - light[2]*plane[0];
    result[1,2] :=     - light[2]*plane[1];
    result[2,2] := dot - light[2]*plane[2];
    result[3,2] :=     - light[2]*plane[3];

    result[0,3] :=     - light[3]*plane[0];
    result[1,3] :=     - light[3]*plane[1];
    result[2,3] :=     - light[3]*plane[2];
    result[3,3] := dot - light[3]*plane[3];
  end;


// multiplies two 4x4 matrices
function MatrixMultiply(M1, M2: TMatrix4f): TMatrix4f;

  begin
    result[0,0] := M1[0,0]*M2[0,0] + M1[0,1]*M2[1,0] + M1[0,2]*M2[2,0] + M1[0,3]*M2[3,0];
    result[0,1] := M1[0,0]*M2[0,1] + M1[0,1]*M2[1,1] + M1[0,2]*M2[2,1] + M1[0,3]*M2[3,1];
    result[0,2] := M1[0,0]*M2[0,2] + M1[0,1]*M2[1,2] + M1[0,2]*M2[2,2] + M1[0,3]*M2[3,2];
    result[0,3] := M1[0,0]*M2[0,3] + M1[0,1]*M2[1,3] + M1[0,2]*M2[2,3] + M1[0,3]*M2[3,3];

    result[1,0] := M1[1,0]*M2[0,0] + M1[1,1]*M2[1,0] + M1[1,2]*M2[2,0] + M1[1,3]*M2[3,0];
    result[1,1] := M1[1,0]*M2[0,1] + M1[1,1]*M2[1,1] + M1[1,2]*M2[2,1] + M1[1,3]*M2[3,1];
    result[1,2] := M1[1,0]*M2[0,2] + M1[1,1]*M2[1,2] + M1[1,2]*M2[2,2] + M1[1,3]*M2[3,2];
    result[1,3] := M1[1,0]*M2[0,3] + M1[1,1]*M2[1,3] + M1[1,2]*M2[2,3] + M1[1,3]*M2[3,3];

    result[2,0] := M1[2,0]*M2[0,0] + M1[2,1]*M2[1,0] + M1[2,2]*M2[2,0] + M1[2,3]*M2[3,0];
    result[2,1] := M1[2,0]*M2[0,1] + M1[2,1]*M2[1,1] + M1[2,2]*M2[2,1] + M1[2,3]*M2[3,1];
    result[2,2] := M1[2,0]*M2[0,2] + M1[2,1]*M2[1,2] + M1[2,2]*M2[2,2] + M1[2,3]*M2[3,2];
    result[2,3] := M1[2,0]*M2[0,3] + M1[2,1]*M2[1,3] + M1[2,2]*M2[2,3] + M1[2,3]*M2[3,3];

    result[3,0] := M1[3,0]*M2[0,0] + M1[3,1]*M2[1,0] + M1[3,2]*M2[2,0] + M1[3,3]*M2[3,0];
    result[3,1] := M1[3,0]*M2[0,1] + M1[3,1]*M2[1,1] + M1[3,2]*M2[2,1] + M1[3,3]*M2[3,1];
    result[3,2] := M1[3,0]*M2[0,2] + M1[3,1]*M2[1,2] + M1[3,2]*M2[2,2] + M1[3,3]*M2[3,2];
    result[3,3] := M1[3,0]*M2[0,3] + M1[3,1]*M2[1,3] + M1[3,2]*M2[2,3] + M1[3,3]*M2[3,3];
  end;



// postmultiplies two 4x4 matrices, using only rotation/scale/translation coefficients
function MatrixPostMultiply(M1, M2: TMatrix4f): TMatrix4f;

  begin
    result[0,0] := M1[0,0]*M2[0,0] + M1[1,0]*M2[0,1] + M1[2,0]*M2[0,2];
    result[0,1] := M1[0,1]*M2[0,0] + M1[1,1]*M2[0,1] + M1[2,1]*M2[0,2];
    result[0,2] := M1[0,2]*M2[0,0] + M1[1,2]*M2[0,1] + M1[2,2]*M2[0,2];
    result[0,3] := 0.0;

    result[1,0] := M1[0,0]*M2[1,0] + M1[1,0]*M2[1,1] + M1[2,0]*M2[1,2];
    result[1,1] := M1[0,1]*M2[1,0] + M1[1,1]*M2[1,1] + M1[2,1]*M2[1,2];
    result[1,2] := M1[0,2]*M2[1,0] + M1[1,2]*M2[1,1] + M1[2,2]*M2[1,2];
    result[1,3] := 0.0;

    result[2,0] := M1[0,0]*M2[2,0] + M1[1,0]*M2[2,1] + M1[2,0]*M2[2,2];
    result[2,1] := M1[0,1]*M2[2,0] + M1[1,1]*M2[2,1] + M1[2,1]*M2[2,2];
    result[2,2] := M1[0,2]*M2[2,0] + M1[1,2]*M2[2,1] + M1[2,2]*M2[2,2];
    result[2,3] := 0.0;

    result[3,0] := M1[0,0]*M2[3,0] + M1[1,0]*M2[3,1] + M1[2,0]*M2[3,2] + M1[3,0];
    result[3,1] := M1[0,1]*M2[3,0] + M1[1,1]*M2[3,1] + M1[2,1]*M2[3,2] + M1[3,1];
    result[3,2] := M1[0,2]*M2[3,0] + M1[1,2]*M2[3,1] + M1[2,2]*M2[3,2] + M1[3,2];
    result[3,3] := 1.0;
  end;



// creates a rotation matrix around the x-axis
function MatrixRotationX(AngleX: float) : TMatrix4f;

  var
    sine, cosine: float;

  begin
    cosine := cos(AngleX * PI / 180.0);
    sine := sin(AngleX * PI / 180.0);

    result := MatrixIdentity;
    result[1,1] := cosine;
    result[1,2] := sine;
    result[2,1] := -sine;
    result[2,2] := cosine;
  end;


// creates a rotation matrix around the y-axis
function MatrixRotationY(AngleY: float) : TMatrix4f;

  var
    sine, cosine: float;

  begin
    cosine := cos(AngleY * PI / 180.0);
    sine := sin(AngleY * PI / 180.0);

    result := MatrixIdentity;
    result[0,0] := cosine;
    result[0,2] := -sine;
    result[2,0] := sine;
    result[2,2] := cosine;
  end;


// creates a rotation matrix around the z-axis
function MatrixRotationZ(AngleZ: float) : TMatrix4f;

  var
    sine, cosine: float;

  begin
    cosine := cos(AngleZ * PI / 180.0);
    sine := sin(AngleZ * PI / 180.0);

    result := MatrixIdentity;
    result[0,0] := cosine;
    result[0,1] := sine;
    result[1,0] := -sine;
    result[1,1] := cosine;
  end;


// scales a matrix by given factors
procedure MatrixScale(var M: TMatrix4f; ScaleX, ScaleY, ScaleZ: float);

  begin
    M[0,0] := M[0,0] * ScaleX;
    M[0,1] := M[0,1] * ScaleX;
    M[0,2] := M[0,2] * ScaleX;

    M[1,0] := M[1,0] * ScaleY;
    M[1,1] := M[1,1] * ScaleY;
    M[1,2] := M[1,2] * ScaleY;

    M[2,0] := M[2,0] * ScaleZ;
    M[2,1] := M[2,1] * ScaleZ;
    M[2,2] := M[2,2] * ScaleZ;
  end;



// sets translation of M to -x,-y,-z
procedure MatrixSetInverseTranslation(var M: TMatrix4f; x, y, z: float);

  begin
    M[3,0] := -x;
    M[3,1] := -y;
    M[3,2] := -z;
  end;



// sets rotation of M to inverted xa->ya->za
procedure MatrixSetInverseRotation(var M: TMatrix4f; xa, ya, za: float);

  var
    cr,sr,cp,sp,cy,sy,srsp,crsp: float;

  begin
    xa := xa * PI/180.0;
    ya := ya * PI/180.0;
    za := za * PI/180.0;

    cr := cos(xa);
    sr := sin(xa);
    cp := cos(ya);
    sp := sin(ya);
    cy := cos(za);
    sy := sin(za);

    M[0,0] := cp*cy;
    M[1,0] := cp*sy;
    M[2,0] := -sp;

    srsp := sr*sp;
    crsp := cr*sp;

    M[0,1] := srsp*cy-cr*sy;
    M[1,1] := srsp*sy+cr*cy;
    M[2,1] := sr*cp;

    M[0,2] := crsp*cy+sr*sy;
    M[1,2] := crsp*sy-sr*cy;
    M[2,2] := cr*cp;
  end;



// sets rotation of M to xa->ya->za
procedure MatrixSetRotation(var M: TMatrix4f; xa, ya, za: float);

  var
    cr,sr,cp,sp,cy,sy,srsp,crsp: float;

  begin
    xa := xa * PI/180.0;
    ya := ya * PI/180.0;
    za := za * PI/180.0;

    cr := cos(xa);
    sr := sin(xa);
    cp := cos(ya);
    sp := sin(ya);
    cy := cos(za);
    sy := sin(za);

    M[0,0] := cp*cy;
    M[0,1] := cp*sy;
    M[0,2] := -sp;

    srsp := sr*sp;
    crsp := cr*sp;

    M[1,0] := srsp*cy-cr*sy;
    M[1,1] := srsp*sy+cr*cy;
    M[1,2] := sr*cp;

    M[2,0] := crsp*cy+sr*sy;
    M[2,1] := crsp*sy-sr*cy;
    M[2,2] := cr*cp;
  end;



// sets translation of M to x, y, z
procedure MatrixSetTranslation(var M: TMatrix4f; x, y, z: float);

  begin
    M[3,0] := x;
    M[3,1] := y;
    M[3,2] := z;
  end;



// sets rotation of M by quaternion Q
procedure MatrixSetRotationQuaternion(var M: TMatrix4f; Q: TQuaternion);

  var
    xx,yy,zz,
    xy,xz,xw,
    yz,yw,
    zw : float;

  begin
    xx := Q.x * Q.x;
    xy := Q.x * Q.y;
    xz := Q.x * Q.z;
    xw := Q.x * Q.w;

    yy := Q.y * Q.y;
    yz := Q.y * Q.z;
    yw := Q.y * Q.w;

    zz := Q.z * Q.z;
    zw := Q.z * Q.w;

    // first row
    M[0,0] := 1.0 - 2.0 * (yy + zz);
    M[0,1] :=       2.0 * (xy - zw);
    M[0,2] :=       2.0 * (xz + yw);
    // second row
    M[1,0] :=       2.0 * (xy + zw);
    M[1,1] := 1.0 - 2.0 * (xx + zz);
    M[1,2] :=       2.0 * (yz - xw);
    // third row
    M[2,0] :=       2.0 * (xz - yw);
    M[2,1] :=       2.0 * (yz + xw);
    M[2,2] := 1.0 - 2.0 * (xx + yy);
  end;



// translates matrix by a given vector
function MatrixTranslation(TranslateX, TranslateY, TranslateZ: float): TMatrix4f;

  begin
    result := MatrixIdentity;
    result[3,0] := TranslateX;
    result[3,1] := TranslateY;
    result[3,2] := TranslateZ;
  end;


// returns the rotation of a matrix in xa, ya and za
procedure MatrixExtractRotation(M : TMatrix4f; var xa, ya, za : float);

  var
    row : array[0..2] of TVec;
    i   : integer;

  begin
    for i := 0 to 2 do begin
      row[i].x := M[i,0];
      row[i].y := M[i,1];
      row[i].z := M[i,2];
    end;

    // Check for a coordinate system flip.  If the determinant
    // is -1, then negate the matrix and the scaling factors.
    if VectorDot(row[0], VectorCross(row[1], row[2])) < 0 then
      for i := 0 to 2 do begin
        row[i].x := -row[i].x;
        row[i].y := -row[i].y;
        row[i].z := -row[i].z;
      end;

    // now, get the rotations out
    ya := ArcSin(-row[0].z);
    if cos(ya) <> 0 then
    begin
      xa := ArcTan2(row[1].z, row[2].z);
      za := ArcTan2(row[0].y, row[0].x);
    end
    else
    begin
      xa := arctan2(row[1].x, row[1].y);
      za := 0;
    end;

    xa := xa * (180.0 / PI);
    ya := ya * (180.0 / PI);
    za := za * (180.0 / PI);
  end;



// returns the translation of a matrix in x,y,z
procedure MatrixExtractTranslation(M: TMatrix4f; var x, y, z : float);

  begin
    x := M[3,0];
    y := M[3,1];
    z := M[3,2];
  end;



// generates a rotation matrix by a quaternion
function MatrixFromQuaternion(Q: TQuaternion): TMatrix4f;

  var
    xx,yy,zz,
    xy,xz,xw,
    yz,yw,
    zw : float;

  begin
    xx := Q.x * Q.x;
    xy := Q.x * Q.y;
    xz := Q.x * Q.z;
    xw := Q.x * Q.w;

    yy := Q.y * Q.y;
    yz := Q.y * Q.z;
    yw := Q.y * Q.w;

    zz := Q.z * Q.z;
    zw := Q.z * Q.w;

    // first row
    result[0,0] := 1.0 - 2.0 * (yy + zz);
    result[0,1] :=       2.0 * (xy - zw);
    result[0,2] :=       2.0 * (xz + yw);
    result[0,3] := 0.0;
    // second row
    result[1,0] :=       2.0 * (xy + zw);
    result[1,1] := 1.0 - 2.0 * (xx + zz);
    result[1,2] :=       2.0 * (yz - xw);
    result[1,3] := 0.0;
    // third row
    result[2,0] :=       2.0 * (xz - yw);
    result[2,1] :=       2.0 * (yz + xw);
    result[2,2] := 1.0 - 2.0 * (xx + yy);
    result[2,3] := 0.0;
    // fourth row
    result[3,0] := 0.0;
    result[3,1] := 0.0;
    result[3,2] := 0.0;
    result[3,3] := 1.0;
    // now result is a 4x4 homogeneous matrix that can be applied to an OpenGL matrix
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  quaternion routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



function QuaternionAdd(q1, q2: TQuaternion) : TQuaternion;

  begin
    result.x := q1.x + q2.x;
    result.y := q1.y + q2.y;
    result.z := q1.z + q2.z;
    result.w := q1.w + q2.w;
  end;



procedure QuaternionConjugate(var Q: TQuaternion);

  begin
    Q.x := -Q.x;
    Q.y := -Q.y;
    Q.z := -Q.z;
  end;



procedure QuaternionNormalize(var Q: TQuaternion);

  var
    n : float;

  begin
    n := sqrt(Q.x*Q.x + Q.y*Q.y + Q.z*Q.z + Q.w*Q.w);
    Q.x := Q.x / n;
    Q.y := Q.y / n;
    Q.z := Q.z / n;
    Q.w := Q.w / n;
  end;



// generates a quaternion from three euler angles
function QuaternionFromAngles(xa, ya, za: float) : TQuaternion;

  var
    angle: float;
    sr,sp,sy,cr,cp,cy,crcp,srsp: float;

  begin
    angle := -za*(PI/180.0)*0.5;
    sy := sin(angle);
    cy := cos(angle);
    angle := -ya*(PI/180.0)*0.5;
    sp := sin(angle);
    cp := cos(angle);
    angle := -xa*(PI/180.0)*0.5;
    sr := sin(angle);
    cr := cos(angle);

    crcp := cr*cp;
    srsp := sr*sp;

    result.x := sr*cp*cy-cr*sp*sy;
    result.y := cr*sp*cy+sr*cp*sy;
    result.z := crcp*sy-srsp*cy;
    result.w := crcp*cy+srsp*sy;
  end;



// generates a quaternion from a rotation matrix
function QuaternionFromMatrix(M: TMatrix4f): TQuaternion;

  var
    diagonal, scale : float;

  begin
    // find the diagonal of the matrix by adding up it's diagonal indices.
    diagonal := M[0,0] + M[1,1] + M[2,2] + 1.0;
    // check if the diagonal is greater than zero, perform "instant" calculation
    // if this is the case, else use diagonal element with greatest value
    if diagonal > 0.0 then begin
      // calculate the scale of the diagonal
      scale := sqrt(diagonal) * 2;
      // Calculate the x, y, x and w of the quaternion through the respective equation
      result.x := (M[2,1] - M[1,2]) / scale;
      result.y := (M[0,2] - M[2,0]) / scale;
      result.z := (M[1,0] - M[0,1]) / scale;
      result.w := 0.25 * scale;
    end
    else begin
      // if the first element of the diagonal is the greatest value
      if (M[0,0]>M[1,1]) and (M[0,0]>M[2,2]) then begin
        // find the scale according to the first element, and double that value
        scale := sqrt(1.0 + M[0,0] - M[1,1] - M[2,2]) * 2.0;
        // calculate the x, y, x and w of the quaternion through the respective equation
        result.x := 0.25 * scale;
        result.y := (M[1,0] + M[0,1]) / scale;
        result.z := (M[0,2] + M[2,0]) / scale;
        result.w := (M[2,1] - M[1,2]) / scale;
      end
      // else if the second element of the diagonal is the greatest value
      else if M[1,1]>M[2,2] then begin
        // find the scale according to the second element, and double that value
        scale := sqrt(1.0 + M[1,1] - M[0,0] - M[2,2]) * 2.0;
        // calculate the x, y, x and w of the quaternion through the respective equation
        result.x := (M[1,0] + M[0,1]) / scale;
        result.y := 0.25 * scale;
        result.z := (M[2,1] + M[1,2]) / scale;
        result.w := (M[0,2] - M[2,0]) / scale;
      end
      // else the third element of the diagonal is the greatest value
      else begin
        // find the scale according to the third element, and double that value
        scale := sqrt(1.0 + M[2,2] - M[0,0] - M[1,1]) * 2.0;
        // calculate the x, y, x and w of the quaternion through the respective equation
        result.x := (M[0,2] + M[2,0]) / scale;
        result.y := (M[2,1] + M[1,2]) / scale;
        result.z := 0.25 * scale;
        result.w := (M[1,0] - M[0,1]) / scale;
      end;
    end;
  end;



function QuaternionMultiply(q1, q2: TQuaternion) : TQuaternion;

  begin
    result.x :=  q1.x * q2.w + q1.y * q2.z - q1.z * q2.y + q1.w * q2.x;
    result.y := -q1.x * q2.z + q1.y * q2.w + q1.z * q2.x + q1.w * q2.y;
    result.z :=  q1.x * q2.y - q1.y * q2.x + q1.z * q2.w + q1.w * q2.z;
    result.w := -q1.x * q2.x - q1.y * q2.y - q1.z * q2.z + q1.w * q2.w;
  end;



procedure QuaternionScale(var Q: TQuaternion; s: float);

  begin
    Q.x := Q.x * s;
    Q.y := Q.y * s;
    Q.z := Q.z * s;
    Q.w := Q.w * s;
  end;



// interpolates R between Q1 und Q2 by t ranging from 0 to 1
function QuaternionSlerp(q1, q2: TQuaternion; t: float) : TQuaternion;

  var
    r,
    scale0, scale1,
    theta, sintheta : float;

  begin
    // do a dot product between q1 and q2.
    r := (q1.x*q2.x) + (q1.y*q2.y) + (q1.z*q2.z) + (q1.w*q2.w);
    // if the dot product is less than 0, the angle is greater than 90 degrees
    if (r<0.0) then begin
      // negate the second quaternion and the result of the dot product
      q2.x:=-q2.x; q2.y:=-q2.y; q2.z:=-q2.z; q2.w:=-q2.w;
      r := -r;
    end;
    // set the first and second scale for the interpolation
    scale0 := 1.0 - t;
    scale1 := t;
    // check if angle between the 2 quaternions was big enough to warrant spherical calculations
    if ((1.0-r) > 0.1) then begin
      // get the angle between the 2 quaternions, and store the sin of that angle
      theta := ArcCos(r);
      sintheta := sin(theta);
      // calculate the scale for q1 and q2, according to the angle and it's sine value
      scale0 := sin((1.0 - t)*theta) / sintheta;
      scale1 := sin(t*theta) / sintheta;
    end;
    // calculate the x, y, z and w values for the quaternion by using a special
    // form of linear interpolation for quaternions.
    result.x := (scale0 * q1.x) + (scale1 * q2.x);
    result.y := (scale0 * q1.y) + (scale1 * q2.y);
    result.z := (scale0 * q1.z) + (scale1 * q2.z);
    result.w := (scale0 * q1.w) + (scale1 * q2.w);
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  axis routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



// creates a rotation matrix from an axis rotation
function AxisRotationToMatrix(Axis: TVec; Angle: float) : TMatrix4f;

  var
      cosine,
      sine,
      Len,
      one_minus_cosine: Extended;

  begin
    Angle := Angle * PI / 180;
    sine := sin(Angle);
    cosine := cos(Angle);
    one_minus_cosine := 1 - cosine;
    Len := VectorStandard(Axis);

    if Len = 0 then Result := MatrixIdentity
               else
    begin
      Result[0, 0] := (one_minus_cosine * Axis.x * Axis.x) + cosine;
      Result[0, 1] := (one_minus_cosine * Axis.x * Axis.y) - (Axis.z * Sine);
      Result[0, 2] := (one_minus_cosine * Axis.z * Axis.x) + (Axis.y * Sine);
      Result[0, 3] := 0;

      Result[1, 0] := (one_minus_cosine * Axis.x * Axis.y) + (Axis.z * Sine);
      Result[1, 1] := (one_minus_cosine * Axis.y * Axis.y) + cosine;
      Result[1, 2] := (one_minus_cosine * Axis.y * Axis.z) - (Axis.x * Sine);
      Result[1, 3] := 0;

      Result[2, 0] := (one_minus_cosine * Axis.z * Axis.x) - (Axis.y * Sine);
      Result[2, 1] := (one_minus_cosine * Axis.y * Axis.z) + (Axis.x * Sine);
      Result[2, 2] := (one_minus_cosine * Axis.z * Axis.z) + cosine;
      Result[2, 3] := 0;

      Result[3, 0] := 0;
      Result[3, 1] := 0;
      Result[3, 2] := 0;
      Result[3, 3] := 1;
    end;
  end;



// converts three rotations around the main axii to a single axis rotation
function AxisRotation(ax,ay,az: float; var axis: TVec) : float;

  var
      Axis1, Axis2: TVec;
      M, M1, M2: TMatrix4f;
      cost, cost1,
      sint,
      s1, s2, s3: Single;
      I: Integer;

  begin
    // see if we are only rotating about a single Axis
    if Abs(ax) < NearZeroValue then
    begin
      if Abs(ay) < NearZeroValue then
      begin
        axis := Vector(0, 0, 1);
        result := az;
        Exit;
      end
      else
        if Abs(az) < NearZeroValue then
        begin
          axis := Vector(0, 1, 0);
          result := ay;
          Exit;
        end
     end
     else
       if (Abs(ay) < NearZeroValue) and
          (Abs(az) < NearZeroValue) then
       begin
         axis := Vector(1, 0, 0);
         result := ax;
         Exit;
       end;

    // make the rotation matrix
    Axis1 := Vector(1, 0, 0);
    M := AxisRotationToMatrix(Axis1, ax);

    Axis2 := Vector(0, 1, 0);
    M2 := AxisRotationToMatrix(Axis2, ay);
    M1 := MatrixMultiply(M, M2);

    Axis2 := Vector(0, 0, 1);
    M2 := AxisRotationToMatrix(Axis2, az);
    M := MatrixMultiply(M1, M2);

    cost := ((M[0, 0] + M[1, 1] + M[2, 2])-1) / 2;
    if cost < -1 then
      cost := -1
    else
      if cost > (1 - NearZeroValue) then
      begin
        // Bad Angle - this would cause a crash
        axis := Vector(1, 0, 0);
        result := 0;
        Exit;
      end;

    cost1 := 1 - cost;
    axis := Vector(sqrt((M[0, 0]-cost) / cost1),
                   sqrt((M[1, 1]-cost) / cost1),
                   sqrt((M[2, 2]-cost) / cost1));
    result := arccos(cost) * 180 / PI;

    sint := 2 * sqrt(1 - cost * cost); // This is actually 2 Sin(t)

    // Determine the proper signs
    for I := 0 to 7 do
    begin
      if (I and 1) > 1 then s1 := -1 else s1 := 1;
      if (I and 2) > 1 then s2 := -1 else s2 := 1;
      if (I and 4) > 1 then s3 := -1 else s3 := 1;
      if (Abs(s1 * axis.x * sint-M[1, 2] + M[2, 1]) < NearZeroValue) and
         (Abs(s2 * axis.y * sint-M[2, 0] + M[0, 2]) < NearZeroValue) and
         (Abs(s3 * axis.z * sint-M[0, 1] + M[1, 0]) < NearZeroValue) then
          begin
            // We found the right combination of signs
            axis.x := axis.x * s1;
            axis.y := axis.y * s2;
            axis.z := axis.z * s3;
            Exit;
          end;
    end;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  plane routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



// prepares plane by a given triangle
procedure PlanePrepareTri(vecs : PVecs; tri : TTri);

  begin
    with bgPlane do begin
      Poly[0] := vecs[tri[0]];
      Poly[1] := vecs[tri[1]];
      Poly[2] := vecs[tri[2]];
      with Normal do Begin
        x := Poly[0].y*(Poly[1].z-Poly[2].z) + Poly[1].y*(Poly[2].z-Poly[0].z) + Poly[2].y*(Poly[0].z-Poly[1].z);
        y := Poly[0].z*(Poly[1].x-Poly[2].x) + Poly[1].z*(Poly[2].x-Poly[0].x) + Poly[2].z*(Poly[0].x-Poly[1].x);
        z := Poly[0].x*(Poly[1].y-Poly[2].y) + Poly[1].x*(Poly[2].y-Poly[0].y) + Poly[2].x*(Poly[0].y-Poly[1].y);
      End;
      VectorStandard(Normal);
      OrgDist := - Normal.x*Poly[0].x - Normal.y*Poly[0].y - Normal.z*Poly[0].z;
    end;
  end;



// prepares plane by three given points
procedure PlanePrepare(p1,p2,p3 : PVec);

  begin
    with bgPlane do begin
      Poly[0] := p1^;
      Poly[1] := p2^;
      Poly[2] := p3^;
      with Normal do Begin
        x := Poly[0].y*(Poly[1].z-Poly[2].z) + Poly[1].y*(Poly[2].z-Poly[0].z) + Poly[2].y*(Poly[0].z-Poly[1].z);
        y := Poly[0].z*(Poly[1].x-Poly[2].x) + Poly[1].z*(Poly[2].x-Poly[0].x) + Poly[2].z*(Poly[0].x-Poly[1].x);
        z := Poly[0].x*(Poly[1].y-Poly[2].y) + Poly[1].x*(Poly[2].y-Poly[0].y) + Poly[2].x*(Poly[0].y-Poly[1].y);
      End;
      VectorStandard(Normal);
      OrgDist := - Normal.x*Poly[0].x - Normal.y*Poly[0].y - Normal.z*Poly[0].z;
    end;
  end;



// requires normal, OrgDist precomputed, returns if v1-->v2 intersects plane
function PlaneIntersectLine(v1, v2 : TVec; var Intersection : TVec) : boolean;

  var
    dist1, dist2 : Float;
    LineDir, Intersect : TVec;
    Numerator, Denominator, dist : Double;
    angle : Double;
    A,B,C  : TVec;

  begin
    with bgPlane do begin
      dist1 := (Normal.x * v1.x) +  // Ax +
               (Normal.y * v1.y) +  // By +
               (Normal.z * v1.z) +  // Cz +
               OrgDist;             // D

      dist2 := (Normal.x * v2.x) +  // Ax +
               (Normal.y * v2.y) +  // By +
               (Normal.z * v2.z) +  // Cz +
               OrgDist;             // D

      if dist1 * dist2 >= 0 then begin
        result := false;
        Exit;
      end;

      LineDir.x:=v2.x-v1.x; LineDir.y:=v2.y-v1.y; LineDir.z:=v2.z-v1.z;
      VectorStandard(LineDir);
      Numerator := -(Normal.x*v1.x + Normal.y*v1.y + Normal.z*v1.z + OrgDist);
      Denominator := Normal.x*LineDir.x + Normal.y*LineDir.y + Normal.z*LineDir.z;
      if Abs(Denominator) < 0.000001 then begin
        result := false;
        Exit;
      end;

      dist := Numerator / Denominator;
      Intersect.x := v1.x + LineDir.x * dist;
      Intersect.y := v1.y + LineDir.y * dist;
      Intersect.z := v1.z + LineDir.z * dist;

      A.x:=Poly[0].x-Intersect.x; A.y:=Poly[0].y-Intersect.y; A.z:=Poly[0].z-Intersect.z;
      B.x:=Poly[1].x-Intersect.x; B.y:=Poly[1].y-Intersect.y; B.z:=Poly[1].z-Intersect.z;
      C.x:=Poly[2].x-Intersect.x; C.y:=Poly[2].y-Intersect.y; C.z:=Poly[2].z-Intersect.z;
      if (abs(A.x) + abs(A.y) + abs(A.z) < 0.00001) or
         (abs(B.x) + abs(B.y) + abs(B.z) < 0.00001) or
         (abs(C.x) + abs(C.y) + abs(C.z) < 0.00001) then begin
        Intersection := Intersect;
        result := true;
        Exit;
      end;
      Angle := VectorAngle(A, B) + VectorAngle(B, C) + VectorAngle(C, A);
      if Angle >= (0.9999*2.0*PI) then begin
        Intersection := Intersect;
        result := true;
      end
      else
        result := false;
    end;
  end;


// requires normal, OrgDist precomputed, returns if y,z intersect plane
function PlaneX(y, z : Float; var x : Float) : boolean;

  var
    Intersect : TVec;
    Numerator, dist : Double;
    angle : Double;
    A,B,C  : TVec;

  begin
    with bgPlane do begin
      Numerator := -(Normal.y*y + Normal.z*z + OrgDist);
      if Abs(Normal.x) < (0.000001) then begin
        result := false;
        Exit;
      end;

      dist := Numerator / Normal.x;
      Intersect.x := dist;
      Intersect.y := y;
      Intersect.z := z;

      A.x:=Poly[0].x-Intersect.x; A.y:=Poly[0].y-Intersect.y; A.z:=Poly[0].z-Intersect.z;
      B.x:=Poly[1].x-Intersect.x; B.y:=Poly[1].y-Intersect.y; B.z:=Poly[1].z-Intersect.z;
      C.x:=Poly[2].x-Intersect.x; C.y:=Poly[2].y-Intersect.y; C.z:=Poly[2].z-Intersect.z;
      if (abs(A.x) + abs(A.y) + abs(A.z) < 0.00001) or
         (abs(B.x) + abs(B.y) + abs(B.z) < 0.00001) or
         (abs(C.x) + abs(C.y) + abs(C.z) < 0.00001) then begin
        x := Intersect.x;
        result := true;
        Exit;
      end;
      Angle := VectorAngle(A, B) + VectorAngle(B, C) + VectorAngle(C, A);
      if Angle >= (0.9999*2.0*PI) then begin
        x := Intersect.x;
        result := true;
      end
      else
        result := false;
    end;
  end;


// requires normal, OrgDist precomputed, returns if x,z intersect plane
function PlaneY(x, z : Float; var y : Float) : boolean;

  var
    Intersect : TVec;
    Numerator, dist : Double;
    angle : Double;
    A,B,C  : TVec;

  begin
    with bgPlane do begin
      Numerator := -(Normal.x*x + Normal.z*z + OrgDist);
      if Abs(Normal.y) < (0.000001) then begin
        result := false;
        Exit;
      end;

      dist := Numerator / Normal.y;
      Intersect.x := x;
      Intersect.y := dist;
      Intersect.z := z;

      A.x:=Poly[0].x-Intersect.x; A.y:=Poly[0].y-Intersect.y; A.z:=Poly[0].z-Intersect.z;
      B.x:=Poly[1].x-Intersect.x; B.y:=Poly[1].y-Intersect.y; B.z:=Poly[1].z-Intersect.z;
      C.x:=Poly[2].x-Intersect.x; C.y:=Poly[2].y-Intersect.y; C.z:=Poly[2].z-Intersect.z;
      if (abs(A.x) + abs(A.y) + abs(A.z) < 0.00001) or
         (abs(B.x) + abs(B.y) + abs(B.z) < 0.00001) or
         (abs(C.x) + abs(C.y) + abs(C.z) < 0.00001) then begin
        y := Intersect.y;
        result := true;
        Exit;
      end;
      Angle := VectorAngle(A, B) + VectorAngle(B, C) + VectorAngle(C, A);
      if Angle >= (0.9999*2.0*PI) then begin
        y := Intersect.y;
        result := true;
      end
      else
        result := false;
    end;
  end;


// requires normal, OrgDist precomputed, returns if x,y intersect plane
function PlaneZ(x, y : Float; var z : Float) : boolean;

  var
    Intersect : TVec;
    Numerator, dist : Double;
    angle : Double;
    A,B,C  : TVec;

  begin
    with bgPlane do begin
      Numerator := -(Normal.x*x + Normal.y*y + OrgDist);
      if Abs(Normal.z) < (0.000001) then begin
        result := false;
        Exit;
      end;

      dist := Numerator / Normal.z;
      Intersect.x := x;
      Intersect.y := y;
      Intersect.z := dist;

      A.x:=Poly[0].x-Intersect.x; A.y:=Poly[0].y-Intersect.y; A.z:=Poly[0].z-Intersect.z;
      B.x:=Poly[1].x-Intersect.x; B.y:=Poly[1].y-Intersect.y; B.z:=Poly[1].z-Intersect.z;
      C.x:=Poly[2].x-Intersect.x; C.y:=Poly[2].y-Intersect.y; C.z:=Poly[2].z-Intersect.z;
      if (abs(A.x) + abs(A.y) + abs(A.z) < 0.00001) or
         (abs(B.x) + abs(B.y) + abs(B.z) < 0.00001) or
         (abs(C.x) + abs(C.y) + abs(C.z) < 0.00001) then begin
        z := Intersect.z;
        result := true;
        Exit;
      end;
      Angle := VectorAngle(A, B) + VectorAngle(B, C) + VectorAngle(C, A);
      if Angle >= (0.9999*2.0*PI) then begin
        z := Intersect.z;
        result := true;
      end
      else
        result := false;
    end;
  end;


// classifies sphere against any plane
function ClassifySphere(var Center, Normal, Point : TVec; radius : float) : TSphereClassification;

  var
    distance : float;

  begin
    distance := Normal.x*Center.x + Normal.y*Center.y + Normal.z*Center.z -
                Normal.x*Point.x - Normal.y*Point.y - Normal.z*Point.z;
    if (abs(distance) < radius) then
      result := scIntersects
    else begin
      if distance>=radius then
        result := scFront
      else
        result := scBehind;
    end;
  end;


// requires normal, OrgDist precomputed, returns if sphere intersects plane
function PlaneIntersectSphere(Center : TVec; radius : float) : boolean;

  var
    distance : float;
    Offset, Intersect : TVec;
    angle  : Double;
    A,B,C  : TVec;
    Vector, EdgeNormal : TVec;

  begin
    with bgPlane do begin
      distance := Normal.x*Center.x + Normal.y*Center.y + Normal.z*Center.z + OrgDist;
      if (abs(distance) < radius) then begin
        Offset.x:=Normal.x*distance; Offset.y:=Normal.y*distance; Offset.z:=Normal.z*distance;
        Intersect.x:=Center.x-Offset.x; Intersect.y:=Center.y-Offset.y; Intersect.z:=Center.z-Offset.z;

        A.x:=Poly[0].x-Intersect.x; A.y:=Poly[0].y-Intersect.y; A.z:=Poly[0].z-Intersect.z;
        B.x:=Poly[1].x-Intersect.x; B.y:=Poly[1].y-Intersect.y; B.z:=Poly[1].z-Intersect.z;
        C.x:=Poly[2].x-Intersect.x; C.y:=Poly[2].y-Intersect.y; C.z:=Poly[2].z-Intersect.z;
        Angle := VectorAngle(A, B);
        Angle := Angle + VectorAngle(B, C);
        Angle := Angle + VectorAngle(C, A);
        if Angle >= (0.9999*2.0*PI) then begin
          result := true;
          exit;
        end
        else begin
          //////////////// Calculate Edge 1 ////////////////
          Vector.x:=Poly[1].x-Poly[0].x; Vector.y:=Poly[1].y-Poly[0].y; Vector.z:=Poly[1].z-Poly[0].z;
          EdgeNormal.x := Vector.y*Normal.z - Vector.z*Normal.y;
          EdgeNormal.y := Vector.z*Normal.x - Vector.x*Normal.z;
          EdgeNormal.z := Vector.x*Normal.y - Vector.y*Normal.x;
          VectorStandard(EdgeNormal);
          if ClassifySphere(Center, EdgeNormal, Poly[0], radius)=scBehind then begin
            result := false;
            exit;
          end;
          //////////////// Calculate Edge 2 ////////////////
          Vector.x:=Poly[2].x-Poly[1].x; Vector.y:=Poly[2].y-Poly[1].y; Vector.z:=Poly[2].z-Poly[1].z;
          EdgeNormal.x := Vector.y*Normal.z - Vector.z*Normal.y;
          EdgeNormal.y := Vector.z*Normal.x - Vector.x*Normal.z;
          EdgeNormal.z := Vector.x*Normal.y - Vector.y*Normal.x;
          VectorStandard(EdgeNormal);
          if ClassifySphere(Center, EdgeNormal, Poly[1], radius)=scBehind then begin
            result := false;
            exit;
          end;
          //////////////// Calculate Edge 3 ////////////////
          Vector.x:=Poly[0].x-Poly[2].x; Vector.y:=Poly[0].y-Poly[2].y; Vector.z:=Poly[0].z-Poly[2].z;
          EdgeNormal.x := Vector.y*Normal.z - Vector.z*Normal.y;
          EdgeNormal.y := Vector.z*Normal.x - Vector.x*Normal.z;
          EdgeNormal.z := Vector.x*Normal.y - Vector.y*Normal.x;
          VectorStandard(EdgeNormal);
          if ClassifySphere(Center, EdgeNormal, Poly[1], radius)=scBehind then begin
            result := false;
            exit;
          end;
          // We only collide if NONE of the classifications are BEHIND
          result := true;
        end;
      end
      else
        result := false;
    end;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  various help routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



function Color4f(r, g, b, a : GLfloat) : TColor4f;

  begin
    result.r := r;
    result.g := g;
    result.b := b;
    result.a := a;
  end;



// return if a point in world space is occluded by on-screen geometry
function PointIsOccluded(p: TVec) : boolean;

  var
    viewport: TVector4i;
    mvmatrix, projmatrix: TMatrix4d;
    winx, winy, winz: GLdouble;
    bufferZ: GLfloat;

  begin
    glGetIntegerv (GL_VIEWPORT, addr(viewport));
    glGetDoublev (GL_MODELVIEW_MATRIX, addr(mvmatrix));
    glGetDoublev (GL_PROJECTION_MATRIX, addr(projmatrix));
    // project world space onto screen
    gluProject(p.x, p.y, p.z, mvmatrix, projmatrix, viewport, @winx, @winy, @winz);
      // read back pixel from depth buffer
      glReadPixels(round(winx), round(winy),1,1,GL_DEPTH_COMPONENT, GL_FLOAT, @bufferZ);
    result := bufferZ < winZ;
  end;


// retunrns the average occlusion of a rectangle by on-screen geometry
function RectIsOccluded(Center: TVec; hw, hh: integer) : float;

  var
    viewport: TVector4i;
    mvmatrix, projmatrix: TMatrix4d;
    winx, winy, winz: GLdouble;
    left, bottom, width, height, i, n: integer;
    bufferZ: array[0..16383] of GLfloat;
    occ: integer;

  begin
    glGetIntegerv (GL_VIEWPORT, addr(viewport));
    glGetDoublev (GL_MODELVIEW_MATRIX, addr(mvmatrix));
    glGetDoublev (GL_PROJECTION_MATRIX, addr(projmatrix));
    // project world space onto screen
    gluProject(Center.x, Center.y, Center.z, mvmatrix, projmatrix, viewport, @winx, @winy, @winz);
      // read back pixel from depth buffer
    left := round(winx) - hw;
    bottom := round(winy) - hh;
    width := hw*2;
    height := hh*2;
    if left<viewport[0] then begin
      width := width - (viewport[0] - left);
      left := viewport[0];
    end;
    if left + width > viewport[0] + viewport[2] then begin
      width := width - (left + width - viewport[0] - viewport[2]);
    end;
    if bottom<viewport[1] then begin
      height := height - (viewport[1] - height);
      bottom := viewport[1];
    end;
    if bottom + height > viewport[1] + viewport[3] then begin
      height := height - (bottom + height - viewport[1] - viewport[3]);
    end;
    if (width>0) and (height>0) then begin
        glReadPixels(left, bottom,width,height,GL_DEPTH_COMPONENT, GL_FLOAT, @bufferZ);
      n := width * height;
      occ := 0;
      for i:=0 to n-1 do begin
        if bufferZ[i] < winz then inc(occ);
      end;
      result := occ / n;
    end
    else
      result := 1.0;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  32 bit TGA image loader
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



function LoadTGAImage(FileName : ShortString) : PRGBAImage;

  var
      Header : TTGAHeader;
      F : File;
      img : PRGBAImage;
      l : integer;
      h : byte;

  begin
    img := NIL;
    IOResult; // clear IOResult
    Assign(F, FileName);
    Reset(F, 1);
    if IOResult=0 then begin
      BlockRead(F, Header, SizeOf(Header));
      with Header do
      if (datatypecode=2) and (bitsperpixel=32) then begin
        Seek(F, SizeOf(Header) + idlength + colourmaptype * colourmaplength);
        New(img);
        img.width := width;
        img.height := height;
        l := img.width * img.height;
        GetMem(img.data, l * 4);
        BlockRead(F, img.data^, l * 4);
        repeat
          dec(l);
          with img.data[l] do begin h:=r; r:=b; b:=h; end;
        until l=0;
      end;
      Close(F);
    end
    else begin
      New(img);
      img.width := 16;
      img.height := 16;
      GetMem(img.data, 1024);
      FillChar(img.data^, 1024, $FF);
    end;
    result := img;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  RenderSettings help routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



function NewRenderSettings : TRenderSettings;

  begin
    inc(bgRenderSets);
    ReallocMem(bgRenderSet, bgRenderSets * SizeOf(TRenderSettings));
    bgRenderSet[bgRenderSets-1] := TRenderSettings.create;
    result := bgRenderSet[bgRenderSets-1];
  end;



function RemoveRenderSettings(rs : TRenderSettings) : boolean;

  var
    i : integer;
    found : boolean;

  begin
    found := false;
    for i := 0 to bgRenderSets-1 do
      if bgRenderSet[i]=rs then begin
        found := true;
        break;
      end;
    if found then begin
      move(bgRenderSet[i+1], bgRenderSet[i], (bgRenderSets - i - 1) * sizeof(TRenderSettings));
      dec(bgRenderSets);
      reallocmem(bgRenderSet, bgRenderSets * sizeof(TRenderSettings));
      rs.Free;
    end;
    result := found;
  end;



function GetRenderSettingsIndex(rs : TRenderSettings) : integer;

  var
    i : integer;

  begin
    if rs<>NIL then
      for i := 0 to bgRenderSets-1 do
        if bgRenderSet[i] = rs then begin
          result := i;
          exit;
        end;
    result := -1;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  TextureSettings help routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



function NewTextureSettings : TTextureSettings;

  begin
    inc(bgTextureSets);
    reallocmem(bgTextureSet, bgTextureSets * sizeof(TTextureSettings));
    bgTextureSet[bgTextureSets-1] := TTextureSettings.create;
    result := bgTextureSet[bgTextureSets-1];
  end;



function RemoveTextureSettings(ts : TTextureSettings) : boolean;

  var
    i : integer;
    found : boolean;

  begin
    found := false;
    for i := 0 to bgTextureSets-1 do
      if bgTextureSet[i]=ts then begin
        found := true;
        break;
      end;
    if found then begin
      move(bgTextureSet[i+1], bgTextureSet[i], (bgTextureSets - i - 1) * sizeof(TTextureSettings));
      dec(bgTextureSets);
      ReallocMem(bgTextureSet, bgTextureSets * sizeof(TTextureSettings));
      ts.Free;
    end;
    result := found;
  end;



function GetTextureSettingsIndex(ts : TTextureSettings) : integer;

  var
    i : integer;

  begin
    if ts<>NIL then
      for i := 0 to bgTextureSets-1 do
        if bgTextureSet[i] = ts then begin
          result := i;
          exit;
        end;
    result := -1;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Texture help routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



{$IFDEF GL_11}
// load bitmap and convert to texture, returning pointer to texture
function NewTexture1D(FName : ShortString; components : GLint) : TTexture1D;

  var
    image   : PRGBAImage;
    line    : PRGBALine;
    tex     : TTexture1d;

  begin
    // Load Texture
    image := LoadTGAImage(FName);
    if (not Assigned(image)) then begin
      result := NIL;
      exit;
    end;

    tex := TTexture1d.Create;
    tex.SetComponents(components);
    if image.height>0 then begin
      New(Line);
      Line.Length := image.Width;
      GetMem(Line.Data, Line.Length * SizeOf(TRGBA));
      Move(image.Data^, Line.Data^, Line.Length * SizeOf(TRGBA));
      tex.SetLine(Line);
    end;
    FreeMem(image.data);
    FreeMem(image);
    tex.Compile;

    // Save Texture in Texturelist
    inc(bgTextures);
    ReallocMem(bgTexture, bgTextures * SizeOf(TTexture));
    bgTexture[bgTextures-1] := tex;

    result := tex;
  end;
{$ENDIF}



{$IFDEF GL_11}
// load bitmap and convert to texture, returning pointer to texture
function NewTexture2D(FName : ShortString; components : GLint) : TTexture2D;

  var
    image   : PRGBAimage;
    tex     : TTexture2d;

  begin
    // Load Texture
    image := LoadTGAImage(FName);
    if (not Assigned(image)) then begin
      result := NIL;
      exit;
    end;

    tex := TTexture2d.Create;
    tex.SetComponents(components);
    tex.SetImage(image);
    tex.Compile;

    // Save Texture in Texturelist
    inc(bgTextures);
    ReallocMem(bgTexture, bgTextures * SizeOf(TTexture));
    bgTexture[bgTextures-1] := tex;

    result := tex;
  end;
{$ENDIF}


{$IFDEF GL_12}
function NewTexture3D(FName : ShortString; components : GLint) : TTexture3D;

  var
    F : File;
    tex : TTexture3d;
    box : PRGBABox;
    pixels : Array[0..31] of Array[0..31] of TRGBA;
    i : integer;

  begin
    IOResult; // clear IOResult
    Assign(F, FName);
    Reset(F, 1);
    tex := NIL;
    if IOResult=0 then begin
      tex := TTexture3d.Create;
      BlockRead(F, pixels, 4096);
      New(box);
      BlockRead(F, box.width, 4);
      BlockRead(F, box.height, 4);
      BlockRead(F, box.depth, 4);
      i := box.width * box.height * box.depth * 4;
      GetMem(box.data, i);
      BlockRead(F, box.data^, i);
      Close(F);
      tex.SetBox(box);
      tex.SetComponents(components);
      tex.Compile;

      // Save Texture in Texturelist
      inc(bgTextures);
      ReallocMem(bgTexture, bgTextures * SizeOf(TTexture));
      bgTexture[bgTextures-1] := tex;
    end;
    result := tex;
  end;
{$ENDIF}


{$IFDEF GL_13}
function NewTextureCubeMap(FName : ShortString; components : GLint) : TTextureCubeMap;

  var
    F : File;
    tex : TTextureCubeMap;
    pixels : Array[0..31] of Array[0..31] of TRGBA;
    i : integer;
    cubemap : PRGBACubeMap;

  begin
    IOResult; // clear IOResult
    Assign(F, FName);
    Reset(F, 1);
    tex := NIL;
    if IOResult=0 then begin
      tex := TTextureCubeMap.Create;
      BlockRead(F, pixels, 4096);
      New(cubemap);
      BlockRead(F, cubemap.side, 4);
      i := cubemap.side * cubemap.side * 4;
      GetMem(cubemap.positive_x, i);
      GetMem(cubemap.negative_x, i);
      GetMem(cubemap.positive_y, i);
      GetMem(cubemap.negative_y, i);
      GetMem(cubemap.positive_z, i);
      GetMem(cubemap.negative_z, i);
      BlockRead(F, cubemap.positive_x^, i);
      BlockRead(F, cubemap.negative_x^, i);
      BlockRead(F, cubemap.positive_y^, i);
      BlockRead(F, cubemap.negative_y^, i);
      BlockRead(F, cubemap.positive_z^, i);
      BlockRead(F, cubemap.negative_z^, i);
      Close(F);
      tex.SetCubeMap(cubemap);
      tex.SetComponents(components);
      tex.Compile;

      // Save Texture in Texturelist
      inc(bgTextures);
      ReallocMem(bgTexture, bgTextures * SizeOf(TTexture));
      bgTexture[bgTextures-1] := tex;
    end;
    result := tex;
  end;
{$ENDIF}


{$IFDEF GL_13}
function NewTextureCubemapFromTGAs(N1,N2,N3,N4,N5,N6: ShortString; components : GLint) : TTextureCubeMap;

  var
    cubemap : PRGBACubeMap;
    i1,i2,i3,i4,i5,i6: PRGBAImage;
    tex: TTextureCubemap;

  begin
    i1 := LoadTGAImage(N1);
    i2 := LoadTGAImage(N2);
    i3 := LoadTGAImage(N3);
    i4 := LoadTGAImage(N4);
    i5 := LoadTGAImage(N5);
    i6 := LoadTGAImage(N6);
    if not (Assigned(i1) and Assigned(i2) and Assigned(i3) and Assigned(i4) and Assigned(i5) and Assigned(i6)) then begin
      if Assigned(i1) then begin FreeMem(i1.data); FreeMem(i1); end;
      if Assigned(i2) then begin FreeMem(i2.data); FreeMem(i2); end;
      if Assigned(i3) then begin FreeMem(i3.data); FreeMem(i3); end;
      if Assigned(i4) then begin FreeMem(i4.data); FreeMem(i4); end;
      if Assigned(i5) then begin FreeMem(i5.data); FreeMem(i5); end;
      if Assigned(i6) then begin FreeMem(i6.data); FreeMem(i6); end;
      result := NIL;
      exit;
    end;
    if (i1.width<>i2.width) or (i1.width<>i3.width) or (i1.width<>i4.width) or (i1.width<>i5.width) or (i1.width<>i6.width) or
       (i1.width<>i1.height) or (i2.width<>i2.height) or (i3.width<>i3.height) or (i4.width<>i4.height) or (i5.width<>i5.height) or (i6.width<>i6.height) then
    begin
      FreeMem(i1.data); FreeMem(i1);
      FreeMem(i2.data); FreeMem(i2);
      FreeMem(i3.data); FreeMem(i3);
      FreeMem(i4.data); FreeMem(i4);
      FreeMem(i5.data); FreeMem(i5);
      FreeMem(i6.data); FreeMem(i6);
      result := NIL;
      exit;
    end;
    tex := TTextureCubeMap.Create;
    New(cubemap);
    cubemap.side := i1.width;
    cubemap.positive_x:=i1.data; FreeMem(i1);
    cubemap.negative_x:=i2.data; FreeMem(i2);
    cubemap.positive_y:=i3.data; FreeMem(i3);
    cubemap.negative_y:=i4.data; FreeMem(i4);
    cubemap.positive_z:=i5.data; FreeMem(i5);
    cubemap.negative_z:=i6.data; FreeMem(i6);

    tex.SetCubeMap(cubemap);
    tex.SetComponents(components);
    tex.Compile;

    // Save Texture in Texturelist
    inc(bgTextures);
    ReallocMem(bgTexture, bgTextures * SizeOf(TTexture));
    bgTexture[bgTextures-1] := tex;
    result := tex;
  end;
{$ENDIF}


{$IFDEF GL_11}
function NewDynamicTexture2D(width, height: integer; components : GLint) : TDynamicTexture2D;

  var
    tex : TDynamicTexture2d;

  begin
    tex := TDynamicTexture2d.Create;
    tex.SetComponents(components);
    tex.Rescale(width, height);
    tex.Compile;

    // Save Texture in Texturelist
    inc(bgTextures);
    ReallocMem(bgTexture, bgTextures * SizeOf(TTexture));
    bgTexture[bgTextures-1] := tex;
    bgDynTexture[bgDynTextures] := tex;
    inc(bgDynTextures);

    result := tex;
  end;
{$ENDIF}


{$IFDEF GL_13}
function NewDynamicTextureCubeMap(side: integer; components : GLint) : TDynamicTextureCubeMap;

  var
    tex : TDynamicTextureCubeMap;

  begin
    tex := TDynamicTextureCubeMap.Create;
    tex.SetComponents(components);
    tex.Rescale(side);
    tex.Compile;

    // Save Texture in Texturelist
    inc(bgTextures);
    ReallocMem(bgTexture, bgTextures * SizeOf(TTexture));
    bgTexture[bgTextures-1] := tex;
    bgDynTexture[bgDynTextures] := tex;
    inc(bgDynTextures);

    result := tex;
  end;
{$ENDIF}


function NewTextureDefault : TTextureDefault;

  var
    tex : TTextureDefault;

  begin
    tex := TTextureDefault.Create;

    // Save Texture in Texturelist
    inc(bgTextures);
    ReallocMem(bgTexture, bgTextures * SizeOf(TTexture));
    bgTexture[bgTextures-1] := tex;
    result := tex;
  end;



// removes texture with a given handle
function RemoveTexture(t : TTexture) : Boolean;

  var
    i : integer;
    l, f: integer;
    found : boolean;
    tex : TTexture;

  begin
    found := false;
    for i := 0 to bgTextures-1 do
      if bgTexture[i] = t then begin
        found := true;
        break;
      end;
    if found then begin
      tex := bgTexture[i];
      for l:=0 to MAXLIGHTS-1 do begin
        f := 0;
        while f<bgLightFlare[l].Flares do begin
          if bgLightFlare[l].Flare[f].Tex=tex then begin
            Move(bgLightFlare[l].Flare[f+1], bgLightFlare[l].Flare[f], (bgLightFlare[l].Flares - f - 1) * sizeof(TLightFlare));
            dec(bgLightFlare[l].Flares);
          end
          else
            inc(f);
        end;
      end;
      Move(bgTexture[i+1], bgTexture[i], (bgTextures - i - 1) * sizeof(TTexture));
      Dec(bgTextures);
      ReallocMem(bgTexture, bgTextures * sizeof(TTexture));
    end;

    found := false;
    for i := 0 to bgDynTextures-1 do
      if bgDynTexture[i] = t then begin
        found := true;
        break;
      end;
    if found then begin
      Move(bgDynTexture[i+1], bgDynTexture[i], (bgDynTextures - i - 1) * sizeof(TTexture));
      Dec(bgDynTextures);
    end;

    if t<>NIL then t.Free;
    result := found;
  end;



function GetTextureIndex(t : TTexture) : integer;

  var
    i : integer;

  begin
    if t<>NIL then
      for i := 0 to bgTextures-1 do
        if bgTexture[i] = t then begin
          result := i;
          exit;
        end;
    result := -1;
  end;



procedure UpdateDynamicTextures;

  var
    i : integer;
    tex : TTexture;

  begin
    for i := 0 to bgDynTextures-1 do begin
      tex := bgDynTexture[i];
      if tex<>NIL then begin
        case tex.GetIdentity of
          SD_TEX_DYN_2D   : TDynamicTexture2d(tex).Compile;
{$IFDEF GL_13}
          SD_TEX_DYN_CUBE : TDynamicTextureCubemap(tex).Compile;
{$ENDIF}
        end;
      end;
    end;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Material help routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



function NewMaterial : TMaterial;

  begin
    inc(bgMaterials);
    reallocmem(bgMaterial, bgMaterials * sizeof(TMaterial));
    bgMaterial[bgMaterials-1] := TMaterial.Create;
    result := bgMaterial[bgMaterials-1];
  end;



function RemoveMaterial(m : TMaterial) : boolean;

  var
    i : integer;
    found : boolean;

  begin
    found := false;
    for i := 0 to bgMaterials-1 do
      if bgMaterial[i]=m then begin
        found := true;
        break;
      end;
    if found then begin
      Move(bgMaterial[i+1], bgMaterial[i], (bgMaterials - i - 1) * sizeof(TMaterial));
      Dec(bgMaterials);
      ReallocMem(bgMaterial, bgMaterials * sizeof(TMaterial));
      m.Free;
    end;
    result := found;
  end;



function GetMaterialIndex(m : TMaterial) : integer;

  var
    i : integer;

  begin
    if m<>NIL then
      for i := 0 to bgMaterials-1 do
        if bgMaterial[i] = m then begin
          result := i;
          exit;
        end;
    result := -1;
  end;



{$IFDEF GL_PROGRAM}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  vertex program help routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



function NewVertexProgram : TVertexProgram;

  begin
    inc(bgVertexPrograms);
    reallocmem(bgVertexProgram, bgVertexPrograms * sizeof(TVertexProgram));
    bgVertexProgram[bgVertexPrograms-1] := TVertexProgram.Create;
    result := bgVertexProgram[bgVertexPrograms-1];
  end;



function RemoveVertexProgram(vp : TVertexProgram) : boolean;

  var
    i : integer;
    found : boolean;

  begin
    found := false;
    for i := 0 to bgVertexPrograms-1 do
      if bgVertexProgram[i]=vp then begin
        found := true;
        break;
      end;
    if found then begin
      Move(bgVertexProgram[i+1], bgVertexProgram[i], (bgVertexPrograms - i - 1) * sizeof(TVertexProgram));
      Dec(bgVertexPrograms);
      ReallocMem(bgVertexProgram, bgVertexPrograms * sizeof(TVertexProgram));
      vp.Free;
    end;
    result := found;
  end;



function GetVertexProgramIndex(vp : TVertexProgram) : integer;

  var
    i : integer;

  begin
    if vp<>NIL then
      for i := 0 to bgVertexPrograms-1 do
        if bgVertexProgram[i] = vp then begin
          result := i;
          exit;
        end;
    result := -1;
  end;



procedure UpdateVPEnvParams;

  var
    i: integer;

  begin
    if GL_PROGRAM then begin
      for i:=0 to bgVPEnvParams-1 do
        if bgVPEnvParam[i].Index>=0 then
          glProgramEnvParameter4fvARB(GL_VERTEX_PROGRAM_ARB, bgVPEnvParam[i].Index, addr(bgVPEnvParam[i].Param));
      for i:=0 to bgProgramMatrices do
        if bgProgramMatrix[i].Index>=0 then begin
          glMatrixMode(GL_MATRIX0_ARB + bgProgramMatrix[i].Index);
          if bgProgramMatrix[i].bound<>NIL then
            bgProgramMatrix[i].Matrix := MatrixMultiply(bgViewMatrix, bgProgramMatrix[i].Bound.objMatrix);
          glLoadMatrixf(Addr(bgProgramMatrix[i].Matrix));
        end;
      glMatrixMode(GL_MODELVIEW);
    end;
  end;



procedure SetVPEnvParam(index: integer; a,b,c,d: float);

  var
    i: integer;

  begin
    for i:=0 to bgVPEnvParams-1 do begin
      if bgVPEnvParam[i].Index=index then with bgVPEnvParam[i] do begin
        Param[0] := a;
        Param[1] := b;
        Param[2] := c;
        Param[3] := d;
        exit;
      end;
    end;
    with bgVPEnvParam[bgVPEnvParams] do begin
      Param[0] := a;
      Param[1] := b;
      Param[2] := c;
      Param[3] := d;
    end;
    inc(bgVPEnvParams);
  end;



procedure SetVPEnvParamv(index: integer; p: TVec4f);

  var
    i: integer;

  begin
    for i:=0 to bgVPEnvParams-1 do begin
      if bgVPEnvParam[i].Index=index then with bgVPEnvParam[i] do begin
        Param := p;
        exit;
      end;
    end;
    bgVPEnvParam[bgVPEnvParams].Param := p;
    inc(bgVPEnvParams);
  end;



procedure SetProgramMatrix(index: integer; m: TMatrix4f);

  var
    i: integer;

  begin
    for i:=0 to bgProgramMatrices-1 do begin
      if bgProgramMatrix[i].Index=index then with bgProgramMatrix[i] do begin
        Matrix := m;
        bound := NIL;
        exit;
      end;
    end;
    bgProgramMatrix[bgProgramMatrices].Matrix := m;
    bgProgramMatrix[bgProgramMatrices].bound := NIL;
    inc(bgProgramMatrices);
  end;



procedure BindProgramMatrix(index: integer; obj: TObj);

  var
    i: integer;

  begin
    for i:=0 to bgProgramMatrices-1 do begin
      if bgProgramMatrix[i].Index=index then with bgProgramMatrix[i] do begin
        bound := obj;
        exit;
      end;
    end;
    bgProgramMatrix[bgProgramMatrices].bound := obj;
    inc(bgProgramMatrices);
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  fragment program help routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



function NewFragmentProgram : TFragmentProgram;

  begin
    inc(bgFragmentPrograms);
    reallocmem(bgFragmentProgram, bgFragmentPrograms * sizeof(TFragmentProgram));
    bgFragmentProgram[bgFragmentPrograms-1] := TFragmentProgram.Create;
    result := bgFragmentProgram[bgFragmentPrograms-1];
  end;



function RemoveFragmentProgram(fp : TFragmentProgram) : boolean;

  var
    i : integer;
    found : boolean;

  begin
    found := false;
    for i := 0 to bgFragmentPrograms-1 do
      if bgFragmentProgram[i]=fp then begin
        found := true;
        break;
      end;
    if found then begin
      Move(bgFragmentProgram[i+1], bgFragmentProgram[i], (bgFragmentPrograms - i - 1) * sizeof(TFragmentProgram));
      Dec(bgFragmentPrograms);
      ReallocMem(bgFragmentProgram, bgFragmentPrograms * sizeof(TFragmentProgram));
      fp.Free;
    end;
    result := found;
  end;



function GetFragmentProgramIndex(fp : TFragmentProgram) : integer;

  var
    i : integer;

  begin
    if fp<>NIL then
      for i := 0 to bgFragmentPrograms-1 do
        if bgFragmentProgram[i] = fp then begin
          result := i;
          exit;
        end;
    result := -1;
  end;



procedure UpdateFPEnvParams;

  var
    i: integer;

  begin
    if GL_PROGRAM then
      for i:=0 to bgFPEnvParams-1 do
        if bgFPEnvParam[i].Index>=0 then
          glProgramEnvParameter4fvARB(GL_FRAGMENT_PROGRAM_ARB, bgFPEnvParam[i].Index, addr(bgFPEnvParam[i].Param));
  end;



procedure SetFPEnvParam(index: integer; a,b,c,d: float);

  var
    i: integer;

  begin
    for i:=0 to bgFPEnvParams-1 do begin
      if bgFPEnvParam[i].Index=index then with bgFPEnvParam[i] do begin
        Param[0] := a;
        Param[1] := b;
        Param[2] := c;
        Param[3] := d;
        exit;
      end;
    end;
    with bgFPEnvParam[bgFPEnvParams] do begin
      Param[0] := a;
      Param[1] := b;
      Param[2] := c;
      Param[3] := d;
    end;
    inc(bgFPEnvParams);
  end;



procedure SetFPEnvParamv(index: integer; p: TVec4f);

  var
    i: integer;

  begin
    for i:=0 to bgFPEnvParams-1 do begin
      if bgFPEnvParam[i].Index=index then with bgFPEnvParam[i] do begin
        Param := p;
        exit;
      end;
    end;
    bgFPEnvParam[bgFPEnvParams].Param := p;
    inc(bgFPEnvParams);
  end;
{$ENDIF}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  obj help routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



// returns obj with corresponding handle
function GetObj(handle : GLuint) : TObj;

  begin
    if (handle>0) and (handle<=GLuint(bgObjects)) then
      result := bgObject[handle-1]
    else
      result := NIL;
  end;



// add Obj in show list
procedure ShowObj(b:TObj);

  var
    l : PObjList;

  begin
    if b=NIL then exit;
    if b.GetParent<>NIL then exit;
    if bgObjList<>NIL then begin   // run through list if initialized
      l := bgObjList;
      while (l.next<>NIL) and (l.me<>b) do // find last element
        l := l.next;
      if l.me = b then exit;     // leave at once if b already in list
      new(l.next);               // define new element
      l := l.next;               // l points to last element
    end
    else begin                   // build list if not initialized
      new(bgObjList) ;             // define first element
      l := bgObjList;              // l points to last (and only) element
    end;
    l.me := b;                   // set actual Obj
    l.next := NIL;               // mark list end
  end;



// remove b from show list
procedure HideObj(b:TObj);

  var
    l,h : PObjList;

  begin
    if bgObjList<>NIL then  // don't do anything if list not initialized
      if bgObjList.me = b then begin
        l := bgObjList;
        bgObjList := bgObjList.next;
        dispose(l);
      end
      else begin
        l := bgObjlist;
        while l.next <> NIL do begin
          if l.next.me=b then begin
            h := l.next;
            l.next := l.next.next;
            dispose(h);
            exit;
          end; // if
        end; // while
      end; // else
  end;



// binds light to obj
procedure BindLight(index : integer; obj : TObj);

  begin
    bgBoundLight[index] := obj;
  end;



// binds spot light to obj
procedure BindSpotLight(index : integer; obj : TObj);

  begin
    bgBoundSpotLight[index] := obj;
  end;



// binds view to obj
procedure BindView(obj : TObj);

  begin
    bgBoundView := obj;
  end;



// return the root of the object list
function DisplayedList : PObjList;

  begin
    DisplayedList := bgObjList;
  end;



// read an OpenGL primitive out of a file
procedure ReadPrimitive(var F : File);

  var
    token : byte;
    mode  : GLenum;
    data  : array[0..3] of float;
    index,
    texunit,
    texcomponents : integer;

  begin
    repeat
      BlockRead(F, token, 1);
      case token of
        TOKEN_BEGIN : begin
                        BlockRead(F, mode, 4);
                        glBegin(mode);
                      end;
        TOKEN_V3F   : begin
                        BlockRead(F, data, 12);
                        glVertex3fv(Addr(data));
                      end;
        TOKEN_V4F   : begin
                        BlockRead(F, data, 16);
                        glVertex4fv(Addr(data));
                      end;
        TOKEN_N3F   : begin
                        BlockRead(F, data, 12);
                        glNormal3fv(Addr(data));
                      end;
        TOKEN_C3F   : begin
                        BlockRead(F, data, 12);
                        glColor3fv(Addr(data));
                      end;
        TOKEN_C4F   : begin
                        BlockRead(F, data, 16);
                        glColor4fv(Addr(data));
                      end;
        TOKEN_F1F   : begin
                        BlockRead(F, data, 4);
                        {$IFDEF GL_14} if GL_14 then
                        glFogCoordf(data[0]);
                        {$ENDIF}
                      end;
        TOKEN_A04F..TOKEN_A314F : begin
                        index := token - TOKEN_A04F;
                        BlockRead(F, data, 16);
                        {$IFDEF GL_PROGRAM} if GL_PROGRAM then
                        glVertexAttrib4fvARB(index, Addr(data));
                        {$ENDIF}
                      end;
        TOKEN_U0T1F..TOKEN_U31T4F : begin
                        texunit := (token - TOKEN_U0T1F) shr 2;
                        texcomponents := (token and $03) + 1;
                        BlockRead(F, data, texcomponents * 4);
                        if texunit=0 then begin
                          case texcomponents of
                            1 : glTexCoord1f(data[0]);
                            2 : glTexCoord2fv(Addr(data));
                            3 : glTexCoord3fv(Addr(data));
                            4 : glTexCoord4fv(Addr(data));
                          end;
                        end;
                        {$IFDEF GL_13}
                        if GL_13 and (texunit>0) then begin
                          case texcomponents of
                            1 : glMultiTexCoord1f(GL_TEXTURE0 + texunit, data[0]);
                            2 : glMultiTexCoord2fv(GL_TEXTURE0 + texunit, Addr(data));
                            3 : glMultiTexCoord3fv(GL_TEXTURE0 + texunit, Addr(data));
                            4 : glMultiTexCoord4fv(GL_TEXTURE0 + texunit, Addr(data));
                          end;
                        end;
                        {$ENDIF}
                      end;
        TOKEN_END   : glEnd;
      end;
    until (token=TOKEN_END) or (token=TOKEN_NONE);
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  display routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



// animate all objects
procedure bgAnimate(TimeDifference: integer);

  var
    i: integer;

  begin
    for i:=0 to bgObjAnimations-1 do
      bgObjAnimation[i].me.Animate(bgObjAnimation[i].AnimationIndex, TimeDifference);
  end;


// set window background color
procedure bgSetBackColor(r,g,b,a : GLfloat);

  begin
    bgBackColor.r := r;
    bgBackColor.g := g;
    bgBackColor.b := b;
    bgBackColor.a := a;
    glClearColor(r, g, b, a);    // Clear The Background Color
  end;



// clear screen and zbuffer
procedure bgClearScreen;

  begin
    glClear(bgClearBufferBits);  // clear used OpenGL buffers
  end;



// delete the rendering context, destroy the window and quit
procedure bgCloseWindow;

  begin
    glDeleteLists(bgViewInvertedTextureMatrixList, 1);
    ChangeDisplaySettings(DEVMODE(nil^), 0);
    wglMakeCurrent(bgDC, 0);
    wglDeleteContext(bgRC);
    ReleaseDC(bgWND,bgDC);
    DestroyWindow(bgWND);
    PostQuitMessage(0);
    bgRun := False;
  end;



// make sure that the bg window is displayed and has the focus
procedure bgDisplayWindow;

  begin
    ShowWindow(bgWND, SW_SHOW);
    UpdateWindow(bgWND);
    SetFocus(bgWND);
  end;



// draw or clear the background
procedure bgDrawBackground;

  begin
    if bgBoundSkyBox<>NIL then begin
      glClear(bgClearBufferBits AND NOT (GL_COLOR_BUFFER_BIT));
      glLoadIdentity;
      case bgBoundSkyBox.GetIdentity of
        SD_TEX_CUBE:     {$IFDEF GL_13} if GL_13 then TTextureCubeMap(bgBoundSkyBox).DisplaySkyBox(bgZNear*4) {$ENDIF};
        SD_TEX_DYN_CUBE: {$IFDEF GL_13} if GL_13 then TDynamicTextureCubeMap(bgBoundSkyBox).DisplaySkyBox(bgZNear*4) {$ENDIF};
        else glClear(GL_COLOR_BUFFER_BIT);
      end;
    end
    else
      glClear(bgClearBufferBits);
  end;



// draw all objects of the OpenGL scene
procedure bgDrawObjects;

  var
    l : PObjList;
    i, j : integer;
    LightPos : TVec4f;
    v : TVec;
    f : float;
    CanDisplay: boolean;

  begin
    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixf(Addr(bgViewMatrix));
    for i := 0 to MAXLIGHTS - 1 do begin
      if bgBoundLight[i]<>NIL then begin
        v.x := 0; v.y := 0; v.z := 0;
        bgBoundLight[i].Transform(v);
        LightPosition(i, v.x, v.y, v.z);
      end;
      if bgBoundSpotLight[i]<>NIL then begin
        v.x := 0;
        v.y := 0;
        if bgBoundSpotLight[i]=bgBoundLight[i] then
          v.z := -1
        else
          v.z := 0;
        bgBoundSpotLight[i].Transform(v);
        v.x := v.x - bgLight[i].Position[0];
        v.y := v.y - bgLight[i].Position[1];
        v.z := v.z - bgLight[i].Position[2];
        VectorStandard(v);
        LightSpotDirection(i, v.x, v.y, v.z);
      end;
    end;

    bgTransparentObjects := 0;
    bgShadowObjects := 0;
    l := bgObjList;
    while l <> NIL do begin
      l.me.Display;
      l := l.next;
    end;

    if bgDrawShadows and (bgShadowObjects > 0) then begin
      i := 0;
      while i < bgShadowObjects do begin
        VectorTransformTo(bgShadowTable[i].objCent, v, bgShadowTable[i].objMatrix);
        VectorTransformTo(v, bgShadowTable[i].objMesh.meshPosition, bgViewMatrix);
        if VectorLength(bgShadowTable[i].objMesh.meshPosition) > bgShadowViewDist then begin
          dec(bgShadowObjects);
          if i<bgShadowObjects-1 then
            Move(bgShadowTable[i+1], bgShadowTable[i], (bgShadowObjects - i) * SizeOf(TObj));
        end
        else
          inc(i);
      end;

      if (bgShadowObjects > 0) then begin
        glPushAttrib(GL_COLOR_BUFFER_BIT + GL_DEPTH_BUFFER_BIT + GL_ENABLE_BIT + GL_POLYGON_BIT + GL_STENCIL_BUFFER_BIT);
        glDisable(GL_LIGHTING);
        glDepthMask(FALSE);
        glDepthFunc(GL_LESS);
        glEnable(GL_STENCIL_TEST);
        glDisable(GL_POLYGON_STIPPLE);
        for j := bgMaxTextureUnits - 1 downto 0 do begin
          {$IFDEF GL_13} if GL_13 then glActiveTexture(GL_TEXTURE0 + j); {$ENDIF}
          glDisable(GL_TEXTURE_1D);
          glDisable(GL_TEXTURE_2D);
          {$IFDEF GL_12} if GL_12 then glDisable(GL_TEXTURE_3D); {$ENDIF}
          {$IFDEF GL_13} if GL_13 then glDisable(GL_TEXTURE_CUBE_MAP); {$ENDIF}
          glDisable(GL_TEXTURE_GEN_S);
          glDisable(GL_TEXTURE_GEN_T);
          glDisable(GL_TEXTURE_GEN_R);
          glDisable(GL_TEXTURE_GEN_Q);
        end;

        for j := 0 to MAXLIGHTS - 1 do begin
          if bgLight[j].shadowing then begin
            glEnable(GL_DEPTH_TEST);
            glStencilFunc(GL_ALWAYS, bgShadowStencilStart, Cardinal($ffffffff));
            glColorMask(FALSE, FALSE, FALSE, FALSE);

            glClear(GL_STENCIL_BUFFER_BIT);
            glGetLightfv(GL_LIGHT0 + j, GL_POSITION, Addr(LightPos));
            v.x := LightPos[0];
            v.y := LightPos[1];
            v.z := LightPos[2];

            for i := 0 to bgShadowObjects-1 do begin
              glLoadMatrixf(Addr(bgViewMatrix));
              bgShadowTable[i].MultGLMatrix;
              case bgShadowTable[i].objShadowState.State of
                Shadowing : bgShadowTable[i].objMesh.CastShadow(v, bgShadowTable[i].objShadowState.SelfShadowing);
                else        bgShadowTable[i].objMesh.CastFastShadow(v, bgShadowTable[i].objShadowState.SelfShadowing);
              end;
            end;
            //draw a shadowing rectangle covering the entire screen
            glColorMask(TRUE, TRUE, TRUE, TRUE);
            glColor4fv(Addr(bgLight[j].shadow));
            glEnable(GL_BLEND);
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
            glStencilFunc(GL_NOTEQUAL, 0, Cardinal($ffffffff));
            glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
            glLoadIdentity;
            glDisable(GL_DEPTH_TEST);

            f := bgZNear * 4.0;
            glBegin(GL_TRIANGLE_STRIP);
              glVertex3f(-f, f, -bgZNEAR);
              glVertex3f(-f,-f, -bgZNEAR);
              glVertex3f( f, f, -bgZNEAR);
              glVertex3f( f,-f, -bgZNEAR);
            glEnd;
          end;
        end;
        glColor3f(1.0, 1.0, 1.0);
        glDisable(GL_STENCIL_TEST);
        glPopAttrib;
      end;
    end;

    if bgTransparentObjects > 0 then begin
      While bgTransparentList <> NIL do begin
        glLoadMatrixf(Addr(bgViewMatrix));
        bgTransparentList.me.MultGLMatrix;
        CanDisplay := true;
        if Assigned(bgTransparentList.me.objOnBeforeDrawObj) then
          bgTransparentList.me.objOnBeforeDrawObj(bgTransparentList.me, CanDisplay);
        if CanDisplay then begin
          if bgTransparentList.me.objTransparency = Transparent then
            bgTransparentList.me.DrawObj
          else begin
            glEnable(GL_CULL_FACE);
            glCullFace(GL_FRONT);
            bgTransparentList.me.DrawObj;
            glCullFace(GL_BACK);
            bgTransparentList.me.DrawObj;
          end;
          if Assigned(bgTransparentList.me.objOnAfterDrawObj) then
            bgTransparentList.me.objOnAfterDrawObj(bgTransparentList.me);
        end;
        bgTransparentList := bgTransparentList.next;
      end;
    end;

  end;



// draw the whole scene and swap buffers
procedure bgDrawScene;

  begin
    if bgBoundView <> NIL then bgBoundView.SetAsCamera;
    bgSetMatrices;
    if bgDynamicTextures then UpdateDynamicTextures;
    bgDrawBackground;
{$IFDEF GL_PROGRAM}
    if GL_PROGRAM then begin
      UpdateVPEnvParams;
      UpdateFPEnvParams;
    end;
{$ENDIF}
    bgDrawObjects;
    if bgLightFlares then LightDrawFlares;
    bgSwapBuffers;
  end;



// try to set fast OpenGL hints
procedure bgFastHint;
begin
  if Assigned(glHint) then begin
    glHint(GL_FOG_HINT, GL_FASTEST);
    glHint(GL_LINE_SMOOTH_HINT, GL_FASTEST);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
    glHint(GL_POINT_SMOOTH_HINT, GL_FASTEST);
    glHint(GL_POLYGON_SMOOTH_HINT, GL_FASTEST);
    {$IFDEF GL_13} if GL_13 then glHint(GL_TEXTURE_COMPRESSION_HINT, GL_FASTEST); {$ENDIF}
    {$IFDEF GL_14} if GL_14 then glHint(GL_GENERATE_MIPMAP_HINT, GL_FASTEST); {$ENDIF}
  end;
end;



// this will be called right after the GL window is created
procedure bgInitGL(Width: GLsizei; Height: GLsizei);

  var
    i : integer;
    S : String;

  begin
    glGetIntegerv(GL_MAX_TEXTURE_SIZE, @bgMaxTextureSize);
    {$IFDEF GL_13} if GL_13 then glGetIntegerv(GL_MAX_TEXTURE_UNITS, @bgMaxTextureUnits); {$ENDIF}

    for i := 1 to ParamCount do begin
      S := ParamStr(i);
      if (S='FAST') then bgFastHint;
      if (S='NICE') then bgNiceHint;
      if (S='TEX32') then bgMaxTextureSize := 32;
      if (S='TEX64') then bgMaxTextureSize := 64;
      if (S='TEX128') then bgMaxTextureSize := 128;
      if (S='TEX256') then bgMaxTextureSize := 256;
    end;

    glEnable(GL_TEXTURE_1D);             // Enable Texture Mapping
    glEnable(GL_TEXTURE_2D);

    glClearColor(0.0, 0.0, 1.0, 0.0);    // Clear The Background Color To Blue
    glClearDepth(1.0);                   // Enables Clearing Of The Depth Buffer
    glDepthFunc(GL_LESS);                // The Type Of Depth Test To Do
    glEnable(GL_DEPTH_TEST);             // Enables Depth Testing
    glShadeModel(GL_SMOOTH);             // Enables Smooth Color Shading

    {$IFDEF GL_12} glLightModeli (GL_LIGHT_MODEL_COLOR_CONTROL,GL_SEPARATE_SPECULAR_COLOR); {$ENDIF}
    glEnable(GL_LIGHTING); // Enable Lighting
    LightEnable(0);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity;
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(bgViewMatrix));

    bgResizeViewport(Width, Height);
  end;



// init OpenGL window with given parameters
Function bgInitWindow(Left, Top, Width, Height : Integer; Title : ShortString;
                       FullScreen: Boolean; Parent : HWnd; WndProc : PWndProc) : Boolean;

  var
    wc: TWndClass;  // Windows Class Structure Used To Set Up The Type Of Window
    dmScreenSettings: DEVMODE;
    Screen: TRECT;
    Style : Integer;
    i, j : integer;
    S : String;

  begin
    if not InitOpenGL then begin
      ShowError('could not load OpenGL DLLs');
      Result := False;
      exit;
    end;

    // clear global data
    ZeroMemory(@bgLightFlare, sizeof(bgLightFlare));

    // make window class
    ZeroMemory(@wc, sizeof(wc));
    wc.style := CS_HREDRAW or CS_VREDRAW or CS_OWNDC;
    wc.lpfnWndProc := WndProc;
    wc.hInstance := hInstance;
    wc.hCursor := LoadCursor(0, bgMouseCursor);
    wc.lpszClassName := 'bgOpenGLClass';
    if(Windows.RegisterClass(wc)=0) then begin
      ShowError('could not create window class');
      Result := False;
      exit;
    end;

    // make window handle
    if FullScreen then
      Style := Integer(WS_POPUPWINDOW or WS_CLIPCHILDREN or WS_CLIPSIBLINGS)
    else
      Style := Integer(WS_OVERLAPPEDWINDOW or WS_CLIPCHILDREN or WS_CLIPSIBLINGS);
    if Parent <> 0 then
      Style := WS_CHILD or WS_DISABLED;
    if bgUseDlgFrame then Style := Style or WS_DLGFRAME;
    Title := Title + #0;
    bgWND := CreateWindow(
      'bgOpenGLClass',         // Class Name
      @Title[1],                // Title Appearing At The Top Of The Window
       Style,
       Left, Top,               // The Position Of The Window On The Screen
       Width, Height,           // The Width And Height Of The WIndow
       Parent, 0, hInstance, NIL);
    if (bgWND = 0) then begin
      ShowError('could not create window');
      Result := False;
      exit;
    end;

    // switch to fullscreen if needed
    if FullScreen and (Parent=0) then begin
      ZeroMemory( @dmScreenSettings, sizeof( DEVMODE ));
      With dmScreenSettings do begin
        dmSize := sizeof( DEVMODE );
        dmPelsWidth  := Width;  // Width
        dmPelsHeight := Height;  // Height
        dmFields     := DM_PELSWIDTH or DM_PELSHEIGHT; // Color Depth
      end;
      ChangeDisplaySettings(dmScreenSettings, CDS_FULLSCREEN); // Switch To Fullscreen Mode
    end;

    // get device context
    bgDC := GetDC(bgWND);  // Gets a Device Context For The Window

    // get rendering context
    if bgPixelFormat = 0 then begin
      for i := 1 To ParamCount do begin
        S := ParamStr(i);
        if Length(S)>2 then begin
          if (S[1]='P') and (S[2]='F') then begin
            Delete(S, 1, 2);
            Val(S, bgPixelFormat, j);
            if j<>0 then bgPixelFormat := 0;
          end
          else
          if (S[1]='Z') and (S[2]='B') then begin
            Delete(S, 1, 2);
            val(S, bgPFD.cDepthBits, j);
            if j<>0 then bgPFD.cDepthBits := 16;
          end;
        end;
      end;
    end; // if

    bgRC := CreateRenderingContext(bgDC, bgPixelFormat, bgPFD);
    if bgRC=0 then
      begin
        ShowError('could not create rendering context');
        SendMessage(bgWND, WM_CLOSE, 0, 0);
        Result := False;
        exit;
      end;
    ActivateRenderingContext(bgDC, bgRC);

    GetClientRect(bgWND, Screen);
    bgInitGL(Screen.right, Screen.bottom);

    if Parent<>0 then
      bgDisplayWindow;
    bgRun := True;
    bgViewInvertedTextureMatrixList := glGenLists(1);

    result := True;
  end;



// try to set nice OpenGL hints
procedure bgNiceHint;
begin
  if Assigned(glHint) then begin
    glHint(GL_FOG_HINT, GL_NICEST);
    glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
    glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
    {$IFDEF GL_13} if GL_13 then glHint(GL_TEXTURE_COMPRESSION_HINT, GL_NICEST); {$ENDIF}
    {$IFDEF GL_14} if GL_14 then glHint(GL_GENERATE_MIPMAP_HINT, GL_NICEST); {$ENDIF}
  end;
end;



// resize the viewport to a given size
procedure bgResizeViewport(Width: GLsizei; Height: GLsizei);

  var
    fWidth, fHeight: GLdouble;

  begin
    if (Height=0) then            // Prevent A Divide By Zero If The Window Is Too Small
      Height:=1;
    glViewport(0, 0, Width, Height);    // Reset The Current Viewport And Perspective Transformation

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    fWidth := Width;
    fHeight := Height;
    gluPerspective(bgFieldOfView,bgAspectRatio*fWidth/fHeight,bgZNear,bgZFar);

    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixf(Addr(bgViewMatrix));
    FrustumCalculate;
  end;



// prepare the object matrices
procedure bgSetMatrices;

  var
    l : PObjList;

  begin
    l := bgObjList;
    while l <> NIL do begin
      l.me.SetMatrix(NIL);
      l := l.next;
    end;
  end;



// when running in a client window rather use bgShowScene than bgDrawScene
procedure bgShowScene;

  begin
    InvalidateRect(bgWND, NIL, true);
    SendMessage(bgWND, WM_PAINT, 0, 0);
  end;



// swap screen and back buffer display
procedure bgSwapBuffers;

  begin
    SwapBuffers(bgDC);
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  view routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



// compute the view texture list for reflections
procedure ViewComputeInvertedTextureMatrixList;

  begin
    bgViewInvertedMatrix := bgViewMatrix;
    MatrixInvert(bgViewInvertedMatrix);
    bgViewTextureMatrix := bgViewInvertedMatrix;
    bgViewTextureMatrix[3][0] := 0.0;
    bgViewTextureMatrix[3][1] := 0.0;
    bgViewTextureMatrix[3][2] := 0.0;
    glNewList(bgViewInvertedTextureMatrixList, GL_COMPILE);
      glMatrixMode(GL_TEXTURE);
      glLoadMatrixf(Addr(bgViewTextureMatrix));
      glMatrixMode(GL_MODELVIEW);
    glEndList;
    glMatrixMode(GL_TEXTURE);
    glLoadIdentity;
    glMatrixMode(GL_MODELVIEW);
  end;



// returns the visible objects in a given rectangle
function ViewContainsAt(left,top,width,height: GLint; bt: PGLuintTable; max: integer) : integer;

  var
    l : PObjList;
    viewport : TVector4i;

  begin
    glSelectBuffer(max, PGLuint(bt));
    glRenderMode(GL_SELECT);
    glInitNames;
    glPushName(0);
    glGetIntegerv(GL_VIEWPORT, Addr(viewport));

    glMatrixMode(GL_PROJECTION);
    glPushMatrix;
    glLoadIdentity();
    gluPickMatrix(left, viewport[3]-top, width, height, viewport);
    gluPerspective(bgFieldOfView,bgAspectRatio*viewport[2]/viewport[3],bgZNear,bgZFar);

    l := bgObjList;
    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixf(Addr(bgViewMatrix));
    FrustumCalculate;
    while l <> NIL do begin
      l.me.FillSelectionBuffer;
      l := l.next;
    end;
    glMatrixMode(GL_PROJECTION);
    glPopMatrix;
    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixf(Addr(bgViewMatrix));
    FrustumCalculate;
    result := glRenderMode(GL_RENDER);
  end;



// returns a vector in direction of the view x axis
procedure ViewDirectionX(length : float; var v : TVec);

  begin
    v.x := length*bgViewMatrix[0,0];
    v.y := length*bgViewMatrix[1,0];
    v.z := length*bgViewMatrix[2,0];
  end;



// returns a vector in direction of the view y axis
procedure ViewDirectionY(length : float; var v : TVec);

  begin
    v.x := length*bgViewMatrix[0,1];
    v.y := length*bgViewMatrix[1,1];
    v.z := length*bgViewMatrix[2,1];
  end;



// returns a vector in direction of the view z axis
procedure ViewDirectionZ(length : float; var v : TVec);

  begin
    v.x := length*bgViewMatrix[0,2];
    v.y := length*bgViewMatrix[1,2];
    v.z := length*bgViewMatrix[2,2];
  end;



// translates the view
procedure ViewMove(x,y,z : float);

  begin
    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixf(Addr(bgViewMatrix));
    glTranslatef(x, y, z);
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(bgViewMatrix));
    FrustumCalculate;
    ViewComputeInvertedTextureMatrixList;
  end;



// sets the current perspective
procedure ViewPerspective(FoV, Aspect, NearZ, FarZ : GLDouble);

  var
    viewport : TVector4i;

  begin
    bgFieldOfView := FoV;
    bgAspectRatio := Aspect;
    bgZNear := NearZ;
    bgZFar := FarZ;

    glGetIntegerv(GL_VIEWPORT, Addr(viewport));
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity;
    gluPerspective(bgFieldOfView,bgAspectRatio*viewport[2]/viewport[3],bgZNear,bgZFar);

    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixf(Addr(bgViewMatrix));
    FrustumCalculate;
  end;



// rotates the view
procedure ViewRotate(xa,ya,za : float);

  begin
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity;
    glRotatef(xa,1.0,0.0,0.0);
    glRotatef(ya,0.0,1.0,0.0);
    glRotatef(za,0.0,0.0,1.0);
    glMultMatrixf(Addr(bgViewMatrix));
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(bgViewMatrix));
    FrustumCalculate;
    ViewComputeInvertedTextureMatrixList;
  end;



// sets the current view
procedure ViewSet(x,y,z, xa,ya,za : Float);

  begin
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity;
    glRotatef(xa,1.0,0.0,0.0);
    glRotatef(ya,0.0,1.0,0.0);
    glRotatef(za,0.0,0.0,1.0);
    glTranslatef(x, y, z);
    glGetFloatv(GL_MODELVIEW_MATRIX, Addr(bgViewMatrix));
    FrustumCalculate;
    ViewComputeInvertedTextureMatrixList;
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  light routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



procedure LightAmbient(index : integer; r, g, b, a : GLfloat);

  begin
    bgLight[index].ambient[0] := r;
    bgLight[index].ambient[1] := g;
    bgLight[index].ambient[2] := b;
    bgLight[index].ambient[3] := a;
    glLightfv(GL_LIGHT0 + index, GL_AMBIENT, Addr(bgLight[index].ambient));
  end;



procedure LightDiffuse(index : integer; r, g, b, a : GLfloat);

  begin
    bgLight[index].diffuse[0] := r;
    bgLight[index].diffuse[1] := g;
    bgLight[index].diffuse[2] := b;
    bgLight[index].diffuse[3] := a;
    glLightfv(GL_LIGHT0 + index, GL_DIFFUSE, Addr(bgLight[index].diffuse));
  end;



procedure LightSpecular(index : integer; r, g, b, a : GLfloat);

  begin
    bgLight[index].specular[0] := r;
    bgLight[index].specular[1] := g;
    bgLight[index].specular[2] := b;
    bgLight[index].specular[3] := a;
    glLightfv(GL_LIGHT0 + index, GL_SPECULAR, Addr(bgLight[index].specular));
  end;



procedure LightShadow(index: integer; r, g, b, a : GLfloat);

  begin
    bgLight[index].shadow[0] := r;
    bgLight[index].shadow[1] := g;
    bgLight[index].shadow[2] := b;
    bgLight[index].shadow[3] := a;
  end;



procedure LightPosition(index : integer; x, y, z : GLfloat);

  begin
    bgLight[index].position[0] := x;
    bgLight[index].position[1] := y;
    bgLight[index].position[2] := z;
    bgLight[index].position[3] := 1.0;
    glLightfv(GL_LIGHT0 + index, GL_POSITION, Addr(bgLight[index].position));
  end;



procedure LightDirection(index : integer; x, y, z : GLfloat);

  begin
    bgLight[index].position[0] := x;
    bgLight[index].position[1] := y;
    bgLight[index].position[2] := z;
    bgLight[index].position[3] := 0.0;
    glLightfv(GL_LIGHT0 + index, GL_POSITION, Addr(bgLight[index].position));
  end;



procedure LightSpotExponent(index : integer; v :  GLfloat);

  begin
    bgLight[index].spotexponent := v;
    glLightf(GL_LIGHT0 + index, GL_SPOT_EXPONENT, v);
  end;



procedure LightSpotCutOff(index : integer; v : GLfloat);

  begin
    bgLight[index].spotcutoff := v;
    glLightf(GL_LIGHT0 + index, GL_SPOT_CUTOFF, v);
  end;



procedure LightSpotDirection(index : integer; x, y, z : GLfloat);

  begin
    bgLight[index].spotdirection[0] := x;
    bgLight[index].spotdirection[1] := y;
    bgLight[index].spotdirection[2] := z;
    glLightfv(GL_LIGHT0 + index, GL_SPOT_DIRECTION, Addr(bgLight[index].spotdirection));
  end;



procedure LightConstantAttenuation(index : integer; v : GLfloat);

  begin
    bgLight[index].constantattenuation := v;
    glLightf(GL_LIGHT0 + index, GL_CONSTANT_ATTENUATION, v);
  end;



procedure LightLinearAttenuation(index : integer; v : GLfloat);

  begin
    bgLight[index].linearattenuation := v;
    glLightf(GL_LIGHT0 + index, GL_LINEAR_ATTENUATION, v);
  end;



procedure LightQuadraticAttenuation(index : integer; v : GLfloat);

  begin
    bgLight[index].quadraticattenuation := v;
    glLightf(GL_LIGHT0 + index, GL_QUADRATIC_ATTENUATION, v);
  end;



procedure LightEnable(index : integer);

  begin
    bgLight[index].enabled := True;
    glEnable(GL_LIGHT0 + index);
  end;



procedure LightDisable(index : integer);

  begin
    bgLight[index].enabled := False;
    glDisable(GL_LIGHT0 + index);
  end;



function LightAddFlare(index : integer; Position, HSize, VSize, cr,cg,cb,ca: float; Tex: TTexture2D): integer;

  var
    Flare: PLightFlare;
    i: integer;

  begin
    i := 0;
    if bgLightFlare[index].Flares = MAXLIGHTFLARES then begin
      result := MAXLIGHTFLARES;
      exit;
    end;
    while i<bgLightFlare[index].Flares do begin
      if bgLightFlare[index].Flare[i].Position > Position then begin
        Move(bgLightFlare[index].Flare[i], bgLightFlare[index].Flare[i+1], (bgLightFlare[index].Flares-i)*sizeof(TLightFlare));
        break;
      end;
      inc(i);
    end;
    inc(bgLightFlare[index].Flares);
    Flare := @bgLightFlare[index].Flare[i];
    Flare.Position := Position;
    Flare.HSize := HSize;
    Flare.VSize := VSize;
    Flare.Color.r := cr;
    Flare.Color.g := cg;
    Flare.Color.b := cb;
    Flare.Color.a := ca;
    Flare.Tex := Tex;
    result := i;
  end;



procedure LightFlareOcclusion(index: integer; HOccSize, VOccSize: integer);

  begin
    bgLightFlare[index].HOccSize := HOccSize;
    bgLightFlare[index].VOccSize := VOccSize;
  end;



procedure LightRemoveFlare(index, FlareIndex : integer);

  begin
    if FlareIndex < bgLightFlare[index].Flares then begin
      Move(bgLightFlare[index].Flare[FlareIndex+1],
           bgLightFlare[index].Flare[FlareIndex],
           (bgLightFlare[index].Flares - FlareIndex - 1) * sizeof(TLightFlare));
      dec(bgLightFlare[index].Flares);
    end;
  end;



procedure LightRemoveFlares(index : integer);

  begin
    FillChar(bgLightFlare[index], sizeof(TLightFlares), 0);
  end;



procedure LightDrawFlares;

  var
    i, l, f: integer;
    light: PLight;
    LightPos, CamPos: TVec;
    LightToCam: TVec;
    LightToCamLen: float;
    CamDir: TVec;
    Intersection: TVec;
    LightToIntersection: TVec;
    Flare: PLightFlare;
    FlarePos: TVec;
    XV, YV: TVec;
    Occlusion: float;
    rad: float;

  begin
{$IFDEF GL_13}
    if GL_13 then
    for i:=bgMaxTextureUnits-1  downto 0 do begin
      glActiveTexture(GL_TEXTURE0 + i);
      glDisable(GL_TEXTURE_1D);
      glDisable(GL_TEXTURE_2D);
      glDisable(GL_TEXTURE_3D);
      glDisable(GL_TEXTURE_CUBE_MAP);
    end;
{$ENDIF}
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    glDisable(GL_LIGHTING);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glDisable(GL_DEPTH_TEST);
    glEnable(GL_TEXTURE_2D);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);
    glMatrixMode(GL_MODELVIEW);
    glLoadMatrixf(addr(bgViewMatrix));

    for l := 0 to MAXLIGHTS-1 do begin
      light := @bgLight[l];
      LightPos := Vector(light.Position[0], light.Position[1], light.Position[2]);
      rad := 0.0;
      for f:=0 to bgLightFlare[l].Flares-1 do begin
        Flare := @bgLightFlare[l].Flare[f];
        if rad<Flare.HSize then rad:=Flare.HSize;
        if rad<Flare.VSize then rad:=Flare.VSize;
      end;
      if (light.Enabled) and
         (bgLightFlare[l].Flares>0) and
         (light.Position[3]=1.0) and
         (SphereInFrustum(LightPos.x, LightPos.y, LightPos.z, rad)) then begin

        if (bgLightFlare[l].HOccSize>0) and (bgLightFlare[l].VOccSize>0) then
          Occlusion := RectIsOccluded(LightPos, bgLightFlare[l].HOccSize, bgLightFlare[l].VOccSize)
        else
          Occlusion := 0.0;

        MatrixExtractTranslation(bgViewInvertedMatrix, CamPos.x, CamPos.y, CamPos.z);
        LightToCam := VectorSub(CamPos, TVec(addr(light.Position)^));
        LightToCamLen := VectorLength(LightToCam);
        CamDir := Vector(bgViewMatrix[0,2],bgViewMatrix[1,2],bgViewMatrix[2,2]);
        Intersection := VectorScale(CamDir, LightToCamLen);
        Intersection := VectorAdd(Intersection, CamPos);
        LightToIntersection := VectorSub(Intersection, LightPos);

        for f:=0 to bgLightFlare[l].Flares-1 do begin
          Flare := @bgLightFlare[l].Flare[f];
          FlarePos := VectorAdd(LightPos, VectorScale(LightToIntersection, Flare.Position));

          glPushMatrix;
          glTranslatef(FlarePos.x, FlarePos.y, FlarePos.z);
          XV := Vector(bgViewMatrix[0][0], bgViewMatrix[1][0], bgViewMatrix[2][0]);
          YV := Vector(bgViewMatrix[0][1], bgViewMatrix[1][1], bgViewMatrix[2][1]);
          XV := VectorScale(XV, Flare.HSize);
          YV := VectorScale(YV, Flare.VSize);
          Flare.Tex.Display;
          glColor4f(Flare.Color.r, Flare.Color.g, Flare.Color.b, Flare.Color.a * (1-Occlusion));
          glBegin(GL_QUADS);
            glTexCoord2f(0.0, 0.0);
            glVertex3f(-XV.x - YV.x, -XV.y - YV.y, -XV.z - YV.z);
            glTexCoord2f(1.0, 0.0);
            glVertex3f( XV.x - YV.x,  XV.y - YV.y,  XV.z - YV.z);
            glTexCoord2f(1.0, 1.0);
            glVertex3f( XV.x + YV.x,  XV.y + YV.y,  XV.z + YV.z);
            glTexCoord2f(0.0, 1.0);
            glVertex3f(-XV.x + YV.x, -XV.y + YV.y, -XV.z + YV.z);
          glEnd;
          glPopMatrix;
        end;
      end;
    end;

    glDisable(GL_BLEND);
    glEnable(GL_DEPTH_TEST);
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  fog routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



procedure FogColor(r, g, b, a : GLfloat);

  var
    fog : TVec4f;

  begin
    fog[0]:=r; fog[1]:=g; fog[2]:=b; fog[3]:=a;
    glFogfv(GL_FOG_COLOR, Addr(fog));
  end;



procedure FogDensity(density : GLfloat);

  begin
    glFogf(GL_FOG_DENSITY, density);
  end;



procedure FogStart(start : GLfloat);

  begin
    glFogf(GL_FOG_START, start);
  end;



procedure FogEnd(fogend : GLfloat);

  begin
    glFogf(GL_FOG_END, fogend);
  end;



procedure FogLinear;

  begin
    glFogi(GL_FOG_MODE, GL_LINEAR);
  end;



procedure FogExp;

  begin
    glFogi(GL_FOG_MODE, GL_EXP);
  end;



procedure FogExp2;

  begin
    glFogi(GL_FOG_MODE, GL_EXP2);
  end;



procedure FogEnable;

  begin
    glEnable(GL_FOG);
  end;



procedure FogDisable;

  begin
    glDisable(GL_FOG);
  end;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  frustum routines
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



// normalize a plane from a given frustum
procedure FrustumNormalizePlane(side : TFrustumSide);

  var
    magnitude : float;
    A, B, C : float;
    
  begin
    // magnitude = sqrt( x^2 + y^2 + z^2)
    A := bgFrustum[side][pdA];
    B := bgFrustum[side][pdB];
    C := bgFrustum[side][pdC];
    magnitude := sqrt(A*A + B*B + C*C);
    // divide the plane's values by it's magnitude.
    bgFrustum[side][pdA] := A / magnitude;
    bgFrustum[side][pdB] := B / magnitude;
    bgFrustum[side][pdC] := C / magnitude;
    bgFrustum[side][pdD] := bgFrustum[side][pdD] / magnitude;
  end;



// extracts frustum from the projection and modelview matrix
procedure FrustumCalculate;

  var
    proj,                            // projection matrix
    modl,                            // modelview matrix
    clip : array[0..15] of GLfloat; // clipping planes

  begin
    glGetFloatv( GL_PROJECTION_MATRIX, Addr(proj));
    glGetFloatv( GL_MODELVIEW_MATRIX, Addr(modl));

    // multiply modl with proj

    clip[ 0] := modl[ 0]*proj[ 0] + modl[ 1]*proj[ 4] + modl[ 2]*proj[ 8] + modl[ 3]*proj[12];
    clip[ 1] := modl[ 0]*proj[ 1] + modl[ 1]*proj[ 5] + modl[ 2]*proj[ 9] + modl[ 3]*proj[13];
    clip[ 2] := modl[ 0]*proj[ 2] + modl[ 1]*proj[ 6] + modl[ 2]*proj[10] + modl[ 3]*proj[14];
    clip[ 3] := modl[ 0]*proj[ 3] + modl[ 1]*proj[ 7] + modl[ 2]*proj[11] + modl[ 3]*proj[15];

    clip[ 4] := modl[ 4]*proj[ 0] + modl[ 5]*proj[ 4] + modl[ 6]*proj[ 8] + modl[ 7]*proj[12];
    clip[ 5] := modl[ 4]*proj[ 1] + modl[ 5]*proj[ 5] + modl[ 6]*proj[ 9] + modl[ 7]*proj[13];
    clip[ 6] := modl[ 4]*proj[ 2] + modl[ 5]*proj[ 6] + modl[ 6]*proj[10] + modl[ 7]*proj[14];
    clip[ 7] := modl[ 4]*proj[ 3] + modl[ 5]*proj[ 7] + modl[ 6]*proj[11] + modl[ 7]*proj[15];

    clip[ 8] := modl[ 8]*proj[ 0] + modl[ 9]*proj[ 4] + modl[10]*proj[ 8] + modl[11]*proj[12];
    clip[ 9] := modl[ 8]*proj[ 1] + modl[ 9]*proj[ 5] + modl[10]*proj[ 9] + modl[11]*proj[13];
    clip[10] := modl[ 8]*proj[ 2] + modl[ 9]*proj[ 6] + modl[10]*proj[10] + modl[11]*proj[14];
    clip[11] := modl[ 8]*proj[ 3] + modl[ 9]*proj[ 7] + modl[10]*proj[11] + modl[11]*proj[15];

    clip[12] := modl[12]*proj[ 0] + modl[13]*proj[ 4] + modl[14]*proj[ 8] + modl[15]*proj[12];
    clip[13] := modl[12]*proj[ 1] + modl[13]*proj[ 5] + modl[14]*proj[ 9] + modl[15]*proj[13];
    clip[14] := modl[12]*proj[ 2] + modl[13]*proj[ 6] + modl[14]*proj[10] + modl[15]*proj[14];
    clip[15] := modl[12]*proj[ 3] + modl[13]*proj[ 7] + modl[14]*proj[11] + modl[15]*proj[15];

    // extract the sides from clipping planes.

    bgFrustum[fsRight][pdA] := clip[ 3] - clip[ 0];
    bgFrustum[fsRight][pdB] := clip[ 7] - clip[ 4];
    bgFrustum[fsRight][pdC] := clip[11] - clip[ 8];
    bgFrustum[fsRight][pdD] := clip[15] - clip[12];
    FrustumNormalizePlane(fsRight);

    bgFrustum[fsLeft][pdA] := clip[ 3] + clip[ 0];
    bgFrustum[fsLeft][pdB] := clip[ 7] + clip[ 4];
    bgFrustum[fsLeft][pdC] := clip[11] + clip[ 8];
    bgFrustum[fsLeft][pdD] := clip[15] + clip[12];
    FrustumNormalizePlane(fsLeft);

    bgFrustum[fsBottom][pdA] := clip[ 3] + clip[ 1];
    bgFrustum[fsBottom][pdB] := clip[ 7] + clip[ 5];
    bgFrustum[fsBottom][pdC] := clip[11] + clip[ 9];
    bgFrustum[fsBottom][pdD] := clip[15] + clip[13];
    FrustumNormalizePlane(fsBottom);

    bgFrustum[fsTop][pdA] := clip[ 3] - clip[ 1];
    bgFrustum[fsTop][pdB] := clip[ 7] - clip[ 5];
    bgFrustum[fsTop][pdC] := clip[11] - clip[ 9];
    bgFrustum[fsTop][pdD] := clip[15] - clip[13];
    FrustumNormalizePlane(fsTop);

    bgFrustum[fsBack][pdA] := clip[ 3] - clip[ 2];
    bgFrustum[fsBack][pdB] := clip[ 7] - clip[ 6];
    bgFrustum[fsBack][pdC] := clip[11] - clip[10];
    bgFrustum[fsBack][pdD] := clip[15] - clip[14];
    FrustumNormalizePlane(fsBack);

    bgFrustum[fsFront][pdA] := clip[ 3] + clip[ 2];
    bgFrustum[fsFront][pdB] := clip[ 7] + clip[ 6];
    bgFrustum[fsFront][pdC] := clip[11] + clip[10];
    bgFrustum[fsFront][pdD] := clip[15] + clip[14];
    FrustumNormalizePlane(fsFront);
end;



// determines if a point is inside of the frustum
function PointInFrustum(x, y, z : float) : boolean;

  begin
    // go through all the sides of the frustum
    // Calculate the plane equation and check if the point is behind a side of the frustum
    if (bgFrustum[fsRight][pdA]*x + bgFrustum[fsRight][pdB]*y + bgFrustum[fsRight][pdC]*z + bgFrustum[fsRight][pdD] <= 0) then begin
      result := false;
      exit;
    end;
    if (bgFrustum[fsLeft][pdA]*x + bgFrustum[fsLeft][pdB]*y + bgFrustum[fsLeft][pdC]*z + bgFrustum[fsLeft][pdD] <= 0) then begin
      result := false;
      exit;
    end;
    if (bgFrustum[fsTop][pdA]*x + bgFrustum[fsTop][pdB]*y + bgFrustum[fsTop][pdC]*z + bgFrustum[fsTop][pdD] <= 0) then begin
      result := false;
      exit;
    end;
    if (bgFrustum[fsBottom][pdA]*x + bgFrustum[fsBottom][pdB]*y + bgFrustum[fsBottom][pdC]*z + bgFrustum[fsBottom][pdD] <= 0) then begin
      result := false;
      exit;
    end;
    if (bgFrustum[fsBack][pdA]*x + bgFrustum[fsBack][pdB]*y + bgFrustum[fsBack][pdC]*z + bgFrustum[fsBack][pdD] <= 0) then begin
      result := false;
      exit;
    end;
    if (bgFrustum[fsFront][pdA]*x + bgFrustum[fsFront][pdB]*y + bgFrustum[fsFront][pdC]*z + bgFrustum[fsFront][pdD] <= 0) then begin
      result := false;
      exit;
    end;
    result := true;
  end;



//  determines if a sphere is inside of the frustum
function SphereInFrustum(x, y, z, radius : float) : boolean;

  begin
    // Go through all the sides of the frustum
    // If the center of the sphere is farther away from the plane than the radius then the sphere is outside
    radius := -radius;
    if (bgFrustum[fsRight][pdA]*x + bgFrustum[fsRight][pdB]*y + bgFrustum[fsRight][pdC]*z + bgFrustum[fsRight][pdD] <= radius) then begin
      result := false;
      exit;
    end;
    if (bgFrustum[fsLeft][pdA]*x + bgFrustum[fsLeft][pdB]*y + bgFrustum[fsLeft][pdC]*z + bgFrustum[fsLeft][pdD] <= radius) then begin
      result := false;
      exit;
    end;
    if (bgFrustum[fsTop][pdA]*x + bgFrustum[fsTop][pdB]*y + bgFrustum[fsTop][pdC]*z + bgFrustum[fsTop][pdD] <= radius) then begin
      result := false;
      exit;
    end;
    if (bgFrustum[fsBottom][pdA]*x + bgFrustum[fsBottom][pdB]*y + bgFrustum[fsBottom][pdC]*z + bgFrustum[fsBottom][pdD] <= radius) then begin
      result := false;
      exit;
    end;
    if (bgFrustum[fsBack][pdA]*x + bgFrustum[fsBack][pdB]*y + bgFrustum[fsBack][pdC]*z + bgFrustum[fsBack][pdD] <= radius) then begin
      result := false;
      exit;
    end;
    if (bgFrustum[fsFront][pdA]*x + bgFrustum[fsFront][pdB]*y + bgFrustum[fsFront][pdC]*z + bgFrustum[fsFront][pdD] <= radius) then begin
      result := false;
      exit;
    end;
    result := true;
  end;



end.
